/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.IJobChangeEvent;
import org.eclipse.core.runtime.jobs.IJobManager;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.jobs.JobChangeAdapter;
import org.eclipse.jface.resource.FontRegistry;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.progress.UIJob;

import de.bsvrz.buv.plugin.konfigass.KonfigBereichDaten.EnumStatusKonfigBereich;
import de.bsvrz.buv.plugin.konfigass.assistenten.InfoKonfigAenderungAssistent;
import de.bsvrz.buv.plugin.konfigass.assistenten.InfoKonfigAenderungModell;
import de.bsvrz.buv.plugin.konfigass.assistenten.KbAuswahlAssistent;
import de.bsvrz.buv.plugin.konfigass.assistenten.KbAuswahlModell;
import de.bsvrz.buv.plugin.konfigass.assistenten.KbEntfernenHinzufuegenAssistent;
import de.bsvrz.buv.plugin.konfigass.assistenten.KbEntfernenHinzufuegenModell;
import de.bsvrz.buv.plugin.konfigass.assistenten.KbImportAssistent;
import de.bsvrz.buv.plugin.konfigass.assistenten.KbImportModell;
import de.bsvrz.buv.plugin.konfigass.assistenten.KvAoeNeuAssistent;
import de.bsvrz.buv.plugin.konfigass.assistenten.KvAoeNeuModell;
import de.bsvrz.buv.plugin.konfigass.assistenten.WizardDialog2;
import de.bsvrz.dav.daf.communication.dataRepresentation.AttributeBaseValueDataFactory;
import de.bsvrz.dav.daf.communication.dataRepresentation.AttributeHelper;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationAuthority;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.ObjectTimeSpecification;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.puk.config.configFile.datamodel.ConfigConfigurationArea;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigSystemObject;
import de.bsvrz.puk.config.main.managementfile.ConfigurationAreaManagementInfo;
import de.bsvrz.puk.config.main.managementfile.ConfigurationManagementFile;
import de.bsvrz.puk.config.main.managementfile.ManagementFile;
import de.bsvrz.puk.config.main.managementfile.VersionInfo;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Erzeugt die Darstellung die im Reiter Konfiguration angezeigt wird
 * 
 * @author Pittner
 * 
 */
public class KonfigAssDarstellungKonfiguration extends Composite
{

  private Debug _debug = Debug.getLogger(); // @jve:decl-index=0:

  private Composite _composite;

  private Composite composite1;

  private TableViewer _tvZufluesse1;

  private Composite composite = null;

  private Group group11 = null;

  private Composite composite6; // @jve:decl-index=0:visual-constraint="650,10"

  private Composite composite7;

  private Group group511 = null;

  private Composite composite22 = null;

  private Composite composite23 = null;

  private Composite composite24 = null;

  private Composite composite25 = null;

  private Composite composite26 = null;

  private Button buttonLadeKonfiguration = null;

  private Button buttonEntladeKonfiguration;

  private Button buttonNeuerKvAoe = null;

  private Button buttonKbNeu = null;

  private Button buttonKbEntfernen = null;

  private Button buttonKbHinzufuegen = null;

  private Button buttonImport = null;

  private Button buttonKonsistenzPruefung = null;
  
  private Button buttonUebepruefeAktivierung;

  private Button buttonAktivierung = null;

  private Button buttonFreigabeZurUebernahme = null;

  private Button buttonFreigabeZurAktivierung = null;

  private Button buttonFreigabeZurAktivierungOhne = null;

  private Button buttonExport = null;

  private Composite composite27 = null;

  private Composite composite28 = null;

  private Group group01 = null;

  private Label label11 = null;

  private Text text11 = null;

  private Label label31 = null;

  private Text text21 = null;

  private Composite composite211 = null;

  // private Button buttonAbbrechen = null;

  private XmlElementKonfigVerwaltung xmlVerwaltung = null;

  private Composite composite11 = null;

  private Composite composite2 = null;

  private Composite composite3 = null;

  private Group group1 = null;

  private ConfigDataModel _config;

  private String _pidAoeKonfig; // @jve:decl-index=0:

  private SortedMap<String, KonfigBereichDaten> _mapKonfigBereichKapsel = null; // @jve:decl-index=0:

  private List<KonfigBereichDaten> _listKonfigBereichKapsel = null; // @jve:decl-index=0:

  protected String _dateiPfad;
  
//Liste der fr diesen KA geffneten Extensions
  private List<String> _zugeordnetePluginIds = new LinkedList<String>();
  

  private final static Color color2 = new Color(null, 165, 165, 165);
//  
//  private final static Color color2 = new Color(null, 20, 240, 120);
//
//  private final static Color color3 = new Color(null, 255, 255, 255);

  private Thread _tAutoCloser; // @jve:decl-index=0:

  private IStructuredSelection _selection;
  
//  private int  _hbarsel = 0;
//  
//  private int  _vbarsel = 0;

  // private int[] _selIndex;

  private MultiStatus _status = null; // @jve:decl-index=0:

  private String _secondId = null;
  
  public KonfigAssDarstellungKonfiguration(Composite parent, int style, String secondId)
  {
    super(parent, style);
    _secondId = secondId;
    xmlVerwaltung = new XmlElementKonfigVerwaltung(_config);
    initialisiere();
  }
  
  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialisiere()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    this.setSize(new Point(770, 611));
    setLayoutData(gridData1);
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.FILL;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.heightHint = -1;
    gridData3.widthHint = -1;
    gridData3.horizontalIndent = 0;
    gridData3.verticalAlignment = GridData.FILL;

    _composite = new Composite(this, SWT.NONE);
    _composite.setLayoutData(gridData2);
    _composite.setLayout(gridLayout);

    erzeugeCompositeOben();
    erzeugeCompositeMitte();
    // erzeugeCompositeUnten();

    //Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp(this, Activator.PLUGIN_ID + ".konfiguration");
    
    updateButtonKeinDatenModell();
  }

  public TableViewer holeTableViewerKonfigurationsbereiche()
  {
    return _tvZufluesse1;
  }

  public ConfigDataModel getConfig()
  {
    return _config;
  }

  /**
   * This method initializes composite
   * 
   */
  private void erzeugeCompositeMitte()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;

    composite = new Composite(_composite, SWT.NONE);
    composite.setLayout(new GridLayout(2, false));
    composite.setLayoutData(gridData);

    erzeugeGroupLinks();
    erzeugeCompositeGroupRechts();
  }

  /**
   * This method initializes group11
   * 
   */
  private void erzeugeGroupLinks()
  {
    GridData gridData27 = new GridData();
    gridData27.horizontalAlignment = GridData.FILL;
    gridData27.grabExcessHorizontalSpace = true;
    gridData27.grabExcessVerticalSpace = true;
    gridData27.verticalAlignment = GridData.FILL;
    GridData gridData26 = new GridData();
    gridData26.horizontalAlignment = GridData.FILL;
    gridData26.grabExcessHorizontalSpace = true;
    gridData26.grabExcessVerticalSpace = true;
    gridData26.verticalAlignment = GridData.FILL;
    GridLayout gridLayout3 = new GridLayout();
    gridLayout3.numColumns = 2;
    GridLayout gridLayout2 = new GridLayout();
    gridLayout2.numColumns = 1;
    GridData gridData7 = new GridData();
    gridData7.horizontalAlignment = GridData.FILL;
    gridData7.grabExcessHorizontalSpace = true;
    gridData7.grabExcessVerticalSpace = true;
    gridData7.verticalAlignment = GridData.FILL;
    GridData gridData8 = new GridData();
    gridData8.horizontalAlignment = GridData.FILL;
    gridData8.grabExcessHorizontalSpace = true;
    gridData8.grabExcessVerticalSpace = true;
    gridData8.verticalAlignment = GridData.FILL;
    GridData gridData6 = new GridData();
    gridData6.grabExcessHorizontalSpace = true;
    gridData6.grabExcessVerticalSpace = true;
    gridData6.horizontalAlignment = GridData.FILL;
    gridData6.verticalAlignment = GridData.FILL;
    composite6 = new Composite(composite, SWT.NONE);
    composite6.setLayout(gridLayout2);
    composite6.setLayoutData(gridData8);

    group11 = new Group(composite6, SWT.NONE);
    group11.setLayout(new GridLayout());
    group11.setText("Konfigurationsbereiche");
    group11.setLayoutData(gridData6);

    erzeugeCompositeGroupLinks();

  }

  /**
   * This method initializes group11
   * 
   */
  private void erzeugeCompositeGroupRechts()
  {
    GridLayout gridLayout2 = new GridLayout();
    gridLayout2.marginWidth = 5;
    gridLayout2.horizontalSpacing = 5;
    GridData gridData7 = new GridData();
    gridData7.widthHint = 252;
    gridData7.horizontalAlignment = SWT.FILL;
    gridData7.grabExcessVerticalSpace = true;
    gridData7.verticalAlignment = GridData.FILL;
    composite7 = new Composite(composite, SWT.NONE);
    composite7.setLayout(gridLayout2);
    composite7.setLayoutData(gridData7);

    erzeugeGroupOperationen();

    erzeugeTabelleKonfigurationsbereiche();
  }

  /**
   * This method initializes group511
   * 
   */
  private void erzeugeGroupOperationen()
  {
    GridLayout gridLayout4 = new GridLayout();
    gridLayout4.marginWidth = 5;
    GridData gridData8 = new GridData();
    gridData8.widthHint = 241;
    gridData8.heightHint = 373;
    gridData8.horizontalAlignment = SWT.FILL;
    gridData8.grabExcessVerticalSpace = true;
    gridData8.grabExcessHorizontalSpace = true;
    gridData8.verticalAlignment = GridData.FILL;
    group511 = new Group(composite7, SWT.NONE);
    group511.setText("Operationen");
    group511.setLayoutData(gridData8);
    createCompositeOperationen1();
    createCompositeOperationen2();
    createCompositeOperationen3();
    createCompositeOperationen4();
    group511.setLayout(gridLayout4);
    createCompositeOperationen5();
  }

  /**
   * This method initializes composite22
   * 
   */
  private void createCompositeOperationen1()
  {
    GridData gridData4 = new GridData();
    gridData4.horizontalAlignment = GridData.FILL;
    gridData4.grabExcessHorizontalSpace = true;
    gridData4.verticalAlignment = GridData.CENTER;
    GridData gridData17 = new GridData();
    gridData17.horizontalAlignment = SWT.FILL;
    gridData17.widthHint = 214;
    gridData17.verticalAlignment = SWT.FILL;
    GridData gridData10 = new GridData();
    gridData10.widthHint = 215;
    gridData10.horizontalAlignment = GridData.FILL;
    gridData10.grabExcessHorizontalSpace = true;
    gridData10.grabExcessVerticalSpace = true;
    gridData10.verticalAlignment = GridData.FILL;
    composite22 = new Composite(group511, SWT.NONE);
    composite22.setLayout(new GridLayout());
    composite22.setLayoutData(gridData10);
    buttonNeuerKvAoe = new Button(composite22, SWT.NONE);
    buttonNeuerKvAoe.setText("Neuer KV/AOE...");
    buttonNeuerKvAoe.setAlignment(SWT.LEFT);
    buttonNeuerKvAoe.setLayoutData(gridData4);
    buttonNeuerKvAoe.setLayoutData(gridData17);
    buttonNeuerKvAoe.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        updateButtonJobAktiv();
        aktionNeuerKvAoe();
      }
    });
  }

  /**
   * This method initializes composite23
   * 
   */
  private void createCompositeOperationen2()
  {
    GridData gridData25 = new GridData();
    gridData25.widthHint = 204;
    gridData25.grabExcessHorizontalSpace = true;
    gridData25.verticalAlignment = GridData.CENTER;
    gridData25.horizontalAlignment = GridData.FILL;
    GridData gridData16 = new GridData();
    gridData16.horizontalAlignment = GridData.FILL;
    gridData16.verticalAlignment = GridData.CENTER;
    GridData gridData15 = new GridData();
    gridData15.widthHint = 203;
    gridData15.horizontalAlignment = SWT.FILL;
    gridData15.verticalAlignment = GridData.CENTER;
    GridData gridData11 = new GridData();
    gridData11.horizontalAlignment = GridData.FILL;
    gridData11.grabExcessHorizontalSpace = true;
    gridData11.grabExcessVerticalSpace = true;
    gridData11.verticalAlignment = GridData.FILL;
    composite23 = new Composite(group511, SWT.NONE);
    composite23.setLayout(new GridLayout());
    composite23.setLayoutData(gridData11);
    buttonKbNeu = new Button(composite23, SWT.NONE);
    buttonKbNeu.setText("KB Neu...");
    buttonKbNeu.setAlignment(SWT.LEFT);
    buttonKbNeu.setLayoutData(gridData15);
    buttonKbNeu.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionKbNeu();

      }
    });
    buttonKbEntfernen = new Button(composite23, SWT.NONE);
    buttonKbEntfernen.setText("KB Entfernen...");
    buttonKbEntfernen.setAlignment(SWT.LEFT);
    buttonKbEntfernen.setLayoutData(gridData16);
    buttonKbEntfernen.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionKbEntfernen();
      }
    });
    buttonKbHinzufuegen = new Button(composite23, SWT.NONE);
    buttonKbHinzufuegen.setText("KB Hinzufgen...");
    buttonKbHinzufuegen.setAlignment(SWT.LEFT);
    buttonKbHinzufuegen.setLayoutData(gridData25);
    buttonKbHinzufuegen.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionKbHinzufuegen();
      }
    });
  }

  /**
   * This method initializes composite24
   * 
   */
  private void createCompositeOperationen3()
  {
    GridData gridData21 = new GridData();
    gridData21.horizontalAlignment = GridData.FILL;
    gridData21.grabExcessHorizontalSpace = true;
    gridData21.verticalAlignment = GridData.CENTER;
    GridData gridData20 = new GridData();
    gridData20.horizontalAlignment = GridData.FILL;
    gridData20.grabExcessHorizontalSpace = true;
    gridData20.verticalAlignment = GridData.CENTER;
    GridData gridData19 = new GridData();
    gridData19.horizontalAlignment = GridData.FILL;
    gridData19.grabExcessHorizontalSpace = true;
    gridData19.verticalAlignment = GridData.CENTER;
    GridData gridData18 = new GridData();
    gridData18.widthHint = 222;
    gridData18.horizontalAlignment = GridData.FILL;
    gridData18.grabExcessHorizontalSpace = true;
    gridData18.verticalAlignment = GridData.CENTER;
    GridData gridData12 = new GridData();
    gridData12.widthHint = 215;
    gridData12.horizontalAlignment = GridData.FILL;
    gridData12.grabExcessHorizontalSpace = true;
    gridData12.grabExcessVerticalSpace = true;
    gridData12.verticalAlignment = GridData.FILL;
    composite24 = new Composite(group511, SWT.NONE);
    composite24.setLayout(new GridLayout());
    composite24.setLayoutData(gridData12);
    buttonImport = new Button(composite24, SWT.NONE);
    buttonImport.setText("Import...");
    buttonImport.setAlignment(SWT.LEFT);
    buttonImport.setLayoutData(gridData18);
    buttonImport.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        updateButtonJobAktiv();
        aktionImport();
      }
    });
    buttonKonsistenzPruefung = new Button(composite24, SWT.NONE);
    buttonKonsistenzPruefung.setText("Konsistenzprfung");
    buttonKonsistenzPruefung.setAlignment(SWT.LEFT);
    buttonKonsistenzPruefung.setLayoutData(gridData19);
    buttonKonsistenzPruefung.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        updateButtonJobAktiv();
        aktionKonsistenzPruefung();
      }
    });
  
    buttonUebepruefeAktivierung = new Button(composite24, SWT.NONE);
    buttonUebepruefeAktivierung.setText("\u00DCberpr\u00FCfe Aktivierung");     
    buttonUebepruefeAktivierung.setAlignment(SWT.LEFT);
    buttonUebepruefeAktivierung.setLayoutData(gridData21);
    buttonUebepruefeAktivierung.addSelectionListener(new SelectionAdapter() {
     @Override
      public void widgetSelected(SelectionEvent e)
      {
        updateButtonJobAktiv();
        aktionPruefungAktivierung();
      }
    });
 
    buttonAktivierung = new Button(composite24, SWT.NONE);
    buttonAktivierung.setText("Aktivierung...");
    buttonAktivierung.setAlignment(SWT.LEFT);
    buttonAktivierung.setLayoutData(gridData20);
    buttonAktivierung.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        updateButtonJobAktiv();
        aktionAktivierung();
      }
    });

  }

  /**
   * This method initializes composite25
   * 
   */
  private void createCompositeOperationen4()
  {
    GridData gridData23 = new GridData();
    gridData23.widthHint = 205;
    gridData23.horizontalAlignment = GridData.FILL;
    GridData gridData22 = new GridData();
    gridData22.horizontalAlignment = GridData.FILL;
    gridData22.grabExcessHorizontalSpace = true;
    gridData22.verticalAlignment = GridData.CENTER;
    GridData gridData21 = new GridData();
    gridData21.horizontalAlignment = GridData.FILL;
    gridData21.grabExcessHorizontalSpace = true;
    gridData21.verticalAlignment = GridData.CENTER;
    GridData gridData13 = new GridData();
    gridData13.widthHint = 212;
    gridData13.horizontalAlignment = GridData.FILL;
    gridData13.grabExcessVerticalSpace = true;
    gridData13.verticalAlignment = GridData.FILL;
    composite25 = new Composite(group511, SWT.NONE);
    composite25.setLayout(new GridLayout());
    composite25.setLayoutData(gridData13);
    buttonFreigabeZurUebernahme = new Button(composite25, SWT.NONE);
    buttonFreigabeZurUebernahme.setText("Freigabe zur bernahme...");
    buttonFreigabeZurUebernahme.setAlignment(SWT.LEFT);
    buttonFreigabeZurUebernahme.setLayoutData(gridData21);
    buttonFreigabeZurUebernahme.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        updateButtonJobAktiv();
        aktionFreigabeZurUebernahme();
      }
    });
    buttonFreigabeZurAktivierung = new Button(composite25, SWT.NONE);
    buttonFreigabeZurAktivierung.setText("Freigabe zur Aktivierung...");
    buttonFreigabeZurAktivierung.setAlignment(SWT.LEFT);
    buttonFreigabeZurAktivierung.setLayoutData(gridData22);
    buttonFreigabeZurAktivierung.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        updateButtonJobAktiv();
        aktionFreigabeZurAktivierung();
      }
    });
    buttonFreigabeZurAktivierungOhne = new Button(composite25, SWT.NONE);
    buttonFreigabeZurAktivierungOhne.setText("F.z.A. ohne lokale Aktivierung durch KV");
    buttonFreigabeZurAktivierungOhne.setAlignment(SWT.LEFT);
    buttonFreigabeZurAktivierungOhne.setLayoutData(gridData23);
    buttonFreigabeZurAktivierungOhne.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        updateButtonJobAktiv();
        aktionFreigabeZurAktivierungKv();
      }
    });

  }

  /**
   * This method initializes composite26
   * 
   */
  private void createCompositeOperationen5()
  {
    GridData gridData24 = new GridData();
    gridData24.grabExcessHorizontalSpace = true;
    gridData24.widthHint = 208;
    gridData24.horizontalAlignment = GridData.FILL;
    GridData gridData14 = new GridData();
    gridData14.horizontalAlignment = SWT.FILL;
    gridData14.grabExcessVerticalSpace = true;
    gridData14.verticalAlignment = SWT.FILL;
    composite26 = new Composite(group511, SWT.NONE);
    composite26.setLayout(new GridLayout());
    composite26.setLayoutData(gridData14);
    buttonExport = new Button(composite26, SWT.NONE);
    buttonExport.setText("Export");
    buttonExport.setAlignment(SWT.LEFT);
    buttonExport.setLayoutData(gridData24);
    buttonExport.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        updateButtonJobAktiv();
        aktionExport();
      }
    });
  }

  /**
   * This method initializes composite27
   * 
   */
  private void erzeugeCompositeOben()
  {
    GridData gridData28 = new GridData();
    gridData28.horizontalAlignment = GridData.FILL;
    gridData28.grabExcessHorizontalSpace = true;
    gridData28.verticalAlignment = GridData.CENTER;
    composite27 = new Composite(_composite, SWT.NONE);
    composite27.setLayout(new GridLayout());
    erzeugeCompositeGroupKonfiguration();
    composite27.setLayoutData(gridData28);
  }

  /**
   * This method initializes composite28
   * 
   */
  private void erzeugeCompositeGroupKonfiguration()
  {
    GridData gridData29 = new GridData();
    gridData29.horizontalAlignment = GridData.FILL;
    gridData29.grabExcessHorizontalSpace = true;
    gridData29.verticalAlignment = GridData.CENTER;
    composite28 = new Composite(composite27, SWT.NONE);
    composite28.setLayout(new GridLayout());
    erzeugeGroupKonfiguration();
    composite28.setLayoutData(gridData29);
  }

  /**
   * This method initializes group01
   * 
   */
  private void erzeugeGroupKonfiguration()
  {
    GridData gridData30 = new GridData();
    gridData30.horizontalAlignment = GridData.FILL;
    gridData30.grabExcessHorizontalSpace = true;
    gridData30.verticalAlignment = GridData.CENTER;
    GridData gridData51 = new GridData();
    gridData51.horizontalAlignment = GridData.FILL;
    gridData51.grabExcessHorizontalSpace = true;
    gridData51.verticalAlignment = GridData.CENTER;
    GridData gridData41 = new GridData();
    gridData41.horizontalAlignment = GridData.FILL;
    gridData41.grabExcessHorizontalSpace = true;
    gridData41.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout11 = new GridLayout();
    gridLayout11.numColumns = 4;
    gridLayout11.makeColumnsEqualWidth = false;
    gridLayout11.marginHeight = 5;
    gridLayout11.marginWidth = 5;
    gridLayout11.verticalSpacing = 5;
    gridLayout11.horizontalSpacing = 5;
    group01 = new Group(composite28, SWT.NONE);
    group01.setText("Konfiguration");
    group01.setLayoutData(gridData30);
    group01.setLayout(gridLayout11);
    label11 = new Label(group01, SWT.NONE);
    label11.setText("Konfiguration (Verwaltungsdatei)");
    text11 = new Text(group01, SWT.BORDER);
    text11.setLayoutData(gridData41);
    buttonLadeKonfiguration = new Button(group01, SWT.NONE);
    buttonLadeKonfiguration.setText("...");
    buttonLadeKonfiguration.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {

        // System.err.print("Dies ist eine separate Ausgabe!");

        composite1.layout(true);

        FileDialog dialog = new FileDialog(Display.getCurrent().getActiveShell(), SWT.APPLICATION_MODAL
            | SWT.DIALOG_TRIM | SWT.RESIZE | SWT.OPEN);
        dialog.setFilterNames(new String[] { "XML-Dateien" });
        dialog.setFilterExtensions(new String[] { "*.xml" });
       
        if (_dateiPfad != null)
        {  
          File temp = new File(_dateiPfad);
          String path = temp.getParent();
          if (path != null)
          {
            temp = new File(path + "/../" + "config");
            if (temp.exists())
            {
              path += "/../" + "config";
            }
            else
            {  
              temp = new File(path + "/../" + "konfiguration");
              if (temp.exists())
              {
                path += "/../" + "konfiguration";
              }
            }  
            dialog.setFilterPath(path);
          }
        }
//        else
//        {
//          String dateiPfad = System.getProperty("ConfigurationPathBenutzer");
//          if (dateiPfad == null)
//          {
//            dateiPfad = System.getProperty("user.dir");
//          }
//          dialog.setFilterPath(dateiPfad);
//        }
          
        String s = dialog.open();

        if (s != null)
        {
          _dateiPfad = s.replace("\\", "/");
          updateButtonJobAktiv();
          aktionLadeDatenModell(true, null);
        }
        else
        {
          if (_config != null)
            updateButtonKeineSelektion();
          else
            updateButtonKeinDatenModell();
        }

      }
    });
    GridData gridData = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
    gridData.widthHint = 24;
    buttonEntladeKonfiguration = new Button(group01, SWT.NONE);
    buttonEntladeKonfiguration.setLayoutData(gridData);
    buttonEntladeKonfiguration.setText("X");
    buttonEntladeKonfiguration.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionEntladeDatenModell();
      }
    });

    label31 = new Label(group01, SWT.NONE);
    label31.setText("Konfigurationsverantwortlicher");
    text21 = new Text(group01, SWT.BORDER);
    text21.setLayoutData(gridData51);
    text21.setEditable(false);
    new Label(group01, SWT.NONE);
    new Label(group01, SWT.NONE);
  }

  public List<KonfigBereichDaten> getListKonfigBereichKapsel()
  {
    return _listKonfigBereichKapsel;
  }

  /**
   * Erzeugt das Offline-Datenmodell der Konfiguration aus der XML-Verwaltungsdatei
   * 
   * @param configurationFile
   *          die Verwaltungsdatei
   * @return das Offline-Datenmodell
   */
  private ConfigDataModel erzeugeKonfigDatenModellAusDatei(final File configurationFile)
  {
    try
    {
      final ConfigDataModel configuration = new ConfigDataModel(configurationFile);

      Runnable autoCloser = new Runnable() {
        public void run()
        {

//          System.out.println("AutoCloser: close!!!");
          configuration.close();

//          Display.getDefault().asyncExec(new Runnable() {
//
//            @Override
//            public void run()
//            {
//              ConfigurationManagementFile managementFile = configuration.getManagementFile();
//
//              List<ConfigurationAreaManagementInfo> list = managementFile.getAllConfigurationAreaManagementInfos();
//
//              for (ConfigurationAreaManagementInfo cami : list)
//              {
//                File dir = cami.getDirectory();
//                if (dir.isDirectory())
//                {
//
//                  String[] fileNames = dir.list();
//                  for (String fileName : fileNames)
//                  {
//                    if (fileName.contains(".config.lock"))
//                    {
//                      String sub = fileName.substring(0, fileName.indexOf(".config.lock"));
//                      if (cami.getPid().equals(sub))
//                      {
//                        System.out.println("Lock noch vorhanden nach dem Schliessen der Konfiguration: " + fileName);
//                        File lock = new File(fileName);
//                        System.out.println("Lock vorhanden: " + lock.exists());
//                        lock.delete();
//                        System.out.println("Lschen erfolgreich: " + lock.exists());
//                      }
//                    }
//                    else if (fileName.contains(".xml.lock"))
//                    {
//                      System.out.println("Lock noch vorhanden nach dem Schliessen der Konfiguration: " + fileName);
//                      File lock = new File(fileName);
//                      System.out.println("Lock vorhanden: " + lock.exists());
//                      lock.delete();
//                      System.out.println("Lschen erfolgreich: " + lock.exists());
//                    }
//                  }
//                }
//              }
//
//            }
//          });
        }
      };

      if (_tAutoCloser != null)
      {
        Runtime.getRuntime().removeShutdownHook(_tAutoCloser);
      }

      _tAutoCloser = new Thread(autoCloser);

      Runtime.getRuntime().addShutdownHook(_tAutoCloser);

      return configuration;
    }
    catch (RuntimeException e)
    {
      if (_tAutoCloser != null)
      {
        Runtime.getRuntime().removeShutdownHook(_tAutoCloser);
        _tAutoCloser = null;
      }

      _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID, "Fehler beim Laden des Datenmodells", e));
    }

    return null;

  }

  // /**
  // * This method initializes composite211
  // *
  // */
  // private void erzeugeCompositeAbbruch()
  // {
  // GridData gridData110 = new GridData();
  // gridData110.horizontalAlignment = GridData.END;
  // gridData110.grabExcessHorizontalSpace = true;
  // gridData110.verticalAlignment = GridData.CENTER;
  // GridData gridData2 = new GridData();
  // gridData2.horizontalAlignment = GridData.FILL;
  // gridData2.grabExcessHorizontalSpace = true;
  // gridData2.verticalAlignment = GridData.CENTER;
  // // composite211 = new Composite(composite29, SWT.NONE);
  // composite211 = new Composite(group1, SWT.NONE);
  // composite211.setLayout(new GridLayout());
  // composite211.setLayoutData(gridData2);
  // buttonAbbrechen = new Button(composite211, SWT.NONE);
  // buttonAbbrechen.setText("Abbruch");
  // buttonAbbrechen.setLayoutData(gridData110);
  // buttonAbbrechen.setEnabled(false);
  // buttonAbbrechen.addSelectionListener(new SelectionAdapter() {
  // @Override
  // public void widgetSelected(SelectionEvent e)
  // {
  // aktionAbbruch();
  // }
  // });
  //
  // }

  private Job _currentJob = null;

  private ISelectionChangedListener _selChangedlistener;
  

  private void erzeugeTabelleKonfigurationsbereiche()
  {
    GridData gridData111 = new GridData();
    gridData111.grabExcessVerticalSpace = true;
    gridData111.heightHint = 321;
    gridData111.horizontalAlignment = GridData.FILL;
    gridData111.grabExcessHorizontalSpace = true;
    gridData111.verticalAlignment = GridData.FILL;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    GridLayout gridLayout = new GridLayout();
    composite1 = new Composite(composite11, SWT.None);
    composite1.setLayout(gridLayout);
    composite1.setLayoutData(gridData);

    _tvZufluesse1 = new TableViewer(composite1, SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvZufluesse1.setContentProvider(new TabelleKonfigurationsbereicheInhaltsVersorger());

    TableViewerColumn tc1 = new TableViewerColumn(_tvZufluesse1, SWT.NONE);
    tc1.getColumn().setText("Modelle");
    tc1.getColumn().setToolTipText("Modelldefinitionen vorhanden");

    TableViewerColumn tc2 = new TableViewerColumn(_tvZufluesse1, SWT.NONE);
    tc2.getColumn().setText("Objekte");
    tc2.getColumn().setToolTipText("Objektdefinitionen vorhanden");

    TableViewerColumn tc3 = new TableViewerColumn(_tvZufluesse1, SWT.NONE);
    tc3.getColumn().setText("KV");
    tc3.getColumn().setToolTipText("Konfigurationsverantworlicher");

    TableViewerColumn tc4 = new TableViewerColumn(_tvZufluesse1, SWT.NONE);
    tc4.getColumn().setText("KB");
    tc4.getColumn().setToolTipText("Konfigurationsbereich");

    TableViewerColumn tc5 = new TableViewerColumn(_tvZufluesse1, SWT.NONE);
    tc5.getColumn().setText("Zeitp. aktiv. Vers.");
    tc5.getColumn().setToolTipText("Zeitpunkt aktivierte Version");

    TableViewerColumn tc6 = new TableViewerColumn(_tvZufluesse1, SWT.NONE);
    tc6.getColumn().setText("Aktivierbare Vers.");
    tc6.getColumn().setToolTipText("Aktivierbare Version");

    TableViewerColumn tc7 = new TableViewerColumn(_tvZufluesse1, SWT.NONE);
    tc7.getColumn().setText("Aktuelle Vers.");
    tc7.getColumn().setToolTipText("Aktuelle Version");

    TableViewerColumn tc8 = new TableViewerColumn(_tvZufluesse1, SWT.NONE);
    tc8.getColumn().setText("Vers. F.z.bern.");
    tc8.getColumn().setToolTipText("Version der Freigabe zur bernahme");

    TableViewerColumn tc9 = new TableViewerColumn(_tvZufluesse1, SWT.NONE);
    tc9.getColumn().setText("Vers. F.z.Aktiv.");
    tc9.getColumn().setToolTipText("Version der Freigabe zur Aktivierung");

    TableViewerColumn tc10 = new TableViewerColumn(_tvZufluesse1, SWT.NONE);
    tc10.getColumn().setText("Vers. F.z.Akt.o.l.A");
    tc10.getColumn().setToolTipText(
        "Version der Freigabe zur Aktivierung ohne lokale Aktivierung durch den Konfigurationsverantwortlichen");

//    TableViewerColumn tc11 = new TableViewerColumn(_tvZufluesse1, SWT.NONE);
//    tc11.getColumn().setText("Status");
//    tc11.getColumn().setToolTipText("Status des Konfigurationsbereiches");

    TableViewerColumn tc12 = new TableViewerColumn(_tvZufluesse1, SWT.NONE);
    tc12.getColumn().setText("Pfad");
    tc12.getColumn().setToolTipText("Pfadangabe zur *.config-Datei");

    _tvZufluesse1.getTable().setLinesVisible(true);
    _tvZufluesse1.getTable().setLayoutData(gridData111);
    _tvZufluesse1.getTable().setHeaderVisible(true);

    _selChangedlistener = new ISelectionChangedListener() {

      @Override
      public void selectionChanged(SelectionChangedEvent event)
      {
        // TODO Auto-generated method stub

        IStructuredSelection selection = (IStructuredSelection)_tvZufluesse1.getSelection();

        if (selection != null && !selection.isEmpty()){
          _selection = selection;
          Object obj = _selection.getFirstElement();
          if (obj instanceof KonfigBereichDaten)
          {
            Iterator it = _selection.iterator();
            boolean selektionMitRechte = true;
            
            while (it.hasNext())
            {
              KonfigBereichDaten kbd = (KonfigBereichDaten)it.next();
              if (!kbd.getBearbeitung())
              {
                selektionMitRechte = false;
              }
            }
            updateButtonSelektion(selektionMitRechte);
          }
          else
          {
            updateButtonKeineSelektion();
          }
        }
         

      }
    };
    
    
    _tvZufluesse1.addSelectionChangedListener(_selChangedlistener);

    TableColumn[] columns = _tvZufluesse1.getTable().getColumns();
    for (TableColumn tc : columns)
    {
      tc.pack();
    }
    
//    final ScrollBar hbar = _tvZufluesse1.getTable().getHorizontalBar();
//    final ScrollBar vbar = _tvZufluesse1.getTable().getVerticalBar();
//    
//    SelectionListener listener1 = new SelectionAdapter () {
//      @Override
//      public void widgetSelected(SelectionEvent e)
//      {
//        _hbarsel = hbar.getSelection();
//        
//      }
//    };
//    SelectionListener listener2 = new SelectionAdapter () {
//      @Override
//      public void widgetSelected(SelectionEvent e)
//      {
//        _vbarsel = vbar.getSelection();
//      }
//    };
//    
//
//    
//    hbar.addSelectionListener(listener1);
//    vbar.addSelectionListener(listener2);

  }

  /**
   * * {@link LabelProvider} fr die Tabelle.
   * 
   * @author Pittner
   * 
   */
  private class TabelleKonfigurationsbereicheInhaltsVersorger implements IStructuredContentProvider
  {

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
     */
    public Object[] getElements(Object inputElement)
    {
      java.util.List list = (java.util.List)inputElement;

      Object[] re = new Object[list.size()];

      for (int i = 0; i < list.size(); i++)
      {
        re[i] = list.get(i);
      }

      return re;
    }

    /*
     * (non-Javadoc) *
     * 
     * @see org.eclipse.jface.viewers.IContentProvider#dispose()
     */
    public void dispose()
    {
      // System.setOut(outBackup);
      // System.setErr(errBackup);
    }

    /*
     * (non-Javadoc) *
     * 
     * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object,
     * java.lang.Object)
     */
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {

    }
  }

  /**
   * This method initializes composite11
   * 
   */
  private void erzeugeCompositeGroupLinks()
  {
    GridData gridData9 = new GridData();
    gridData9.heightHint = 355;
    gridData9.horizontalAlignment = GridData.FILL;
    gridData9.grabExcessHorizontalSpace = true;
    gridData9.grabExcessVerticalSpace = true;
    gridData9.verticalAlignment = GridData.FILL;
    composite11 = new Composite(group11, SWT.NONE);
    // FillLayout fillLayout = new FillLayout();
    // fillLayout.type = org.eclipse.swt.SWT.HORIZONTAL;
    composite11.setLayout(new GridLayout());
    composite11.setLayoutData(gridData9);
  }

  // /**
  // * This method initializes composite2
  // *
  // */
  // private void erzeugeCompositeUnten()
  // {
  // GridData gridData32 = new GridData();
  // gridData32.horizontalAlignment = GridData.FILL;
  // gridData32.grabExcessHorizontalSpace = true;
  // gridData32.verticalAlignment = GridData.CENTER;
  // composite2 = new Composite(_composite, SWT.NONE);
  // composite2.setLayout(new GridLayout());
  // erzeugeCompositeUnten2();
  // composite2.setLayoutData(gridData32);
  // }

  // /**
  // * This method initializes composite3
  // *
  // */
  // private void erzeugeCompositeUnten2()
  // {
  // GridData gridData33 = new GridData();
  // gridData33.horizontalAlignment = GridData.FILL;
  // gridData33.grabExcessHorizontalSpace = true;
  // gridData33.verticalAlignment = GridData.CENTER;
  // composite3 = new Composite(composite2, SWT.NONE);
  // composite3.setLayout(new GridLayout());
  // erzeugeGroupUnten();
  // composite3.setLayoutData(gridData33);
  // }

  // /**
  // * This method initializes group1
  // *
  // */
  // private void erzeugeGroupUnten()
  // {
  // GridData gridData34 = new GridData();
  // gridData34.horizontalAlignment = GridData.FILL;
  // gridData34.grabExcessHorizontalSpace = true;
  // gridData34.verticalAlignment = GridData.CENTER;
  // group1 = new Group(composite3, SWT.NONE);
  // group1.setLayout(new GridLayout());
  // // erzeugeCompositeGroupUnten();
  // erzeugeCompositeAbbruch();
  // group1.setLayoutData(gridData34);
  // }

  /**
   * Setzt das Konfig-Datenmodell fr alle registrierten Teilkonfigurationsassistenten
   * 
   * @param dataModel
   *          das Datenmodell
   * @param pfad
   *          der Pfad zur Verwaltungsdatei, wird bei den Teilkonfigurationsassistenten angezeigt
   */
  public void setzeKonfigDatenModellFuerTka(ConfigDataModel dataModel, String pfad)
  {
    // if (dataModel != null)
    {
      IExtensionRegistry er = Platform.getExtensionRegistry();

      IConfigurationElement[] configurationElementsFor = er.getExtensionPoint(
          "de.bsvrz.buv.plugin.konfigass.konfigAssDatenModell").getConfigurationElements();

      for (IConfigurationElement ce : configurationElementsFor)
      {
        try
        {
          IKonfigAssDatenModell operation = (IKonfigAssDatenModell)ce
              .createExecutableExtension("konfigAssDatenModellKlasse");

          String s = operation.holePluginId();
          
          if (_zugeordnetePluginIds.contains(s))
          {
            if (dataModel != null)
              operation.setzeKonfigDatenModell(dataModel, pfad);
            else
              operation.setzeKonfigDatenModell(null, null);        
          }
//          String pfadTka = operation.holePfad();
//          String secId = operation.holeSecId();
//          if (!s.equals("de.bsvrz.buv.plugin.tkabasis")){
//            
//         
//            
//              if (!ermittleObTkaGeoeffnet(s)){
//                if (dataModel != null)
//                  operation.setzeKonfigDatenModell(dataModel, pfad, _secondId);
//                else
//                  operation.setzeKonfigDatenModell(null, null, _secondId);        
//                
//                System.out.println("---------> setzte DM fr TKA:" + s);
//                System.out.println("secId: " + secId + " -> " + _secondId);
//              }              
//              else if (pfadTka != null && pfadTka.equals(pfad) || pfadTka == null)
//              {                
//                if (dataModel != null)
//                  operation.setzeKonfigDatenModell(dataModel, pfad, _secondId);
//                else
//                  operation.setzeKonfigDatenModell(null, null, _secondId);
//                
//                System.out.println("---------> setzte DM fr TKA:" + s);
//                System.out.println("secId: " + secId + " -> " + _secondId);
//                  
//              }
//              else if (secId != null && secId.equals(_secondId)){
//                if (dataModel != null)
//                  operation.setzeKonfigDatenModell(dataModel, pfad, _secondId);
//                else
//                  operation.setzeKonfigDatenModell(null, null, _secondId);
//                
//                System.out.println("---------> setzte DM fr TKA:" + s);
//                System.out.println("secId: " + secId + " -> " + _secondId);
//              }
//          }
        }
        catch (CoreException e)
        {
          // TODO Auto-generated catch block
          e.printStackTrace();
        }
      }
    }
  }

  /**
   * Setzt das Konfig-Datenmodell fr den Teilkonfigurationsassistenten mit der bergebenen Id
   * 
   * @param pluginId
   *          Die Id des TKA-Plugins, fr das das Datenmodel gesetzt werden soll
   * 
   * @param dataModel
   *          das Datenmodell
   * @param pfad
   *          der Pfad zur Verwaltungsdatei, wird bei den Teilkonfigurationsassistenten angezeigt
   */
  public void setzeKonfigDatenModellFuerTka(String pluginId, ConfigDataModel dataModel, String pfad)
  {
    // if (dataModel != null)
    {
      IExtensionRegistry er = Platform.getExtensionRegistry();

      IConfigurationElement[] configurationElementsFor = er.getExtensionPoint(
          "de.bsvrz.buv.plugin.konfigass.konfigAssDatenModell").getConfigurationElements();

      for (IConfigurationElement ce : configurationElementsFor)
      {
        try
        {
          IKonfigAssDatenModell operation = (IKonfigAssDatenModell)ce
              .createExecutableExtension("konfigAssDatenModellKlasse");

          String s = operation.holePluginId();
          
          if (s.equals(pluginId))
          {
            if (dataModel != null)
              operation.setzeKonfigDatenModell(dataModel, pfad);
            else
              operation.setzeKonfigDatenModell(null, null);     
            
            break;
          }
        }
        catch (CoreException e)
        {
          // TODO Auto-generated catch block
          e.printStackTrace();
        }
      }
    }
  }

  
  
//  public boolean ermittleObTkaGeoeffnet(String pluginId)
//  {
//    boolean re = false;
//
//    IWorkbench wb = PlatformUI.getWorkbench();
//    IWorkbenchWindow ww = wb.getActiveWorkbenchWindow();
//    IWorkbenchPage page = ww.getActivePage();
//
//    IViewReference[] vrs = page.getViewReferences();
//    for (IViewReference ref : vrs)
//    {       
//      String sec = ref.getSecondaryId();
//      
//      if (sec != null && pluginId.contains("tka"))
//      {
//        String sub = pluginId.substring(pluginId.indexOf("tka") + 3);
//        sec = sec.toLowerCase();
//        if (sec != null && sec.equals(sub))
//        {
//          re = true;          
//        }
//      }
//    }
//    return re;
//  }
  
  
  
  
//  public boolean ermittleEigenerKA(String secId)
//  {
//    
//    System.out.println("KonfigAssDarstellungKonfiguration.ermittleEigenerKA(): " + secId);
//    
//    
//    
//    
//    this.getParent();
//    
//    
//    boolean re = false;
//    
//    IWorkbench wb = PlatformUI.getWorkbench();
//    IWorkbenchWindow ww = wb.getActiveWorkbenchWindow();
//    IWorkbenchPage page = ww.getActivePage();
//    
//    IViewReference[] vrs = page.getViewReferences();
//    for (IViewReference ref : vrs)
//    {       
//      String sec = ref.getSecondaryId();
//      
//      if (sec != null && sec.equals(secId))
//      {        
//        re = true;                  
//      }
//    }
//    return re;
//  }
  
//  public void ermittleEigeneView(Object me, String sec)
//  {
//    if (me instanceof KonfigAssSicht)
//    {
//      KonfigAssSicht view = (KonfigAssSicht)me;
//      sec = view.getViewSite().getSecondaryId();      
//    }else{
//      Composite comp = (Composite)me;      
//      ermittleEigeneView(comp, sec);
//    }
//  }

  /**
   * Setzt das Konfig-Datenmodell fr alle registrierten Teilkonfigurationsassistenten
   * 
   * @param dataModel
   *          das Datenmodell
   * @param pfad
   *          der Pfad zur Verwaltungsdatei, wird bei den Teilkonfigurationsassistenten angezeigt
   */
  public void informiereUeberLadenDatenmodell(String pfad)
  {
    // if (dataModel != null)
    {
      IExtensionRegistry er = Platform.getExtensionRegistry();

      IConfigurationElement[] configurationElementsFor = er.getExtensionPoint(
          "de.bsvrz.buv.plugin.konfigass.konfigAssDatenModell").getConfigurationElements();

      for (IConfigurationElement ce : configurationElementsFor)
      {
        try
        {
          final IKonfigAssDatenModell operation = (IKonfigAssDatenModell)ce
              .createExecutableExtension("konfigAssDatenModellKlasse");

          String s = operation.holePluginId();
          Display.getDefault().asyncExec(new Runnable() {

            @Override
            public void run()
            {
              operation.ladeKonfigDatenModell(_dateiPfad);
            }
          });

        }
        catch (CoreException e)
        {
          // TODO Auto-generated catch block
          e.printStackTrace();
        }

      }

    }

  }

  private void aktuelleInfoKbAnzeigen()
  {
    _pidAoeKonfig = _config.getConfigurationAuthorityPid();

    _mapKonfigBereichKapsel = new TreeMap<String, KonfigBereichDaten>();

    List liste = _config.getManagementFile().getAllConfigurationAreaManagementInfos();

    for (int i = 0; i < liste.size(); i++)
    {
      ConfigurationAreaManagementInfo cami = (ConfigurationAreaManagementInfo)liste.get(i);

      String pid = cami.getPid();

      ConfigurationArea ca = _config.getAllConfigurationAreas().get(pid);

      if (ca != null)
      {
        String name = ca.getName();
        ConfigurationAuthority kv = null;
        try
        {
          kv = ca.getConfigurationAuthority();
          if (kv != null)
          {
            String pidAoeKb = kv.getPid();

            boolean bearbeiten = false;
            if (_pidAoeKonfig.equals(pidAoeKb))
              bearbeiten = true;
            long z2 = ca.getTimeOfLastActiveConfigurationChange();
//            short v2 = ca.getActivatableVersion();
//            short v3 = ca.getActiveVersion();
//            short v4 = ca.getModifiableVersion();
//            short v6 = ca.getTransferableVersion();
//            short v7 = ca.getValidSince();

            
            short v2 = ca.getModifiableVersion();
            short v3 = ca.getActiveVersion();
            short v4 = ca.getTransferableVersion();
            short v6 = ca. getActivatableVersion();
            short v7 = ca. getActivatableVersion();
            
//            ConfigurationAreaManagementInfo cami = _config.getConfigurationAreaManagementInfo(pid);
            List<VersionInfo> versions = cami.getVersions();
             
            String pfad = cami.getDirectory().getAbsolutePath();

            KonfigBereichBestimmeModellObjekt kbBestimmeModellObjekt = new KonfigBereichBestimmeModellObjekt(_config,
                ca);

            Boolean isModell = kbBestimmeModellObjekt.getIsModell();
            Boolean isObjekt = kbBestimmeModellObjekt.getIsObjekt();

            KonfigBereichDaten kbk = new KonfigBereichDaten(pid, name, bearbeiten, isModell, isObjekt, pidAoeKb, z2,
                v2, v3, v4, v6, v7, EnumStatusKonfigBereich.IMPORT, pfad);

            String key = kbk.getPidAOE() + kbk.getPid();

            _mapKonfigBereichKapsel.put(key, kbk);
          }
        }
        catch (IllegalStateException e)
        {
          // TODO Auto-generated catch block
          System.err.println(e.getMessage());
        }
      }
      Collection<KonfigBereichDaten> col = _mapKonfigBereichKapsel.values();

      _listKonfigBereichKapsel = new ArrayList<KonfigBereichDaten>();
      for (KonfigBereichDaten kbk : col)
      {
        _listKonfigBereichKapsel.add(kbk);
      }
    }
  }

  @Override
  public void dispose()
  {
    if (_config != null){
      _config.close();
      _config = null;
    }
  }

  private void erzeugeInhaltTabelle(final List<KonfigBereichDaten> liste)
  {
//    System.out.println("KonfigAssDarstellungKonfiguration.erzeugeInhaltTabelle()");

    _tvZufluesse1.setInput(new ArrayList<KonfigBereichDaten>());
    FontRegistry registry = new FontRegistry();
    Font fontBold = registry.getBold(Display.getCurrent().getSystemFont().getFontData()[0].getName());
    
    List<KonfigBereichDaten> listSel = new ArrayList<KonfigBereichDaten>();

    if (_selection != null)
    {
      Object obj = _selection.getFirstElement();
      if (obj instanceof KonfigBereichDaten)
      {
        Iterator it = _selection.iterator();

        while (it.hasNext())
        {
          KonfigBereichDaten kbd = (KonfigBereichDaten)it.next();
//          System.out.println("listSel: " + kbd.getPid());
          listSel.add(kbd);
        }
      }
    }
 
    List<TableItem> listItem = new ArrayList<TableItem>();
    
    for (KonfigBereichDaten kapsel : liste)
    {
      TableItem ti = new TableItem(_tvZufluesse1.getTable(), SWT.NONE);
      

      Image a1 = ImageFactory.getInstanz().getImage("icons/a1.png");
      Image c1 = ImageFactory.getInstanz().getImage("icons/c1.png");

      if (kapsel.getModelle())
        ti.setImage(0, a1);
      else
        ti.setImage(0, c1);

      if (kapsel.getObjekte())
        ti.setImage(1, a1);
      else
        ti.setImage(1, c1);

      ti.setText(2, kapsel.getPidAOE());
      ti.setText(3, kapsel.getPid());
      Date d = new Date(kapsel.getDatumAktivierung());
      SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyy HH:mm:ss");
      ti.setText(4, sdf.format(d));
      ti.setText(5, kapsel.getVersionAktivierung().toString());
      ti.setText(6, kapsel.getVersionAktuell().toString());
      ti.setText(7, kapsel.getVersionFreigabeZurUebernahme().toString());
      ti.setText(8, kapsel.getVersionFreigabeZurAktivierung().toString());
      ti.setText(9, kapsel.getVersionFreigabeZurAktivierungOhne().toString());
//      ti.setText(10, kapsel.getStatusKonfigBereich().toString().toString());
      ti.setText(10, kapsel.getPfad());

      ti.setData(kapsel);
      
      if (kapsel.getAktvieren() != null && kapsel.getAktvieren())
      {        
        Color color = Display.getDefault().getSystemColor(SWT.COLOR_RED);        
        ti.setBackground(color);
      }
      else
      {
        Color color = Display.getDefault().getSystemColor(SWT.COLOR_WHITE);
        ti.setBackground(color);        
      }
      
      if (kapsel.getBearbeitung() != null && !kapsel.getBearbeitung())
      {
//        Color color = Display.getDefault().getSystemColor(SWT.COLOR_GRAY);
        ti.setForeground(color2);
      }
      else
      {
        ti.setFont(fontBold);
      } 
      
      
      if (listSel.contains(kapsel)){
//        System.out.println("gefunden: " + kapsel.getPid());
        listItem.add(ti);
      }
      
    }
    if (listItem.size() > 0)
    {
      TableItem[] items = listItem.toArray(new TableItem[listItem.size()]);
//      System.out.println("items: " + items.length);
      _tvZufluesse1.getTable().setSelection(items);
      
      _selChangedlistener.selectionChanged(new SelectionChangedEvent(_tvZufluesse1, _tvZufluesse1.getSelection()));
            
    }else{
      updateButtonKeineSelektion();
    }
    
    
    
//    int x =  hBar1.getSelection() * (hBar2.getMaximum() - hBar2.getThumb()) / Math.max(1, hBar1.getMaximum() - hBar1.getThumb());
//    int y =  vBar1.getSelection() * (vBar2.getMaximum() - vBar2.getThumb()) / Math.max(1, vBar1.getMaximum() - vBar1.getThumb());
// 
//    if (_hbarsel > 0) 
//      _tvZufluesse1.getTable().getHorizontalBar().setSelection(_hbarsel);
//    if (_vbarsel > 0) 
//      _tvZufluesse1.getTable().getVerticalBar().setSelection(_vbarsel);
      
    
  }

  private void aktionLadeDatenModell(boolean benutzerInfo, final List<String> list)
  {
    if (_dateiPfad != null)
    {

      final File file = new File(_dateiPfad);

      if (_config != null)
      {
        if (benutzerInfo)
        {
          String meldung = Activator.PLUGIN_ID + " Zugriff auf das bisherige Datenmodell "
              + "ist nicht mehr mglich, da ein neues Datenmodell geladen wird!";

          IWorkbench wb = PlatformUI.getWorkbench();
          if (wb != null)
          {
            Display display = wb.getDisplay();

            if (display != null)
            {
              Shell shell = display.getActiveShell();
              if (shell != null)
              {
                MessageBox msg = new MessageBox(shell, SWT.OK | SWT.CANCEL | SWT.ICON_WARNING);
                msg.setText("Neues Datenmodell wird geladen");
                msg.setMessage(meldung);
                int open = msg.open();

                if (open != SWT.OK)
                {
                  updateButtonKeineSelektion();
                  return;
                }
              }
            }
          }
        }        
        informiereUeberLadenDatenmodell("Test");
        _tAutoCloser.run();
      }

      _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion Laden Datenmodell", null);
      MeldungenSpeicher.getInstanz().setStatus(_status);

      final Job job = new Job("KonfigAss: Lade Datenmodell") {

        protected IStatus run(IProgressMonitor monitor)
        {
          monitor.beginTask("Start mit Laden...", 5);
          monitor.worked(1);
          
          try
          {
            if (_tAutoCloser != null)
            {
              _tAutoCloser.join();
            }
            MeldungenSpeicher.getInstanz().setKonfiguration(_dateiPfad);
            _config = erzeugeKonfigDatenModellAusDatei(file);            
            monitor.worked(1);
          }
          catch (Exception e)
          {
            // _status = new MultiStatus(Activator.PLUGIN_ID, Status.ERROR, "Fehler Aktion Laden Datenmodell", e);
            MeldungenSpeicher.getInstanz().setKonfiguration("");
            _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID, "Fehler Aktion Laden Datenmodell", e));

            return Status.CANCEL_STATUS;
          }
          finally
          {
            monitor.worked(3);
            monitor.done();
            
            MeldungenSpeicher.getInstanz().setStatus(_status);
          }

          if (_config == null)
          {
            return Status.CANCEL_STATUS;
          }
          else
          {            
            aktuelleInfoKbAnzeigen();

            boolean fehler = false;

            if (list != null)
            {
              for (String pidKb : list)
              {
                ConfigurationArea ca1 = _config.getAllConfigurationAreas().get(pidKb);

                if (ca1 != null)
                {
                  AttributeGroup atg = _config.getAttributeGroup("atg.konfigurationsBereichAbhngigkeiten");
                  ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion((ConfigDataModel)_config);
                  Data data = ((ConfigSystemObject)ca1).getConfigurationData(atg, lookup);
                  if (data != null)
                  {
                    Data item = data.getItem("KonfigurationsAbhngigkeiten");
                    Array array = item.asArray();
                    for (int i = 0; i < array.getLength(); i++)
                    {
                      item = array.getItem(i);

                      String pid = item.getItem("BereichNotwendig").asTextValue().getValueText();
                      if (pid != null)
                      {
                        ConfigurationArea kb = _config.getAllConfigurationAreas().get(pid);
                        if (kb != null)
                        {
                          Short activeVersion = kb.getActiveVersion();
                          String text1 = item.getItem("VersionNotwendig").asTextValue().getValueText();
                          String text2 = item.getItem("Kennung").asTextValue().getValueText();
                          Short neededVersion = new Short(text1);

                          if (text2.equals("notwendig"))
                          {
                            Map<String, ConfigurationArea> map = _config.getAllConfigurationAreas();
                            if (!map.containsKey(kb.getPid()))
                            {
                              String meldung = "Fehler bei den Abhngigkeiten des KB: " + kb.getPid()
                                  + " ist nicht vorhanden in: " + pidKb;
                              _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID, meldung));
                              fehler = true;
                            }
                          }
                        }
                        else
                        {
                          String meldung = "Fehler bei den Abhngigkeiten des KB: " + pid
                              + " es ist kein SystemObject fr den KB vorhanden!";
                          _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID, meldung));
                          fehler = true;
                        }
                      }
                    }
                  }
                }
                // else{
                // String meldung = "Fehler bei den Abhngigkeiten des KB: " + pidKb +
                // " es ist kein SystemObject fr den KB vorhanden!";
                // _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID, meldung));
                // fehler = true;
                // }
              }
            }

            if (!fehler)
              return Status.OK_STATUS;
            else
              return Status.CANCEL_STATUS;
          }

        }
      };

      job.addJobChangeListener(new JobChangeAdapter() {

        public void done(IJobChangeEvent event)
        {
          if (event.getResult().isOK())
          {
            _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
                "Aktion Laden Datenmodell beendet, bitte beachten Sie die Ausgaben der Konfiguration"));

            UIJob uiJob = new UIJob("aktualisiere GUI") {

              @Override
              public IStatus runInUIThread(IProgressMonitor monitor)
              {
                try
                {
                  aktuelleInfoKbAnzeigen();
                  erzeugeInhaltTabelle(_listKonfigBereichKapsel);
                  updateTextKonfiguration(_dateiPfad, _pidAoeKonfig);
//                  updateButtonKeineSelektion();
                  setzeKonfigDatenModellFuerTka(_config, _dateiPfad);
                }
                catch (Exception e)
                {
                  _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
                      "Fehler beim aktualisieren des GUI, es sind Ausnahmen aufgetreten. ", e));

                  if (_config != null)
                  {
                    _config.close();
                    _config = null;
                  }
                  erzeugeInhaltTabelle(new ArrayList<KonfigBereichDaten>());
                  updateTextKonfiguration("", "");
                  updateButtonKeinDatenModell();
//                  setzeKonfigDatenModellFuerTka(null, "");
                  setzeKonfigDatenModellFuerTka(null, _dateiPfad);

                  if (list != null)
                  {
                    XmlElementKonfigVerwaltung xmlElementKonfigVerwaltung = new XmlElementKonfigVerwaltung(_config);
                    xmlElementKonfigVerwaltung.einlesenXmlAusDatei(_dateiPfad);

                    LinkedList<ElementKonfig> listeKbXml = xmlElementKonfigVerwaltung.getElementKonfigBereich();

                    List<ElementKonfig> delList = new ArrayList<ElementKonfig>();

                    if (listeKbXml != null)
                    {
                      for (ElementKonfig ek : listeKbXml)
                      {
                        if (ek != null)
                        {
                          String b = ek.getBezeichnung();
                          for (String s : list)
                          {
                            if (s.equals(b))
                            {
                              delList.add(ek);
                            }
                          }
                        }
                      }

                    }
                    if (delList.size() > 0)
                    {
                      for (ElementKonfig ek2 : delList)
                      {
                        String meldung = "Entferne KB wegen Fehler aus den Veraltungdaten: " + ek2.getBezeichnung();
                        _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID, meldung));
                        xmlElementKonfigVerwaltung.entferneElementKonfig(ek2);
                      }

                      String s = xmlElementKonfigVerwaltung.ausgabeXmlFormatiert();
                      xmlElementKonfigVerwaltung.schreibeXMLStringInDatei(_dateiPfad, s, true);
                    }
                  }

                  return Status.CANCEL_STATUS;
                }
                return Status.OK_STATUS;
              }
            };
            uiJob.setUser(false);
            uiJob.setSystem(true);
            uiJob.schedule();

          }
          else
          // Aufrumen
          {
            _status
                .merge(new Status(
                    Status.ERROR,
                    Activator.PLUGIN_ID,
                    "Aktion Laden Datenmodell abgebrochen! Grund: es ist eine Ausnahme bei der Ausfhrung aufgetreten. Die evt. geladene Konfiguration wurde geschlossen. "));

            UIJob uiJob = new UIJob("aktualisiere GUI") {

              @Override
              public IStatus runInUIThread(IProgressMonitor monitor)
              {              
                try
                {
                  if (_config != null)
                  {
                    _config.close();
                    _config = null;
                  }
                  erzeugeInhaltTabelle(new ArrayList<KonfigBereichDaten>());
                  updateTextKonfiguration("", "");
                  updateButtonKeinDatenModell();
//                  setzeKonfigDatenModellFuerTka(null, "");
                  setzeKonfigDatenModellFuerTka(null, _dateiPfad);

                  if (list != null)
                  {
                    XmlElementKonfigVerwaltung xmlElementKonfigVerwaltung = new XmlElementKonfigVerwaltung(_config);
                    xmlElementKonfigVerwaltung.einlesenXmlAusDatei(_dateiPfad);

                    LinkedList<ElementKonfig> listeKbXml = xmlElementKonfigVerwaltung.getElementKonfigBereich();

                    List<ElementKonfig> delList = new ArrayList<ElementKonfig>();

                    if (listeKbXml != null)
                    {
                      for (ElementKonfig ek : listeKbXml)
                      {
                        if (ek != null)
                        {
                          String b = ek.getBezeichnung();
                          for (String s : list)
                          {
                            if (s.equals(b))
                            {
                              delList.add(ek);
                            }
                          }
                        }
                      }

                    }
                    if (delList.size() > 0)
                    {
                      for (ElementKonfig ek2 : delList)
                      {
                        String meldung = "Entferne KB aus den Veraltungdaten: " + ek2.getBezeichnung();
                        _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID, meldung));
                        xmlElementKonfigVerwaltung.entferneElementKonfig(ek2);
                      }

                      String s = xmlElementKonfigVerwaltung.ausgabeXmlFormatiert();
                      xmlElementKonfigVerwaltung.schreibeXMLStringInDatei(_dateiPfad, s, true);
                    }
                  }

                }
                catch (Exception e)
                {
                  _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
                      "Fehler beim aktualisieren des GUI, es sind Ausnahmen aufgetreten. ", e));
                  return Status.CANCEL_STATUS;
                }
                return Status.OK_STATUS;
              }
            };
            uiJob.setUser(false);
            uiJob.setSystem(true);
            uiJob.schedule();
          }
          {
            MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);
          }
        }
      });
      job.setSystem(false);
      job.setUser(true);
      job.schedule(); // start as soon as possible
    }
  }

  /**
   * Speichert die Info als konfigurierenden Datensatz an einem Objekt.
   * 
   * @param systemObject
   *          Objekt, an dem die Info gespeichert werden soll
   * @param info
   *          die zu speichernde Info
   * 
   * @throws ConfigurationChangeException
   *           Falls der konfigurierende Datensatz nicht am Objekt gespeichert werden konnte.
   */
  private void setInfo(SystemObject systemObject, KvAoeNeuModell model) throws ConfigurationChangeException
  {
    // wenn Info nicht definiert ist, braucht es auch nicht abgespeichert zu werden
    if (model != null)
    {
      // wenn der Info-Datensatz bereits existiert, kann dieser gendert werden
      final AttributeGroup atg = _config.getAttributeGroup("atg.info");
      Data data = AttributeBaseValueDataFactory.createAdapter(atg, AttributeHelper.getAttributesValues(atg));
      data.getTextValue("kurzinfo").setText(model.getKurzInfo());
      data.getTextValue("beschreibung").setText(model.getBeschreibung());
      systemObject.setConfigurationData(atg, data);
    }
  }

  /**
   * Speichert die Info als konfigurierenden Datensatz an einem Objekt.
   * 
   * @param systemObject
   *          Objekt, an dem die Info gespeichert werden soll
   * @param info
   *          die zu speichernde Info
   * 
   * @throws ConfigurationChangeException
   *           Falls der konfigurierende Datensatz nicht am Objekt gespeichert werden konnte.
   */
  private void setInfoDef(SystemObject systemObject, KvAoeNeuModell model) throws ConfigurationChangeException
  {
    // wenn Info nicht definiert ist, braucht es auch nicht abgespeichert zu werden
    if (model != null)
    {
      // wenn der Info-Datensatz bereits existiert, kann dieser gendert werden
      final AttributeGroup atg = _config.getAttributeGroup("atg.info");
      Data data = AttributeBaseValueDataFactory.createAdapter(atg, AttributeHelper.getAttributesValues(atg));
      data.getTextValue("kurzinfo").setText(model.getInfoDef());
      data.getTextValue("beschreibung").setText(model.getBeschreibungDef());
      systemObject.setConfigurationData(atg, data);
    }
  }

  private void aktionKonsistenzPruefung()
  {
    _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion Konsistenzprfung", null);
    MeldungenSpeicher.getInstanz().setStatus(_status);

    final Job job = new Job("KonfigAss: Pruefe Konsistenz") {

      protected IStatus run(IProgressMonitor monitor)
      {
        monitor.beginTask("Starte Konsistenzpruefung...", 5);
        monitor.worked(1);

        try
        {

          KonfigApplikation app = new KonfigApplikation(_config);
          app.pruefeKonsistenz(false, false);
          monitor.worked(1);
        }
        catch (Exception e2)
        {
          _status
              .merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
                  "Aktion Konsistenzprfung abgebrochen! Grund: es ist eine Ausnahme bei der Ausfhrung aufgetreten. "
                      + e2));
          return Status.CANCEL_STATUS;
        }
        finally
        {
          monitor.worked(3);
          monitor.done();
        }

        return Status.OK_STATUS;
      }
    };

    job.addJobChangeListener(new JobChangeAdapter() {

      @Override
      public void running(IJobChangeEvent event)
      {
        UIJob uiJob = new UIJob("aktualisiere GUI") {

          @Override
          public IStatus runInUIThread(IProgressMonitor monitor)
          {
            // TODO Auto-generated method stub
            // updateButtonJobAktiv();
            return Status.OK_STATUS;
          }
        };
        uiJob.setUser(false);
        uiJob.setSystem(true);
        uiJob.schedule();
      }

      public void done(IJobChangeEvent event)
      {
        if (event.getResult().isOK())
        {
          _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
              "Aktion Konsistenzprfung beendet, bitte beachten Sie die Ausgaben der Konfiguration"));
          MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);
          UIJob uiJob = new UIJob("aktualisiere GUI") {

            @Override
            public IStatus runInUIThread(IProgressMonitor monitor)
            {
              // TODO Auto-generated method stub
              aktuelleInfoKbAnzeigen();
              erzeugeInhaltTabelle(_listKonfigBereichKapsel);
//              updateButtonKeineSelektion();
              return Status.OK_STATUS;
            }
          };
          uiJob.setUser(false);
          uiJob.setSystem(true);
          uiJob.schedule();

        }
        else
        {
          MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);
          UIJob uiJob = new UIJob("aktualisiere GUI") {

            @Override
            public IStatus runInUIThread(IProgressMonitor monitor)
            {
              // TODO Auto-generated method stub
              updateButtonKeineSelektion();
              return Status.OK_STATUS;
            }
          };
          uiJob.setUser(false);
          uiJob.setSystem(true);
          uiJob.schedule();
        }

      }
    });
    job.setSystem(false);
    job.setUser(true);
    job.schedule(); // start as soon as possible
  }
  
  private void aktionPruefungAktivierung()
  {
    _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion Ueberprfung Aktivierung", null);
    MeldungenSpeicher.getInstanz().setStatus(_status);

    final Job job = new Job("KonfigAss: Pruefe Aktivierung") {

      protected IStatus run(IProgressMonitor monitor)
      {
        Map<String, ConfigurationArea> areas = _config.getAllConfigurationAreas();
        
        monitor.beginTask("Starte Aktivierungsprfung...", areas.size() + 2);
        monitor.worked(1);

        try        
        {
          List<ConfigurationArea> list = new ArrayList<ConfigurationArea>();
          
          
          for (Map.Entry<String, ConfigurationArea> me : areas.entrySet())
          {             
            boolean aktivieren = false;
            ConfigurationArea ca = me.getValue();
            if (ca != null){
              
              if (!ca.isValid()){
                aktivieren = true;
              }
              else{
                Collection<SystemObject> objects = ca.getObjects(null, ObjectTimeSpecification.valid());
                
                for (SystemObject so : objects)
                {
                  if (!so.isValid()){
                    aktivieren = true;
                    break;
                  }
                }  
                
                Collection<SystemObject> newObjects = ca.getNewObjects();
                
                for (SystemObject so : newObjects)
                {
                  if (!so.isValid()){
                    aktivieren = true;
                    break;
                  }              
                }                 
              }
               
              if (aktivieren){
                list.add(ca);
              }              
            }  
            monitor.worked(1);
          }
          
          aktuelleInfoKbAnzeigen();          
          for (ConfigurationArea ca : list)
          {
//            System.out.println("muss aktiviert werden: " + ca.getPid());
            
            for (KonfigBereichDaten kbd : _listKonfigBereichKapsel)
            {
              if (ca.getPid().equals(kbd.getPid())){                                 
                kbd.setAktvieren(true);             
              }
            }
          }          
          
          monitor.worked(1);
        }
        catch (Exception e2)
        {
          _status
              .merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
                  "Aktion Aktivierungsprfung abgebrochen! Grund: es ist eine Ausnahme bei der Ausfhrung aufgetreten. "
                      + e2));
          return Status.CANCEL_STATUS;
        }
        finally
        {
          monitor.done();
        }

        return Status.OK_STATUS;
      }
    };

    job.addJobChangeListener(new JobChangeAdapter() {

      @Override
      public void running(IJobChangeEvent event)
      {
        UIJob uiJob = new UIJob("aktualisiere GUI") {

          @Override
          public IStatus runInUIThread(IProgressMonitor monitor)
          {
            // TODO Auto-generated method stub
            // updateButtonJobAktiv();
            return Status.OK_STATUS;
          }
        };
        uiJob.setUser(false);
        uiJob.setSystem(true);
        uiJob.schedule();
      }

      public void done(IJobChangeEvent event)
      {
        if (event.getResult().isOK())
        {
          _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
              "Aktion Aktivierungsprfung beendet, bitte beachten Sie die Ausgaben der Konfiguration"));
          MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);
          UIJob uiJob = new UIJob("aktualisiere GUI") {

            @Override
            public IStatus runInUIThread(IProgressMonitor monitor)
            {
              // TODO Auto-generated method stub
              erzeugeInhaltTabelle(_listKonfigBereichKapsel);              
              return Status.OK_STATUS;
            }
          };
          uiJob.setUser(false);
          uiJob.setSystem(true);
          uiJob.schedule();

        }
        else
        {
          MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);
          UIJob uiJob = new UIJob("aktualisiere GUI") {

            @Override
            public IStatus runInUIThread(IProgressMonitor monitor)
            {
              // TODO Auto-generated method stub
              updateButtonKeineSelektion();
              return Status.OK_STATUS;
            }
          };
          uiJob.setUser(false);
          uiJob.setSystem(true);
          uiJob.schedule();
        }

      }
    });
    job.setSystem(false);
    job.setUser(true);
    job.schedule(); // start as soon as possible
    
  }

  private void aktionAktivierung()
  {
    if (_selection != null)
    {
      Object obj = _selection.getFirstElement();
      if (obj instanceof KonfigBereichDaten)
      {
        Iterator it = _selection.iterator();

        Map<String, File> list = new HashMap<String, File>();

        while (it.hasNext())
        {
          KonfigBereichDaten kbd = (KonfigBereichDaten)it.next();
          String pid = kbd.getPid();
          Map<String, ConfigurationArea> map = _config.getAllConfigurationAreas();
          if (map.containsKey(pid))
          {
            ConfigurationArea ca = map.get(pid);
            list.put(ca.getPid(), new File(""));
          }

        }
        if (list.size() > 0)
        {                
          KbAuswahlAssistent wizard = new KbAuswahlAssistent("Aktivieren", "aktiviert", list);

          wizard.init(PlatformUI.getWorkbench(), null);
          // Instantiates the wizard container with the wizard and opens it
          WizardDialog dialog = new WizardDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell(), wizard);
          dialog.create();
          int ret = dialog.open();

          if (ret == 0)
          {
             _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion Aktivierung", null);
             MeldungenSpeicher.getInstanz().setStatus(_status);
            
             KbAuswahlModell model = wizard.getModel();
             final Map<String, File> list2 = model.getList();
              
             final Job job = new Job("KonfigAss: Aktiviere nderungen") {

               protected IStatus run(IProgressMonitor monitor)
               {
                 monitor.beginTask("Starte Aktivierung...", 5);
                 monitor.worked(1);

                 try
                 {
                   
                   List<ConfigurationArea> listCa = new ArrayList<ConfigurationArea>();
                   
                   for (Map.Entry<String, File> me : list2.entrySet())
                   {
                     String pid = me.getKey();
                     
//                     System.out.println("ca: " + "-" + pid + "-");
                     
                     ConfigurationArea ca = _config.getAllConfigurationAreas().get(pid);
                     if (ca != null){
                       listCa.add(ca);
//                       System.out.println("ca: " + ca.getPid());
                     }
                   }
                   
                   for (ConfigurationArea ca : listCa)
                   {                     
                     _status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Aktiviere KB: " + ca.getPid()));
                   }
                   
                   KonfigApplikation app = new KonfigApplikation(_config);
//                   System.out.println("4: " + listCa.size());
                   app.starteAktivierung(listCa, false, false);
                   monitor.worked(1);
                   
                   // _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion Aktivierung", null);
                   // MeldungenSpeicher.getInstanz().setStatus(_status);

                 }
                 catch (Exception e2)
                 {
                   _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
                       "Aktion Aktivierung abgebrochen! Grund: es ist eine Ausnahme bei der Ausfhrung aufgetreten. " + e2));
                   return Status.CANCEL_STATUS;
                 }
                 finally
                 {
                   monitor.worked(3);
                   monitor.done();
                 }

                 return Status.OK_STATUS;
               }
             };

             job.addJobChangeListener(new JobChangeAdapter() {

               @Override
               public void running(IJobChangeEvent event)
               {
                 UIJob uiJob = new UIJob("aktualisiere GUI") {

                   @Override
                   public IStatus runInUIThread(IProgressMonitor monitor)
                   {
                     // TODO Auto-generated method stub
                     // updateButtonJobAktiv();
                     return Status.OK_STATUS;
                   }
                 };
                 uiJob.setUser(false);
                 uiJob.setSystem(true);
                 uiJob.schedule();
               }

               public void done(IJobChangeEvent event)
               {
                 if (event.getResult().isOK())
                 {
                   _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
                       "Aktion Aktivierung beendet, bitte beachten Sie die Ausgaben der Konfiguration"));
                   MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);

                   updateButtonJobAktiv();
                   aktionLadeDatenModell(false, null);
                 }
                 else
                 {
                   MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), false);
                   UIJob uiJob = new UIJob("aktualisiere GUI") {

                     @Override
                     public IStatus runInUIThread(IProgressMonitor monitor)
                     {
                       // TODO Auto-generated method stub
                       updateButtonKeineSelektion();
                       return Status.OK_STATUS;
                     }
                   };
                   uiJob.setUser(false);
                   uiJob.setSystem(true);
                   uiJob.schedule();
                 }

               }
             });
             job.setSystem(false);
             job.setUser(true);
             job.schedule(); // start as soon as possible             
             
          }else{
            _selChangedlistener.selectionChanged(new SelectionChangedEvent(_tvZufluesse1, _tvZufluesse1.getSelection()));
          }
        }
      }
    } 
  }

  private void aktionAktivierung(final List<ConfigurationArea> list)
  {
    _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion Aktivierung", null);
    MeldungenSpeicher.getInstanz().setStatus(_status);

    final Job job = new Job("KonfigAss: Aktiviere nderungen") {

      protected IStatus run(IProgressMonitor monitor)
      {
        monitor.beginTask("Starte Aktivierung...", 5);
        monitor.worked(1);

        try
        {
          if (list.size() > 0)
          {
            // _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion Aktivierung", null);
            // MeldungenSpeicher.getInstanz().setStatus(_status);

            KonfigApplikation app = new KonfigApplikation(_config);

            for (ConfigurationArea ca : list)
            {
              _status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Aktiviere KB: " + ca.getPid()));
            }

            app.starteAktivierung(list, false, false);
            monitor.worked(1);

          }
        }
        catch (Exception e2)
        {
          _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
              "Aktion Aktivierung abgebrochen! Grund: es ist eine Ausnahme bei der Ausfhrung aufgetreten. " + e2));
          return Status.CANCEL_STATUS;
        }
        finally
        {
          monitor.worked(3);
          monitor.done();
        }

        return Status.OK_STATUS;
      }
    };

    job.addJobChangeListener(new JobChangeAdapter() {

      @Override
      public void running(IJobChangeEvent event)
      {
        UIJob uiJob = new UIJob("aktualisiere GUI") {

          @Override
          public IStatus runInUIThread(IProgressMonitor monitor)
          {
            // TODO Auto-generated method stub
            // updateButtonJobAktiv();
            return Status.OK_STATUS;
          }
        };
        uiJob.setUser(false);
        uiJob.setSystem(true);
        uiJob.schedule();
      }

      public void done(IJobChangeEvent event)
      {
        if (event.getResult().isOK())
        {
          _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
              "Aktion Aktivierung beendet, bitte beachten Sie die Ausgaben der Konfiguration"));
          MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);

          updateButtonJobAktiv();
          aktionLadeDatenModell(false, null);
        }
        else
        {
          MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), false);
          UIJob uiJob = new UIJob("aktualisiere GUI") {

            @Override
            public IStatus runInUIThread(IProgressMonitor monitor)
            {
              // TODO Auto-generated method stub
              updateButtonKeineSelektion();
              return Status.OK_STATUS;
            }
          };
          uiJob.setUser(false);
          uiJob.setSystem(true);
          uiJob.schedule();
        }

      }
    });
    job.setSystem(false);
    job.setUser(true);
    job.schedule(); // start as soon as possible
  }

  private void aktionImport()
  {
    if (_config != null)
    {

      KbImportAssistent wizard = new KbImportAssistent(_dateiPfad, true);

      // IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
      // IWorkbenchPage page = dw.getActivePage();
      // IViewReference ref = page.findViewReference(KonfigAssSicht.ID);
      // IWorkbenchPart part = ref.getPart(false);

      wizard.init(PlatformUI.getWorkbench(), null);
      // Instantiates the wizard container with the wizard and opens it
      WizardDialog dialog = new WizardDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell(), wizard);
      dialog.create();

      int ret = dialog.open();

      if (ret == 0)
      {
        _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion Import", null);
        MeldungenSpeicher.getInstanz().setStatus(_status);

        KbImportModell model = wizard.getModel();

        final Map<String, File> files = model.getList();

        List<String> list = new ArrayList<String>();

        final Collection<String> values = files.keySet();

        for (String s : values)
        {
          if (!list.contains(s))
          {

            if (s.contains(".xml"))
            {
              int i = s.indexOf(".xml");
              String neu = s.subSequence(0, i).toString();
              // String neu = s.substring(0, s.length() - 4);
              list.add(neu);
            }

          }
        }

        if (list.size() > 0)
        {
          boolean vorhanden = false;

          for (String pid : list)
          {
            File verzeichnis = new File(_dateiPfad).getParentFile();
            if (verzeichnis.exists() && verzeichnis.isDirectory())
            {
              String[] ss = verzeichnis.list();
              for (String s : ss)
              {
                if (s.contains(".config"))
                {
                  int i = s.indexOf(".config");
                  String neu = s.subSequence(0, i).toString();
                  if (neu.equals(pid))
                  {
                    vorhanden = true;
                  }
                }
              }
            }
          }
          
          if (true)
          {
            final List<String> flist = list;

            final Job job = new Job("KonfigAss: Importiere") {
              protected IStatus run(IProgressMonitor monitor)
              {
                monitor.beginTask("Starte Import...", 5);
                monitor.worked(1);
                
                try
                {

                  KonfigApplikation app = new KonfigApplikation(_config);

                  File[] ff = files.values().toArray(new File[files.values().size()]);

                  File file = ff[0];

                  String path = file.getParent();
                  File folder = new File(path);

                  for (String string : flist)
                  {
                    _status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Importiere KB : " + string));
                  }
                  app.startImport(folder, flist);
                  monitor.worked(1);
                }
                catch (Exception e2)
                {
                  _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
                      "Aktion Import abgebrochen! Grund: es ist eine Ausnahme bei der Ausfhrung aufgetreten. ", e2));
                  return Status.CANCEL_STATUS;
                }
                finally
                {
                  monitor.worked(3);
                  monitor.done();
                }

                return Status.OK_STATUS;
              }
            };

            job.addJobChangeListener(new JobChangeAdapter() {

              @Override
              public void running(IJobChangeEvent event)
              {
                UIJob uiJob = new UIJob("aktualisiere GUI") {

                  @Override
                  public IStatus runInUIThread(IProgressMonitor monitor)
                  {
                    // TODO Auto-generated method stub
                    // updateButtonJobAktiv();
                    return Status.OK_STATUS;
                  }
                };
                uiJob.setUser(false);
                uiJob.setSystem(true);
                uiJob.schedule();
              }

              public void done(IJobChangeEvent event)
              {
                if (event.getResult().isOK())
                {
                  _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
                  "Aktion Import beendet, bitte beachten Sie die Ausgaben der Konfiguration"));
                  MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);
                  UIJob uiJob = new UIJob("aktualisiere GUI") {

                    @Override
                    public IStatus runInUIThread(IProgressMonitor monitor)
                    {
                      // TODO Auto-generated method stub
                      aktuelleInfoKbAnzeigen();
                      erzeugeInhaltTabelle(_listKonfigBereichKapsel);

                      updateTextKonfiguration(_dateiPfad, _pidAoeKonfig);
//                      updateButtonKeineSelektion();

                      setzeKonfigDatenModellFuerTka(_config, _dateiPfad);
                      return Status.OK_STATUS;
                    }
                  };
                  uiJob.setUser(false);
                  uiJob.setSystem(true);
                  uiJob.schedule();
                }
                else
                {
                  MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), false);
                  UIJob uiJob = new UIJob("aktualisiere GUI") {

                    @Override
                    public IStatus runInUIThread(IProgressMonitor monitor)
                    {
                      // TODO Auto-generated method stub
                      updateButtonKeineSelektion();
                      return Status.OK_STATUS;
                    }
                  };
                  uiJob.setUser(false);
                  uiJob.setSystem(true);
                  uiJob.schedule();
                }

              }
            });
            job.setSystem(false);
            job.setUser(true);
            job.schedule(); // start as soon as possible
          }
          else
          {
            _status
                .merge(new Status(
                    Status.ERROR,
                    Activator.PLUGIN_ID,
                    "Aktion Import abgebrochen! Grund: Zu mind. einem zu importierenden Konfigurationsbereich gibt es bereits eine *.config-Datei im Konfigurationsverzeichnis. "));
            MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), false);
            updateButtonKeineSelektion();
          }
        }
      }
      else
      {
        _selChangedlistener.selectionChanged(new SelectionChangedEvent(_tvZufluesse1, _tvZufluesse1.getSelection()));
      }
    }
  }

  private void aktionKbHinzufuegen()
  {
    KbEntfernenHinzufuegenAssistent wizard = new KbEntfernenHinzufuegenAssistent(_dateiPfad, true);

    wizard.init(PlatformUI.getWorkbench(), null);
    // Instantiates the wizard container with the wizard and opens it
    WizardDialog dialog = new WizardDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell(), wizard);
    dialog.create();

    int ret = dialog.open();

    if (ret == 0)
    {
      _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion KB Hinzufgen. ", null);
      MeldungenSpeicher.getInstanz().setStatus(_status);

      // XmlElementKonfigVerwaltung xmlElementKonfigVerwaltung = new XmlElementKonfigVerwaltung(_config);
      //
      // xmlElementKonfigVerwaltung.einlesenXmlAusDatei(_dateiPfad);

      KbEntfernenHinzufuegenModell model = wizard.getModel();

      Map<String, File> list = model.getList();
//      String pfadConfigDatei = model.getPfad();

      if (_tAutoCloser != null)
        _tAutoCloser.start();

      List<ConfigurationAreaManagementInfo> list3 = _config.getManagementFile()
          .getAllConfigurationAreaManagementInfos();

      List<String> listPidsCa = new ArrayList<String>();

      for (ConfigurationAreaManagementInfo cami : list3)
      {
        String pid = cami.getPid();
        listPidsCa.add(pid);
      }

      // Warten, bis der Thread beendet ist: finish
      try
      {
        if (_tAutoCloser != null)
          _tAutoCloser.join();

        _config = null;

        ManagementFile managementFile = new ManagementFile(new File(_dateiPfad));

        Set<String> keySet = list.keySet();
        List<String> pids = new ArrayList<String>();
        for (String s : keySet)
        {
          if (s.contains(".config"))
          {
            int i = s.indexOf(".config");
            String neu = s.subSequence(0, i).toString();

            if (listPidsCa.contains(neu))
            {
              _status.merge(new Status(Status.WARNING, Activator.PLUGIN_ID,
                  "KB ist schon in der aktuellen Konfiguration vorhanden: " + neu));
            }
            else
            {
              ConfigurationAreaManagementInfo cami = managementFile.addConfigurationAreaManagementInfo(neu);
//              String pfadParent = new File(pfadConfigDatei).getParent();
              String pfadParent = list.get(s).getParent();
              cami.setDirectory(new File(pfadParent));

              pids.add(neu);

              // xmlElementKonfigVerwaltung.hinzufuegenElementKonfig(new ElementKonfig(1, neu));
              _status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Fge KB hinzu: " + neu + 
                  " - Pfad: " + pfadParent));
            }

          }
          else
          {
            _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID, "Fehler beim Hinzufgen der Datei: " + s));
          }

        }

        
        if (pids.size() > 0)
        {
          // String s = xmlElementKonfigVerwaltung.ausgabeXmlFormatiert();
          // xmlElementKonfigVerwaltung.schreibeXMLStringInDatei(_dateiPfad, s, true);

          managementFile.save();
//          managementFile.close();

//          _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
//              "Aktion Laden Datenmodell beendet, bitte beachten Sie evtl. Ausgaben der Konfiguration"));
//          MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);
//
//          updateButtonJobAktiv();
//
//          aktionLadeDatenModell(false, pids);

        }
        
        managementFile.close();
        
        _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
        "Aktion Laden Datenmodell beendet, bitte beachten Sie evtl. Ausgaben der Konfiguration"));
        MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);

        updateButtonJobAktiv();

        aktionLadeDatenModell(false, pids);
      }
      catch (Exception ex)
      {
        _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
            "Aktion KB Hinzufgen abgebrochen! Grund: es ist eine Ausnahme bei der Ausfhrung aufgetreten. ", ex));
        MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), false);
      }
      // finally
      // {
      // MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis());
      // }
    }
    else
    {
       _selChangedlistener.selectionChanged(new SelectionChangedEvent(_tvZufluesse1, _tvZufluesse1.getSelection()));
    }
  }

  private void aktionKbEntfernen()
  {
    if (_config != null)
    {
      Map<String, File> list = new HashMap<String, File>();

      List<String> listFehler = new ArrayList<String>();

      IStructuredSelection selection = (IStructuredSelection)_tvZufluesse1.getSelection();

      if (selection.size() > 0)
      {
        Object firstElement = selection.getFirstElement();

        if (firstElement instanceof KonfigBereichDaten)
        {
          ArrayList<KonfigBereichDaten> listSel = new ArrayList<KonfigBereichDaten>();
          
          Iterator it = selection.iterator();
          while (it.hasNext())
          {
            KonfigBereichDaten kbd = (KonfigBereichDaten)it.next();
            listSel.add(kbd); 
          }
          
          boolean fehler = false;
          for (KonfigBereichDaten kbd : listSel)
          {
            for (KonfigBereichDaten kbd2 : _listKonfigBereichKapsel)
            {
              // ConfigurationArea ca = _config.getConfigurationArea(kbd2.getPid());
              ConfigurationArea ca = _config.getAllConfigurationAreas().get(kbd2.getPid());
              if (ca != null)
              {
                AttributeGroup atg = _config.getAttributeGroup("atg.konfigurationsBereichAbhngigkeiten");
                ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion((ConfigDataModel)_config);
                Data data = ((ConfigSystemObject)ca).getConfigurationData(atg, lookup);
                if (data != null)
                {
                  Data item = data.getItem("KonfigurationsAbhngigkeiten");
                  Array array = item.asArray();
                  for (int i = 0; i < array.getLength(); i++)
                  {
                    item = array.getItem(i);

                    String pid = item.getItem("BereichNotwendig").asTextValue().getValueText();
                    if (pid != null)
                    {
                      ConfigurationArea kb = _config.getConfigurationArea(pid);
                      if (kb != null)
                      {
                        Short activeVersion = kb.getActiveVersion();
                        String text1 = item.getItem("VersionNotwendig").asTextValue().getValueText();
                        String text2 = item.getItem("Kennung").asTextValue().getValueText();
                        Short neededVersion = new Short(text1);

                        if (text2.equals("notwendig"))
                        {

                          if (kbd.getPid().equals(kb.getPid()))
                          {
                            // System.out.println(kbd2.getPid() + " -> " + kb.getPid() + " : " + activeVersion + " : " +
                            // neededVersion);
                            // String meldung = "Fehler bei den Abhngigkeiten des KB: " + kbd.getPid() +
                            // " ist abhngig von: " + kb.getPid();
                            // _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID, meldung));
                            
                            if (!listFehler.contains(kbd2.getPid()) && !listSel.contains(kbd2)){                              
                              fehler = true;
                              listFehler.add(kbd2.getPid());
                            }
                          }
                        }
                      }
                    }
                  }
                }

              }
              else
              {
                String meldung = "Fehler bei den Abhngigkeiten des KB: " + kbd2.getPid()
                    + " es ist kein SystemObject fr den KB vorhanden!";
                _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID, meldung));
                fehler = true;
              }
            }
            list.put(kbd.getPid(), null);
          }
          if (fehler)
          {
            String meldung = "Das Entfernen des/der KB fhrt zu einem inkonsistenten Kernsystem,  "
                + "da dieser/diese von folgenden KB bentigt werden:\n\n";

            for (int j = 0; j < listFehler.size(); j++)
            {
              String s = listFehler.get(j);

              if (j < listFehler.size() - 1)
                s += ", ";

              meldung += s;
            }

            meldung += "\n\nSoll die Aktion trotzdem fortgefhrt werden?";

            MessageBox msg = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), SWT.OK
                | SWT.CANCEL | SWT.ERROR);
            msg.setText("Fehler KB: Abhngigkeiten");
            msg.setMessage(meldung);
            int open = msg.open();

            if (open != SWT.OK)
            {
              return;
            }

          }
        }
      }

      KbEntfernenHinzufuegenAssistent wizard = new KbEntfernenHinzufuegenAssistent(_pidAoeKonfig, false, list);

      wizard.init(PlatformUI.getWorkbench(), null);
      // Instantiates the wizard container with the wizard and opens it
      WizardDialog dialog = new WizardDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell(), wizard);
      dialog.create();
      int ret = dialog.open();

      if (ret == 0)
      {
        _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion KB Entfernen. ", null);
        MeldungenSpeicher.getInstanz().setStatus(_status);

        XmlElementKonfigVerwaltung xmlElementKonfigVerwaltung = new XmlElementKonfigVerwaltung(_config);

        xmlElementKonfigVerwaltung.einlesenXmlAusDatei(_dateiPfad);

        LinkedList<ElementKonfig> listeKbXml = xmlElementKonfigVerwaltung.getElementKonfigBereich();

        KbEntfernenHinzufuegenModell model = wizard.getModel();
        Map<String, File> list2 = model.getList();

        List<ElementKonfig> delList = new ArrayList<ElementKonfig>();

        if (listeKbXml != null)
        {
          for (ElementKonfig ek : listeKbXml)
          {
            if (ek != null)
            {
              String b = ek.getBezeichnung();
              Set<String> keySet = list2.keySet();
              for (String s : keySet)
              {
                if (s.equals(b))
                {
                  {
                    delList.add(ek);
                  }
                }
              }
            }
          }
        }

        if (_tAutoCloser != null)
          _tAutoCloser.start();

        // Warten, bis der Thread beendet ist: finish
        try
        {
          if (_tAutoCloser != null)
            _tAutoCloser.join();

          _config = null;

          if (delList.size() > 0)
          {
            for (ElementKonfig ek2 : delList)
            {
              xmlElementKonfigVerwaltung.entferneElementKonfig(ek2);
              _status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Entferne KB: " + ek2.getBezeichnung()));
            }

            String s = xmlElementKonfigVerwaltung.ausgabeXmlFormatiert();
            xmlElementKonfigVerwaltung.schreibeXMLStringInDatei(_dateiPfad, s, true);

            _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
                "Aktion KB Entfernen beendet, bitte beachten Sie evtl. die Ausgaben der Konfiguration"));
            MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);

            updateButtonJobAktiv();
            aktionLadeDatenModell(false, null);

          }
        }
        catch (Exception ex)
        {
          _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
              "Aktion KB Entfernen abgebrochen! Grund: es ist eine Ausnahme bei der Ausfhrung aufgetreten. ", ex));
          MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), false);
        }
        // finally
        // {
        // MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis());
        // }
      }else{
        _selChangedlistener.selectionChanged(new SelectionChangedEvent(_tvZufluesse1, _tvZufluesse1.getSelection()));
      }
    }
  }

  private void aktionKbNeu()
  {
    if (_config != null)
    {
      // KbNeuAssistent wizard = new KbNeuAssistent(_pidAoeKonfig);
      //
      // WizardDialog dialog = new WizardDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell(), wizard);
      //
      // IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
      // IWorkbenchPage page = dw.getActivePage();
      // IViewReference ref = page.findViewReference(KonfigAssSicht.ID);
      // IWorkbenchPart part = ref.getPart(false);
      //
      // wizard.init(part.getSite().getWorkbenchWindow().getWorkbench(), null);
      //
      // dialog.create();
      // int ret = dialog.open();
      //
      // if (ret == 0)
      // {
      // KbNeuModell model = wizard.getModel();

      // AttributeGroup atg = _config.getAttributeGroup("atg.konfigurationsnderungen");
      // Data data = kb.getConfigurationData(atg);

      InfoKonfigAenderungAssistent wizard = new InfoKonfigAenderungAssistent(null, true);

      // IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
      // IWorkbenchPage page = dw.getActivePage();
      // IViewReference ref = page.findViewReference(KonfigAssSicht.ID);
      // IWorkbenchPart part = ref.getPart(false);

      wizard.init(PlatformUI.getWorkbench(), null);
      // Instantiates the wizard container with the wizard and opens it
      WizardDialog dialog = new WizardDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell(), wizard);
      dialog.create();

      int ret = dialog.open();

      if (ret == 0)
      {
        InfoKonfigAenderungModell model = wizard.getModel();
        String pid = model.getPid();
        String name = model.getName();
        long stand = model.getStand();
        String autor = model.getAutor();
        String grund = model.getGrund();
        String text = model.getText();
        String info = model.getInfo();
        String beschreibung = model.getBeschreibung();

        _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion KB Neu", null);
        MeldungenSpeicher.getInstanz().setStatus(_status);

        String aoePid = _config.getConfigurationAuthorityPid();

        try
        {
          _status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Erstelle KB: " + pid + " unter dem KV: " + aoePid));

          // ConfigurationAreaFile[] files = _config.getConfigurationFileManager().getConfigurationAreas();
          //          
          // for (ConfigurationAreaFile caf : files)
          // {
          // System.out.println("caf: " + caf.getConfigurationAreaInfo().getPid());
          // if (pid.equals(caf.getConfigurationAreaInfo().getPid())){
          // System.out.println("vorhanden1");
          // vorhanden = true;
          // }
          // }

          boolean vorhanden = false;
          File verzeichnis = new File(_dateiPfad).getParentFile();
          if (verzeichnis.exists() && verzeichnis.isDirectory())
          {
            String[] list = verzeichnis.list();
            for (String s : list)
            {
              if (s.contains(".config"))
              {
                int i = s.indexOf(".config");
                String neu = s.subSequence(0, i).toString();
                if (neu.equals(pid))
                {
                  vorhanden = true;
                }
              }
            }
          }

          if (!vorhanden)
          {
            ConfigurationArea ca = _config.createConfigurationArea(name, pid, aoePid);

            AttributeGroup atg = _config.getAttributeGroup("atg.konfigurationsnderungen");
            Data data = AttributeBaseValueDataFactory.createAdapter(atg, AttributeHelper.getAttributesValues(atg));

            Array array = data.getItem("Konfigurationsnderung").asArray();
            array.setLength(1);
            array.getItem(0).getItem("Stand").asTimeValue().setMillis(stand);
            array.getItem(0).getItem("Autor").asTextValue().setText(autor);
            array.getItem(0).getItem("Version").asTextValue().setText("1");
            array.getItem(0).getItem("Grund").asTextValue().setText(grund);
            array.getItem(0).getItem("Text").asTextValue().setText(text);
            ca.setConfigurationData(atg, data);

            atg = _config.getAttributeGroup("atg.info");
            data = AttributeBaseValueDataFactory.createAdapter(atg, AttributeHelper.getAttributesValues(atg));

            data.getTextValue("kurzinfo").setText(info);
            data.getTextValue("beschreibung").setText(beschreibung);
            ca.setConfigurationData(atg, data);

            ConfigurationAreaManagementInfo cami = _config.getManagementFile().getConfigurationAreaManagementInfo(
                ca.getPid());
            cami.setDirectory(verzeichnis);

            _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
                "Aktion KB Neu beendet, bitte beachten Sie evtl. Ausgaben der Konfiguration"));

            UIJob uiJob = new UIJob("aktualisiere GUI") {

              @Override
              public IStatus runInUIThread(IProgressMonitor monitor)
              {
                // TODO Auto-generated method stub
                aktuelleInfoKbAnzeigen();
                erzeugeInhaltTabelle(_listKonfigBereichKapsel);

                updateTextKonfiguration(_dateiPfad, _pidAoeKonfig);
//                updateButtonKeineSelektion();

                setzeKonfigDatenModellFuerTka(_config, _dateiPfad);
                return Status.OK_STATUS;
              }
            };
            uiJob.setUser(false);
            uiJob.setSystem(true);
            uiJob.schedule();

          }
          else
          {
            _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
                "Aktion KB Neu abgebrochen! Grund: Es ist bereits ein Konfigurationsbereich vorhanden: " + pid));
          }
          MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);

        }
        catch (Exception ex)
        {
          _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
              "Aktion KB Neu abgebrochen! Grund: es ist eine Ausnahme bei der Ausfhrung aufgetreten. ", ex));
          MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), false);
        }
      }else{
        _selChangedlistener.selectionChanged(new SelectionChangedEvent(_tvZufluesse1, _tvZufluesse1.getSelection()));
      }
    }
  }

  private void aktionNeuerKvAoe()
  {
    KvAoeNeuAssistent wizard = new KvAoeNeuAssistent(PlatformUI.getWorkbench(), _config);

    WizardDialog2 dialog = new WizardDialog2(PlatformUI.getWorkbench().getDisplay().getActiveShell(), wizard);
    dialog.create();

    int ret = dialog.open();

    if (ret == 0)
    {
      final KvAoeNeuModell model = wizard.getModel();
      final String ziel = model.getZielVerzeichnis();
      _dateiPfad = ziel.replace("\\", "/") + "/config/verwaltungsdaten.xml";

      if (_dateiPfad != null)
      {
        final File file = new File(_dateiPfad);

        final String jobname = "KonfigAss: Neuer KV/AOE";

        final Job job = new Job(jobname) {
          protected IStatus run(IProgressMonitor monitor)
          {
            if (model.isUseKV())
            {
              monitor.beginTask("Start Erstellung KV...", 4);

              _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion Neuer KV", null);
              MeldungenSpeicher.getInstanz().setStatus(_status);
            }
            else
            {
              monitor.beginTask("Start Erstellung AOE...", 8);
              _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion Neuer AOE", null);
              MeldungenSpeicher.getInstanz().setStatus(_status);
            }

            if (model.getConfig() != null && model.getAutoCloser() != null)
            {
              _config = model.getConfig();
              _tAutoCloser = model.getAutoCloser();
            }
            else
            {
              _config = erzeugeKonfigDatenModellAusDatei(file);
            }
            monitor.worked(1);

            if (_config == null)
            {
              _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
                  "Aktion Neuer KV/AOE abgebrochen! Grund: es konnte kein Datenmodell erzeugt werden."));
              return Status.CANCEL_STATUS;
            }

            ConfigurationManagementFile managementFile = _config.getManagementFile();

            if (managementFile != null)
            {

              String pid = model.getPid();
              managementFile.setConfigurationAuthority(pid);

              if (model.isUseKV())
              {
                try
                {
                  String name = "KB fuer " + model.getName();
                  String kb = "kb." + model.getPid();
                  String code = model.getKodierung();
                  Long l = new Long(code);
                  String kv = managementFile.getConfigurationAuthority();

                  // Bereich und Konfigurationsverantwortlicher werden erstellt
                  SystemObjectType type = _config.getType("typ.konfigurationsVerantwortlicher");

                  monitor.setTaskName("Erzeuge KB fr KV");
                  ConfigurationArea ca = _config.createConfigurationArea(name, kb, type, kv, model.getName(), l);
                  monitor.worked(1);

                  _status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Starte Erzeugung KB fr KV", null));

                  setInfo(ca, model);

                  // die Zeiten, wann sich ein Datensatz gendert oder ein Objekt erzeugt wurde, mssen
                  // initialisiert
                  // werden
                  ((ConfigConfigurationArea)ca).initialiseTimeOfLastChanges();

                  final AttributeGroup atg = _config
                      .getAttributeGroup("atg.konfigurationsVerantwortlicherEigenschaften");
                  Data data = AttributeBaseValueDataFactory
                      .createAdapter(atg, AttributeHelper.getAttributesValues(atg));
                  data.getItem("kodierung").asTextValue().setText(code);

                  Collection<SystemObject> zuukuenftige = ca.getNewObjects();

                  for (SystemObject so : zuukuenftige)
                  {
                    if (so.getPid().equals(kv))
                    {

                      ConfigurationObject co = (ConfigurationObject)so;

                      co.setConfigurationData(atg, data);

                      _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
                          "Erzeugung KB fr KV erfolgreich beendet", null));
                    }

                  }

                  List<ConfigurationArea> list = new ArrayList<ConfigurationArea>();

                  list.addAll(_config.getAllConfigurationAreas().values());
                  // list.add(ca);

                  KonfigApplikation app = new KonfigApplikation(_config);

                  monitor.setTaskName("Aktiviere KB: " + ca.getPid());

                  _status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Aktiviere nderungen", null));

                  app.starteAktivierung(list, false, false);

                  monitor.worked(1);

                  _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
                      "Aktivierung  beendet, bitte beachten Sie evtl. die Ausgaben der Konfiguration", null));

                }
                catch (ConfigurationChangeException e1)
                {
                  _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID, "Fehler bei Erzeugung KB fr KV", e1));
                }

              }
              else
              {

                try
                {
                  String name = "AOE fuer " + model.getName();
                  String kb = "kb." + model.getPid();
                  String code = model.getKodierung();
                  Long l = new Long(code);
                  String kv = managementFile.getConfigurationAuthority();

                  _status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Starte Erzeugung KB fr AOE", null));

                  monitor.setTaskName("Erzeuge KB fr AOE");
                  // Bereich und AOE werden erstellt
                  SystemObjectType type = _config.getType("typ.autarkeOrganisationsEinheit");
                  ConfigurationArea ca = _config.createConfigurationArea(name, kb, type, kv, model.getName(), l);
                  monitor.worked(1);

                  setInfo(ca, model);

                  // die Zeiten, wann sich ein Datensatz gendert oder ein Objekt erzeugt wurde, mssen
                  // initialisiert
                  // werden
                  ((ConfigConfigurationArea)ca).initialiseTimeOfLastChanges();

                  _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
                      "Erzeugung KB fr AOE erfolgreich abgeschlossen", null));

                  final AttributeGroup atg = _config
                      .getAttributeGroup("atg.konfigurationsVerantwortlicherEigenschaften");
                  Data data = AttributeBaseValueDataFactory
                      .createAdapter(atg, AttributeHelper.getAttributesValues(atg));
                  data.getItem("kodierung").asTextValue().setText(code);

                  _status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Starte Erzeugung Mengen fr AOE", null));

                  Collection<SystemObject> zuukuenftige = ca.getNewObjects();
                  monitor.setTaskName("Erzeuge Mengen");

                  Map<String, String> mengen = model.getMengen();

                  for (SystemObject so : zuukuenftige)
                  {
                    if (so.getPid().equals(kv))
                    {
                      ConfigurationObject co = (ConfigurationObject)so;

                      co.setConfigurationData(atg, data);

                      for (Map.Entry<String, String> me : mengen.entrySet())
                      {

                        String value = me.getValue();
                        SystemObjectType typ = _config.getType(value);

                        if (typ != null)
                        {
                          ObjectSet set = (ObjectSet)ca.createConfigurationObject((ConfigurationObjectType)typ, "", me
                              .getKey(), null);

                          if (set != null)
                            co.addSet(set);
                        }
                        else
                        {
                          _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
                              "Konnte Typ der Menge nicht ermitteln: " + me.getKey(), null));
                        }

                      }
                    }
                  }
                  monitor.worked(1);

                  _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
                      "Erzeugung Mengen fr AOE erfolgreich beendet", null));

                  List<ConfigurationArea> list = new ArrayList<ConfigurationArea>();
                  list.add(ca);

                  KonfigApplikation app = new KonfigApplikation(_config);

                  _status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Starte Aktivierung", null));

                  monitor.setTaskName("Aktiviere: " + ca.getPid());
                  app.starteAktivierung(list, false, false);
                  monitor.worked(1);

                  _status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Aktivierung erfolgreich beendet", null));
                  _status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Starte Entladen Datenmodell", null));

                  monitor.setTaskName("Schliesse Konfiguration...");
                  _tAutoCloser.run();
                  try
                  {
                    _tAutoCloser.join();
                    monitor.worked(1);
                    _config = null;
                    monitor.setTaskName("Lade Datenmodell...");
                    _config = erzeugeKonfigDatenModellAusDatei(file);
                    monitor.worked(1);

                    _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
                        "Entladen Datenmodell erfolgreich beendet", null));

                    _status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Starte Erzeugung Default-KB fr AOE",
                        null));

                    monitor.setTaskName("Erzeuge Default KB fr AOE");

                    String nameDefaultKb = "Default-Bereich fr: " + model.getName();
                    String pidDefaultKb = "kb.default." + model.getPid();

                    ConfigurationArea caDef = _config.createConfigurationArea(nameDefaultKb, pidDefaultKb, kv);
                    monitor.worked(1);

                    setInfoDef(caDef, model);

                    _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
                        "Erzeugung Default-KB fr AOE erfolgreich beendet", null));

                    monitor.setTaskName("Aktualisiere KB fr AOE");

                    _status
                        .merge(new Status(Status.OK, Activator.PLUGIN_ID, "Starte Aktualisierung KB fr AOE ", null));
                    SystemObject so = _config.getObject(kv);
                    if (so != null)
                    {
                      ConfigurationObject co = (ConfigurationObject)so;
                      Data cd = co.getConfigurationData(atg);
                      cd.getItem("kodierung").asTextValue().setText(code);
                      Array array = cd.getItem("defaultBereich").asArray();
                      array.setLength(1);
                      array.getItem(0).asTextValue().setText("kb.default." + model.getPid());
                      co.setConfigurationData(atg, cd);
                    }

                    _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
                        "Aktualisierung KB fr AOE erfolgreich beendet", null));

                    monitor.worked(1);

                    list.add(caDef);

                  }
                  catch (InterruptedException e)
                  {
                    _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
                        "Fehler beim Schliessen des Datenmodells!", e));
                  }

                }
                catch (ConfigurationChangeException e1)
                {
                  _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
                      "Fehler beim ndern der Konfigurationsdaten fr den Default-KB!", e1));
                }

              }

            }

            monitor.done();

            if (_config == null)
            {
              return Status.CANCEL_STATUS;
            }
            else
            {
              return Status.OK_STATUS;
            }
          }
        };

        job.addJobChangeListener(new JobChangeAdapter() {

          @Override
          public void running(IJobChangeEvent event)
          {
            UIJob uiJob = new UIJob("aktualisiere GUI") {

              @Override
              public IStatus runInUIThread(IProgressMonitor monitor)
              {
                // TODO Auto-generated method stub
                // updateButtonJobAktiv();
                return Status.OK_STATUS;
              }
            };
            uiJob.setUser(false);
            uiJob.setSystem(true);
            uiJob.schedule();
          }

          public void done(IJobChangeEvent event)
          {
            if (event.getResult().isOK())
            {
              if (model.isUseKV())
              {
                _tAutoCloser.start();

                // Warten, bis der Thread beendet ist: finish
                try
                {
                  _tAutoCloser.join();

                  _config = null;

                  updateButtonJobAktiv();

//                  aktionLadeDatenModell(false, null);

                  _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
                      "Aktion Neuer KV beendet, bitte beachten Sie die Ausgaben der Konfiguration", null));
                  MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);
                  
                  aktionLadeDatenModell(false, null);
                }
                catch (InterruptedException ex)
                {
                  _status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Fehler beim Schliessen des Datenmodells!",
                      ex));
                  MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), false);
                }

              }
              else
              {
                _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
                    "Aktion Neuer AOE beendet, bitte beachten Sie die Ausgaben der Konfiguration", null));
                MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);

                Map<String, ConfigurationArea> areas = _config.getAllConfigurationAreas();

                List<ConfigurationArea> list = new ArrayList<ConfigurationArea>();

                for (Map.Entry<String, ConfigurationArea> me : areas.entrySet())
                {
                  ConfigurationArea ca = me.getValue();
                  if (!ca.isValid())
                  {
                    list.add(ca);
                  }
                }

                aktionAktivierung(list);
              }
            }
            else
            {
              if (model.isUseKV())
              {
                _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
                    "Aktion Neuer KV abgebrochen! Grund: es ist mind. eine Ausnahme bei der Ausfhrung aufgetreten. ",
                    null));
                MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);
              }
              else
              {
                _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
                    "Aktion Neuer AOE abgebrochen! Grund: es ist mind. eine Ausnahme bei der Ausfhrung aufgetreten. ",
                    null));
                MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);
              }

              UIJob uiJob = new UIJob("aktualisiere GUI") {

                @Override
                public IStatus runInUIThread(IProgressMonitor monitor)
                {
                  // TODO Auto-generated method stub
                  erzeugeInhaltTabelle(new ArrayList<KonfigBereichDaten>());
                  updateTextKonfiguration("", "");
                  updateButtonKeinDatenModell();
//                  setzeKonfigDatenModellFuerTka(null, "");
                  setzeKonfigDatenModellFuerTka(null, _dateiPfad);
                
                  // Lschen Ordnerstruktur
                  KonfigAssDateiOperationen.loescheVerzeichnis(new File(ziel), monitor, 1);
                                  
                  return Status.OK_STATUS;
                }
              };
              uiJob.setUser(false);
              uiJob.setSystem(true);
              uiJob.schedule();

            }

          }
        });
        job.setSystem(false);
        job.setUser(true);
        job.schedule(); // start as soon as possible
      }
    }
    else
    {
      updateButtonKeinDatenModell();
    }
  }

//  private void aktionFreigabeZurUebernahme()
//  {
//    _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion Freigabe zur bernahme", null);
//    MeldungenSpeicher.getInstanz().setStatus(_status);
//
//    final Job job = new Job("KonfigAss: Freigabe zur bernahme") {
//      protected IStatus run(IProgressMonitor monitor)
//      {
//
//        monitor.beginTask("Starte Freigabe zur bernahme...", 1);
//
//        try
//        {
//          if (_selection != null)
//          {
//            Object obj = _selection.getFirstElement();
//
//            if (obj instanceof KonfigBereichDaten)
//            {
//              Iterator it = _selection.iterator();
//
//              List<ConfigurationArea> list = new ArrayList<ConfigurationArea>();
//
//              while (it.hasNext())
//              {
//                KonfigBereichDaten kbd = (KonfigBereichDaten)it.next();
//                String pid = kbd.getPid();
//
//                Map<String, ConfigurationArea> map = _config.getAllConfigurationAreas();
//                if (map.containsKey(pid))
//                {
//                  ConfigurationArea ca = map.get(pid);
//                  list.add(ca);
//                }
//              }
//
//              if (list.size() > 0)
//              {
//                KonfigApplikation app = new KonfigApplikation(_config);
//
//                for (ConfigurationArea ca : list)
//                {
//                  _status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Freigabe zur bernahme fr KB: "
//                      + ca.getPid()));
//                }
//
//                app.starteFreigabeZurbernahme(list, false, false);
//                monitor.worked(1);
//
//              }
//            }
//          }
//        }
//        catch (Exception e2)
//        {
//          _status
//              .merge(new Status(
//                  Status.ERROR,
//                  Activator.PLUGIN_ID,
//                  "Aktion Freigabe zur bernahme abgebrochen! Grund: es ist eine Ausnahme bei der Ausfhrung aufgetreten. ",
//                  e2));
//          return Status.CANCEL_STATUS;
//        }
//        finally
//        {
//          monitor.done();
//        }
//
//        return Status.OK_STATUS;
//      }
//    };
//
//    job.addJobChangeListener(new JobChangeAdapter() {
//
//      @Override
//      public void running(IJobChangeEvent event)
//      {
//        // TODO Auto-generated method stub
//        UIJob uiJob = new UIJob("aktualisiere GUI") {
//
//          @Override
//          public IStatus runInUIThread(IProgressMonitor monitor)
//          {
//            // TODO Auto-generated method stub
//            // updateButtonJobAktiv();
//            return Status.OK_STATUS;
//          }
//        };
//        uiJob.setUser(false);
//        uiJob.setSystem(true);
//        uiJob.schedule();
//
//      }
//
//      public void done(IJobChangeEvent event)
//      {
//        if (event.getResult().isOK())
//        {
//          _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
//              "Aktion Freigabe zur bernahme beendet, bitte beachten Sie die Ausgaben der Konfiguration"));
//          MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);
//          UIJob uiJob = new UIJob("aktualisiere GUI") {
//
//            @Override
//            public IStatus runInUIThread(IProgressMonitor monitor)
//            {
//              // TODO Auto-generated method stub
//              aktuelleInfoKbAnzeigen();
//              erzeugeInhaltTabelle(_listKonfigBereichKapsel);
////              updateButtonKeineSelektion();
//              return Status.OK_STATUS;
//            }
//          };
//          uiJob.setUser(false);
//          uiJob.setSystem(true);
//          uiJob.schedule();
//
//        }
//        else
//        {
//          MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), false);
//          UIJob uiJob = new UIJob("aktualisiere GUI") {
//
//            @Override
//            public IStatus runInUIThread(IProgressMonitor monitor)
//            {
//              // TODO Auto-generated method stub
//              updateButtonKeineSelektion();
//              return Status.OK_STATUS;
//            }
//          };
//          uiJob.setUser(false);
//          uiJob.setSystem(true);
//          uiJob.schedule();
//        }
//      }
//    });
//    job.setSystem(false);
//    job.setUser(true);
//    job.schedule(); // start as soon as possible
//  }
  
  private void aktionFreigabeZurUebernahme()
  {
    if (_selection != null)
    {
      Object obj = _selection.getFirstElement();
      if (obj instanceof KonfigBereichDaten)
      {
        Iterator it = _selection.iterator();

        Map<String, File> list = new HashMap<String, File>();

        while (it.hasNext())
        {
          KonfigBereichDaten kbd = (KonfigBereichDaten)it.next();
          String pid = kbd.getPid();
          Map<String, ConfigurationArea> map = _config.getAllConfigurationAreas();
          if (map.containsKey(pid))
          {
            ConfigurationArea ca = map.get(pid);
            list.put(ca.getPid(), new File(""));
          }

        }
        if (list.size() > 0)
        {                
          KbAuswahlAssistent wizard = new KbAuswahlAssistent("Freigabe zur bernahme", "freigegeben", list);

          wizard.init(PlatformUI.getWorkbench(), null);
          // Instantiates the wizard container with the wizard and opens it
          WizardDialog dialog = new WizardDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell(), wizard);
          dialog.create();
          int ret = dialog.open();

          if (ret == 0)
          {
             _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion Freigabe zur bernahme", null);
             MeldungenSpeicher.getInstanz().setStatus(_status);
            
             KbAuswahlModell model = wizard.getModel();
             final Map<String, File> list2 = model.getList();
              
             final Job job = new Job("KonfigAss: Freigabe zur bernahme") {

               protected IStatus run(IProgressMonitor monitor)
               {
                 monitor.beginTask("Starte Freigabe...", 5);
                 monitor.worked(1);

                 try
                 {
                   
                   List<ConfigurationArea> listCa = new ArrayList<ConfigurationArea>();
                   
                   for (Map.Entry<String, File> me : list2.entrySet())
                   {
                     String pid = me.getKey();
                     
                     ConfigurationArea ca = _config.getAllConfigurationAreas().get(pid);
                     if (ca != null){
                       listCa.add(ca);                      
                     }
                   }
                   
                   for (ConfigurationArea ca : listCa)
                   {                     
                     _status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Freigabe zur bernahme von KB: " + ca.getPid()));
                   }
                   
                   KonfigApplikation app = new KonfigApplikation(_config);                               
                   app.starteFreigabeZurUebernahme(listCa, false, false);

                   monitor.worked(1);
                   
                   // _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion Aktivierung", null);
                   // MeldungenSpeicher.getInstanz().setStatus(_status);

                 }
                 catch (Exception e2)
                 {
                   _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
                       "Aktion Freigabe zur bernahme abgebrochen! Grund: es ist eine Ausnahme bei der Ausfhrung aufgetreten. " + e2));
                   return Status.CANCEL_STATUS;
                 }
                 finally
                 {
                   monitor.worked(3);
                   monitor.done();
                 }

                 return Status.OK_STATUS;
               }
             };

             job.addJobChangeListener(new JobChangeAdapter() {

               @Override
               public void running(IJobChangeEvent event)
               {
                 UIJob uiJob = new UIJob("aktualisiere GUI") {

                   @Override
                   public IStatus runInUIThread(IProgressMonitor monitor)
                   {
                     // TODO Auto-generated method stub
                     // updateButtonJobAktiv();
                     return Status.OK_STATUS;
                   }
                 };
                 uiJob.setUser(false);
                 uiJob.setSystem(true);
                 uiJob.schedule();
               }

               public void done(IJobChangeEvent event)
               {
                 if (event.getResult().isOK())
                 {
                   _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
                       "Aktion Freigabe zur bernahme beendet, bitte beachten Sie die Ausgaben der Konfiguration"));
                   MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);

                   updateButtonJobAktiv();
                   aktionLadeDatenModell(false, null);
                 }
                 else
                 {
                   MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), false);
                   UIJob uiJob = new UIJob("aktualisiere GUI") {

                     @Override
                     public IStatus runInUIThread(IProgressMonitor monitor)
                     {
                       // TODO Auto-generated method stub
                       updateButtonKeineSelektion();
                       return Status.OK_STATUS;
                     }
                   };
                   uiJob.setUser(false);
                   uiJob.setSystem(true);
                   uiJob.schedule();
                 }

               }
             });
             job.setSystem(false);
             job.setUser(true);
             job.schedule(); // start as soon as possible             
             
          }else{
            _selChangedlistener.selectionChanged(new SelectionChangedEvent(_tvZufluesse1, _tvZufluesse1.getSelection()));
          }
        }
      }
    } 
  }

//  private void aktionFreigabeZurAktivierung()
//  {
//    _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion Freigabe zur Aktivierung", null);
//    MeldungenSpeicher.getInstanz().setStatus(_status);
//
//    final Job job = new Job("KonfigAss: Freigabe zur Aktivierung") {
//      protected IStatus run(IProgressMonitor monitor)
//      {
//
//        monitor.beginTask("Starte Freigabe zur Aktivierung...", 1);
//
//        try
//        {
//          if (_selection != null)
//          {
//            Object obj = _selection.getFirstElement();
//
//            if (obj instanceof KonfigBereichDaten)
//            {
//              Iterator it = _selection.iterator();
//
//              List<ConfigurationArea> list = new ArrayList<ConfigurationArea>();
//
//              while (it.hasNext())
//              {
//                KonfigBereichDaten kbd = (KonfigBereichDaten)it.next();
//                String pid = kbd.getPid();
//
//                Map<String, ConfigurationArea> map = _config.getAllConfigurationAreas();
//                if (map.containsKey(pid))
//                {
//                  ConfigurationArea ca = map.get(pid);
//                  list.add(ca);
//                }
//              }
//
//              if (list.size() > 0)
//              {
//                KonfigApplikation app = new KonfigApplikation(_config);
//
//                for (ConfigurationArea ca : list)
//                {
//                  _status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Freigabe zur Aktivierung fr KB: "
//                      + ca.getPid()));
//                }
//
//                app.starteFreigabeZurAktivierung(list);
//                monitor.worked(1);
//
//              }
//            }
//          }
//        }
//        catch (Exception e2)
//        {
//          _status
//              .merge(new Status(
//                  Status.ERROR,
//                  Activator.PLUGIN_ID,
//                  "Aktion Freigabe zur Aktivierung abgebrochen! Grund: es ist eine Ausnahme bei der Ausfhrung aufgetreten. ",
//                  e2));
//          return Status.CANCEL_STATUS;
//        }
//        finally
//        {
//          monitor.done();
//        }
//
//        return Status.OK_STATUS;
//      }
//    };
//
//    job.addJobChangeListener(new JobChangeAdapter() {
//
//      @Override
//      public void running(IJobChangeEvent event)
//      {
//        // TODO Auto-generated method stub
//        UIJob uiJob = new UIJob("aktualisiere GUI") {
//
//          @Override
//          public IStatus runInUIThread(IProgressMonitor monitor)
//          {
//            // TODO Auto-generated method stub
//            // updateButtonJobAktiv();
//            return Status.OK_STATUS;
//          }
//        };
//        uiJob.setUser(false);
//        uiJob.setSystem(true);
//        uiJob.schedule();
//
//      }
//
//      public void done(IJobChangeEvent event)
//      {
//        if (event.getResult().isOK())
//        {
//          _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
//              "Aktion Freigabe zur Aktivierung beendet, bitte beachten Sie evtl. die Ausgaben der Konfiguration. "));
//          MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);
//          UIJob uiJob = new UIJob("aktualisiere GUI") {
//
//            @Override
//            public IStatus runInUIThread(IProgressMonitor monitor)
//            {
//              // TODO Auto-generated method stub
//              aktuelleInfoKbAnzeigen();
//              erzeugeInhaltTabelle(_listKonfigBereichKapsel);
////              updateButtonKeineSelektion();
//              return Status.OK_STATUS;
//            }
//          };
//          uiJob.setUser(false);
//          uiJob.setSystem(true);
//          uiJob.schedule();
//
//        }
//        else
//        {
//          MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), false);
//          UIJob uiJob = new UIJob("aktualisiere GUI") {
//
//            @Override
//            public IStatus runInUIThread(IProgressMonitor monitor)
//            {
//              // TODO Auto-generated method stub
//              updateButtonKeineSelektion();
//              return Status.OK_STATUS;
//            }
//          };
//          uiJob.setUser(false);
//          uiJob.setSystem(true);
//          uiJob.schedule();
//        }
//
//      }
//    });
//    job.setSystem(false);
//    job.setUser(true);
//    job.schedule(); // start as soon as possible
//  }
  private void aktionFreigabeZurAktivierung()
  {
    if (_selection != null)
    {
      Object obj = _selection.getFirstElement();
      if (obj instanceof KonfigBereichDaten)
      {
        Iterator it = _selection.iterator();

        Map<String, File> list = new HashMap<String, File>();

        while (it.hasNext())
        {
          KonfigBereichDaten kbd = (KonfigBereichDaten)it.next();
          String pid = kbd.getPid();
          Map<String, ConfigurationArea> map = _config.getAllConfigurationAreas();
          if (map.containsKey(pid))
          {
            ConfigurationArea ca = map.get(pid);
            list.put(ca.getPid(), new File(""));
          }

        }
        if (list.size() > 0)
        {                
          KbAuswahlAssistent wizard = new KbAuswahlAssistent("Freigabe zur Aktivierung", "freigegeben", list);

          wizard.init(PlatformUI.getWorkbench(), null);
          // Instantiates the wizard container with the wizard and opens it
          WizardDialog dialog = new WizardDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell(), wizard);
          dialog.create();
          int ret = dialog.open();

          if (ret == 0)
          {
             _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion Freigabe zur Aktivierung", null);
             MeldungenSpeicher.getInstanz().setStatus(_status);
            
             KbAuswahlModell model = wizard.getModel();
             final Map<String, File> list2 = model.getList();
              
             final Job job = new Job("KonfigAss: Freigabe zur Aktivierung") {

               protected IStatus run(IProgressMonitor monitor)
               {
                 monitor.beginTask("Starte Freigabe...", 5);
                 monitor.worked(1);

                 try
                 {
                   
                   List<ConfigurationArea> listCa = new ArrayList<ConfigurationArea>();
                   
                   for (Map.Entry<String, File> me : list2.entrySet())
                   {
                     String pid = me.getKey();
                     
//                     System.out.println("ca: " + "-" + pid + "-");
                     
                     ConfigurationArea ca = _config.getAllConfigurationAreas().get(pid);
                     if (ca != null){
                       listCa.add(ca);
                     }
                   }
                   
                   for (ConfigurationArea ca : listCa)
                   {                     
                     _status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Aktiviere KB: " + ca.getPid()));
                   }
                   
                   KonfigApplikation app = new KonfigApplikation(_config);                               
                   app.starteFreigabeZurAktivierung(listCa);

                   monitor.worked(1);
                   
                   // _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion Aktivierung", null);
                   // MeldungenSpeicher.getInstanz().setStatus(_status);

                 }
                 catch (Exception e2)
                 {
                   _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
                       "Aktion Freigabe zur Aktivierung abgebrochen! Grund: es ist eine Ausnahme bei der Ausfhrung aufgetreten. " + e2));
                   return Status.CANCEL_STATUS;
                 }
                 finally
                 {
                   monitor.worked(3);
                   monitor.done();
                 }

                 return Status.OK_STATUS;
               }
             };

             job.addJobChangeListener(new JobChangeAdapter() {

               @Override
               public void running(IJobChangeEvent event)
               {
                 UIJob uiJob = new UIJob("aktualisiere GUI") {

                   @Override
                   public IStatus runInUIThread(IProgressMonitor monitor)
                   {
                     // TODO Auto-generated method stub
                     // updateButtonJobAktiv();
                     return Status.OK_STATUS;
                   }
                 };
                 uiJob.setUser(false);
                 uiJob.setSystem(true);
                 uiJob.schedule();
               }

               public void done(IJobChangeEvent event)
               {
                 if (event.getResult().isOK())
                 {
                   _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
                       "Aktion Freigabe zur Aktivierung beendet, bitte beachten Sie die Ausgaben der Konfiguration"));
                   MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);

                   updateButtonJobAktiv();
                   aktionLadeDatenModell(false, null);
                 }
                 else
                 {
                   MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), false);
                   UIJob uiJob = new UIJob("aktualisiere GUI") {

                     @Override
                     public IStatus runInUIThread(IProgressMonitor monitor)
                     {
                       // TODO Auto-generated method stub
                       updateButtonKeineSelektion();
                       return Status.OK_STATUS;
                     }
                   };
                   uiJob.setUser(false);
                   uiJob.setSystem(true);
                   uiJob.schedule();
                 }

               }
             });
             job.setSystem(false);
             job.setUser(true);
             job.schedule(); // start as soon as possible             
             
          }else{
            _selChangedlistener.selectionChanged(new SelectionChangedEvent(_tvZufluesse1, _tvZufluesse1.getSelection()));
          }
        }
      }
    } 
  }

//  private void aktionFreigabeZurAktivierungKv()
//  {
//    _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion F.z.A. ohne lokale Aktivierung durch KV",
//        null);
//    MeldungenSpeicher.getInstanz().setStatus(_status);
//
//    final Job job = new Job("KonfigAss: F.z.A. ohne lokale Aktivierung durch KV") {
//      protected IStatus run(IProgressMonitor monitor)
//      {
//
//        monitor.beginTask("Starte F.z.A. ohne lokale Aktivierung durch KV...", 1);
//
//        try
//        {
//          if (_selection != null)
//          {
//            Object obj = _selection.getFirstElement();
//
//            if (obj instanceof KonfigBereichDaten)
//            {
//              Iterator it = _selection.iterator();
//
//              List<ConfigurationArea> list = new ArrayList<ConfigurationArea>();
//
//              while (it.hasNext())
//              {
//                KonfigBereichDaten kbd = (KonfigBereichDaten)it.next();
//                String pid = kbd.getPid();
//
//                Map<String, ConfigurationArea> map = _config.getAllConfigurationAreas();
//                if (map.containsKey(pid))
//                {
//                  ConfigurationArea ca = map.get(pid);
//                  list.add(ca);
//                }
//              }
//
//              if (list.size() > 0)
//              {
//                KonfigApplikation app = new KonfigApplikation(_config);
//
//                for (ConfigurationArea ca : list)
//                {
//                  _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
//                      "F.z.A. ohne lokale Aktivierung durch KV fr KB: " + ca.getPid()));
//                }
//
//                app.startFreigabeZurAktivierungOhneKVAktivierung(list, false, false);
//                monitor.worked(1);
//
//              }
//            }
//          }
//        }
//        catch (Exception e2)
//        {
//          _status
//              .merge(new Status(
//                  Status.ERROR,
//                  Activator.PLUGIN_ID,
//                  "Aktion F.z.A. ohne lokale Aktivierung durch KV abgebrochen! Grund: es ist eine Ausnahme bei der Ausfhrung aufgetreten. ",
//                  e2));
//          return Status.CANCEL_STATUS;
//        }
//        finally
//        {
//          monitor.done();
//        }
//
//        return Status.OK_STATUS;
//      }
//    };
//
//    job.addJobChangeListener(new JobChangeAdapter() {
//
//      @Override
//      public void running(IJobChangeEvent event)
//      {
//        // TODO Auto-generated method stub
//        UIJob uiJob = new UIJob("aktualisiere GUI") {
//
//          @Override
//          public IStatus runInUIThread(IProgressMonitor monitor)
//          {
//            // TODO Auto-generated method stub
//            // updateButtonJobAktiv();
//            return Status.OK_STATUS;
//          }
//        };
//        uiJob.setUser(false);
//        uiJob.setSystem(true);
//        uiJob.schedule();
//
//      }
//
//      public void done(IJobChangeEvent event)
//      {
//        if (event.getResult().isOK())
//        {
//          _status
//              .merge(new Status(Status.OK, Activator.PLUGIN_ID,
//                  "Aktion F.z.A. ohne lokale Aktivierung durch KV beendet, bitte beachten Sie die Ausgaben der Konfiguration"));
//          MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);
//          UIJob uiJob = new UIJob("aktualisiere GUI") {
//
//            @Override
//            public IStatus runInUIThread(IProgressMonitor monitor)
//            {
//              // TODO Auto-generated method stub
//              aktuelleInfoKbAnzeigen();
//              erzeugeInhaltTabelle(_listKonfigBereichKapsel);
////              updateButtonKeineSelektion();
//              return Status.OK_STATUS;
//            }
//          };
//          uiJob.setUser(false);
//          uiJob.setSystem(true);
//          uiJob.schedule();
//
//        }
//        else
//        {
//          MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), false);
//          UIJob uiJob = new UIJob("aktualisiere GUI") {
//
//            @Override
//            public IStatus runInUIThread(IProgressMonitor monitor)
//            {
//              // TODO Auto-generated method stub
//              updateButtonKeineSelektion();
//              return Status.OK_STATUS;
//            }
//          };
//          uiJob.setUser(false);
//          uiJob.setSystem(true);
//          uiJob.schedule();
//        }
//
//      }
//    });
//    job.setSystem(false);
//    job.setUser(true);
//    job.schedule(); // start as soon as possible
//  }
  private void aktionFreigabeZurAktivierungKv()
  {
    if (_selection != null)
    {
      Object obj = _selection.getFirstElement();
      if (obj instanceof KonfigBereichDaten)
      {
        Iterator it = _selection.iterator();

        Map<String, File> list = new HashMap<String, File>();

        while (it.hasNext())
        {
          KonfigBereichDaten kbd = (KonfigBereichDaten)it.next();
          String pid = kbd.getPid();
          Map<String, ConfigurationArea> map = _config.getAllConfigurationAreas();
          if (map.containsKey(pid))
          {
            ConfigurationArea ca = map.get(pid);
            list.put(ca.getPid(), new File(""));
          }

        }
        if (list.size() > 0)
        {                
          KbAuswahlAssistent wizard = new KbAuswahlAssistent("F.z.Akt.o.l.A.", "freigegeben", list);

          wizard.init(PlatformUI.getWorkbench(), null);
          // Instantiates the wizard container with the wizard and opens it
          WizardDialog dialog = new WizardDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell(), wizard);
          dialog.create();
          int ret = dialog.open();

          if (ret == 0)
          {
             _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion F.z.Akt.o.l.A.", null);
             MeldungenSpeicher.getInstanz().setStatus(_status);
            
             KbAuswahlModell model = wizard.getModel();
             final Map<String, File> list2 = model.getList();
              
             final Job job = new Job("KonfigAss: F.z.Akt.o.l.A.") {

               protected IStatus run(IProgressMonitor monitor)
               {
                 monitor.beginTask("Starte Freigabe...", 5);
                 monitor.worked(1);

                 try
                 {
                   
                   List<ConfigurationArea> listCa = new ArrayList<ConfigurationArea>();
                   
                   for (Map.Entry<String, File> me : list2.entrySet())
                   {
                     String pid = me.getKey();
                     
//                     System.out.println("ca: " + "-" + pid + "-");
                     
                     ConfigurationArea ca = _config.getAllConfigurationAreas().get(pid);
                     if (ca != null){
                       listCa.add(ca);
                     }
                   }
                   
                   for (ConfigurationArea ca : listCa)
                   {                     
                     _status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Freigabe zur Aktivierung von KB: " + ca.getPid()));
                   }
                   
                   KonfigApplikation app = new KonfigApplikation(_config);                               
                   app.startFreigabeZurAktivierungOhneKVAktivierung(listCa, false, false);

                   monitor.worked(1);
                   
                   // _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion Aktivierung", null);
                   // MeldungenSpeicher.getInstanz().setStatus(_status);

                 }
                 catch (Exception e2)
                 {
                   _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
                       "Aktion Freigabe zur Aktivierung o.l.A.d.K abgebrochen! Grund: es ist eine Ausnahme bei der Ausfhrung aufgetreten. " + e2));
                   return Status.CANCEL_STATUS;
                 }
                 finally
                 {
                   monitor.worked(3);
                   monitor.done();
                 }

                 return Status.OK_STATUS;
               }
             };

             job.addJobChangeListener(new JobChangeAdapter() {

               @Override
               public void running(IJobChangeEvent event)
               {
                 UIJob uiJob = new UIJob("aktualisiere GUI") {

                   @Override
                   public IStatus runInUIThread(IProgressMonitor monitor)
                   {
                     // TODO Auto-generated method stub
                     // updateButtonJobAktiv();
                     return Status.OK_STATUS;
                   }
                 };
                 uiJob.setUser(false);
                 uiJob.setSystem(true);
                 uiJob.schedule();
               }

               public void done(IJobChangeEvent event)
               {
                 if (event.getResult().isOK())
                 {
                   _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
                       "Aktion Freigabe zur bernahme beendet, bitte beachten Sie die Ausgaben der Konfiguration"));
                   MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);

                   updateButtonJobAktiv();
                   aktionLadeDatenModell(false, null);
                 }
                 else
                 {
                   MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), false);
                   UIJob uiJob = new UIJob("aktualisiere GUI") {

                     @Override
                     public IStatus runInUIThread(IProgressMonitor monitor)
                     {
                       // TODO Auto-generated method stub
                       updateButtonKeineSelektion();
                       return Status.OK_STATUS;
                     }
                   };
                   uiJob.setUser(false);
                   uiJob.setSystem(true);
                   uiJob.schedule();
                 }

               }
             });
             job.setSystem(false);
             job.setUser(true);
             job.schedule(); // start as soon as possible             
             
          }else{
            _selChangedlistener.selectionChanged(new SelectionChangedEvent(_tvZufluesse1, _tvZufluesse1.getSelection()));
          }
        }
      }
    } 
  }

//  private void aktionExport()
//  {
//    _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion Export", null);
//    MeldungenSpeicher.getInstanz().setStatus(_status);
//
//    final Job job = new Job("KonfigAss: Export") {
//      protected IStatus run(IProgressMonitor monitor)
//      {
//
//        monitor.beginTask("Starte Export...", 1);
//
//        try
//        {
//          if (_selection != null)
//          {
//            Object obj = _selection.getFirstElement();
//
//            if (obj instanceof KonfigBereichDaten)
//            {
//              Iterator it = _selection.iterator();
//
//              List<String> list = new ArrayList<String>();
//
//              while (it.hasNext())
//              {
//                KonfigBereichDaten kbd = (KonfigBereichDaten)it.next();
//                String pid = kbd.getPid();
//
//                Map<String, ConfigurationArea> map = _config.getAllConfigurationAreas();
//                if (map.containsKey(pid))
//                {
//                  list.add(pid);
//                }
//              }
//
//              if (list.size() > 0)
//              {
//                KonfigApplikation app = new KonfigApplikation(_config);
//
//                for (String s : list)
//                {
//                  _status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Export fr KB: " + s));
//                }
//
//                File file = new File(_dateiPfad);
//
//                String configPfad = file.getParent();
//                file = new File(configPfad);
//                String pfad = file.getParent();
//
//                String xmlPfad = pfad + "/xml";
//
//                file = new File(xmlPfad);
//
//                if (!file.exists())
//                  file.mkdir();
//
//                if (file.isDirectory())
//                {
//                  app.startExport(file, list);
//                  monitor.worked(1);
//                }
//              }
//            }
//          }
//        }
//        catch (Exception e2)
//        {
//          _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
//              "Aktion Export abgebrochen! Grund: es ist eine Ausnahme bei der Ausfhrung aufgetreten. ", e2));
//          return Status.CANCEL_STATUS;
//        }
//        finally
//        {
//          monitor.done();
//        }
//
//        return Status.OK_STATUS;
//      }
//    };
//
//    job.addJobChangeListener(new JobChangeAdapter() {
//
//      @Override
//      public void running(IJobChangeEvent event)
//      {
//        // TODO Auto-generated method stub
//        UIJob uiJob = new UIJob("aktualisiere GUI") {
//
//          @Override
//          public IStatus runInUIThread(IProgressMonitor monitor)
//          {
//            // TODO Auto-generated method stub
//            // updateButtonJobAktiv();
//            return Status.OK_STATUS;
//          }
//        };
//        uiJob.setUser(false);
//        uiJob.setSystem(true);
//        uiJob.schedule();
//
//      }
//
//      public void done(IJobChangeEvent event)
//      {
//        if (event.getResult().isOK())
//        {
//          _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
//              "Aktion Export beendet, bitte beachten Sie die Ausgaben der Konfiguration"));
//          MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);
//          UIJob uiJob = new UIJob("aktualisiere GUI") {
//
//            @Override
//            public IStatus runInUIThread(IProgressMonitor monitor)
//            {
//              // TODO Auto-generated method stub
//              aktuelleInfoKbAnzeigen();
//              erzeugeInhaltTabelle(_listKonfigBereichKapsel);
////              updateButtonKeineSelektion();
//              return Status.OK_STATUS;
//            }
//          };
//          uiJob.setUser(false);
//          uiJob.setSystem(true);
//          uiJob.schedule();
//
//        }
//        else
//        {
//          MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), false);
//          UIJob uiJob = new UIJob("aktualisiere GUI") {
//
//            @Override
//            public IStatus runInUIThread(IProgressMonitor monitor)
//            {
//              // TODO Auto-generated method stub
//              updateButtonKeineSelektion();
//              return Status.OK_STATUS;
//            }
//          };
//          uiJob.setUser(false);
//          uiJob.setSystem(true);
//          uiJob.schedule();
//        }
//
//      }
//    });
//    job.setSystem(false);
//    job.setUser(true);
//    job.schedule(); // start as soon as possible
//  }
  
  private void aktionExport()
  {
    if (_selection != null)
    {
      Object obj = _selection.getFirstElement();
      if (obj instanceof KonfigBereichDaten)
      {
        Iterator it = _selection.iterator();

        Map<String, File> list = new HashMap<String, File>();

        while (it.hasNext())
        {
          KonfigBereichDaten kbd = (KonfigBereichDaten)it.next();
          String pid = kbd.getPid();
          Map<String, ConfigurationArea> map = _config.getAllConfigurationAreas();
          if (map.containsKey(pid))
          {
            ConfigurationArea ca = map.get(pid);
            list.put(ca.getPid(), new File(""));
          }

        }
        if (list.size() > 0)
        {                
          KbAuswahlAssistent wizard = new KbAuswahlAssistent("Export", "exportiert", list);

          wizard.init(PlatformUI.getWorkbench(), null);
          // Instantiates the wizard container with the wizard and opens it
          WizardDialog dialog = new WizardDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell(), wizard);
          dialog.create();
          int ret = dialog.open();

          if (ret == 0)
          {
             _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion Export", null);
             MeldungenSpeicher.getInstanz().setStatus(_status);
            
             KbAuswahlModell model = wizard.getModel();
             final Map<String, File> list2 = model.getList();
              
             final Job job = new Job("KonfigAss: Export") {

               protected IStatus run(IProgressMonitor monitor)
               {
                 monitor.beginTask("Starte Export...", 5);
                 monitor.worked(1);

                 try
                 {
                   
//                   List<ConfigurationArea> listCa = new ArrayList<ConfigurationArea>();
//                   
//                   for (Map.Entry<String, File> me : list2.entrySet())
//                   {
//                     String pid = me.getKey();
//                     
//                     ConfigurationArea ca = _config.getAllConfigurationAreas().get(pid);
//                     if (ca != null){
//                       listCa.add(ca);
//                       System.out.println("ca: " + ca.getPid());
//                     }
//                   }
                   
                   if (list2.size() > 0)
                   {                     
                     List<String> list = new ArrayList<String>();
                     
                     for (String s : list2.keySet())
                     {                        
                       _status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Export fr KB: " + s));
                       list.add(s);
                     }
                     File file = new File(_dateiPfad);
     
                     String configPfad = file.getParent();
                     file = new File(configPfad);
                     String pfad = file.getParent();
     
                     String xmlPfad = pfad + "/xml";     
                     file = new File(xmlPfad);
     
                     if (!file.exists())
                       file.mkdir();
     
                     if (file.isDirectory())
                     {
                       KonfigApplikation app = new KonfigApplikation(_config);
                       app.startExport(file, list);
                       monitor.worked(1);                      
                     }
                   }
                   
                   // _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Starte Aktion Aktivierung", null);
                   // MeldungenSpeicher.getInstanz().setStatus(_status);

                 }
                 catch (Exception e2)
                 {
                   _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
                       "Aktion Export abgebrochen! Grund: es ist eine Ausnahme bei der Ausfhrung aufgetreten. " + e2));
                   return Status.CANCEL_STATUS;
                 }
                 finally
                 {
                   monitor.worked(3);
                   monitor.done();
                 }

                 return Status.OK_STATUS;
               }
             };

             job.addJobChangeListener(new JobChangeAdapter() {

               @Override
               public void running(IJobChangeEvent event)
               {
                 UIJob uiJob = new UIJob("aktualisiere GUI") {

                   @Override
                   public IStatus runInUIThread(IProgressMonitor monitor)
                   {
                     // TODO Auto-generated method stub
                     // updateButtonJobAktiv();
                     return Status.OK_STATUS;
                   }
                 };
                 uiJob.setUser(false);
                 uiJob.setSystem(true);
                 uiJob.schedule();
               }

               public void done(IJobChangeEvent event)
               {
                 if (event.getResult().isOK())
                 {
                   _status.merge(new Status(Status.OK, Activator.PLUGIN_ID,
                       "Aktion Export beendet, bitte beachten Sie die Ausgaben der Konfiguration"));
                   MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), true);

//                   aktuelleInfoKbAnzeigen();
//                   erzeugeInhaltTabelle(_listKonfigBereichKapsel);
                   Display.getDefault().asyncExec(new Runnable() {
                    
                    @Override
                    public void run()
                    {
                      // TODO Auto-generated method stub
                      _selChangedlistener.selectionChanged(new SelectionChangedEvent(_tvZufluesse1, _tvZufluesse1.getSelection()));                      
                    }
                  });
                   
                 }
                 else
                 {
                   MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(_status, System.currentTimeMillis(), false);
                   UIJob uiJob = new UIJob("aktualisiere GUI") {

                     @Override
                     public IStatus runInUIThread(IProgressMonitor monitor)
                     {
                       // TODO Auto-generated method stub
                       updateButtonKeineSelektion();
                       return Status.OK_STATUS;
                     }
                   };
                   uiJob.setUser(false);
                   uiJob.setSystem(true);
                   uiJob.schedule();
                 }

               }
             });
             job.setSystem(false);
             job.setUser(true);
             job.schedule(); // start as soon as possible             
             
          }else{
            _selChangedlistener.selectionChanged(new SelectionChangedEvent(_tvZufluesse1, _tvZufluesse1.getSelection()));
          }
        }
      }
    } 
  }
  

  private void aktionEntladeDatenModell()
  {
    if (_config != null)
    {
      String meldung = Activator.PLUGIN_ID + " Zugriff auf das bisherige Datenmodell "
          + "ist nicht mehr mglich, da das Datenmodell entladen wird!";

      MessageBox msg = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), SWT.OK | SWT.CANCEL
          | SWT.ICON_WARNING);
      msg.setText("Das Datenmodell wird entladen");
      msg.setMessage(meldung);
      int open = msg.open();

      if (open == SWT.OK)
      {

        _config.close();
        _config = null;

        erzeugeInhaltTabelle(new ArrayList<KonfigBereichDaten>());

        updateTextKonfiguration("", "");

        updateButtonKeinDatenModell();

//        setzeKonfigDatenModellFuerTka(null, "");
        setzeKonfigDatenModellFuerTka(null, _dateiPfad);

      }

    }
  }

  private void aktionAbbruch()
  {
    IJobManager jobManager = Job.getJobManager();

    Job job = jobManager.currentJob();
    if (_currentJob != null)
    {
      if (Job.RUNNING == _currentJob.getState())
      {
        _currentJob.cancel();
      }
    }
  }

  private void updateButtonJobAktiv()
  {
    UIJob uiJob = new UIJob("aktualisiere GUI") {

      @Override
      public IStatus runInUIThread(IProgressMonitor monitor)
      {
        buttonLadeKonfiguration.setEnabled(false);
        buttonEntladeKonfiguration.setEnabled(false);

        buttonNeuerKvAoe.setEnabled(false);

        buttonKbNeu.setEnabled(false);
        buttonKbHinzufuegen.setEnabled(false);
        buttonKbEntfernen.setEnabled(false);

        buttonImport.setEnabled(false);
        buttonKonsistenzPruefung.setEnabled(false);         
        buttonUebepruefeAktivierung.setEnabled(false);         
        buttonAktivierung.setEnabled(false);

        buttonFreigabeZurUebernahme.setEnabled(false);
        buttonFreigabeZurAktivierung.setEnabled(false);
        buttonFreigabeZurAktivierungOhne.setEnabled(false);

        buttonExport.setEnabled(false);

        // buttonAbbrechen.setEnabled(true);
        return Status.OK_STATUS;
      }
    };
    uiJob.setUser(false);
    uiJob.setSystem(true);
    uiJob.schedule();
  }

  private void updateButtonSelektion(boolean selektionMitRechte)
  {
    // if (_config != null)
    {
      buttonLadeKonfiguration.setEnabled(true);
      buttonEntladeKonfiguration.setEnabled(true);

      buttonNeuerKvAoe.setEnabled(false);

      buttonKbNeu.setEnabled(true);
      buttonKbHinzufuegen.setEnabled(true);
      buttonKbEntfernen.setEnabled(true);

      buttonImport.setEnabled(true);
      buttonKonsistenzPruefung.setEnabled(true);
      buttonUebepruefeAktivierung.setEnabled(true);         
      buttonAktivierung.setEnabled(true);

      if (selektionMitRechte)
      {
        Iterator it = _selection.iterator();

        Map<String, ConfigurationArea> areas = _config.getAllConfigurationAreas();

        boolean selektionAktiviert = true;
        while (it.hasNext())
        {
          KonfigBereichDaten kbd = (KonfigBereichDaten)it.next();
          String pid = kbd.getPid();
          if (areas.containsKey(pid))
          {
            ConfigurationArea ca = areas.get(pid);
            if (!ca.isValid())
              selektionAktiviert = false;
          }
        }

        buttonFreigabeZurAktivierung.setEnabled(selektionAktiviert);
        buttonFreigabeZurUebernahme.setEnabled(true);
        buttonFreigabeZurAktivierungOhne.setEnabled(true);
      }
      else
      {
        buttonFreigabeZurAktivierung.setEnabled(false);
        buttonFreigabeZurUebernahme.setEnabled(false);
        buttonFreigabeZurAktivierungOhne.setEnabled(false);
      }

      buttonExport.setEnabled(true);

      // buttonAbbrechen.setEnabled(false);

    }
  }

  private void updateButtonKeineSelektion()
  {
    buttonLadeKonfiguration.setEnabled(true);
    buttonEntladeKonfiguration.setEnabled(true);

    buttonNeuerKvAoe.setEnabled(false);

    buttonKbNeu.setEnabled(true);
    buttonKbHinzufuegen.setEnabled(true);
    buttonKbEntfernen.setEnabled(false);

    buttonImport.setEnabled(true);
    buttonKonsistenzPruefung.setEnabled(true);
    buttonUebepruefeAktivierung.setEnabled(true);         
    buttonAktivierung.setEnabled(false);

    buttonFreigabeZurUebernahme.setEnabled(false);
    buttonFreigabeZurAktivierung.setEnabled(false);
    buttonFreigabeZurAktivierungOhne.setEnabled(false);

    buttonExport.setEnabled(false);

    // buttonAbbrechen.setEnabled(false);
  }

  private void updateButtonKeinDatenModell()
  {
    buttonLadeKonfiguration.setEnabled(true);
    buttonEntladeKonfiguration.setEnabled(false);

    buttonNeuerKvAoe.setEnabled(true);

    buttonKbNeu.setEnabled(false);
    buttonKbHinzufuegen.setEnabled(false);
    buttonKbEntfernen.setEnabled(false);

    buttonImport.setEnabled(false);
    buttonKonsistenzPruefung.setEnabled(false);
    buttonUebepruefeAktivierung.setEnabled(false);         
    buttonAktivierung.setEnabled(false);

    buttonFreigabeZurUebernahme.setEnabled(false);
    buttonFreigabeZurAktivierung.setEnabled(false);
    buttonFreigabeZurAktivierungOhne.setEnabled(false);

    buttonExport.setEnabled(false);

    // buttonAbbrechen.setEnabled(false);
  }

  private void updateTextKonfiguration(String pfad, String pidAoe)
  {
    if (text11 != null && pfad != null)
      text11.setText(pfad);

    if (text21 != null && pidAoe != null)
      text21.setText(pidAoe);
  }
  
  public String getPfad()
  {
    return _dateiPfad;
  }
  
  List<String> getZugeordnetePluginIds()
  {
    return _zugeordnetePluginIds;
  }

} // @jve:decl-index=0:visual-constraint="10,10"
