/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintStream;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.konfigass.konsoleAusgabeUpdateListener.KonsoleAusgabeUpdateListener;
import de.bsvrz.sys.funclib.debug.Debug;

class KonsoleAusgabeStream extends ByteArrayOutputStream
{

  private String lineSeparator;

  private MultiStatus _status;

  private KonsoleAusgabeStream _instanz;

  private PrintStream _errBackup;

  protected Debug _debug = Debug.getLogger();

  public MultiStatus getStatus()
  {
    return _status;
  }

  public void setStatus(MultiStatus status)
  {
    _status = status;
  }

  public KonsoleAusgabeStream(PrintStream errBackup)
  {
    // TODO Auto-generated constructor stub
    super();
    _instanz = this;

    // lineSeparator = System.getProperty("line.separator");

    lineSeparator = "\r\n";

    _errBackup = errBackup;

  }

  /**
   * upon flush() write the existing contents of the OutputStream to the logger as a log record.
   * 
   * @throws java.io.IOException
   *           in case of error
   */
  public void flush() throws IOException
  {

    synchronized (this)
    {
      super.flush();
      final String record = this.toString();
      super.reset();

      if (record.length() == 0 || record.equals(lineSeparator))
      {
        // avoid empty records
        return;
      }

      final String replace = record.replaceFirst(lineSeparator, "");

      if (record.contains("WARNUNG:")
          && (record.contains(".konfigass") || record.contains(".tka") || record.contains(".puk.config") || record.contains(".sys.funclib.xml")))
      {
        // System.out.println("hier: " + "Warnung!!!");
        Status status = new Status(Status.WARNING, Activator.PLUGIN_ID, replace);

        // _status.merge(new Status(Status.WARNING, Activator.PLUGIN_ID, replace));
        KonsoleAusgabeUpdateListener.getInstanz()
            .notifyKonsoleAusgabeUpdateListener(status, System.currentTimeMillis());

      }
      else if (record.contains("INFO   :")
          && (record.contains(".konfigass") || record.contains(".tka") || record.contains(".puk.config") || record.contains(".sys.funclib.xml")))
      {
        // System.out.println("hier: " + "Info!!!");
        Status status = new Status(Status.INFO, Activator.PLUGIN_ID, replace);

        // _status.merge(new Status(Status.INFO, Activator.PLUGIN_ID, replace));
        KonsoleAusgabeUpdateListener.getInstanz()
            .notifyKonsoleAusgabeUpdateListener(status, System.currentTimeMillis());
      }
      else if (record.contains("FEHLER :")
          && (record.contains(".konfigass") || record.contains(".tka") || record.contains(".puk.config") || record.contains(".sys.funclib.xml")))
      {
        // System.out.println("hier: " + "Fehler!!!");
        Status status = new Status(Status.ERROR, "de.bsvrz.buv.plugin.tkabasis", replace);

        KonsoleAusgabeUpdateListener.getInstanz()
            .notifyKonsoleAusgabeUpdateListener(status, System.currentTimeMillis());

      }
      else
      {
        PrintStream errAlt = System.err;

        System.setErr(_errBackup);
        Debug.reinstallConsoleHandler();
        // _debug .setLoggerLevel(StdOutErrLevel.FINE);

        System.err.print(record + "\n");

        System.setErr(errAlt);
        Debug.reinstallConsoleHandler();
        // _debug.setLoggerLevel(StdOutErrLevel.FINE);

      }
    }
  }
}