/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass.datenModellUpdateListener;

import java.util.Collections;
import java.util.EventObject;
import java.util.List;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Es wird ein Event (Ereignis) fr die Benachrichtigung von TKA untereinander modelliert
 * 
 * @author Sans
 *
 */
public class DatenModellUpdateEvent extends EventObject
{
	private static final long serialVersionUID = 1365256169927819792L;
	
	/**
	 * Liste der genderten Systemobjekte
	 */
	private List<SystemObject> _listeAktualisiert;
	
	/**
   * Liste der gelschten Systemobjekte
   */
  private List<SystemObject> _listeGeloescht;
	
  /**
   * Liste der neuen Systemobjekte
   */
  private List<SystemObject> _listeNeu;
	
	/**
	 * Konstruktor
	 * @param quelle Quelle des Events
	 * @param typ typ des Ereignisses
	 * @param listeAktualisiert Liste der aktualisierten Systemobjekte
	 * @param listeGeloescht  Liste der gelschten Systemobjekte
	 * @param listeNeu  Liste der neuen Systemobjekte
	 */
	public DatenModellUpdateEvent(Object quelle, List<SystemObject> listeAktualisiert, 
	    List<SystemObject> listeGeloescht, List<SystemObject> listeNeu)
	{
		super(quelle);
		_listeAktualisiert = listeAktualisiert;
		_listeGeloescht = listeGeloescht;
		_listeNeu = listeNeu;
	}
	
	/**
	 * Liefert die Liste der aktualisierten Systemobjekte zurck
	 * 
	 * @return Liste der aktualisierten Systemobjekte
	 */
	public List<SystemObject> getListeSystemObjekte()
	{
	  if (_listeAktualisiert != null)
	  {
	    return Collections.unmodifiableList(_listeAktualisiert);
	  }
	  else
	  {
	    return Collections.emptyList();
	  }
	}

	/**
   * Liefert die Liste der geloeschten Systemobjekte zurck
   * 
   * @return Liste der geloeschten Systemobjekte
   */
  public List<SystemObject> getListeSystemObjekteGeloescht()
  {
    if (_listeGeloescht != null)
    {
      return Collections.unmodifiableList(_listeGeloescht);
    }
    else
    {
      return Collections.emptyList();
    }
  }
	
  /**
   * Liefert die Liste der neu erzeugten Systemobjekte zurck
   * 
   * @return Liste der neu erzeugten Systemobjekte
   */
  public List<SystemObject> getListeSystemObjekteNeu()
  {
    if (_listeNeu != null)
    {
      return Collections.unmodifiableList(_listeNeu);
    }
    else
    {
      return Collections.emptyList();
    }
  }
  
}
