/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass.datenModellUpdateListener;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Die Klasse kapselt die Verwaltungsmethoden fr die Listener-Funktionalitt 
 * des {@link DatenModellUpdateListener}. Diese Klasse implementiert das Singleton-Pattern
 * und stellt dadurch sicher dass der Listener nur einmal Instanziiert wird. 
 * 
 * 
 * @author Pittner
 *
 */
public class DatenModellUpdateListener
{
  
  /**
   * Liste der {@link IDatenModellUpdateListener}, die ber Aktualisierungen informiert werden mchten.
   */
  private LinkedList<IDatenModellUpdateListener> _datenModellUpdateListener = new LinkedList<IDatenModellUpdateListener>();
  
  /**
   * Singleton-Instanz
   */
  private static DatenModellUpdateListener _instanz = null;

  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  public static DatenModellUpdateListener getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new DatenModellUpdateListener();
    }
    return _instanz;
  }
    

  /**
   * Fgt einen Listener zur Liste der {@link DatenModellUpdateListener} hinzu.
   * 
   * @param listener
   */
  public void addDatenModellUpdateListener(IDatenModellUpdateListener listener)
  {
    if (!_datenModellUpdateListener.contains(listener))
    {
      _datenModellUpdateListener.add(listener);
    }
  }
  
  /**
   * Benachrichtigt alle registrierten {@link IDatenModellUpdateListener} darber, dass neue Daten vorhanden sind.
   * 
   * @param src die Quelle, die das Ereignis auslst
   * @param ListeAktualisiert Liste der aktualisierten SystemObjekte
   * @param listeGeloescht Liste der geloeschten SystemObjekte
   * @param listeNeu Liste der neu erstellten SystemObjekte
   */
  public void notifyDatenModellUpdateListener(Object src, List<SystemObject> listeAktualisiert, 
      List<SystemObject> listeGeloescht, List<SystemObject> listeNeu)
  {
    DatenModellUpdateEvent event = new DatenModellUpdateEvent(src, listeAktualisiert, listeGeloescht, listeNeu);

    Iterator<IDatenModellUpdateListener> it = _datenModellUpdateListener.iterator();
    while (it.hasNext())
    {
      IDatenModellUpdateListener listener = it.next();
      listener.datenModellUpdate(event);
    }
  }
  
  /**
   * Entfernt einen {@link IDatenModellUpdateListener} von der Liste der Zuhrer.
   * 
   * @param IKonsoleAusgabeUpdateListener
   *          Zu entfernender {@link IDatenModellUpdateListener}.
   */
  public void removeDatenModellUpdateListener(IDatenModellUpdateListener listener)
  {
    if (_datenModellUpdateListener.contains(listener))
    {
      _datenModellUpdateListener.remove(listener);
    }
  }
  
}
