/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkaaq;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.tkabasis.Activator;
import de.bsvrz.buv.plugin.tkabasis.DataFactory;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AbstractHierarchieManager;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HODaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HOMitStatus;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieManager;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.MengeInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.SystemObjektDaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.TypInfo;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.IPruefungen;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.LoeschenChecker;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.PruefungenTools;
import de.bsvrz.buv.plugin.tkabasis.regeln.IBildungsRegeln;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeGroupUsage;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.ObjectSetType;
import de.bsvrz.dav.daf.main.config.ObjectTimeSpecification;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.puk.config.configFile.datamodel.ConfigConfigurationObject;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigSystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

public class HierachieManagerAQ extends AbstractHierarchieManager implements IHierarchieManager
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();

  // private static IHierarchieObjektTyp[] EBENE0 = new IHierarchieObjektTyp[] {HOT_MQ.KB};

  private static IHierarchieObjektTyp[] EBENE1 = new IHierarchieObjektTyp[] { HOT_AQ.AnzeigeQuerschnittTyp,
      HOT_AQ.Zeichensatz, HOT_AQ.AnzeigeTyp, HOT_AQ.AnzeigeQuerschnitt, HOT_AQ.Dirigent };

  private static IHierarchieObjektTyp[] EBENE2_ZeichenLigaturen = new IHierarchieObjektTyp[] { HOT_AQ.Zeichen,
      HOT_AQ.Ligaturen };

  private static IHierarchieObjektTyp[] EBENE2_WvzInhaltZeichensatz = new IHierarchieObjektTyp[] { HOT_AQ.WvzInhalt,
      HOT_AQ.Zeichensatz };

  private static IHierarchieObjektTyp[] EBENE2_Anzeige = new IHierarchieObjektTyp[] { HOT_AQ.Anzeige };

  // private static IHierarchieObjektTyp[] EBENE2_AnzeigeQuerschnittSub = new IHierarchieObjektTyp[] {
  // HOT_AQ.AnzeigeQuerschnittSub };
  private static IHierarchieObjektTyp[] EBENE2_AnzeigeQuerschnitt = new IHierarchieObjektTyp[] { HOT_AQ.AnzeigeQuerschnitt };

  private static IHierarchieObjektTyp[] EBENE3_WzgInhaltGrafikWechselText = new IHierarchieObjektTyp[] {
      HOT_AQ.WzgInhaltGrafik, HOT_AQ.WechselText };

  private static IHierarchieObjektTyp[] EBENE3_ZeichenLigaturen = new IHierarchieObjektTyp[] { HOT_AQ.Zeichen,
      HOT_AQ.Ligaturen };

  private static IHierarchieObjektTyp[] EBENE3_FS = new IHierarchieObjektTyp[] { HOT_AQ.FS };

  private static IHierarchieObjektTyp[] EBENE3_Anzeige = new IHierarchieObjektTyp[] { HOT_AQ.Anzeige };

  private static IHierarchieObjektTyp[] EBENE4 = new IHierarchieObjektTyp[] { HOT_AQ.FS };

  public void bestimmeUntergeordneteHOs(Object objekt)
  {
  	IHierarchie hierarchie = getHierarchie();
  	ConfigDataModel dataModel = hierarchie.getDataModel();
  	ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
  	
    if (objekt instanceof ConfigurationArea)
    {
      ConfigurationArea kb = (ConfigurationArea)objekt;

      Collection<SystemObjectType> typeSpez = new Vector<SystemObjectType>();
      for (IHierarchieObjektTyp hot : EBENE1)
      {
      	String typPid = hot.getTypen().get(0);
        SystemObjectType typ = dataModel.getType(typPid);
        if (typ == null)
        {
        	throw new RuntimeException("Typ " + typPid + " wurde im aktuellen Datemmodell nicht gefunden");
        }
        
        typeSpez.add(typ);
      }

      Collection<SystemObject> objekte = new LinkedList<SystemObject>();

      for (SystemObject so : kb.getObjects(typeSpez, ObjectTimeSpecification.valid()))
      {
        ConfigurationObject ko = (ConfigurationObject)so;
        if (!(ko.getNotValidSince() >= ko.getConfigurationArea().getModifiableVersion()))
        // Objekt wird in zuknftiger Version nicht ungltig
        {
          boolean gefunden = false;
          for (SystemObject so2 : kb.getObjects(typeSpez, ObjectTimeSpecification.valid()))
          {
            ConfigurationObject ko2 = (ConfigurationObject)so2;
            if (!(ko2.getNotValidSince() >= ko2.getConfigurationArea().getModifiableVersion()))
            // Objekt wird in zuknftiger Version nicht ungltig
            {
              if (ko2.isOfType("typ.dirigent"))
              {
                NonMutableSet aqs = ko2.getNonMutableSet("AnzeigeQuerschnitte");
                if (aqs != null)
                {
                  List<SystemObject> elements = aqs.getElementsInModifiableVersion();
                  if (elements.contains(so))
                  {
                    gefunden = true;
                    // break;
                  }
                }
              }
              else if (ko2.isOfType("typ.anzeigeTyp"))
              {
                AttributeGroup atg = kb.getDataModel().getAttributeGroup("atg.anzeigeTyp");
                Data data = ((ConfigConfigurationObject)ko2).getConfigurationData(atg, lookup);

                if (data != null)
                {
                  ReferenceArray ra = data.getReferenceArray("Font");
                  for (int i = 0; i < ra.getLength(); i++)
                  {
                    SystemObject ref = ra.getSystemObject(i);
                    if (ref.equals(so))
                    {
                      gefunden = true;
                    }
                  }
                }
              }
            }
          }

          if (!gefunden)
            objekte.add(so);
        }
      }

      for (SystemObject so : kb.getNewObjects())
      {
        for (SystemObjectType typ : typeSpez)
        {
          if (so.isOfType(typ))
          {
            boolean gefunden = false;

            for (SystemObject so2 : kb.getNewObjects())
            {
              ConfigurationObject ko2 = (ConfigurationObject)so2;

              if (ko2.isOfType("menge.anzeigeQuerschnitte"))
              {
                NonMutableSet set = (NonMutableSet)ko2;
                if (set != null)
                {

                  List<SystemObject> elements = set.getElementsInModifiableVersion();
                  if (elements.contains(so))
                  {
                    gefunden = true;
                    // break;
                  }
                }
              }
              else if (ko2.isOfType("typ.anzeigeTyp"))
              {
                AttributeGroup atg = kb.getDataModel().getAttributeGroup("atg.anzeigeTyp");
                Data data = ((ConfigConfigurationObject)ko2).getConfigurationData(atg, lookup);

                if (data != null)
                {
                  ReferenceArray ra = data.getReferenceArray("Font");
                  for (int i = 0; i < ra.getLength(); i++)
                  {
                    SystemObject ref = ra.getSystemObject(i);
                    if (ref.equals(so))
                    {
                      gefunden = true;
                    }
                  }
                }
              }
            }

            if (!gefunden)
            {
              objekte.add(so);
            }
          }
        }
      }

      for (SystemObject so : objekte)
      {
        IHierarchieObjektTyp hot = null;

        if (so.isOfType("typ.anzeigeQuerschnittTyp"))
        {
          hot = HOT_AQ.AnzeigeQuerschnittTyp;
        }
        else if (so.isOfType("typ.font"))
        {
          hot = HOT_AQ.Zeichensatz;
        }
        else if (so.isOfType("typ.anzeigeTyp"))
        {
          hot = HOT_AQ.AnzeigeTyp;
        }
        else if (so.isOfType("typ.anzeigeQuerschnitt"))
        {
          hot = HOT_AQ.AnzeigeQuerschnitt;
        }
        else if (so.isOfType("typ.dirigent"))
        {
          hot = HOT_AQ.Dirigent;
        }

        if (hot != null)
        {
          // HierarchieObjekt kind = new HierarchieObjekt(hot, new SystemObject[] { so }, EBENE1);
          HierarchieObjekt kind = new HierarchieObjekt(hot, new SystemObject[] { so }, getTypSortierFolge(hot));
          hierarchie.objektHinzufuegen(objekt, kind);
          bestimmeUntergeordneteHOs(kind);
        }
      }

    }
    else if (objekt instanceof HierarchieObjekt)
    {
      HierarchieObjekt ho = (HierarchieObjekt)objekt;

      if (ho.getHierarchieObjektTyp() == HOT_AQ.Zeichensatz)
      {
        ConfigurationObject co = (ConfigurationObject)ho.getSystemObjekte().get(0);
        NonMutableSet zeichen = co.getNonMutableSet("Zeichen");
        if (zeichen != null)
        {
          for (SystemObject so : zeichen.getElementsInModifiableVersion())
          {
            if (so != null)
            {
              if (so.isOfType("typ.zeichen"))
              {
                IHierarchieObjektTyp hot = HOT_AQ.Zeichen;
                // HierarchieObjekt kind = new HierarchieObjekt(hot, new SystemObject[] { so },
                // EBENE2_ZeichenLigaturen);
                HierarchieObjekt kind = new HierarchieObjekt(hot, new SystemObject[] { so }, getTypSortierFolge(hot));
                hierarchie.objektHinzufuegen(ho, kind);
                // Rekursionsende
              }
              else
              {
                // Log Fehler: Mengenelement falscher Typ
                _debug.warning("Unzulssiger Objekttyp " + so.getPid() + " in Menge Zeichen von Objekt " + co.getPid());
              }
            }
          }
        }
        else
        {
          // Log Fehler: Menge fehlt
          _debug.warning("Menge Zeichen nicht gefunden fr Objekt " + co.getPid());
        }
        NonMutableSet ligaturen = co.getNonMutableSet("Ligaturen");
        if (ligaturen != null)
        {
          for (SystemObject so : ligaturen.getElementsInModifiableVersion())
          {
            if (so != null)
            {
              if (so.isOfType("typ.ligaturen"))
              {
                IHierarchieObjektTyp hot = HOT_AQ.Ligaturen;
                // HierarchieObjekt kind = new HierarchieObjekt(hot, new SystemObject[] { so },
                // EBENE2_ZeichenLigaturen);
                HierarchieObjekt kind = new HierarchieObjekt(hot, new SystemObject[] { so }, getTypSortierFolge(hot));
                hierarchie.objektHinzufuegen(ho, kind);
                // Rekursionsende
              }
              else
              {
                // Log Fehler: Mengenelement falscher Typ
                _debug.warning("Unzulssiger Objekttyp " + so.getPid() + " in Menge Ligaturen von Objekt "
                    + co.getPid());
              }
            }
          }
        }
        else
        {
          // Log Fehler: Menge fehlt
          _debug.warning("Menge Zeichen nicht gefunden fr Objekt " + co.getPid());
        }
      }
      else if (ho.getHierarchieObjektTyp() == HOT_AQ.AnzeigeTyp)
      {
        ConfigurationObject co = (ConfigurationObject)ho.getSystemObjekte().get(0);

        NonMutableSet zeichen = co.getNonMutableSet("WvzInhalt");
        if (zeichen != null)
        {
          for (SystemObject so : zeichen.getElementsInModifiableVersion())
          {
            if (so != null)
            {
              if (so.isOfType("typ.wvzInhalt"))
              {
                IHierarchieObjektTyp hot = HOT_AQ.WvzInhalt;
                // HierarchieObjekt kind = new HierarchieObjekt(hot, new SystemObject[] { so },
                // EBENE2_WvzInhaltZeichensatz);
                HierarchieObjekt kind = new HierarchieObjekt(hot, new SystemObject[] { so }, getTypSortierFolge(hot));
                hierarchie.objektHinzufuegen(ho, kind);
                bestimmeUntergeordneteHOs(kind);
              }
              // else if (so.isOfType("typ.font"))
              // {
              // IHierarchieObjektTyp hot = HOT_AQ.ZeichensatzSub;
              // HierarchieObjekt kind = new HierarchieObjekt(hot, new SystemObject[] { so },
              // EBENE2_WvzInhaltZeichensatz);
              // _hierarchie.objektHinzufuegen(ho, kind);
              // bestimmeUntergeordneteHOs(kind);
              // }
              else
              {
                // Log Fehler: Mengenelement falscher Typ
                _debug.warning("Unzulssiger Objekttyp " + so.getPid() + " in Menge WvzInhalt von Objekt "
                    + co.getPid());
              }
            }
          }
        }
        else
        {
          // Log Fehler: Menge fehlt
          _debug.warning("Menge Zeichen nicht gefunden fr Objekt " + co.getPid());
        }

        AttributeGroup atg = dataModel.getAttributeGroup("atg.anzeigeTyp");

        Data data = ((ConfigConfigurationObject)co).getConfigurationData(atg, lookup);

        if (data != null)
        {
          SystemObject[] refs = data.getReferenceArray("Font").getSystemObjectArray();
          if (refs != null)
          {
            for (SystemObject ref : refs)
            {
              if (ref.isOfType("typ.font"))
              {
                IHierarchieObjektTyp hot = HOT_AQ.Zeichensatz;
                // HierarchieObjekt kind = new HierarchieObjekt(HOT_AQ.ZeichensatzSub, new SystemObject[] { ref },
                // EBENE2_WvzInhaltZeichensatz);
                HierarchieObjekt kind = new HierarchieObjekt(hot, new SystemObject[] { ref }, getTypSortierFolge(hot));
                hierarchie.objektHinzufuegen(ho, kind);
                bestimmeUntergeordneteHOs(kind);
              }
            }
          }
        }
      }
      else if (ho.getHierarchieObjektTyp() == HOT_AQ.AnzeigeQuerschnitt)
      {
        ConfigurationObject co = (ConfigurationObject)ho.getSystemObjekte().get(0);
        NonMutableSet zeichen = co.getNonMutableSet("Anzeigen");
        if (zeichen != null)
        {
          for (SystemObject so : zeichen.getElementsInModifiableVersion())
          {
            if (so != null)
            {
              if (so.isOfType("typ.anzeige"))
              {
                IHierarchieObjektTyp hot = HOT_AQ.Anzeige;

                Object ueo = hierarchie.getUebergeordnetesObjekt(ho);

                HierarchieObjekt kind = null;

                // if (ueo instanceof HierarchieObjekt)
                // {
                // kind = new HierarchieObjekt(hot, new SystemObject[] { so }, EBENE2_Anzeige);
                // }else{
                // kind = new HierarchieObjekt(hot, new SystemObject[] { so }, EBENE3_AnzeigeSub);
                // }

                kind = new HierarchieObjekt(hot, new SystemObject[] { so }, getTypSortierFolge(hot));

                hierarchie.objektHinzufuegen(ho, kind);
                bestimmeUntergeordneteHOs(kind);
              }
              else
              {
                // Log Fehler: Mengenelement falscher Typ
                _debug
                    .warning("Unzulssiger Objekttyp " + so.getPid() + " in Menge Anzeigen von Objekt " + co.getPid());
              }
            }
          }
        }
        else
        {
          // Log Fehler: Menge fehlt
          _debug.warning("Menge Zeichen nicht gefunden fr Objekt " + co.getPid());
        }

      }
      else if (ho.getHierarchieObjektTyp() == HOT_AQ.Dirigent)
      {
        ConfigurationObject co = (ConfigurationObject)ho.getSystemObjekte().get(0);
        NonMutableSet zeichen = co.getNonMutableSet("AnzeigeQuerschnitte");
        if (zeichen != null)
        {
          for (SystemObject so : zeichen.getElementsInModifiableVersion())
          {
            if (so != null)
            {
              if (so.isOfType("typ.anzeigeQuerschnitt"))
              {
                // IHierarchieObjektTyp hot = HOT_AQ.AnzeigeQuerschnittSub;
                IHierarchieObjektTyp hot = HOT_AQ.AnzeigeQuerschnitt;
                // HierarchieObjekt kind = new HierarchieObjekt(hot, new SystemObject[] { so },
                // EBENE2_AnzeigeQuerschnittSub);
                HierarchieObjekt kind = new HierarchieObjekt(hot, new SystemObject[] { so }, getTypSortierFolge(hot));
                hierarchie.objektHinzufuegen(ho, kind);
                bestimmeUntergeordneteHOs(kind);
              }
              else
              {
                // Log Fehler: Mengenelement falscher Typ
                _debug
                    .warning("Unzulssiger Objekttyp " + so.getPid() + " in Menge Anzeigen von Objekt " + co.getPid());
              }
            }
          }
        }
        else
        {
          // Log Fehler: Menge fehlt
          _debug.warning("Menge Zeichen nicht gefunden fr Objekt " + co.getPid());
        }

      }
      else if (ho.getHierarchieObjektTyp() == HOT_AQ.Anzeige)
      {
        ConfigurationObject co = (ConfigurationObject)ho.getSystemObjekte().get(0);
        NonMutableSet zeichen = co.getNonMutableSet("FahrStreifen");
        if (zeichen != null)
        {
          for (SystemObject so : zeichen.getElementsInModifiableVersion())
          {
            if (so != null)
            {
              if (so.isOfType("typ.fahrStreifen"))
              {
                IHierarchieObjektTyp hot = HOT_AQ.FS;
                // HierarchieObjekt kind = new HierarchieObjekt(hot, new SystemObject[] { so }, EBENE3_FS);
                HierarchieObjekt kind = new HierarchieObjekt(hot, new SystemObject[] { so }, getTypSortierFolge(hot));
                hierarchie.objektHinzufuegen(ho, kind);
                // Rekursionsende
              }
              else
              {
                // Log Fehler: Mengenelement falscher Typ
                _debug.warning("Unzulssiger Objekttyp " + so.getPid() + " in Menge FahrStreifen von Objekt "
                    + co.getPid());
              }
            }
          }
        }
        else
        {
          // Log Fehler: Menge fehlt
          _debug.warning("Menge Zeichen nicht gefunden fr Objekt " + co.getPid());
        }
      }
      else if (ho.getHierarchieObjektTyp() == HOT_AQ.WvzInhalt)
      {
        ConfigurationObject co = (ConfigurationObject)ho.getSystemObjekte().get(0);

        AttributeGroup atg = dataModel.getAttributeGroup("atg.wvzInhalt");

        Data data = ((ConfigConfigurationObject)co).getConfigurationData(atg, lookup);

        if (data != null)
        {
          SystemObject ref = data.getReferenceValue("Wechseltext").getSystemObject();
          if (ref != null)
          {
            if (ref.isOfType("typ.wechseltext"))
            {
              IHierarchieObjektTyp hot = HOT_AQ.WechselText;
              // HierarchieObjekt kind = new HierarchieObjekt(HOT_AQ.WechselText, new SystemObject[] { ref },
              // EBENE3_WzgInhaltGrafikWechselText);
              HierarchieObjekt kind = new HierarchieObjekt(HOT_AQ.WechselText, new SystemObject[] { ref },
                  getTypSortierFolge(hot));
              hierarchie.objektHinzufuegen(ho, kind);

            }
          }

          String s = data.getItem("IstGrafik").asTextValue().getValueText();

          if (s.equals("Ja"))
          {
            SystemObject[] sos = data.getReferenceArray("GrafikDarstellungen").getSystemObjectArray();

            if (sos != null && sos.length > 0)
            {
              for (int i = 0; i < sos.length; i++)
              {
                SystemObject r = sos[i];
                if (r.isOfType("typ.wzgInhaltGrafik"))
                {
                  IHierarchieObjektTyp hot = HOT_AQ.WzgInhaltGrafik;
                  // HierarchieObjekt kind = new HierarchieObjekt(HOT_AQ.WzgInhaltGrafik, new SystemObject[] { r },
                  // EBENE3_WzgInhaltGrafikWechselText);
                  HierarchieObjekt kind = new HierarchieObjekt(HOT_AQ.WzgInhaltGrafik, new SystemObject[] { r },
                      getTypSortierFolge(hot));
                  hierarchie.objektHinzufuegen(ho, kind);
                }
              }
            }
          }
        }
      }
    }
  }

  public IHierarchieObjektTyp[] getUntergeordneteHOTs(Object objekt)
  {

    if (objekt instanceof ConfigurationArea)
    {
      return EBENE1;
    }
    else if (objekt instanceof HierarchieObjekt)
    {
    	IHierarchie hierarchie = getHierarchie();
      HierarchieObjekt ho = (HierarchieObjekt)objekt;
      IHierarchieObjektTyp typ = ho.getHierarchieObjektTyp();

      if (typ.equals(HOT_AQ.Zeichensatz))
      {
        return EBENE2_ZeichenLigaturen;
      }
      else if (typ.equals(HOT_AQ.AnzeigeTyp))
      {
        return EBENE2_WvzInhaltZeichensatz;
      }
      else if (typ.equals(HOT_AQ.AnzeigeQuerschnitt))
      {
        Object ueo = hierarchie.getUebergeordnetesObjekt(ho);
        if (ueo instanceof ConfigurationArea)
        {
          return EBENE2_Anzeige;
        }
        else
        {
          return EBENE3_Anzeige;
        }
      }
      else if (typ.equals(HOT_AQ.Dirigent))
      {
        return EBENE2_AnzeigeQuerschnitt;
      }
      else if (typ.equals(HOT_AQ.WvzInhalt))
      {
        return EBENE3_WzgInhaltGrafikWechselText;
      }
      else if (typ.equals(HOT_AQ.Zeichensatz))
      {
        return EBENE3_ZeichenLigaturen;
      }
      else if (typ.equals(HOT_AQ.Anzeige))
      {
        Object ueo = hierarchie.getUebergeordnetesObjekt(ho);
        Object uueo = hierarchie.getUebergeordnetesObjekt((HierarchieObjekt)ueo);
        if (uueo instanceof ConfigurationArea)
        {
          return EBENE3_FS;
        }
        else
        {
          return EBENE4;
        }
      }
    }
    return null;
  }

  private List<ObjectSet> erzeugeMengen(IHierarchieObjektTyp hot, SystemObjectType typ, ConfigurationArea kb,
      MultiStatus status)
  {
    List<ObjectSet> result = new Vector<ObjectSet>();
    ConfigDataModel dataModel = getHierarchie().getDataModel();

    for (String mengenName : hot.getTypInfo(typ.getPid()).getMengen())
    {
      MengeInfo info = hot.getTypInfo(typ.getPid()).getMengeInfo(mengenName);

      SystemObjectType mengenTyp = dataModel.getType(info.getTyp());
      ObjectSet menge;
      try
      {
        menge = (ObjectSet)kb.createConfigurationObject((ConfigurationObjectType)mengenTyp, "", mengenName, null);

        status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Menge " + mengenName + " vom Typ "
            + mengenTyp.getPid() + " erzeugt"));

      }
      catch (ConfigurationChangeException e)
      {
        status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID, "Fehler beim Erzeugen der Menge " + mengenName
            + " vom Typ " + mengenTyp.getName() + " erzeugt", e));
        return result;
      }

      result.add(menge);
    }

    return result;
  }

  private List<ObjectSet> erzeugeMengenFuerTyp(IHierarchieObjektTyp hot, String typ, ConfigurationArea kb,
      MultiStatus status, List<SystemObject> undoListe)
  {
    List<ObjectSet> result = new Vector<ObjectSet>();

    for (String mengenName : hot.getTypInfo(typ).getMengen())
    {
      MengeInfo info = hot.getTypInfo(typ).getMengeInfo(mengenName);
      String mengenTyp = info.getTyp();
      ObjectSetType setTyp = kb.getDataModel().getObjectSetType(mengenTyp);
      ObjectSet menge;
      try
      {
        menge = (ObjectSet)kb.createConfigurationObject(setTyp, "", mengenName, null);

        undoListe.add(0, menge);
        status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Menge " + mengenName + " vom Typ " + setTyp.getPid()
            + " erzeugt"));
      }
      catch (ConfigurationChangeException e)
      {
        status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Fehler beim Erzeugen der Menge " + mengenName
            + " vom Typ " + setTyp.getName() + " erzeugt", e));
        return result;
      }

      result.add(menge);
    }

    return result;
  }

  private SystemObject erzeugeSystemObjekt(String pid, ConfigurationObjectType typ, ConfigurationArea kb,
      List<ObjectSet> mengen, MultiStatus status)
  {
    SystemObject result = null;

    try
    {
      result = kb.createConfigurationObject(typ, pid, "", mengen);

      if (mengen != null && !mengen.isEmpty())
      {
        MultiStatus unterStatus = new MultiStatus(Activator.PLUGIN_ID, IStatus.OK, "Systemobjekt " + pid + " vom Typ "
            + typ.getPid() + " erzeugt", null);
        status.add(unterStatus);

        for (ObjectSet set : mengen)
        {
          unterStatus.add(new Status(Status.OK, Activator.PLUGIN_ID, "Menge " + set.getName() + " dem Systemobjekt "
              + pid + " vom Typ " + typ.getPid() + " zugewiesen"));
        }
      }
      else
      {
        status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + pid + " vom Typ " + typ.getPid()
            + " erzeugt"));
      }
    }
    catch (ConfigurationChangeException e)
    {
      status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + pid + " vom Typ " + typ.getPid()
          + " konnte nicht erzeugt werden", e));
    }

    return result;
  }

  private ObjectSet erzeugeBeliebigeMenge(String pidTyp, String name, ConfigurationObject ko, MultiStatus status)
  {
    // TODO Auto-generated method stub
    ObjectSet result = null;

    ObjectSetType setTyp = getHierarchie().getDataModel().getObjectSetType(pidTyp);
    try
    {
      result = (ObjectSet)ko.getConfigurationArea().createConfigurationObject(setTyp, null, name, null);
      ko.addSet(result);

      status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Menge " + name + " vom Typ " + setTyp.getPid()
          + " fr Systemobjekt " + ko.getPid() + " erzeugt"));

    }
    catch (ConfigurationChangeException e)
    {
      status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Menge " + name + " vom Typ " + setTyp.getPid()
          + " konnte fr das Systemobjekt " + ko.getPid() + " nicht erstellt werden", e));
    }

    return result;
  }

  private void behandleMengenZuordnung(Object vater, SystemObject so, MultiStatus status)
  {
    if (vater instanceof HierarchieObjekt)
    {
    	ConfigDataModel dataModel = getHierarchie().getDataModel();
    	
      HierarchieObjekt ho = (HierarchieObjekt)vater;

      SystemObject vaterSo = ho.getSystemObjekte().get(0);

      ObjectSet menge = null;

      if (vaterSo.isOfType("typ.font"))
      {

        if (so.isOfType("typ.zeichen"))
        {

          menge = ((ConfigurationObject)vaterSo).getNonMutableSet("Zeichen");

          if (menge == null)
          {
            menge = erzeugeBeliebigeMenge("menge.zeichen", "Zeichen", (ConfigurationObject)vaterSo, status);
          }

          if (menge == null || status.matches(IStatus.ERROR | IStatus.CANCEL))
          {
            return;
          }
          try
          {
            menge.add(so);

            status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
                + " wird der Menge Zeichen des Systemobjekts " + vaterSo.getPid() + " hinzugefgt"));
          }
          catch (ConfigurationChangeException e)
          {
            status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
                + " konnte nicht der Menge Zeichen des Systemobjekts " + vaterSo.getPid() + " hinzugefgt werden", e));
          }
        }
        else if (so.isOfType("typ.ligaturen"))
        {

          menge = ((ConfigurationObject)vaterSo).getNonMutableSet("Ligaturen");

          if (menge == null)
          {
            menge = erzeugeBeliebigeMenge("menge.ligaturen", "Ligaturen", (ConfigurationObject)vaterSo, status);
          }

          if (menge == null || status.matches(IStatus.ERROR | IStatus.CANCEL))
          {
            return;
          }
          try
          {
            menge.add(so);

            status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
                + " wird der Menge Ligaturen des Systemobjekts " + vaterSo.getPid() + " hinzugefgt"));
          }
          catch (ConfigurationChangeException e)
          {
            status
                .add(new Status(Status.ERROR, Activator.PLUGIN_ID,
                    "Systemobjekt " + so.getPid() + " konnte nicht der Menge Ligaturen des Systemobjekts "
                        + vaterSo.getPid() + " hinzugefgt werden", e));
          }
        }
        else
        {
          status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Unzulssiger Typ: " + so.getType()
              + " bei der Beahndlung der Mengenzuordnung des bergeordneten HO " + ho.getHierarchieObjektTyp()));
        }
      }
      else if (vaterSo.isOfType("typ.anzeigeTyp"))
      {

        if (so.isOfType("typ.wvzInhalt"))
        {
          menge = ((ConfigurationObject)vaterSo).getNonMutableSet("WvzInhalt");

          if (menge == null)
          {
            menge = erzeugeBeliebigeMenge("menge.wvzInhalt", "WvzInhalt", (ConfigurationObject)vaterSo, status);
          }

          if (menge == null || status.matches(IStatus.ERROR | IStatus.CANCEL))
          {
            return;
          }
          try
          {
            menge.add(so);

            status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
                + " wird der Menge Zeichen des Systemobjekts " + vaterSo.getPid() + " hinzugefgt"));
          }
          catch (ConfigurationChangeException e)
          {
            status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
                + " konnte nicht der Menge Zeichen des Systemobjekts " + vaterSo.getPid() + " hinzugefgt werden", e));
          }
        }
        else if (so.isOfType("typ.font"))
        {
          AttributeGroup atg = dataModel.getAttributeGroup("atg.anzeigeTyp");

          ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
          Data data = ((ConfigConfigurationObject)vaterSo).getConfigurationData(atg, lookup);

          if (data == null)
          {
            data = DataFactory.createData(atg);
            // Default und undefiniert Werte setzen
            data.setToDefault();
          }

          ReferenceArray ra = data.getReferenceArray("Font");
          int len = ra.getLength();
          ra.setLength(len + 1);
          ra.getReferenceValue(len).setSystemObject(so);

          try
          {
					  if (!HierarchieObjekt.isAttributgruppeAenderbar(vaterSo, atg))
					  {  
					    vaterSo = behandleSystemObjektAenderung(ho, (ConfigurationObject)vaterSo, vaterSo.getPid());
					    ho.setSystemObjekt(0, vaterSo);
					  }  
          	
            vaterSo.setConfigurationData(atg, data);
            status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + vaterSo.getPid() + " als Referenz "
                + so.getPid() + " gesetzt"));
          }
          catch (ConfigurationChangeException e)
          {
            status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + vaterSo
                + " konnt nicht als Referenz " + so.getPid() + " gesetzt werden", e));
          }

        }

      }
      else if (vaterSo.isOfType("typ.wvzInhalt"))
      {
        AttributeGroup atg = dataModel.getAttributeGroup("atg.wvzInhalt");

        ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
        Data data = ((ConfigConfigurationObject)vaterSo).getConfigurationData(atg, lookup);

        if (data == null)
        {
          data = DataFactory.createData(atg);
          // Default und undefiniert Werte setzen
          data.setToDefault();
        }

        if (so.isOfType("typ.wechseltext"))
        {
          SystemObject sx = data.getReferenceValue("Wechseltext").getSystemObject();
          List<HierarchieObjekt> uos = getHierarchie().getUntergeordneteObjekte(vater);

          HierarchieObjekt del = null;
          for (HierarchieObjekt hox : uos)
          {
            SystemObject kind = hox.getSystemObjekt(0);
            if (kind.equals(sx))
            {
              del = hox;
            }
          }
          if (del != null)
          {
            getHierarchie().objektEntfernen(vater, del);
          }
          
          // data.getItem("IstGrafik").asTextValue().setText("Nein");
          data.getReferenceValue("Wechseltext").setSystemObject(so);
          // ReferenceArray ra = data.getReferenceArray("GrafikDarstellungen");
          // ra.setLength(0);
        }
        else if (so.isOfType("typ.wzgInhaltGrafik"))
        {
          data.getItem("IstGrafik").asTextValue().setText("Ja");
          ReferenceArray ra = data.getReferenceArray("GrafikDarstellungen");
          int len = ra.getLength();
          ra.setLength(len + 1);
          ra.getReferenceValue(len).setSystemObject(so);
        }

        try
        {
        	if (!HierarchieObjekt.isAttributgruppeAenderbar(vaterSo, atg))
				  {  
				    vaterSo = behandleSystemObjektAenderung(ho, (ConfigurationObject)vaterSo, vaterSo.getPid());
				    ho.setSystemObjekt(0, vaterSo);
				  }  
        	
          vaterSo.setConfigurationData(atg, data);
          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + vaterSo.getPid()
              + " als KommunikationsPartner in APKP " + vaterSo.getPid() + " gesetzt"));
        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + vaterSo
              + " konnt nicht als KommunikationsPartner in APKP " + vaterSo.getPid() + " gesetzt werden", e));
        }
      }
      else if (vaterSo.isOfType("typ.dirigent"))
      {

        menge = ((ConfigurationObject)vaterSo).getNonMutableSet("AnzeigeQuerschnitte");

        if (menge == null)
        {
          menge = erzeugeBeliebigeMenge("menge.anzeigeQuerschnitte", "AnzeigeQuerschnitte", (ConfigurationObject)vaterSo, status);
        }

        if (menge == null || status.matches(IStatus.ERROR | IStatus.CANCEL))
        {
          return;
        }
        try
        {
          menge.add(so);

          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
              + " wird der Menge Zeichen des Systemobjekts " + vaterSo.getPid() + " hinzugefgt"));
        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
              + " konnte nicht der Menge Zeichen des Systemobjekts " + vaterSo.getPid() + " hinzugefgt werden", e));
        }
      }
      else if (vaterSo.isOfType("typ.anzeigeQuerschnitt"))
      {

        menge = ((ConfigurationObject)vaterSo).getNonMutableSet("Anzeigen");

        if (menge == null)
        {
          menge = erzeugeBeliebigeMenge("menge.anzeigen", "Anzeigen", (ConfigurationObject)vaterSo, status);
        }

        if (menge == null || status.matches(IStatus.ERROR | IStatus.CANCEL))
        {
          return;
        }
        try
        {
          menge.add(so);

          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
              + " wird der Menge Anzeigen des Systemobjekts " + vaterSo.getPid() + " hinzugefgt"));
        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
              + " konnte nicht der Menge Anzeigen des Systemobjekts " + vaterSo.getPid() + " hinzugefgt werden", e));
        }
      }
      else if (vaterSo.isOfType("typ.anzeige"))
      {
        menge = ((ConfigurationObject)vaterSo).getNonMutableSet("FahrStreifen");

        if (menge == null)
        {
          menge = erzeugeBeliebigeMenge("menge.fahrStreifen", "FahrStreifen", (ConfigurationObject)vaterSo, status);
        }

        if (menge == null || status.matches(IStatus.ERROR | IStatus.CANCEL))
        {
          return;
        }
        try
        {
          menge.add(so);

          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
              + " wird der Menge FahrStreifen des Systemobjekts " + vaterSo.getPid() + " hinzugefgt"));
        }
        catch (ConfigurationChangeException e)
        {
          status
              .add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
                  + " konnte nicht der Menge FahrStreifen des Systemobjekts " + vaterSo.getPid()
                  + " hinzugefgt werden", e));
        }
      }
    }
    // else
    // {
//    System.out.println("Vater-Objekt ist kein HO :" + vater);
    // }

  }
  
  public HOMitStatus erzeugeNeuesHO(ConfigurationArea kb, Object vater, String pid[], IHierarchieObjektTyp hot, boolean pruefePid)
  {
    ConfigDataModel dataModel = getHierarchie().getDataModel();

    ConfigurationObjectType typ = (ConfigurationObjectType)dataModel.getType(hot.getTypen().get(0));
    if (typ == null)
    {
      throw new IllegalArgumentException("Fr " + hot.getTypen().get(0)
          + " kann kein Typ im Datenmodell bestimmt werden");
    }

    IHierarchieObjektTyp[] ebene = getUntergeordneteHOTs(vater);

    if (!Arrays.asList(ebene).contains(hot))
    {
      throw new IllegalArgumentException("Unzulssiger HOT " + hot + " fr bergeordnetes Objekt " + vater);
    }

    // EbeneInfo info = new EbeneInfo(vater, hot);
    MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Erzeuge neues Hierarchieobjekt vom Typ "
        + hot, null);

    HierarchieObjekt vaterHO = null;

    if (vater instanceof HierarchieObjekt)
    {
      vaterHO = (HierarchieObjekt)vater;
    }
    else if (!(vater instanceof ConfigurationArea))
    {
      throw new IllegalArgumentException("bergebenes Objekt " + vater + " ist von unzulssigem Typ");
    }

    HierarchieObjekt ho = null;

    if (!hot.isSystemObjektVorhanden())
    {    	
      if (pruefePid)
    	{	
    		status.add(new PruefungenTools(dataModel).pruefePid(pid[0]));
    	}	

      
      if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
      {
        List<ObjectSet> mengen;
        
        if (hot.equals(HOT_AQ.Anzeige))
    		{
        	mengen = Collections.emptyList();
    		}
        else
        {	
        	mengen = erzeugeMengen(hot, typ, kb, status);
        }	
        
        if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
        {
          SystemObject so = erzeugeSystemObjekt(pid[0], typ, kb, mengen, status);
          
          if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
          {            
            if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
            {
              
              // if (hot.equals(HOT_AQ.Zeichen) || hot.equals(HOT_AQ.Ligaturen) || hot.equals(HOT_AQ.WzgInhaltGrafik)
              // || hot.equals(HOT_AQ.WechselText) || hot.equals(HOT_AQ.WvzInhalt) || hot.equals(HOT_AQ.Zeichensatz)
              // || hot.equals(HOT_AQ.Anzeige) || hot.equals(HOT_AQ.AnzeigeQuerschnittSub))
              {
                behandleMengenZuordnung(vater, so, status);
              }
              
              ho = new HierarchieObjekt(hot, new SystemObject[] { so }, getTypSortierFolge(hot));
              
              getHierarchie().objektHinzufuegen(vater, ho);
            }
          }
        }        
      }
    }
    else
    {
      SystemObject so = dataModel.getObject(pid[0]);

      if (so == null)
      {
        Collection<ConfigurationArea> areas = dataModel.getAllConfigurationAreas().values();
        for (ConfigurationArea ca : areas)
        {
          Collection<SystemObject> elements = ca.getNewObjects();
          for (SystemObject soNeu : elements)
          {
            if (pid[0].equals(soNeu.getPid()))
            {
              so = soNeu;
            }
          }
        }
      }

      if (so != null)
      {
        behandleMengenZuordnung(vater, so, status);
        ho = new HierarchieObjekt(hot, new SystemObject[] { so }, getTypSortierFolge(hot));
        getHierarchie().objektHinzufuegen(vater, ho);
      }
    }

    return new HOMitStatus(ho, status);
  }

  public HOMitStatus kopiereHO(HierarchieObjekt ho, ConfigurationArea kb, Object neuerVater, 
  		String neuePid[],	boolean rekursiv, boolean isWurzel)
  {
    return kopiereHO(ho, kb, neuerVater, neuePid, rekursiv, true, true, isWurzel);
  }

  private HOMitStatus kopiereHO(HierarchieObjekt ho, ConfigurationArea kb, Object neuerVater, 
  		String neuePid[],	boolean rekursiv, boolean anwendenBR, boolean pruefePid, boolean isWurzel)
  {
//    if (neuePid == null && !(neuerVater instanceof HierarchieObjekt))
//    {
//      throw new IllegalArgumentException("Wenn fr die PID null bergeben wird"
//          + ", muss das neue bergeordnete Objekt vom Typ " + HierarchieObjekt.class + " sein");
//    }

    MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Hierarchieobjekt " + ho, null);

    String[] pid = neuePid;

    if (pid == null)
    {
      pid = new String[ho.getSystemObjekte().size()];

      for (int i = 0; i < pid.length; i += 1)
      {
        pid[i] = ho.getSystemObjekt(i).getPid();
      }
      
//      IBildungsRegeln bildungsRegeln = DatenSpeicher.getInstanz().getBildungsRegeln();
//      bildungsRegeln.setWerte(ho.getHierarchieObjektTyp(), (HierarchieObjekt)neuerVater, ho);
//
//      if (!ho.getHierarchieObjektTyp().isSystemObjektVorhanden())
//      {
//
//        pid = bildungsRegeln.getPids(ho.getHierarchieObjektTyp(), (HierarchieObjekt)neuerVater);
//
//      }
//      else
//      {
//        pid = new String[1];
//        pid[0] = ho.getSystemObjekt(0).getPid();
//      }
    }

    HOMitStatus result = erzeugeNeuesHO(kb, neuerVater, pid, ho.getHierarchieObjektTyp(), pruefePid);
    HierarchieObjekt neu = result.getHO();
    status.add(result.getStatus());

    if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
    {
      status.merge(kopiereDaten(ho, neu, anwendenBR, anwendenBR));
    }

    if (!status.matches(IStatus.ERROR | IStatus.CANCEL) && rekursiv)
    {
      for (HierarchieObjekt lauf : getHierarchie().getUntergeordneteObjekte(ho))
      {
        MultiStatus meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK,
            "Kopiere untergeordnetes Hierarchieobjekt " + lauf + " zu " + ho, null);
        result = kopiereHO(lauf, kb, neu, null, true, anwendenBR, pruefePid, false);
        meldung.merge(result.getStatus());
        status.add(meldung);
        if (status.matches(IStatus.ERROR | IStatus.CANCEL))
        {
          break;
        }
      }
    }

    return new HOMitStatus(neu, status);
  }

  private MultiStatus kopiereDaten(HierarchieObjekt srcHO, HierarchieObjekt destHO, boolean bildungsRegelnFuerName,
      boolean bildungsRegelnFuerAtg)
  {
    if (!srcHO.getHierarchieObjektTyp().equals(destHO.getHierarchieObjektTyp()))
    {
      throw new IllegalArgumentException(
          "Beide bergebenen Hierarchieobjekte mssen vom gleichen Hierarchieobjekttyp sein");
    }

    int index = 0;
    MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Daten von " + srcHO + " nach "
        + destHO, null);
    IBildungsRegeln br = DatenSpeicher.getInstanz().getBildungsRegeln();
    ConfigDataModel dataModel = getHierarchie().getDataModel();

    ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);

    for (SystemObject src : srcHO.getSystemObjekte())
    {
      SystemObject dest = destHO.getSystemObjekte().get(index);
      String name;
      if (bildungsRegelnFuerName)
      {
        name = br.getName(dest);
      }
      else
      {
        name = src.getName();
      }

      if (!destHO.getHierarchieObjektTyp().isSystemObjektVorhanden())
      {
        try
        {

          dest.setName(name);
          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Setze Name \"" + name + "\" fr Systemobjekt "
              + dest.getPid()));
        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Name \"" + name + "\" fr Systemobjekt "
              + dest.getPid() + " konnte nicht gesetzt werden", e));
          return status;
        }

        for (AttributeGroupUsage atgu : src.getUsedAttributeGroupUsages())
        {
          Data dataSrc = ((ConfigSystemObject)src).getConfigurationData(atgu, lookup);
          if (dataSrc != null)
          {
            try
            {
              Data dataDest = DataFactory.createModifiableCopy(dataSrc, lookup);
              if (bildungsRegelnFuerAtg)
              {
                dataDest = br.getDatensatz(destHO, dest, atgu.getAttributeGroup(), dataDest, true);
              }
              dest.setConfigurationData(atgu, dataDest);
              status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Kopiere Daten der ATG-Verwendung "
                  + atgu.getAttributeGroup().getPid() + "/" + atgu.getAspect().getPid() + " fr Systemobjekt "
                  + dest.getPid()));
            }
            catch (ConfigurationChangeException e)
            {
              status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Daten der ATG-Verwendung "
                  + atgu.getAttributeGroup().getPid() + "/" + atgu.getAspect().getPid() + " fr Systemobjekt "
                  + dest.getPid() + " konnten nicht kopiert werden", e));
              return status;
            }
          }
        }

      }

      index++;
    }

    return status;
  }

   
  @Override
  protected void behandleVerschiebenInAnderenKB(HierarchieObjekt ho, Object neuerVater, ConfigurationArea srcKB,
  		ConfigurationArea destKB, MultiStatus status)
	{
  	IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
  	IHierarchie hierarchie = getHierarchie();
    Object alterVater = hierarchie.getUebergeordnetesObjekt(ho);
    
  	 if (hot.equals(HOT_AQ.AnzeigeQuerschnittTyp) ||
  			 hot.equals(HOT_AQ.AnzeigeTyp) || 
  			 hot.equals(HOT_AQ.Dirigent))
     {
       Assert.isLegal(alterVater instanceof ConfigurationArea);
       Assert.isLegal(neuerVater instanceof ConfigurationArea);
       
       hierarchie.objektVerschieben(alterVater, neuerVater, ho);
     }
  	 else if (hot.equals(HOT_AQ.AnzeigeQuerschnitt) ||
  			 hot.equals(HOT_AQ.Zeichensatz))
		 {
  		 if (alterVater instanceof ConfigurationArea &&
  			 neuerVater instanceof ConfigurationArea)
  		 {
  			 hierarchie.objektVerschieben(alterVater, neuerVater, ho);
  		 }
  		 else
  		 {
  			 behandleVerschieben(ho, neuerVater, status);
  		 }
		 }
  	 else
  	 {
  		 behandleVerschieben(ho, neuerVater, status);
  	 }
	}
  
  

      
//  private HOMitStatus kopiereHORekursiv(HierarchieObjekt ho, ConfigurationArea kb, Object vater,
//      HOHierarchie hierarchie, boolean isWurzel)
//  {
//    MultiStatus meldung;
//    
//    if (ho == hierarchie.getWurzel())
//    {  
//      meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Hierarchieobjekt " + ho, null);
//    }
//    else
//    {
//      meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere untergeordnetes Hierarchieobjekt " + 
//          ho + " zu " + vater, null);
//    }
//    
//    HOMitStatus result = kopiereHO(ho, kb, vater, null, false, false, false, true);
//    meldung.merge( result.getStatus() );
//    HierarchieObjekt neu = result.getHO();
//
//    if (neu != null)
//    {  
//      for (HierarchieObjekt kind: hierarchie.getUntergeordneteObjekte( ho ))
//      {
//        result = kopiereHORekursiv(kind, kb, neu, hierarchie, false);
//        meldung.add( result.getStatus() );
//      }
//    }
//    
//    return new HOMitStatus(neu, meldung);
//  }
  
  @Override
  protected void behandleVerschieben(HierarchieObjekt ho, Object neuerVater, MultiStatus status)
  {
    IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
    IHierarchie hierarchie = getHierarchie();
    Object alterVater = hierarchie.getUebergeordnetesObjekt(ho);

    if (hot.equals(HOT_AQ.WvzInhalt))
    {
      // EAK
      Assert.isLegal(alterVater instanceof HierarchieObjekt
          && ((HierarchieObjekt)alterVater).getHierarchieObjektTyp().equals(HOT_AQ.AnzeigeTyp));
      Assert.isLegal(neuerVater instanceof HierarchieObjekt
          && ((HierarchieObjekt)neuerVater).getHierarchieObjektTyp().equals(HOT_AQ.AnzeigeTyp));

      behandleMengenVerschieben(ho, (HierarchieObjekt)alterVater, (HierarchieObjekt)neuerVater, "menge.wvzInhalt",
          "WvzInhalt", 0, status);
    }
    if (hot.equals(HOT_AQ.WzgInhaltGrafik))
    {
      // EAK
      Assert.isLegal(alterVater instanceof HierarchieObjekt
          && ((HierarchieObjekt)alterVater).getHierarchieObjektTyp().equals(HOT_AQ.WvzInhalt));
      Assert.isLegal(neuerVater instanceof HierarchieObjekt
          && ((HierarchieObjekt)neuerVater).getHierarchieObjektTyp().equals(HOT_AQ.WvzInhalt));

      behandleAtgVerschieben(ho, (HierarchieObjekt)alterVater, (HierarchieObjekt)neuerVater, "atg.wvzInhalt",
          "GrafikDarstellungen", true, status);

    }
    if (hot.equals(HOT_AQ.WechselText))
    {
      // EAK
      Assert.isLegal(alterVater instanceof HierarchieObjekt
          && ((HierarchieObjekt)alterVater).getHierarchieObjektTyp().equals(HOT_AQ.WvzInhalt));
      Assert.isLegal(neuerVater instanceof HierarchieObjekt
          && ((HierarchieObjekt)neuerVater).getHierarchieObjektTyp().equals(HOT_AQ.WvzInhalt));

      behandleAtgVerschieben(ho, (HierarchieObjekt)alterVater, (HierarchieObjekt)neuerVater, "atg.wvzInhalt",
          "Wechseltext", false, status);
    }
    else if (hot.equals(HOT_AQ.AnzeigeQuerschnitt))
    {
      if (alterVater instanceof HierarchieObjekt && neuerVater instanceof HierarchieObjekt)
      {
        Assert.isLegal(alterVater instanceof HierarchieObjekt
            && ((HierarchieObjekt)alterVater).getHierarchieObjektTyp().equals(HOT_AQ.Dirigent));
        Assert.isLegal(neuerVater instanceof HierarchieObjekt
            && ((HierarchieObjekt)neuerVater).getHierarchieObjektTyp().equals(HOT_AQ.Dirigent));

        behandleMengenVerschieben(ho, (HierarchieObjekt)alterVater, (HierarchieObjekt)neuerVater,
            "menge.anzeigeQuerschnitte", "AnzeigeQuerschnitte", 0, status);
      }
      else if (alterVater instanceof HierarchieObjekt && neuerVater instanceof ConfigurationArea)
      {
        // behandleVerschiebenAQNachOben(ho, (HierarchieObjekt)alterVater, neuerVater, status);
        Object ueo = hierarchie.getUebergeordnetesObjekt(ho);
        if (ueo instanceof HierarchieObjekt)
        {
          Assert.isLegal(HOT_AQ.AnzeigeQuerschnitt.equals(ho.getHierarchieObjektTyp()));
        }
        behandleVerschiebenHONachObenUeberMenge(ho, (HierarchieObjekt)alterVater, neuerVater, "AnzeigeQuerschnitte",
            status);
      }
      else if (alterVater instanceof ConfigurationArea && neuerVater instanceof HierarchieObjekt)
      {
        // behandleVerschiebenAQNachUnten(ho, alterVater, (HierarchieObjekt)neuerVater, status);
        Assert.isLegal(HOT_AQ.AnzeigeQuerschnitt.equals(ho.getHierarchieObjektTyp()));
        behandleVerschiebenHONachUntenUeberMenge(ho, alterVater, (HierarchieObjekt)neuerVater, status);
      }
    }
    else if (hot.equals(HOT_AQ.Anzeige))
    {
      if (alterVater instanceof HierarchieObjekt && neuerVater instanceof HierarchieObjekt)
      {

        Assert.isLegal(alterVater instanceof HierarchieObjekt
            && ((HierarchieObjekt)alterVater).getHierarchieObjektTyp().equals(HOT_AQ.AnzeigeQuerschnitt));
        Assert.isLegal(neuerVater instanceof HierarchieObjekt
            && ((HierarchieObjekt)neuerVater).getHierarchieObjektTyp().equals(HOT_AQ.AnzeigeQuerschnitt));

        behandleMengenVerschieben(ho, (HierarchieObjekt)alterVater, (HierarchieObjekt)neuerVater, "menge.anzeigen",
            "Anzeigen", 0, status);
      }
      else if (alterVater instanceof HierarchieObjekt && neuerVater instanceof ConfigurationArea)
      {
        Object ueo = hierarchie.getUebergeordnetesObjekt(ho);
        if (ueo instanceof HierarchieObjekt)
        {
          Assert.isLegal(HOT_AQ.Anzeige.equals(ho.getHierarchieObjektTyp()));
        }
        behandleVerschiebenHONachObenUeberMenge(ho, (HierarchieObjekt)alterVater, neuerVater, "Anzeigen", status);
      }
      else if (alterVater instanceof ConfigurationArea && neuerVater instanceof HierarchieObjekt)
      {
        Assert.isLegal(HOT_AQ.Anzeige.equals(ho.getHierarchieObjektTyp()));
        behandleVerschiebenHONachUntenUeberMenge(ho, alterVater, (HierarchieObjekt)neuerVater, status);
      }
    }
    else if (hot.equals(HOT_AQ.FS))
    {
      if (alterVater instanceof HierarchieObjekt && neuerVater instanceof HierarchieObjekt)
      {
        Assert.isLegal(alterVater instanceof HierarchieObjekt
            && ((HierarchieObjekt)alterVater).getHierarchieObjektTyp().equals(HOT_AQ.Anzeige));
        Assert.isLegal(neuerVater instanceof HierarchieObjekt
            && ((HierarchieObjekt)neuerVater).getHierarchieObjektTyp().equals(HOT_AQ.Anzeige));

        behandleMengenVerschieben(ho, (HierarchieObjekt)alterVater, (HierarchieObjekt)neuerVater, "menge.fahrStreifen",
            "FahrStreifen", 0, status);
      }
      else if (alterVater instanceof HierarchieObjekt && neuerVater instanceof ConfigurationArea)
      {
        Object ueo = hierarchie.getUebergeordnetesObjekt(ho);
        if (ueo instanceof HierarchieObjekt)
        {
          Assert.isLegal(HOT_AQ.FS.equals(ho.getHierarchieObjektTyp()));
        }
        behandleVerschiebenHONachObenUeberMenge(ho, (HierarchieObjekt)alterVater, neuerVater, "FahrStreifen", status);
      }
      else if (alterVater instanceof ConfigurationArea && neuerVater instanceof HierarchieObjekt)
      {
        Assert.isLegal(HOT_AQ.FS.equals(ho.getHierarchieObjektTyp()));
        behandleVerschiebenHONachUntenUeberMenge(ho, alterVater, (HierarchieObjekt)neuerVater, status);
      }
    }
    else if (hot.equals(HOT_AQ.Zeichen))
    {
      if (alterVater instanceof HierarchieObjekt && neuerVater instanceof HierarchieObjekt)
      {
        Assert.isLegal(alterVater instanceof HierarchieObjekt
            && ((HierarchieObjekt)alterVater).getHierarchieObjektTyp().equals(HOT_AQ.Zeichensatz));
        Assert.isLegal(neuerVater instanceof HierarchieObjekt
            && ((HierarchieObjekt)neuerVater).getHierarchieObjektTyp().equals(HOT_AQ.Zeichensatz));

        behandleMengenVerschieben(ho, (HierarchieObjekt)alterVater, (HierarchieObjekt)neuerVater, "menge.zeichen",
            "Zeichen", 0, status);
      }
      else if (alterVater instanceof HierarchieObjekt && neuerVater instanceof ConfigurationArea)
      {
        Object ueo = hierarchie.getUebergeordnetesObjekt(ho);
        if (ueo instanceof HierarchieObjekt)
        {
          Assert.isLegal(HOT_AQ.Zeichen.equals(ho.getHierarchieObjektTyp()));
        }
        behandleVerschiebenHONachObenUeberMenge(ho, (HierarchieObjekt)alterVater, neuerVater, "Zeichen", status);
      }
      else if (alterVater instanceof ConfigurationArea && neuerVater instanceof HierarchieObjekt)
      {
        Assert.isLegal(HOT_AQ.Zeichen.equals(ho.getHierarchieObjektTyp()));
        behandleVerschiebenHONachUntenUeberMenge(ho, alterVater, (HierarchieObjekt)neuerVater, status);
      }
    }
    else if (hot.equals(HOT_AQ.Ligaturen))
    {
      if (alterVater instanceof HierarchieObjekt && neuerVater instanceof HierarchieObjekt)
      {
        Assert.isLegal(alterVater instanceof HierarchieObjekt
            && ((HierarchieObjekt)alterVater).getHierarchieObjektTyp().equals(HOT_AQ.Zeichensatz));
        Assert.isLegal(neuerVater instanceof HierarchieObjekt
            && ((HierarchieObjekt)neuerVater).getHierarchieObjektTyp().equals(HOT_AQ.Zeichensatz));

        behandleMengenVerschieben(ho, (HierarchieObjekt)alterVater, (HierarchieObjekt)neuerVater, "menge.ligaturen",
            "Ligaturen", 0, status);
      }
      else if (alterVater instanceof HierarchieObjekt && neuerVater instanceof ConfigurationArea)
      {
        Object ueo = hierarchie.getUebergeordnetesObjekt(ho);
        if (ueo instanceof HierarchieObjekt)
        {
          Assert.isLegal(HOT_AQ.Ligaturen.equals(ho.getHierarchieObjektTyp()));
        }
        behandleVerschiebenHONachObenUeberMenge(ho, (HierarchieObjekt)alterVater, neuerVater, "Zeichen", status);
      }
      else if (alterVater instanceof ConfigurationArea && neuerVater instanceof HierarchieObjekt)
      {
        Assert.isLegal(HOT_AQ.Ligaturen.equals(ho.getHierarchieObjektTyp()));
        behandleVerschiebenHONachUntenUeberMenge(ho, alterVater, (HierarchieObjekt)neuerVater, status);
      }
    }
    else if (hot.equals(HOT_AQ.Zeichensatz))
    {
      if (alterVater instanceof HierarchieObjekt && neuerVater instanceof HierarchieObjekt)
      {
        Assert.isLegal(alterVater instanceof HierarchieObjekt
            && ((HierarchieObjekt)alterVater).getHierarchieObjektTyp().equals(HOT_AQ.AnzeigeTyp));
        Assert.isLegal(neuerVater instanceof HierarchieObjekt
            && ((HierarchieObjekt)neuerVater).getHierarchieObjektTyp().equals(HOT_AQ.AnzeigeTyp));

        behandleAtgVerschieben(ho, (HierarchieObjekt)alterVater, (HierarchieObjekt)neuerVater, "atg.anzeigeTyp",
            "Font", true, status);
      }
      else if (alterVater instanceof HierarchieObjekt && neuerVater instanceof ConfigurationArea)
      {
        // behandleVerschiebenAQNachOben(ho, (HierarchieObjekt)alterVater, neuerVater, status);
        Object ueo = hierarchie.getUebergeordnetesObjekt(ho);
        if (ueo instanceof HierarchieObjekt)
        {
          Assert.isLegal(HOT_AQ.Zeichensatz.equals(ho.getHierarchieObjektTyp()));
        }
        behandleVerschiebenHONachObenUeberAtg(ho, (HierarchieObjekt)alterVater, neuerVater, "atg.anzeigeTyp", "Font",
            true, status);
      }
      else if (alterVater instanceof ConfigurationArea && neuerVater instanceof HierarchieObjekt)
      {
        // behandleVerschiebenAQNachUnten(ho, alterVater, (HierarchieObjekt)neuerVater, status);
        Assert.isLegal(HOT_AQ.Zeichensatz.equals(ho.getHierarchieObjektTyp()));
        behandleVerschiebenHONachUntenUeberMenge(ho, alterVater, (HierarchieObjekt)neuerVater, status);
      }
    }
  }

  public IStatus loescheHO(HierarchieObjekt ho)
  {
    MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Lsche Hierarchieobjekt " + ho, null);

    loescheHO(ho, status);

    return status;
    // Activator.getDefault().getLog().log(status);
  }

  private void loescheHO(HierarchieObjekt ho, MultiStatus status)
  {
    IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();

    if (!Arrays.asList(EBENE1).contains(hot) && !Arrays.asList(EBENE2_Anzeige).contains(hot)
        && !Arrays.asList(EBENE2_WvzInhaltZeichensatz).contains(hot)
        && !Arrays.asList(EBENE2_ZeichenLigaturen).contains(hot) && !Arrays.asList(EBENE3_Anzeige).contains(hot)
        && !Arrays.asList(EBENE3_FS).contains(hot) && !Arrays.asList(EBENE3_WzgInhaltGrafikWechselText).contains(hot)
        && !Arrays.asList(EBENE3_ZeichenLigaturen).contains(hot) && !Arrays.asList(EBENE4).contains(hot))
    {
      throw new IllegalArgumentException("Unzulssiger HOT " + hot + " fr das Lschen in der Hierarchie Verkehr MQ");
    }

    Object vater = getHierarchie().getUebergeordnetesObjekt(ho);
    if (vater == null)
    {
      throw new IllegalArgumentException("Kein bergeordnetes Objekt fr " + ho
          + " gefunden  in der Hierarchie Verkehr MQ");
    }

    // if (Arrays.asList(EBENE1).contains(hot) && !(vater instanceof ConfigurationArea))
    // {
    // throw new IllegalArgumentException("Als bergeordnetes Objekt ist nur ein Konfigurationsbereich zulssig");
    // }

    // if (hot.equals(HOT_AQ.FS) && (!(vater instanceof HierarchieObjekt) ||
    // !((HierarchieObjekt)vater).getHierarchieObjektTyp().equals(HOT_AQ.MQ)))
    // {
    // throw new IllegalArgumentException("Als bergeordnetes Objekt ist nur ein HO vom Typ " +
    // HOT_AQ.MQ + " zulssig");
    // }
    
    IHierarchie hierarchie = getHierarchie();
    
    if (HOT_AQ.Zeichensatz.equals(hot))
    {
    	// Sonderbehandlung AQ: Zuerst alle Ligaturen lschen
    	List<HierarchieObjekt> untergeordneteObjekte = 
    		new LinkedList<HierarchieObjekt>(hierarchie.getUntergeordneteObjekte(ho));
    	for (HierarchieObjekt kind: untergeordneteObjekte)
    	{
    		if (HOT_AQ.Ligaturen.equals(kind.getHierarchieObjektTyp()))
    		{
    		  MultiStatus unterStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Lsche Hierarchieobjekt " + kind,
              null);

          loescheHO(kind, unterStatus);

          status.add(unterStatus);

          if (unterStatus.matches(IStatus.CANCEL | IStatus.ERROR))
          {
            // break;
            return;
          }
    		}
    	}
    }
    
    while (!getHierarchie().getUntergeordneteObjekte(ho).isEmpty())
    {
      HierarchieObjekt objekt = getHierarchie().getUntergeordneteObjekte(ho).get(0);
      MultiStatus unterStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Lsche Hierarchieobjekt " + objekt,
          null);

      loescheHO(objekt, unterStatus);

      status.add(unterStatus);

      if (unterStatus.matches(IStatus.CANCEL | IStatus.ERROR))
      {
        // break;
        return;
      }
    }

    ConfigurationObject ko = (ConfigurationObject)ho.getSystemObjekte().get(0);

    if (hot.equals(HOT_AQ.Anzeige))
    {
      // Lsche Menge FahrStreifen
      ObjectSet menge = ko.getNonMutableSet("FahrStreifen");
      if (menge != null)
      {
        try
        {
          // ko.removeSet(menge);
          menge.invalidate();
          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Menge FahrStreifen des Objekts " + ko.getPid()
              + " vom Typ " + ko.getType().getName() + " gelscht."));

        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Menge FahrStreifen des Objekts " + ko.getPid()
              + " vom Typ " + ko.getType().getName() + " konnte nicht gelscht werden.", e));
          return;
        }
      }

      // Entferne Referenz aus Menge des bergeordneten Objekts
      SystemObject anzeige = ((HierarchieObjekt)vater).getSystemObjekte().get(0);

      menge = ((ConfigurationObject)anzeige).getNonMutableSet("Anzeigen");
      if (menge != null)
      {
        try
        {
          menge.remove(ko);

          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
              + ko.getType().getName() + " aus der Menge Anzeigen des Objekts " + anzeige.getPid() + " vom Typ "
              + anzeige.getType().getName() + " entfernt."));

        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
              + ko.getType().getName() + " konnte nicht aus der Menge Anzeigen des Objekts " + anzeige.getPid()
              + " vom Typ " + anzeige.getType().getName() + " entfernt werden.", e));
          return;
        }
        catch (RuntimeException e)
        {
          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
              + ko.getType().getName() + " konnte nicht aus der Menge Anzeigen des Objekts " + anzeige.getPid()
              + " vom Typ " + anzeige.getType().getName() + " entfernt werden.", e));
          return;
        }
      }

    }
    // else if (hot.equals(HOT_AQ.AnzeigeQuerschnitt) || hot.equals(HOT_AQ.AnzeigeQuerschnittSub))
    else if (hot.equals(HOT_AQ.AnzeigeQuerschnitt))
    {
      // Lsche Menge FahrStreifen
      ObjectSet menge = ko.getNonMutableSet("Anzeigen");
      if (menge != null)
      {
        try
        {
          // ko.removeSet(menge);
          menge.invalidate();
          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Menge Anzeigen des Objekts " + ko.getPid()
              + " vom Typ " + ko.getType().getName() + " gelscht."));

        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Menge Anzeigen des Objekts " + ko.getPid()
              + " vom Typ " + ko.getType().getName() + " konnte nicht gelscht werden.", e));
          return;
        }
      }

      // if (hot.equals(HOT_AQ.AnzeigeQuerschnittSub))
      // {

      Object ueo = hierarchie.getUebergeordnetesObjekt(ho);

      if (ueo instanceof HierarchieObjekt)
      {

        // Entferne Referenz aus Menge des bergeordneten Objekts
        SystemObject dirigent = ((HierarchieObjekt)vater).getSystemObjekte().get(0);

        menge = ((ConfigurationObject)dirigent).getNonMutableSet("AnzeigeQuerschnitte");
        if (menge != null)
        {
          try
          {
            menge.remove(ko);

            status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
                + ko.getType().getName() + " aus der Menge AnzeigeQuerschnitte des Objekts " + dirigent.getPid()
                + " vom Typ " + dirigent.getType().getName() + " entfernt."));

          }
          catch (ConfigurationChangeException e)
          {
            status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid()
                + " vom Typ " + ko.getType().getName() + " konnte nicht aus der Menge AnzeigeQuerschnitte des Objekts "
                + dirigent.getPid() + " vom Typ " + dirigent.getType().getName() + " entfernt werden.", e));
            return;
          }
          catch (RuntimeException e)
          {
            status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid()
                + " vom Typ " + ko.getType().getName() + " konnte nicht aus der Menge AnzeigeQuerschnitte des Objekts "
                + dirigent.getPid() + " vom Typ " + dirigent.getType().getName() + " entfernt werden.", e));
            return;
          }
        }
      }
    }
    else if (hot.equals(HOT_AQ.AnzeigeTyp))
    {
      // Lsche Menge FahrStreifen
      ObjectSet menge = ko.getNonMutableSet("WvzInhalt");
      if (menge != null)
      {
        try
        {
          // ko.removeSet(menge);
          menge.invalidate();
          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Menge WvzInhalt des Objekts " + ko.getPid()
              + " vom Typ " + ko.getType().getName() + " gelscht."));

        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Menge WvzInhalt des Objekts " + ko.getPid()
              + " vom Typ " + ko.getType().getName() + " konnte nicht gelscht werden.", e));
          return;
        }
      }
    }
    else if (hot.equals(HOT_AQ.WvzInhalt))
    {

      // Entferne Referenz aus Menge des bergeordneten Objekts
      SystemObject anzeigeTyp = ((HierarchieObjekt)vater).getSystemObjekte().get(0);

      ObjectSet menge = ((ConfigurationObject)anzeigeTyp).getNonMutableSet("WvzInhalt");
      if (menge != null)
      {
        try
        {
          menge.remove(ko);

          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
              + ko.getType().getName() + " aus der Menge WvzInhalt des Objekts " + anzeigeTyp.getPid() + " vom Typ "
              + anzeigeTyp.getType().getName() + " entfernt."));

        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
              + ko.getType().getName() + " konnte nicht aus der Menge WvzInhalt des Objekts " + anzeigeTyp.getPid()
              + " vom Typ " + anzeigeTyp.getType().getName() + " entfernt werden.", e));
          return;
        }
        catch (RuntimeException e)
        {
          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
              + ko.getType().getName() + " konnte nicht aus der Menge WvzInhalt des Objekts " + anzeigeTyp.getPid()
              + " vom Typ " + anzeigeTyp.getType().getName() + " entfernt werden.", e));
          return;
        }
      }

    }
    else if (hot.equals(HOT_AQ.Dirigent))
    {
      // Lsche Menge FahrStreifen
      ObjectSet menge = ko.getNonMutableSet("AnzeigeQuerschnitte");
      if (menge != null)
      {
        try
        {
          // ko.removeSet(menge);
          menge.invalidate();
          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Menge AnzeigeQuerschnitte des Objekts " + ko.getPid()
              + " vom Typ " + ko.getType().getName() + " gelscht."));

        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Menge AnzeigeQuerschnitte des Objekts "
              + ko.getPid() + " vom Typ " + ko.getType().getName() + " konnte nicht gelscht werden.", e));
          return;
        }
      }
    }
    else if (hot.equals(HOT_AQ.Zeichensatz))
    {
//      // Lsche Menge Zeichen
//      ObjectSet menge = ko.getNonMutableSet("Zeichen");
//      if (menge != null)
//      {
//        try
//        {
//          // ko.removeSet(menge);
//          menge.invalidate();
//          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Menge Zeichen des Objekts " + ko.getPid()
//              + " vom Typ " + ko.getType().getName() + " gelscht."));
//
//        }
//        catch (ConfigurationChangeException e)
//        {
//          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Menge Zeichen des Objekts " + ko.getPid()
//              + " vom Typ " + ko.getType().getName() + " konnte nicht gelscht werden.", e));
//          return;
//        }
//      }

//      // Lsche Menge Ligaturen
//      menge = ko.getNonMutableSet("Ligaturen");
//      if (menge != null)
//      {
//        try
//        {
//          // ko.removeSet(menge);
//          menge.invalidate();
//          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Menge Ligaturen des Objekts " + ko.getPid()
//              + " vom Typ " + ko.getType().getName() + " gelscht."));
//
//        }
//        catch (ConfigurationChangeException e)
//        {
//          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Menge Ligaturen des Objekts " + ko.getPid()
//              + " vom Typ " + ko.getType().getName() + " konnte nicht gelscht werden.", e));
//          return;
//        }
//      }

      Object ueo = hierarchie.getUebergeordnetesObjekt(ho);

      if (!(ueo instanceof ConfigurationArea))
      {
        // if (hot.equals(HOT_AQ.ZeichensatzSub))
        // {
        // Entferne Referenz aus der ATG des bergeordneten Objekts
        HierarchieObjekt vaterHo = (HierarchieObjekt)ueo;
        SystemObject vaterSo = vaterHo.getSystemObjekt(0);
        SystemObject so = ho.getSystemObjekt(0);

        AttributeGroup atg = hierarchie.getDataModel().getAttributeGroup("atg.anzeigeTyp");

        ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(hierarchie.getDataModel());
        Data data = ((ConfigConfigurationObject)vaterSo).getConfigurationData(atg, lookup);

        if (data != null)
        {
          ReferenceArray ra = data.getReferenceArray("Font");

          List<SystemObject> delList = new ArrayList<SystemObject>();
          List<SystemObject> list = new ArrayList<SystemObject>();

          for (int i = 0; i < ra.getLength(); i++)
          {
            SystemObject ref = ra.getReferenceValue(i).getSystemObject();
            if (ref != null)
            {
              list.add(ref);
              if (ref.equals(so))
                delList.add(ref);
            }

          }

          if (delList.size() > 0)
          {
            list.removeAll(delList);

            ra.setLength(list.size());

            for (int i = 0; i < ra.getLength(); i++)
            {
              ra.getReferenceValue(i).setSystemObject(list.get(i));
            }

            try
            {
            	if (!HierarchieObjekt.isAttributgruppeAenderbar(vaterSo, atg))
  					  {  
  					    vaterSo = behandleSystemObjektAenderung(vaterHo, (ConfigurationObject)vaterSo, vaterSo.getPid());
  					    vaterHo.setSystemObjekt(0, vaterSo);
  					  }  
            	
              vaterSo.setConfigurationData(atg, data);
              status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + vaterSo.getPid()
                  + " als Referenz " + so.getPid() + " gesetzt"));
            }
            catch (ConfigurationChangeException e)
            {
              status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + vaterSo
                  + " konnt nicht als Referenz " + so.getPid() + " gesetzt werden", e));
            }
          }
        }
      }
    }
    else if (hot.equals(HOT_AQ.FS))
    {
      // Entferne Referenz aus Menge des bergeordneten Objekts
      SystemObject anzeige = ((HierarchieObjekt)vater).getSystemObjekte().get(0);

      ObjectSet menge = ((ConfigurationObject)anzeige).getNonMutableSet("FahrStreifen");
      if (menge != null)
      {
        try
        {
          menge.remove(ko);

          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
              + ko.getType().getName() + " aus der Menge FahrStreifen des Objekts " + anzeige.getPid() + " vom Typ "
              + anzeige.getType().getName() + " entfernt."));

        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
              + ko.getType().getName() + " konnte nicht aus der Menge FahrStreifen des Objekts " + anzeige.getPid()
              + " vom Typ " + anzeige.getType().getName() + " entfernt werden.", e));
          return;
        }
        catch (RuntimeException e)
        {
          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
              + ko.getType().getName() + " konnte nicht aus der Menge FahrStreifen des Objekts " + anzeige.getPid()
              + " vom Typ " + anzeige.getType().getName() + " entfernt werden.", e));
          return;
        }
      }
    }
    else if (hot.equals(HOT_AQ.Zeichen))
    {
      // Entferne Referenz aus Menge des bergeordneten Objekts
      SystemObject anzeige = ((HierarchieObjekt)vater).getSystemObjekte().get(0);

      ObjectSet menge = ((ConfigurationObject)anzeige).getNonMutableSet("Zeichen");
      if (menge != null)
      {
        try
        {
          menge.remove(ko);

          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
              + ko.getType().getName() + " aus der Menge Zeichen des Objekts " + anzeige.getPid() + " vom Typ "
              + anzeige.getType().getName() + " entfernt."));

        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
              + ko.getType().getName() + " konnte nicht aus der Menge Zeichen des Objekts " + anzeige.getPid()
              + " vom Typ " + anzeige.getType().getName() + " entfernt werden.", e));
          return;
        }
        catch (RuntimeException e)
        {
          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
              + ko.getType().getName() + " konnte nicht aus der Menge Zeichen des Objekts " + anzeige.getPid()
              + " vom Typ " + anzeige.getType().getName() + " entfernt werden.", e));
          return;
        }
      }
    }
    else if (hot.equals(HOT_AQ.Ligaturen))
    {
      // Entferne Referenz aus Menge des bergeordneten Objekts
      SystemObject anzeige = ((HierarchieObjekt)vater).getSystemObjekte().get(0);

      ObjectSet menge = ((ConfigurationObject)anzeige).getNonMutableSet("Ligaturen");
      if (menge != null)
      {
        try
        {
          menge.remove(ko);

          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
              + ko.getType().getName() + " aus der Menge Ligaturen des Objekts " + anzeige.getPid() + " vom Typ "
              + anzeige.getType().getName() + " entfernt."));

        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
              + ko.getType().getName() + " konnte nicht aus der Menge Ligaturen des Objekts " + anzeige.getPid()
              + " vom Typ " + anzeige.getType().getName() + " entfernt werden.", e));
          return;
        }
        catch (RuntimeException e)
        {
          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
              + ko.getType().getName() + " konnte nicht aus der Menge Ligaturen des Objekts " + anzeige.getPid()
              + " vom Typ " + anzeige.getType().getName() + " entfernt werden.", e));
          return;
        }
      }
    }
    else if (hot.equals(HOT_AQ.WzgInhaltGrafik))
    {

      HierarchieObjekt vaterHo = (HierarchieObjekt)hierarchie.getUebergeordnetesObjekt(ho);
      SystemObject vaterSo = vaterHo.getSystemObjekt(0);
      SystemObject so = ho.getSystemObjekt(0);

      AttributeGroup atg = hierarchie.getDataModel().getAttributeGroup("atg.wvzInhalt");

      ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(hierarchie.getDataModel());
      Data data = ((ConfigConfigurationObject)vaterSo).getConfigurationData(atg, lookup);

      if (data != null)
      {
        ReferenceArray ra = data.getReferenceArray("GrafikDarstellungen");

        List<SystemObject> delList = new ArrayList<SystemObject>();
        List<SystemObject> list = new ArrayList<SystemObject>();

        for (int i = 0; i < ra.getLength(); i++)
        {
          SystemObject ref = ra.getReferenceValue(i).getSystemObject();
          if (ref != null)
          {
            list.add(ref);
            if (ref.equals(so))
              delList.add(ref);
          }

        }

        if (delList.size() > 0)
        {
          list.removeAll(delList);

          ra.setLength(list.size());

          for (int i = 0; i < ra.getLength(); i++)
          {
            ra.getReferenceValue(i).setSystemObject(list.get(i));
          }

          try
          {
          	if (!HierarchieObjekt.isAttributgruppeAenderbar(vaterSo, atg))
					  {  
					    vaterSo = behandleSystemObjektAenderung(vaterHo, (ConfigurationObject)vaterSo, vaterSo.getPid());
					    vaterHo.setSystemObjekt(0, vaterSo);
					  }  
          	
            vaterSo.setConfigurationData(atg, data);
            status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + vaterSo.getPid() + " als Referenz "
                + so.getPid() + " gelscht"));
          }
          catch (ConfigurationChangeException e)
          {
            status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + vaterSo
                + " konnt nicht als Referenz " + so.getPid() + " gelscht werden", e));
          }
        }

      }
    }
    else if (hot.equals(HOT_AQ.WechselText))
    {
      HierarchieObjekt vaterHo = (HierarchieObjekt)hierarchie.getUebergeordnetesObjekt(ho);
      SystemObject vaterSo = vaterHo.getSystemObjekt(0);
      SystemObject so = ho.getSystemObjekt(0);

      AttributeGroup atg = hierarchie.getDataModel().getAttributeGroup("atg.wvzInhalt");

      ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(hierarchie.getDataModel());
      Data data = ((ConfigConfigurationObject)vaterSo).getConfigurationData(atg, lookup);

      if (data != null)
      {
        SystemObject ref = data.getReferenceValue("Wechseltext").getSystemObject();

        if (ref != null)
        {
          if (ref.equals(so))
          {
            data.getReferenceValue("Wechseltext").setSystemObject(null);
            try
            {
            	if (!HierarchieObjekt.isAttributgruppeAenderbar(vaterSo, atg))
  					  {  
  					    vaterSo = behandleSystemObjektAenderung(vaterHo, (ConfigurationObject)vaterSo, vaterSo.getPid());
  					    vaterHo.setSystemObjekt(0, vaterSo);
  					  }  
            	
              vaterSo.setConfigurationData(atg, data);
              status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + vaterSo.getPid()
                  + " als Referenz " + so.getPid() + " gelscht"));
            }
            catch (ConfigurationChangeException e)
            {
              status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + vaterSo
                  + " konnt nicht als Referenz " + so.getPid() + " gelscht werden", e));
            }
          }
        }

      }
    }

    if (!hot.equals(HOT_AQ.FS))
    {
      // Lsche Objekt
      try
      {
        ko.invalidate();

        status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
            + ko.getType().getName() + " gelscht."));
      }
      catch (ConfigurationChangeException e)
      {
        status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
            + ko.getType().getName() + " konnte nicht gelscht werden.", e));
        return;
      }
      catch (RuntimeException e)
      {
        status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
            + ko.getType().getName() + " konnte nicht gelscht werden.", e));
        return;
      }
    }

    getHierarchie().objektEntfernen(vater, ho);
  }

  public IStatus pruefeDaten(HierarchieObjekt ho, HODaten daten)
  {
    IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
    IPruefungen pruefInstanz = DatenSpeicher.getInstanz().getPruefungen();
    MultiStatus result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Hierarchieobjekt " + ho, null);

    // Prfe Hierarchieobjekt
    result.merge(pruefInstanz.pruefeHierarchieObjekt(ho, daten));

    // Prfe auf offene Referenzen bei PID-nderungen
    LoeschenChecker loeschenChecker = new LoeschenChecker(this);
    List<SystemObject> liste = new Vector<SystemObject>();
    StringBuffer aend = new StringBuffer();
    StringBuffer loesch = new StringBuffer();

    for (SystemObject so : ho.getSystemObjekte())
    {
      SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);

      if (!so.getPid().equals(soDaten.getPid()))
      {
        liste.add(so);
        if (aend.length() > 0)
        {
          aend.append(", ");
        }
        aend.append(soDaten.getPid());
        if (loesch.length() > 0)
        {
          loesch.append(", ");
        }
        loesch.append(so.getPid());
      }
    }

    if (liste.size() > 0)
    {
      MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Durch PID-nderung(en) auf " + aend
          + " werden folgende Objekte gelscht: " + loesch, null);

      Collection<SystemObjectType> typen = loeschenOffeneReferenzenVerboten(ho.getHierarchieObjektTyp());
      boolean flag = verwendeZuLoeschendeObjekteZurPruefungOffeneReferenzen();

      status.merge(loeschenChecker.bestimmeOffeneReferenzen(liste, typen, flag));
      result.merge(status);
    }

    // Prfe ATG-Daten
    for (SystemObject so : ho.getSystemObjekte())
    {
      SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
      if (soDaten != null)
      {
        MultiStatus objektStatus;
        if (ho.getSystemObjekte().size() > 1)
        {
          objektStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe ATG-Daten fr Systemobjekt "
              + so.getPid() + " vom Typ " + so.getType().getPid(), null);
        }
        else
        {
          objektStatus = result;
        }

        // Prfe ATG-Daten fr das Systemobjekt
        TypInfo typInfo = hot.getTypInfo(so.getType().getPid());
        if (typInfo != null)
        {
          for (String atgPid : typInfo.getAtgs())
          {
            AtgInfo atgInfo = typInfo.getAtgInfo(atgPid);
            if (atgInfo != null)
            {
              Data data = soDaten.getAtgDaten(atgPid);
              if (data == null && (atgInfo.isMandatory() || !atgInfo.isOptional()))
              {
                objektStatus.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Fr die nicht-optionale ATG " + atgPid
                    + " sind keine Daten vorhanden"));
              }
              if (data != null)
              {
                MultiStatus atgStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK,
                    "Prfe Daten der ATG " + atgPid, null);
                atgStatus.merge(pruefInstanz.pruefeDaten(ho, so, atgInfo, data));

                objektStatus.add(atgStatus);
              }
            }
            else
            {
              // Kein AtgInfo fr den aktuellen Typ-Info-Typ im HOT fr aktuellen Systemobjekttyp
              throw new IllegalStateException("AtgInfo fr ATG " + atgPid + " fehlt in TypInfo fr "
                  + so.getType().getPid() + " im HierarchieObjektTyp " + hot);
            }
          }
        }
        else
        {
          // Keine Typ-Info im HOT fr aktuellen Systemobjekttyp
          throw new IllegalStateException("TypInfo fehlt fr  " + so.getType().getPid() + " im HierarchieObjektTyp "
              + hot);
        }
        if (result != objektStatus)
        {
          result.add(objektStatus);
        }

      }
      else
      {
        // Keine Daten fr Systemobjekt so bergeben
        throw new IllegalStateException("Daten fr Systemobjekt " + so.getPid() + " erforderlich, aber nicht bergeben");
      }
    }

    return result;
  }

  

    
  @Override
  protected void bereinigeSystemObjektAenderung(HierarchieObjekt ho, ConfigurationObject koAlt,
  		ConfigurationObject koNeu) throws ConfigurationChangeException
  {
  	
  	IHierarchie hierarchie = getHierarchie();
  	Object vater = hierarchie.getUebergeordnetesObjekt(ho);

    if (vater instanceof HierarchieObjekt)
    {
      HierarchieObjekt vaterHO = (HierarchieObjekt)vater;
	    IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
	    ConfigurationObject objekt = null;
	    ObjectSet menge = null;
	
	    if (hot.equals(HOT_AQ.WvzInhalt))
	    {
	      objekt = (ConfigurationObject)vaterHO.getSystemObjekt(0);
	      menge = objekt.getNonMutableSet("WvzInhalt");
	    }
	    else if (hot.equals(HOT_AQ.AnzeigeQuerschnitt))
	    {
	      objekt = (ConfigurationObject)vaterHO.getSystemObjekt(0);
	      menge = objekt.getNonMutableSet("AnzeigeQuerschnitte");
	    }
	    else if (hot.equals(HOT_AQ.Anzeige))
	    {
	      objekt = (ConfigurationObject)vaterHO.getSystemObjekt(0);
	      menge = objekt.getNonMutableSet("Anzeigen");
	    }
	    else if (hot.equals(HOT_AQ.FS))
	    {
	      objekt = (ConfigurationObject)vaterHO.getSystemObjekt(0);
	      menge = objekt.getNonMutableSet("FahrStreifen");
	    }
	    else if (hot.equals(HOT_AQ.Zeichen))
	    {
	      objekt = (ConfigurationObject)vaterHO.getSystemObjekt(0);
	      menge = objekt.getNonMutableSet("Zeichen");
	    }
	    else if (hot.equals(HOT_AQ.Ligaturen))
	    {
	      objekt = (ConfigurationObject)vaterHO.getSystemObjekt(0);
	      menge = objekt.getNonMutableSet("Ligaturen");
	    }
	    else if (hot.equals(HOT_AQ.Zeichensatz))
	    {
	    	if (koAlt != null && koNeu != null && !koAlt.getPid().equals( koNeu.getPid()))
	    	{	
	        objekt = (ConfigurationObject)vaterHO.getSystemObjekt(0);
	        ConfigDataModel dataModel = hierarchie.getDataModel();
	        AttributeGroup atg = dataModel.getAttributeGroup("atg.anzeigeTyp");
	        SystemObject font = ho.getSystemObjekt(0);
	        ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
	        Data data = ((ConfigSystemObject)font).getConfigurationData(atg, lookup);
	        Data neu = DataFactory.createModifiableCopy(data, lookup);
	        ReferenceArray ra = neu.getReferenceArray("Font");
	
	        for (int i = 0; i < ra.getLength(); i++)
	        {
	          SystemObject ref = ra.getReferenceValue(i).getSystemObject();
	          if (ref != null)
	          {
	            if (ref.equals((SystemObject)koAlt))
	              ra.getReferenceValue(i).setSystemObject(koNeu);
	          }
	        }
	        
	        if (!HierarchieObjekt.isAttributgruppeAenderbar(objekt, atg))
	        {  
	        	objekt = behandleSystemObjektAenderung(vaterHO, (ConfigurationObject)objekt, objekt.getPid());
	        	vaterHO.setSystemObjekt(0, objekt);
	        }  
	        
	        font.setConfigurationData(atg, neu);
	    	}  
	    }
	    else if (hot.equals(HOT_AQ.WzgInhaltGrafik))
	    {
	    	if (koAlt != null && koNeu != null && !koAlt.getPid().equals( koNeu.getPid()))
	    	{	
	        objekt = (ConfigurationObject)vaterHO.getSystemObjekt(0);
	        ConfigDataModel dataModel = hierarchie.getDataModel();
	        AttributeGroup atg = dataModel.getAttributeGroup("atg.wvzInhalt");
	        SystemObject grafik = ho.getSystemObjekt(0);
	        ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
	        Data data = ((ConfigSystemObject)grafik).getConfigurationData(atg, lookup);
	        Data neu = DataFactory.createModifiableCopy(data, lookup);
	        ReferenceArray ra = neu.getReferenceArray("GrafikDarstellungen");
	
	        for (int i = 0; i < ra.getLength(); i++)
	        {
	          SystemObject ref = ra.getReferenceValue(i).getSystemObject();
	          if (ref != null)
	          {
	            if (ref.equals((SystemObject)koAlt))
	              ra.getReferenceValue(i).setSystemObject(koNeu);
	          }
	
	        }
	        
	        if (!HierarchieObjekt.isAttributgruppeAenderbar(objekt, atg))
	        {  
	        	objekt = behandleSystemObjektAenderung(vaterHO, (ConfigurationObject)objekt, objekt.getPid());
	        	vaterHO.setSystemObjekt(0, objekt);
	        }  
	        
	        grafik.setConfigurationData(atg, neu);
	    	}  
	    }
	    else if (hot.equals(HOT_AQ.WechselText))
	    {
	    	if (koAlt != null && koNeu != null && !koAlt.getPid().equals( koNeu.getPid()))
	    	{	
	        ConfigDataModel dataModel = hierarchie.getDataModel();
	        AttributeGroup atg = dataModel.getAttributeGroup("atg.wvzInhalt");
	        SystemObject wt = ho.getSystemObjekt(0);
	        ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
	        Data data = ((ConfigSystemObject)wt).getConfigurationData(atg, lookup);
	        Data neu = DataFactory.createModifiableCopy(data, lookup);
	        neu.getReferenceValue("Wechseltext").setSystemObject(koNeu);
	        
	        if (!HierarchieObjekt.isAttributgruppeAenderbar(objekt, atg))
	        {  
	        	wt = behandleSystemObjektAenderung(ho, (ConfigurationObject)wt, wt.getPid());
	        	ho.setSystemObjekt(0, wt);
	        }  
	        
	        wt.setConfigurationData(atg, neu);
	    	}  
	    }
	
	    if (menge != null)
	    {
	    	if (koNeu != null)
	    	{	
		      try
		      {
		        menge.add(koNeu);
		      }
		      catch (ConfigurationChangeException e)
		      {
		        for (ObjectSet set : koNeu.getObjectSets())
		        {
		          set.invalidate();
		        }
		        koNeu.invalidate();
		        throw (e);
		      }
		      catch (RuntimeException e)
		      {
		        for (ObjectSet set : koNeu.getObjectSets())
		        {
		          set.invalidate();
		        }
		        koNeu.invalidate();
		        throw (e);
		      }
	    	}
	    	
	    	if (koAlt != null)
	    	{	
	    		menge.remove(koAlt);
	    	}	
	    }
	  }
  }
  
  private void behandleMengenVerschieben(HierarchieObjekt ho, HierarchieObjekt alterVater, HierarchieObjekt neuerVater,
      String mengenTyp, String mengenName, int index, MultiStatus status)
  {
    ConfigurationObject koSrc = (ConfigurationObject)alterVater.getSystemObjekte().get(0);
    ConfigurationObject koDest = (ConfigurationObject)neuerVater.getSystemObjekte().get(0);

    ObjectSet mengeSrc = koSrc.getObjectSet(mengenName);
    Assert.isNotNull(mengeSrc);

    ObjectSet mengeDest = koDest.getObjectSet(mengenName);
    if (mengeDest == null)
    {
    	IHierarchie hierarchie = getHierarchie();
      ConfigurationArea kb = hierarchie.getKonfigurationsBereich(neuerVater);
      Assert.isNotNull(kb);

      mengeDest = erzeugeBeliebigeMenge(mengenTyp, mengenName, koDest, status);
    }

    if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
    {
      SystemObject ko = ho.getSystemObjekte().get(index);

      try
      {
        mengeDest.add(ko);
        status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + ko.getPid() + " der Menge \""
            + mengenName + "\" des Systemobjekts " + koDest.getPid() + " hinzugefgt"));
      }
      catch (ConfigurationChangeException e)
      {
        status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + ko.getPid()
            + " konnte nicht der Menge \"" + mengenName + "\" des Systemobjekts " + koDest.getPid()
            + " hinzugefgt werden", e));
      }

      if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
      {
//        _hierarchie.objektHinzufuegen(neuerVater, ho);

        try
        {
          mengeSrc.remove(ko);
          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + ko.getPid() + " aus der Menge \""
              + mengenName + "\" des Systemobjekts " + koSrc.getPid() + " entfernt"));
        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + ko.getPid()
              + " konnte nicht aus der Menge \"" + mengenName + "\" des Systemobjekts " + koSrc.getPid()
              + " entfernt werden", e));
        }

        if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
        {
//          _hierarchie.objektEntfernen(alterVater, ho);
          getHierarchie().objektVerschieben(alterVater, neuerVater, ho);
        }
      }
    }
  }

  private void behandleAtgVerschieben(HierarchieObjekt ho, HierarchieObjekt alterVater, HierarchieObjekt neuerVater,
      String pidAtg, String attributName, boolean isArray, MultiStatus status)
  {
    // Object ueo = _hierarchie.getUebergeordnetesObjekt(ho);
    //    
    // if (!(ueo instanceof ConfigurationArea))
    {
      // Referenz der ATG des bergeordneten Ziel-Objekts hinzufgen
    	IHierarchie hierarchie = getHierarchie();
    	ConfigDataModel dataModel = hierarchie.getDataModel();
      ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);

      AttributeGroup atg = dataModel.getAttributeGroup(pidAtg);

      SystemObject soNeuerVater = neuerVater.getSystemObjekt(0);
      SystemObject soAlterVater = alterVater.getSystemObjekt(0);

      Data destData = ((ConfigConfigurationObject)soNeuerVater).getConfigurationData(atg, lookup);
      Data srcData = ((ConfigConfigurationObject)soAlterVater).getConfigurationData(atg, lookup);
      
      Map<HierarchieObjekt, HierarchieObjekt> delMap = new HashMap<HierarchieObjekt, HierarchieObjekt>();

      SystemObject so = ho.getSystemObjekt(0);

      if (destData == null)
      {
        destData = DataFactory.createData(atg);
        // Default und undefiniert Werte setzen
        destData.setToDefault();
      }

      if (isArray)
      {
        ReferenceArray ra = destData.getReferenceArray(attributName);
        int len = ra.getLength();
        ra.setLength(len + 1);
        ra.getReferenceValue(len).setSystemObject(so);
      }
      else
      {
        if (so.isOfType("typ.wechseltext"))
        {
          SystemObject sx = destData.getReferenceValue("Wechseltext").getSystemObject();
          List<HierarchieObjekt> uos = getHierarchie().getUntergeordneteObjekte(neuerVater);

          for (HierarchieObjekt hox : uos)
          {
            SystemObject kind = hox.getSystemObjekt(0);
            if (kind.equals(sx))
            {
              delMap.put(neuerVater, hox);
            }
          }
        }  
        
        destData.getReferenceValue(attributName).setSystemObject(so);
      }

      try
      {
      	if (!HierarchieObjekt.isAttributgruppeAenderbar(soNeuerVater, atg))
        {  
      		soNeuerVater = behandleSystemObjektAenderung(neuerVater, (ConfigurationObject)soNeuerVater, soNeuerVater.getPid());
      		neuerVater.setSystemObjekt(0, soNeuerVater);
        }  
      	
        soNeuerVater.setConfigurationData(atg, destData);
        status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + soNeuerVater.getPid()
            + " als Referenz " + so.getPid() + " gesetzt"));
      }
      catch (ConfigurationChangeException e)
      {
        status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + soNeuerVater
            + " konnt nicht als Referenz " + so.getPid() + " gesetzt werden", e));
      }

      // Entferne Referenz aus der ATG des bergeordneten Quell-Objekts
      if (srcData != null)
      {
        if (isArray)
        {
          ReferenceArray ra = srcData.getReferenceArray(attributName);

          List<SystemObject> delList = new ArrayList<SystemObject>();
          List<SystemObject> list = new ArrayList<SystemObject>();

          for (int i = 0; i < ra.getLength(); i++)
          {
            SystemObject ref = ra.getReferenceValue(i).getSystemObject();
            if (ref != null)
            {
              list.add(ref);
              if (ref.equals(so))
                delList.add(ref);
            }

          }

          if (delList.size() > 0)
          {
            list.removeAll(delList);

            ra.setLength(list.size());

            for (int i = 0; i < ra.getLength(); i++)
            {
              ra.getReferenceValue(i).setSystemObject(list.get(i));
            }
          }
        }
        else
        {
          srcData.getReferenceValue(attributName).setSystemObject(null);
        }

        try
        {
        	if (!HierarchieObjekt.isAttributgruppeAenderbar(soAlterVater, atg))
          {  
        		soAlterVater = behandleSystemObjektAenderung(alterVater, (ConfigurationObject)soAlterVater, soAlterVater.getPid());
        		alterVater.setSystemObjekt(0, soAlterVater);
          }  
        	
          soAlterVater.setConfigurationData(atg, srcData);
          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + soAlterVater.getPid()
              + " als Referenz " + so.getPid() + " gesetzt"));
        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + soAlterVater
              + " konnt nicht als Referenz " + so.getPid() + " gesetzt werden", e));
        }
        
        if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
        {
        	for (HierarchieObjekt vater: delMap.keySet())
          {
          	HierarchieObjekt kind = delMap.get(vater);
          	hierarchie.objektEntfernen(vater, kind);
          }
        	
        	hierarchie.objektVerschieben(alterVater, neuerVater, ho);
        }
      }
    }
  }

  public IStatus uebernehmeDaten(HierarchieObjekt ho, HODaten daten)
  {
    MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK,
        "bernehme bearbeitete Daten fr Hierarchieobjekt " + ho, null);

    // IStatus unterStatus = pruefeDaten(ho, daten);
    //  	
    // status.add(unterStatus);
    //  	
    // if (unterStatus.matches(IStatus.ERROR | IStatus.CANCEL))
    // {
    // return status;
    // }

    List<SystemObject> objektListe = daten.getSystemObjekte();

//    DatenModellUpdateListener.getInstanz().notifyDatenModellUpdateListener(objektListe);

    for (int i = 0; i < objektListe.size(); i += 1)
    {
      SystemObject so = objektListe.get(i);

      if (ho.getSystemObjekte().contains(so))
      {
        SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
        Object vater = getHierarchie().getUebergeordnetesObjekt(ho);

        // PID
        if (!so.getPid().equals(soDaten.getPid()))
        {
          try
          {
            so = behandleSystemObjektAenderung(ho, (ConfigurationObject)so, soDaten.getPid());
            ho.setSystemObjekt(i, so);
            status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, "Neues Objekt mit PID " + so.getPid() + " vom Typ "
                + so.getType().getPid() + " erzeugt"));
          }
          catch (ConfigurationChangeException e)
          {
            status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Fehler beim behandeln der PID-nderung auf "
                + soDaten.getPid() + " fr Objekt " + so.getPid(), e));
          }
        }

        // Name
  			if (so.getName() == null || !so.getName().equals(soDaten.getName()))
  			{
	        try
	        {
	          so.setName(soDaten.getName());
	          status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, "Name \"" + soDaten.getName() + "\" bernommen"));
	        }
	        catch (ConfigurationChangeException e)
	        {
	          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Name \"" + soDaten.getName()
	              + "\" konnte nicht bernommen werden", e));
	        }
  			}  

        // ATGs

        List<String> atgList = ho.getHierarchieObjektTyp().getTypInfo(so.getType().getPid()).getAtgs();

        for (String atgPid : atgList)
        {
        	ConfigDataModel dataModel = getHierarchie().getDataModel();
          AttributeGroup atg = dataModel.getAttributeGroup(atgPid);
          if (atg != null)
          {
            /*
             * AttributeGroupUsage atgUsage = atg.getAttributeGroupUsage(_dataModel.getAspect("asp.eigenschaften")); if
             * (atgUsage != null) { ConfigurationObject ko = (ConfigurationObject)so; if (ko.getValidSince() <
             * ko.getConfigurationArea().getModifiableVersion()) // Objekt ist bereits aktiviert {
             * AttributeGroupUsage.Usage usage = atgUsage.getUsage(); if (usage ==
             * AttributeGroupUsage.Usage.OptionalConfigurationData || usage ==
             * AttributeGroupUsage.Usage.RequiredConfigurationData) { status.add(new Status(IStatus.INFO,
             * Activator.PLUGIN_ID, "Eventuelle nderungen an konfigurierenden Daten der ATG " + atgPid +
             * " werden IGNORIERT wegen ATG-Verwendung " + usage)); continue; } } }
             */

            
            ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);

            // Data ist = so.getConfigurationData(atg);
            Data ist = ((ConfigSystemObject)so).getConfigurationData(atg, lookup);

            Data soll = soDaten.getAtgDaten(atgPid);

            if (soll != null)
            {
            	if (ist == null || soDaten.isDatenGeaendert(atgPid))
	          	{	
	              if (!HierarchieObjekt.isAttributgruppeAenderbar(so, atg))
	              {
//	                status.add(new Status(IStatus.INFO, Activator.PLUGIN_ID,
//	                    "Eventuelle nderungen an konfigurierenden Daten der ATG " + atgPid
//	                        + " werden IGNORIERT, da wegen ATG-Verwendung ("
//	                        + AttributeGroupUsage.Usage.OptionalConfigurationData + ") nicht mglich"));
//	                continue;
	              	
	              	try
		              {
	              		so = behandleSystemObjektAenderung(ho, (ConfigurationObject)so, so.getPid());
		     	  				ho.setSystemObjekt(i, so);
		     	  				status.add(new Status(IStatus.OK, Activator.PLUGIN_ID,
		     	  						"ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
		     	  						" bedingt Erzeugung neues Objekt mit PID " + so.getPid() + " vom Typ " + 
		     	  						so.getType().getPid()));
		               }
		               catch (ConfigurationChangeException e)
		             	 {
		                 status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
		               			"Bei der durch ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
		     	  						" bedingten Erzeugung neues Objekts mit PID " + so.getPid() + " vom Typ " +	
		     	  						so.getType().getPid() + " ist ein Fehler aufgetreten", e));
		               	 continue;
		             	 }
	              }
	              try
	              {
	                so.setConfigurationData(atg, soll);
	                status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, "Konfigurierende Daten der ATG " + atgPid
	                    + " bernommen"));
	              }
	              catch (ConfigurationChangeException e)
	              {
	                status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID,
	                    "Fehler beim bernehmen konfigurierender Daten der ATG " + atgPid, e));
	              }
	          	}  
            }
            else
            // Keine Daten fr diese ATG bergeben => vorhanden Daten lschen
            {
              if (ist != null)
              {
                if (!HierarchieObjekt.isAttributgruppeAenderbar(so, atg))
                {
                	try
		              {
	              		so = behandleSystemObjektAenderung(ho, (ConfigurationObject)so, so.getPid());
		     	  				ho.setSystemObjekt(i, so);
		     	  				status.add(new Status(IStatus.OK, Activator.PLUGIN_ID,
		     	  						"ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
		     	  						" bedingt Erzeugung neues Objekt mit PID " + so.getPid() + " vom Typ " + 
		     	  						so.getType().getPid()));
		               }
		               catch (ConfigurationChangeException e)
		             	 {
		                 status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
		               			"Bei der durch ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
		     	  						" bedingten Erzeugung neues Objekts mit PID " + so.getPid() + " vom Typ " +	
		     	  						so.getType().getPid() + " ist ein Fehler aufgetreten", e));
		               	 continue;
		             	 }
                	
//                  status.add(new Status(IStatus.INFO, Activator.PLUGIN_ID,
//                      "Eventuelle nderungen an konfigurierenden Daten der ATG " + atgPid
//                          + " werden IGNORIERT, da wegen ATG-Verwendung ("
//                          + AttributeGroupUsage.Usage.OptionalConfigurationData + ") nicht mglich"));
//                  continue;
                }
                try
                {
                  so.setConfigurationData(atg, null);
                  status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, "Konfigurierende Daten der ATG " + atgPid
                      + " gelscht"));
                }
                catch (ConfigurationChangeException e)
                {
                  status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID,
                      "Fehler beim Lschen konfigurierender Daten der ATG " + atgPid, e));
                }
              }
            }
          }
          else
          {
            _debug.warning("Bearbeitung Objekt " + so.getPid() + ": Keine Attributgruppe im Datenmodell gefunden fr "
                + atgPid);
          }
        }
      }
    }

    return status;
  }

  @Override
  public boolean pruefeNeuanlageVerboten(Object objekt, IHierarchieObjektTyp hot)
  {

    // TODO Auto-generated method stub
    if (hot.equals(HOT_AQ.Anzeige))
    {
      boolean b = bestimmeObObjekteInKonfigurationVorhanden("typ.anzeigeTyp");

      return b;
    }
    // else if (hot.equals(HOT_AQ.AnzeigeQuerschnitt) || hot.equals(HOT_AQ.AnzeigeQuerschnittSub))
    else if (hot.equals(HOT_AQ.AnzeigeQuerschnitt))
    {
      boolean b = bestimmeObObjekteInKonfigurationVorhanden("typ.anzeigeQuerschnittTyp");

      return b;
    }

    return false;
  }

  private boolean bestimmeObObjekteInKonfigurationVorhanden(String pidTyp)
  {
  	ConfigDataModel dataModel = getHierarchie().getDataModel();
    Collection<ConfigurationArea> areas = dataModel.getAllConfigurationAreas().values();

    SystemObjectType typ = dataModel.getType(pidTyp);
    Collection<SystemObjectType> typeSpez = new Vector<SystemObjectType>();

    typeSpez.add(typ);

    for (ConfigurationArea kb : areas)
    {

      if (kb != null)
      {
        Collection<SystemObject> elements = kb.getObjects(typeSpez, ObjectTimeSpecification.valid());
        if (elements != null && elements.size() > 0)
          return false;

        for (SystemObject so : kb.getNewObjects())
        {
          if (so.isOfType(pidTyp))
            return false;

        }
      }
    }
    return true;
  }

  @Override
  public Collection<SystemObjectType> loeschenOffeneReferenzenVerboten(IHierarchieObjektTyp hot)
  {
    Collection<SystemObjectType> returnList = null;
    ConfigDataModel dataModel = getHierarchie().getDataModel();
    
    if (hot.equals(HOT_AQ.AnzeigeQuerschnittTyp))
    {

      returnList = new ArrayList<SystemObjectType>();
      SystemObjectType type = dataModel.getType("typ.anzeigeQuerschnitt");
      returnList.add(type);

      return returnList;
    }
    else if (hot.equals(HOT_AQ.AnzeigeTyp))
    {

      returnList = new ArrayList<SystemObjectType>();
      SystemObjectType type = dataModel.getType("typ.anzeige");
      returnList.add(type);

      return returnList;
    }
    // Wird ein Zeichensatz geloescht, duerfen keine offene Referenzen vom Typ
    // typ.wechseltext und typ.anzeigeTyp entstehen
    else if (hot.equals(HOT_AQ.Zeichensatz))
    {

      returnList = new ArrayList<SystemObjectType>();
      SystemObjectType type = dataModel.getType("typ.anzeigeTyp");
      returnList.add(type);
      type = dataModel.getType("typ.wechseltext");
      returnList.add(type);

      return returnList;
    }
    else if (hot.equals(HOT_AQ.WechselText))
    {

      returnList = new ArrayList<SystemObjectType>();
      SystemObjectType type = dataModel.getType("typ.wvzInhalt");
      returnList.add(type);

      return returnList;
    }
    else if (hot.equals(HOT_AQ.WzgInhaltGrafik))
    {

      returnList = new ArrayList<SystemObjectType>();
      SystemObjectType type = dataModel.getType("typ.wvzInhalt");
      returnList.add(type);

      return returnList;
    }
    // Wird ein Zeichen geloescht, duerfen keine offene Referenzen vom Typ
    // typ.wechseltext und typ.ligaturen entstehen
    else if (hot.equals(HOT_AQ.Zeichen))
    {

      returnList = new ArrayList<SystemObjectType>();
      SystemObjectType type = dataModel.getType("typ.wechseltext");
      returnList.add(type);
      type = dataModel.getType("typ.ligaturen");
      returnList.add(type);

      return returnList;
    }

    return null;
  }

  private void behandleVerschiebenHONachObenUeberMenge(HierarchieObjekt ho, HierarchieObjekt alterVater,
      Object neuerVater, String mengenName, MultiStatus status)
  {
    ConfigurationObject koSrc = (ConfigurationObject)alterVater.getSystemObjekte().get(0);

    ObjectSet mengeSrc = koSrc.getObjectSet(mengenName);

    if (mengeSrc != null)
    {
      SystemObject ko = ho.getSystemObjekte().get(0);

      try
      {
        mengeSrc.remove(ko);
        status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + ko.getPid()
            + " aus der Menge \"Anzeigen\" des Systemobjekts " + koSrc.getPid() + " entfernt"));
      }
      catch (ConfigurationChangeException e)
      {
        status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + ko.getPid()
            + " konnte nicht aus der Menge \"Anzeigen\" des Systemobjekts " + koSrc.getPid() + " entfernt werden", e));
      }
    }

    ho.setSystemObjekte(new SystemObject[] { ho.getSystemObjekt(0) });
//    _hierarchie.objektHinzufuegen(neuerVater, ho);
//    _hierarchie.objektEntfernen(alterVater, ho);
    if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
    {
      getHierarchie().objektVerschieben(alterVater, neuerVater, ho);
    }
  }
  
  private void behandleVerschiebenHONachObenUeberAtg(HierarchieObjekt ho, HierarchieObjekt alterVater,
      Object neuerVater, String pidAtg, String attributName, boolean isArray, MultiStatus status)
  {
    SystemObject so = ho.getSystemObjekt(0);
    IHierarchie hierarchie = getHierarchie();
    ConfigDataModel dataModel = hierarchie.getDataModel();

    ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);

    AttributeGroup atg = dataModel.getAttributeGroup(pidAtg);

    SystemObject soAlterVater = alterVater.getSystemObjekt(0);

    Data srcData = ((ConfigConfigurationObject)soAlterVater).getConfigurationData(atg, lookup);

    // Entferne Referenz aus der ATG des bergeordneten Quell-Objekts
    if (srcData != null)
    {
      if (isArray)
      {
        ReferenceArray ra = srcData.getReferenceArray(attributName);

        List<SystemObject> delList = new ArrayList<SystemObject>();
        List<SystemObject> list = new ArrayList<SystemObject>();

        for (int i = 0; i < ra.getLength(); i++)
        {
          SystemObject ref = ra.getReferenceValue(i).getSystemObject();
          if (ref != null)
          {
            list.add(ref);
            if (ref.equals(so))
              delList.add(ref);
          }

        }

        if (delList.size() > 0)
        {
          list.removeAll(delList);

          ra.setLength(list.size());

          for (int i = 0; i < ra.getLength(); i++)
          {
            ra.getReferenceValue(i).setSystemObject(list.get(i));
          }
        }
      }
      else
      {
        srcData.getReferenceValue(attributName).setSystemObject(null);
      }

      try
      {
        if (!HierarchieObjekt.isAttributgruppeAenderbar(soAlterVater, atg))
        {  
        	soAlterVater = behandleSystemObjektAenderung(alterVater, (ConfigurationObject)soAlterVater, soAlterVater.getPid());
        	alterVater.setSystemObjekt(0, soAlterVater);
        }  
      	
        soAlterVater.setConfigurationData(atg, srcData);
        status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + soAlterVater.getPid()
            + " als Referenz " + so.getPid() + " gesetzt"));
      }
      catch (ConfigurationChangeException e)
      {
        status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + soAlterVater
            + " konnt nicht als Referenz " + so.getPid() + " gesetzt werden", e));
      }
    }

    ho.setSystemObjekte(new SystemObject[] { ho.getSystemObjekt(0) });
//    _hierarchie.objektHinzufuegen(neuerVater, ho);
//    _hierarchie.objektEntfernen(alterVater, ho);
    if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
    {
      hierarchie.objektVerschieben(alterVater, neuerVater, ho);
    }
  }

  private void behandleVerschiebenHONachUntenUeberMenge(HierarchieObjekt ho, Object alterVater,
      HierarchieObjekt neuerVater, MultiStatus status)
  {
    IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
    SystemObject[] neueObjekte = new SystemObject[1];
    IBildungsRegeln bildungsRegeln = DatenSpeicher.getInstanz().getBildungsRegeln();
    bildungsRegeln.setWerte(hot, neuerVater, ho);
    neueObjekte[0] = ho.getSystemObjekt(0);

    try
    {
      behandleMengenZuordnung(neuerVater, neueObjekte[0], status);
    }
    finally
    {
      if (status.matches(IStatus.ERROR | IStatus.CANCEL))
      {
        // Alle Aktionen rckgngig
        MultiStatus meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Rollback Erzeugen neue Systemobjekte",
            null);
        status.add(meldung);
      }
    }

    if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
    {
      ho.setSystemObjekte(neueObjekte);
//      _hierarchie.objektHinzufuegen(neuerVater, ho);
//      _hierarchie.objektEntfernen(alterVater, ho);
      if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
      {
        getHierarchie().objektVerschieben(alterVater, neuerVater, ho);
      }
    }
  }

  private static IHierarchieObjektTyp[] SORTIER_FOLGE_1 = new IHierarchieObjektTyp[] { HOT_AQ.AnzeigeQuerschnittTyp,
      HOT_AQ.WvzInhalt, HOT_AQ.Zeichensatz, HOT_AQ.AnzeigeTyp, HOT_AQ.AnzeigeQuerschnitt, HOT_AQ.Dirigent };

  private static IHierarchieObjektTyp[] SORTIER_FOLGE_2 = new IHierarchieObjektTyp[] { HOT_AQ.Anzeige };

  private static IHierarchieObjektTyp[] SORTIER_FOLGE_3 = new IHierarchieObjektTyp[] { HOT_AQ.FS };

  // private static IHierarchieObjektTyp[] SORTIER_FOLGE_4 = new IHierarchieObjektTyp[] {HOT_AQ.WvzInhalt};
  private static IHierarchieObjektTyp[] SORTIER_FOLGE_5 = new IHierarchieObjektTyp[] { HOT_AQ.Zeichen, HOT_AQ.Ligaturen };

  private static IHierarchieObjektTyp[] SORTIER_FOLGE_6 = new IHierarchieObjektTyp[] { HOT_AQ.WzgInhaltGrafik,
      HOT_AQ.WechselText };

  private IHierarchieObjektTyp[] getTypSortierFolge(IHierarchieObjektTyp hot)
  {
    IHierarchieObjektTyp[] sortierFolge = null;

    if (hot.equals(HOT_AQ.AnzeigeQuerschnittTyp) || hot.equals(HOT_AQ.Zeichensatz) || hot.equals(HOT_AQ.WvzInhalt)
        || hot.equals(HOT_AQ.AnzeigeTyp) || hot.equals(HOT_AQ.AnzeigeQuerschnitt) || hot.equals(HOT_AQ.Dirigent))
    {
      sortierFolge = SORTIER_FOLGE_1;
    }
    else if (hot.equals(HOT_AQ.Anzeige))
    {
      sortierFolge = SORTIER_FOLGE_2;
    }
    else if (hot.equals(HOT_AQ.FS))
    {
      sortierFolge = SORTIER_FOLGE_3;
    }
    // else if (hot.equals(HOT_AQ.WvzInhalt))
    // {
    // sortierFolge = SORTIER_FOLGE_4;
    // }
    else if (hot.equals(HOT_AQ.Zeichen) || hot.equals(HOT_AQ.Ligaturen))
    {
      sortierFolge = SORTIER_FOLGE_5;
    }
    else if (hot.equals(HOT_AQ.WzgInhaltGrafik) || hot.equals(HOT_AQ.WechselText))
    {
      sortierFolge = SORTIER_FOLGE_6;
    }

    return sortierFolge;
  }

  @Override
  public boolean verwendeZuLoeschendeObjekteZurPruefungOffeneReferenzen()
  {
    // TODO Auto-generated method stub
    return false;
  }
  
  /**
   * Zeigt an, ob das bergebene Hierarchieobjekt beim rekursiven Kopieren als untergeordnetes 
   * Objekt mitkopiert werden soll oder nicht
   *  
   * @param ho das aktuell zu kopierende HO
   * @return <code>true</code>, wenn das aktuell zu kopierende Objekt nicht kopiert werden soll, 
   * <code>false</code> (default) sonst
   */
  public boolean kopierenNichtErlaubt(HierarchieObjekt ho)
  {
  	return false;
  }

}
