/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkaaq;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Vector;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HODaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IAttributInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.SystemObjektDaten;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.IPruefungen;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.PruefungenTools;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.ObjectTimeSpecification;
import de.bsvrz.dav.daf.main.config.StringAttributeType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Prfungen von Attributwerten fr den TKA Verkehr MQ
 * 
 * @author Sans
 * 
 */
public class PruefungenAQ implements IPruefungen
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();

  /**
   * Die Hierarchie, die fr die Prfungen benutzt werden kann
   */
  private IHierarchie _hierarchie;

  /**
   * Bestimmt die Hierarchie, die fr die Prfungen benutzt werden kann
   * 
   * @return die Hierarchie, die fr die Prfungen benutzt werden kann
   */
  public IHierarchie getHierarchie()
  {
    return _hierarchie;
  }

  /**
   * Setzt die Hierarchie, die fr die Prfungen benutzt werden kann
   * 
   * @param hierachie
   *          die Hierarchie, die fr die Prfungen benutzt werden kann
   */
  public void setHierarchie(IHierarchie hierarchie)
  {
    _hierarchie = hierarchie;
  }

  public IStatus pruefeHierarchieObjekt(HierarchieObjekt ho, HODaten daten)
  {
    MultiStatus result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Hierarchieobjekt " + ho, null);

    for (SystemObject so : ho.getSystemObjekte())
    {
      SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
      if (soDaten != null)
      {
        MultiStatus objektStatus;
        if (ho.getSystemObjekte().size() > 1)
        {
          objektStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Systemobjekt " + so.getPid()
              + " vom Typ " + so.getType().getPid(), null);
        }
        else
        {
          objektStatus = result;
        }

        //objektStatus.add(pruefePid(so, soDaten.getPid()));
        objektStatus.add(new PruefungenTools(getHierarchie().getDataModel()).pruefePid(soDaten.getPid(), so));
        
        if (so.isOfType("typ.anzeige"))
        {
          objektStatus.add(pruefeZuordnungAnzeige(ho, so, soDaten));
        }
        else if (so.isOfType("typ.anzeigeQuerschnitt"))
        {
//          if (ho.getHierarchieObjektTyp().equals(HOT_AQ.AnzeigeQuerschnittSub))
//              objektStatus.add(pruefeZuordnungAnzeigeQuerschnitt(ho, so, soDaten));
            Object ueo = _hierarchie.getUebergeordnetesObjekt(ho);
            if (ueo instanceof HierarchieObjekt)
            {
              objektStatus.add(pruefeZuordnungAnzeigeQuerschnitt(ho, so, soDaten));
              
            }
        }
        else if (so.isOfType("typ.ligaturen"))
        {
          objektStatus.add(pruefeZuordnungLigaturen(ho, so, soDaten));
        }
        else if (so.isOfType("typ.wechseltext"))
        {
          objektStatus.add(pruefeZuordnungWechseltext(ho, so, soDaten));
        }
        else if (so.isOfType("typ.wvzInhalt"))
        {
          objektStatus.add(pruefeZuordnungWvzInhalt(ho, so, soDaten));
        }

        if (objektStatus != result)
        {
          result.add(objektStatus);
        }
      }
    }

    return result;
  }

  public IStatus pruefeDaten(HierarchieObjekt ho, SystemObject so, AtgInfo atgInfo, Data daten)
  {
    MultiStatus result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Daten der ATG " + atgInfo.getAtg(),
        null);

    if (so.isOfType("typ.anzeige"))
    {
      if ("atg.anzeige".equals(atgInfo.getAtg()))
      {
        pruefeDatenDefinition(atgInfo, daten, result);
      }
    }
    else if (so.isOfType("typ.anzeigeQuerschnitt"))
    {
      if ("atg.anzeigeQuerschnitt".equals(atgInfo.getAtg()) || "atg.anzeigeQuerschnittNba".equals(atgInfo.getAtg())
          || "atg.punktKoordinaten".equals(atgInfo.getAtg())
          || "atg.punktLiegtAufLinienObjekt".equals(atgInfo.getAtg()))
      {
        pruefeDatenDefinition(atgInfo, daten, result);
      }
    }
    else if (so.isOfType("typ.anzeigeTyp"))
    {
      if ("atg.anzeigeTyp".equals(atgInfo.getAtg()) || "atg.anzeigeTypTextAusrichtung".equals(atgInfo.getAtg()))
      {
        pruefeDatenDefinition(atgInfo, daten, result);
      }
    }
    else if (so.isOfType("typ.dirigent"))
    {
      if ("atg.dirigent".equals(atgInfo.getAtg()))
      {
        pruefeDatenDefinition(atgInfo, daten, result);
      }
    }
    else if (so.isOfType("typ.font"))
    {
      if ("atg.font".equals(atgInfo.getAtg()))
      {
        pruefeDatenDefinition(atgInfo, daten, result);
      }
    }
    else if (so.isOfType("typ.ligaturen"))
    {
      if ("atg.ligaturen".equals(atgInfo.getAtg()))
      {
        pruefeDatenDefinition(atgInfo, daten, result);
      }
    }
    else if (so.isOfType("typ.wechseltext"))
    {
      if ("atg.wechseltext".equals(atgInfo.getAtg()))
      {
        pruefeDatenDefinition(atgInfo, daten, result);
      }
    }
    else if (so.isOfType("typ.wvzInhalt"))
    {
      if ("atg.wvzInhalt".equals(atgInfo.getAtg()))
      {
        pruefeDatenDefinition(atgInfo, daten, result);
      }
    }
    else if (so.isOfType("typ.wzgInhaltGrafik"))
    {
      if ("atg.wzgInhaltGrafik".equals(atgInfo.getAtg()))
      {
        pruefeDatenDefinition(atgInfo, daten, result);
      }
    }
    else if (so.isOfType("typ.zeichen"))
    {
      if ("atg.zeichen".equals(atgInfo.getAtg()))
      {
        pruefeDatenDefinition(atgInfo, daten, result);
      }
    }
    else if (so.isOfType("typ.anzeigeQuerschnittTyp"))
    {
      if ("atg.anzeigeQuerschnittTyp".equals(atgInfo.getAtg()))
      {
        pruefeDatenDefinition(atgInfo, daten, result);
        
        Array array = daten.getItem("MglicheBetriebsZustnde").asArray();
        
        String[] ss = new String[array.getLength()];
        
        for (int i = 0; i < array.getLength(); i++)
        {
          String s = array.getItem(i).asTextValue().getValueText();
          ss[i] = s;
        }
                
        if (ss.length > 0)
        {           
          if (hatGleicheInhalte(ss))
          {
            result.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Alle Betriebszustnde von " + atgInfo.getAtg()
                + " mssen verschieden sein"));
          }

        }
        
      }
    }
    return result;
  }

  private boolean pruefeDatenDefinition(IAttributInfo attInfo, Data daten, MultiStatus status)
  {
    Data item = daten.getItem(attInfo.getName());
    if (!item.isDefined())
    {
      status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Wert ist nicht korrekt definiert"));
      return false;
    }
    else
    {
      status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, "Prfen Daten-Definition: OK"));
    }

    // Plichtfeld-Prfung:
    if (item.isPlain())
    {
      if (attInfo.isPflichtFeld())
      {
        String wert = item.asTextValue().getText();
        if ("".equals(wert)
            || (!(item.getAttributeType() instanceof StringAttributeType) && "undefiniert".equals(wert) || "_Undefiniert_"
                .equals(wert)))
        {
          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Wert fr Pflichtfeld nicht angegeben"));
          return false;
        }
        else
        {
          status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, "Prfen Pflichtfeld-Eigenschaft: OK"));
        }
      }
    }
    return true;
  }

  private void pruefeDatenDefinition(AtgInfo atgInfo, Data daten, MultiStatus status)
  {
    for (String attribut : atgInfo.getAttribute())
    {
      MultiStatus meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert fr Attribut " + attribut, null);
      IAttributInfo attInfo = atgInfo.getAttributInfo(attribut);
      pruefeDatenDefinition(attInfo, daten, meldung);
      status.add(meldung);
    }
  }

  /**
   * Prft, ob eine PID fr ein Systemobjekt vergeben werden kann
   * 
   * @param so
   *          das Systemobjekt, fr das die PID vergeben werden soll
   * @param pid
   *          die PID, die vergeben werden soll
   * @return eine Instanz vom Typ IStatus, die aussagt, ob es zulssig ist, die PID fr das Systemobjekt zu vergeben,
   *         oder <code>null</code>, wenn keine Prfung implementiert ist#
   *         
   * @deprecated        
   */
  private IStatus pruefePid(SystemObject so, String pid)
  {
    MultiStatus result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe PID " + pid, null);
    pruefePid(so, pid, null, result);
    return result;
  }

  /**
   * @deprecated        
   */
  private void pruefePid(SystemObject so, String pid, ConfigurationArea kb, MultiStatus status)
  {
    if (kb != null)
    {
      for (SystemObject lauf : kb.getCurrentObjects())
      {
        if ((lauf instanceof ConfigurationObject && !(((ConfigurationObject)lauf).getNotValidSince() >= kb
            .getModifiableVersion()))
            || (lauf instanceof DynamicObject && !(((DynamicObject)lauf).getNotValidSince() >= kb
                .getModifiableVersion())))
        {
          // Objekt wird in zuknftiger Version nicht ungltig
          pruefePid(lauf, so, pid, status);
        }
      }
      for (SystemObject lauf : kb.getNewObjects())
      {
        pruefePid(lauf, so, pid, status);
      }
    }
    else
    {
      for (ConfigurationArea lauf : _hierarchie.getDataModel().getAllConfigurationAreas().values())
      {
        pruefePid(so, pid, lauf, status);
      }
    }
  }

  /**
   * @deprecated        
   */
  private void pruefePid(SystemObject gefundenesObjekt, SystemObject vorhandenesObjekt, String pid, MultiStatus status)
  {
    if (pid.equals(gefundenesObjekt.getPid()))
    {
      if (!vorhandenesObjekt.equals(gefundenesObjekt))
      {
        int severity = vorhandenesObjekt.getConfigurationArea().equals(gefundenesObjekt.getConfigurationArea()) ? Status.ERROR
            : Status.WARNING;

        status.add(new Status(severity, Activator.PLUGIN_ID, "PID " + pid
            + " wird bereits verwendet von Objekt vom Typ " + gefundenesObjekt.getType().getPid() + " im KB "
            + gefundenesObjekt.getConfigurationArea().getPid()));
      }
    }
  }

  private IStatus pruefeZuordnungAnzeige(HierarchieObjekt ho, SystemObject so, SystemObjektDaten soDaten)
  {
    Object vater = _hierarchie.getUebergeordnetesObjekt(ho);
    MultiStatus result = null;

//    if (vater instanceof HierarchieObjekt
//        && ((HierarchieObjekt)vater).getHierarchieObjektTyp().equals(HOT_AQ.AnzeigeQuerschnitt)
//        || ((HierarchieObjekt)vater).getHierarchieObjektTyp().equals(HOT_AQ.AnzeigeQuerschnittSub))
      if (vater instanceof HierarchieObjekt
          && ((HierarchieObjekt)vater).getHierarchieObjektTyp().equals(HOT_AQ.AnzeigeQuerschnitt))
    {
      SystemObject aq = ((HierarchieObjekt)vater).getSystemObjekt(0);
      NonMutableSet menge = ((ConfigurationObject)aq).getNonMutableSet("Anzeigen");
      if (menge != null)
      {
        if (menge.getElementsInModifiableVersion().contains(so))
        {
          ConfigDataModel dataModel = _hierarchie.getDataModel();

          for (ConfigurationArea kb : dataModel.getAllConfigurationAreas().values())
          {
            SystemObjectType[] typSpez = new SystemObjectType[] { dataModel.getType("typ.anzeigeQuerschnitt") };
            Collection<SystemObject> mqListe = new Vector<SystemObject>();
            mqListe.addAll(kb.getObjects(Arrays.asList(typSpez), ObjectTimeSpecification.valid()));

            for (SystemObject lauf : kb.getNewObjects())
            {
              if (lauf.isOfType("typ.anzeigeQuerschnitt"))
              {
                mqListe.add(lauf);
              }
            }

            for (SystemObject lauf : mqListe)
            {
              if (!lauf.equals(aq))
              {
                ConfigurationObject ko = (ConfigurationObject)lauf;

                if (!(ko.getNotValidSince() >= kb.getModifiableVersion()))
                // Objekt wird in zuknftiger Version nicht ungltig
                {
                  NonMutableSet set = ((ConfigurationObject)lauf).getNonMutableSet("Anzeigen");
                  if (set != null)
                  {
                    if (set.getElementsInModifiableVersion().contains(so))
                    {
                      if (result == null)
                      {
                        result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Die Anzeige " + so.getPid()
                            + " darf nicht mehreren AQ zugeordnet sein", null);
                      }
                      result.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Die Anzeige " + so.getPid()
                          + " ist zustzlich dem AQ " + lauf.getPid() + " zugeordnet"));
                    }
                  }
                }
              }
            }
          }
        }
      }
      else
      {
        _debug.warning("Menge Anzeigen am Objekt " + aq.getPid() + " nicht vorhanden");
        // throw new IllegalStateException("Menge Fahrstreifen am Objekt " + mq.getPid() + " nicht vorhanden");
      }
    }
    else
    {
      throw new IllegalStateException("bergeordnetes Objekt von " + ho + " vom Typ " + HierarchieObjekt.class
          + " vom HOT " + HOT_AQ.AnzeigeQuerschnitt + " erwartet");

    }

    if (result == null)
    {
      return new Status(IStatus.OK, Activator.PLUGIN_ID, "Prfen Zuordnung Anzeige zum AnzeigeQuerschnitt: OK");
    }
    else
    {
      return result;
    }
  }

  private IStatus pruefeZuordnungAnzeigeQuerschnitt(HierarchieObjekt ho, SystemObject so, SystemObjektDaten soDaten)
  {
    Object vater = _hierarchie.getUebergeordnetesObjekt(ho);
    MultiStatus result = null;

    if (vater instanceof HierarchieObjekt && ((HierarchieObjekt)vater).getHierarchieObjektTyp().equals(HOT_AQ.Dirigent))
    {
      SystemObject aq = ((HierarchieObjekt)vater).getSystemObjekt(0);
      NonMutableSet menge = ((ConfigurationObject)aq).getNonMutableSet("AnzeigeQuerschnitte");
      if (menge != null)
      {
        if (menge.getElementsInModifiableVersion().contains(so))
        {
          ConfigDataModel dataModel = _hierarchie.getDataModel();

          for (ConfigurationArea kb : dataModel.getAllConfigurationAreas().values())
          {
            SystemObjectType[] typSpez = new SystemObjectType[] { dataModel.getType("typ.dirigent") };
            Collection<SystemObject> mqListe = new Vector<SystemObject>();
            mqListe.addAll(kb.getObjects(Arrays.asList(typSpez), ObjectTimeSpecification.valid()));

            for (SystemObject lauf : kb.getNewObjects())
            {
              if (lauf.isOfType("typ.dirigent"))
              {
                mqListe.add(lauf);
              }
            }

            for (SystemObject lauf : mqListe)
            {
              if (!lauf.equals(aq))
              {
                ConfigurationObject ko = (ConfigurationObject)lauf;

                if (!(ko.getNotValidSince() >= kb.getModifiableVersion()))
                // Objekt wird in zuknftiger Version nicht ungltig
                {
                  NonMutableSet set = ((ConfigurationObject)lauf).getNonMutableSet("AnzeigeQuerschnitte");
                  if (set != null)
                  {
                    if (set.getElementsInModifiableVersion().contains(so))
                    {
                      if (result == null)
                      {
                        result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Der AQ " + so.getPid()
                            + " darf nicht mehreren Dirigenten zugeordnet sein", null);
                      }
                      result.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Der AQ " + so.getPid()
                          + " ist zustzlich dem Dirigent " + lauf.getPid() + " zugeordnet"));
                    }
                  }
                }
              }
            }
          }
        }
      }
      else
      {
        _debug.warning("Menge AnzeigeQuerschnitte am Objekt " + aq.getPid() + " nicht vorhanden");
        // throw new IllegalStateException("Menge Fahrstreifen am Objekt " + mq.getPid() + " nicht vorhanden");
      }
    }
    else
    {
      throw new IllegalStateException("bergeordnetes Objekt von " + ho + " vom Typ " + HierarchieObjekt.class
          + " vom HOT " + HOT_AQ.Dirigent + " erwartet");

    }

    if (result == null)
    {
      return new Status(IStatus.OK, Activator.PLUGIN_ID, "Prfen Zuordnung AnzeigeQuerschnitt zum Dirigent: OK");
    }
    else
    {
      return result;
    }
  }

  private IStatus pruefeZuordnungLigaturen(HierarchieObjekt ho, SystemObject so, SystemObjektDaten soDaten)
  {    
    Object vater = _hierarchie.getUebergeordnetesObjekt(ho);
    MultiStatus result = null;

    if (vater instanceof HierarchieObjekt
        && ((HierarchieObjekt)vater).getHierarchieObjektTyp().equals(HOT_AQ.Zeichensatz))
    {
      SystemObject vaterSo = ((HierarchieObjekt)vater).getSystemObjekt(0);
      NonMutableSet menge = ((ConfigurationObject)vaterSo).getNonMutableSet("Zeichen");
      if (menge != null)
      {
        List<SystemObject> listSoZeichenRef = menge.getElementsInModifiableVersion();

        Data atgDaten = soDaten.getAtgDaten("atg.ligaturen");
        SystemObject[] sosZeichen = atgDaten.getItem("Zeichen").asReferenceArray().getSystemObjectArray();
        List<SystemObject> listSoZeichen = Arrays.asList(sosZeichen);     
      
        if (listSoZeichen != null){
          for (SystemObject soZeichen : listSoZeichen)
          {          
            if (!listSoZeichenRef.contains(soZeichen))
            {                                  
              if (result == null)
              {
                result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Mind. ein Zeichen ist nicht in der Menge Zeichen des bergeordneten HO enthalten", null);
              }
              result.merge(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Das Zeichen " + soZeichen.getPid()
                  + " ist nicht dem bergeordneten HO " + vaterSo + " zugeordnet"));
              
            }             
          }        
        }        
      }
      else
      {
        _debug.warning("Menge Zeichen am Objekt " + vaterSo.getPid() + " nicht vorhanden");
        // throw new IllegalStateException("Menge Fahrstreifen am Objekt " + mq.getPid() + " nicht vorhanden");
      }
    }
    else
    {
      throw new IllegalStateException("bergeordnetes Objekt von " + ho + " vom Typ " + HierarchieObjekt.class
          + " vom HOT " + HOT_AQ.Zeichensatz + " erwartet");

    }
    
    if (result == null)
    {
      return new Status(IStatus.OK, Activator.PLUGIN_ID, "Prfen Zuordnung Ligaturen zu Zeichen: OK");
    }
    else
    {
      return result;
    }
  }

  private IStatus pruefeZuordnungWechseltext(HierarchieObjekt ho, SystemObject so, SystemObjektDaten soDaten)
  {
    Object vater = _hierarchie.getUebergeordnetesObjekt(ho);
    Object grossVater = _hierarchie.getUebergeordnetesObjekt((HierarchieObjekt)vater);

    MultiStatus result = null;

    if (grossVater instanceof HierarchieObjekt
        && ((HierarchieObjekt)grossVater).getHierarchieObjektTyp().equals(HOT_AQ.AnzeigeTyp))
    {
      List<HierarchieObjekt> uos1 = _hierarchie.getUntergeordneteObjekte(grossVater);

      List<SystemObject> listSoFont = new ArrayList<SystemObject>();
      List<SystemObject> listSoZeichen = new ArrayList<SystemObject>();

      for (HierarchieObjekt ho1 : uos1)
      {
        if (ho1.getHierarchieObjektTyp().equals(HOT_AQ.Zeichensatz))
        {
          SystemObject soFont = ho1.getSystemObjekt(0);
          listSoFont.add(soFont);

          List<HierarchieObjekt> uos2 = _hierarchie.getUntergeordneteObjekte(ho1);

          for (HierarchieObjekt ho2 : uos2)
          {
            if (ho2.getHierarchieObjektTyp().equals(HOT_AQ.Zeichen))
            {
              SystemObject soZeichen = ho2.getSystemObjekt(0);
              
              listSoZeichen.add(soZeichen);
            }
          }
        }
      }

      Data atgDaten = soDaten.getAtgDaten("atg.wechseltext");
      SystemObject soFontRef = atgDaten.getItem("Font").asReferenceValue().getSystemObject();
      SystemObject[] sosZeichenRef = atgDaten.getItem("Textdefinition").asReferenceArray().getSystemObjectArray();

      if (soFontRef != null){
        if (!listSoFont.contains(soFontRef))
        {                  
          if (result == null)
          {
            result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Ein Zeichensatz ist nicht in der Menge Zeichen des bergeordneten HO enthalten", null);
          }
          result.merge(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Der Font " + soFontRef.getPid()
              + " ist nicht dem bergeordneten HO " + (HierarchieObjekt)grossVater + " zugeordnet"));      
        }        
      }
      

      if (sosZeichenRef != null){
        for (SystemObject soZeichenRef : sosZeichenRef)
        {
          if (!listSoZeichen.contains(soZeichenRef))
          {
            if (result == null)
            {
              result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Ein Zeichen ist nicht in der Menge Zeichen des bergeordneten HO enthalten", null);
            }
            result.merge(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Das Zeichen " + soZeichenRef.getPid()
                + " ist nicht dem bergeordneten HO " + (HierarchieObjekt)grossVater + " zugeordnet"));
           
          }
        }        
      }    
    }
    else
    {
      throw new IllegalStateException("bergeordnetes Objekt von " + ho + " vom Typ " + HierarchieObjekt.class
          + " vom HOT " + HOT_AQ.AnzeigeTyp + " erwartet");

    }

    if (result == null)
    {
      return new Status(IStatus.OK, Activator.PLUGIN_ID, "Prfen Zuordnung Wechseltext zu Zeichensatz und Zeichen: OK");
    }
    else
    {
      return result;
    }
  }

  private IStatus pruefeZuordnungWvzInhalt(HierarchieObjekt ho, SystemObject so, SystemObjektDaten soDaten)
  {
    Object vater = _hierarchie.getUebergeordnetesObjekt(ho);

    MultiStatus result = null;

    if (vater instanceof HierarchieObjekt
        && ((HierarchieObjekt)vater).getHierarchieObjektTyp().equals(HOT_AQ.AnzeigeTyp))
    {
      List<HierarchieObjekt> uos1 = _hierarchie.getUntergeordneteObjekte(vater);

      List<SystemObject> listSoWechseltext = new ArrayList<SystemObject>();
      List<SystemObject> listSoWzgInhaltGrafik = new ArrayList<SystemObject>();

      for (HierarchieObjekt ho1 : uos1)
      {
        if (ho1.getHierarchieObjektTyp().equals(HOT_AQ.WvzInhalt))
        {

          List<HierarchieObjekt> uos2 = _hierarchie.getUntergeordneteObjekte(ho1);

          for (HierarchieObjekt ho2 : uos2)
          {
            if (ho2.getHierarchieObjektTyp().equals(HOT_AQ.WechselText))
            {
              SystemObject soWechseltext = ho2.getSystemObjekt(0);
              
              listSoWechseltext.add(soWechseltext);
            }
            else if (ho2.getHierarchieObjektTyp().equals(HOT_AQ.WzgInhaltGrafik))
            {
              SystemObject soWzgInhaltGrafik = ho2.getSystemObjekt(0);
              
              listSoWzgInhaltGrafik.add(soWzgInhaltGrafik);
            }
          }
           
        }
      }

      Data atgDaten = soDaten.getAtgDaten("atg.wvzInhalt");
      SystemObject soWechseltextRef = atgDaten.getItem("Wechseltext").asReferenceValue().getSystemObject();
      SystemObject[] sosGrafikDarstellungenRef = atgDaten.getItem("GrafikDarstellungen").asReferenceArray().getSystemObjectArray();

      if (soWechseltextRef != null){
        if (!listSoWechseltext.contains(soWechseltextRef))
        {                  
          if (result == null)
          {
            result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Ein Wechseltext ist nicht Bestandteil des bergeordneten HO", null);
          }
          result.merge(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Der Wechseltext " + soWechseltextRef.getPid()
              + " ist nicht dem bergeordneten HO " + (HierarchieObjekt)vater + " zugeordnet"));      
        }        
      }
      

      if (sosGrafikDarstellungenRef != null){
        for (SystemObject soGrafikDarstellungenRef : sosGrafikDarstellungenRef)
        {
          if (!listSoWzgInhaltGrafik.contains(soGrafikDarstellungenRef))
          {
            if (result == null)
            {
              result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Eine GrafikDarstellung ist nicht Bestandteil des bergeordneten HO", null);
            }
            result.merge(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Die WzgInhaltGrafik " + soGrafikDarstellungenRef.getPid()
                + " ist nicht dem bergeordneten HO " + (HierarchieObjekt)vater + " zugeordnet"));
           
          }
        }        
      }    
    }
    else
    {
      throw new IllegalStateException("bergeordnetes Objekt von " + ho + " vom Typ " + HierarchieObjekt.class
          + " vom HOT " + HOT_AQ.AnzeigeTyp + " erwartet");

    }

    if (result == null)
    {
      return new Status(IStatus.OK, Activator.PLUGIN_ID, "Prfen Zuordnung Wechseltext zu Zeichensatz und Zeichen: OK");
    }
    else
    {
      return result;
    }
  }
  
  private boolean hatGleicheInhalte(String[] inhalt)
  {
    for (int i = 0; i < inhalt.length - 1; i++)
    {
      for (int j = i + 1; j < inhalt.length; j++)
      {
        if (inhalt[i] == inhalt[j])
        {
          return true;
        }
      }
    }

    return false;
  }

}
