/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkaaxl;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.tkabasis.Activator;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HODaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IAttributInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.SystemObjektDaten;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.IPruefungen;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.PruefungenTools;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.StringAttributeType;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Prfungen von Attributwerten fr den TKA AXL Achslast
 * 
 * @author Krause
 *
 */
public class PruefungenAXL implements IPruefungen
{

	private IHierarchie _hierarchie;
	
	 /**
   * Bestimmt die Hierarchie, die fr die Prfungen benutzt werden kann
   * 
   * @return die Hierarchie, die fr die Prfungen benutzt werden kann
   */
	public IHierarchie getHierarchie()
	{
		return _hierarchie;
	}
  
	/**
   * Setzt die Hierarchie, die fr die Prfungen benutzt werden kann
   * @param hierachie die Hierarchie, die fr die Prfungen benutzt werden kann
   */
	public void setHierarchie(IHierarchie hierarchie)
	{
		_hierarchie = hierarchie;
	}
	
	public IStatus pruefeHierarchieObjekt(HierarchieObjekt ho, HODaten daten)
	{
		MultiStatus result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
				"Prfe Hierarchieobjekt " + ho, null);
//	  IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
		
		for (SystemObject so: ho.getSystemObjekte())
  	{
			 SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
			 if (soDaten != null)
			 {
				 MultiStatus objektStatus;
				 if (ho.getSystemObjekte().size() > 1)
				 {
					 objektStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Systemobjekt " + 
							 so.getPid() + " vom Typ " + so.getType().getPid(), null);
				 }
				 else
				 {
					 objektStatus = result;
				 }
				 
				 //objektStatus.add(pruefePid(so, soDaten.getPid()));
				 objektStatus.add(new PruefungenTools(getHierarchie().getDataModel()).pruefePid(soDaten.getPid(), so));
			 }	
  	}
		
		return result;
	}

  /**
   * Prft einen Attributwert auf Zulssigkeit
   * 
   * @param ho Hierarchieobjekt, dessen Daten geprft werden sollen
   * @param so Systemobjekt, gehrt zu ho
   * @param atgInfo Info ber das zu prfenden Attribut
   * @param daten Datensatz der Daten, die geprft werden sollen
   * @return eine Instanz vom Typ IStatus, die aussagt, ob es zulssig ist, die 
   * bergebenen Daten als Attributwert zu bernehmen, oder <code>null</code>, wenn
   * keine Prfung fr die bergebenen Parameter implementiert ist
   */
	public IStatus pruefeDaten(HierarchieObjekt ho, SystemObject so, AtgInfo atgInfo, Data daten)
  {
	  
    MultiStatus result = new MultiStatus( Activator.PLUGIN_ID, Status.OK, "Prfe Daten der ATG " + atgInfo.getAtg(),
        null );

    if ( so.isOfType( "typ.achsLastMessStelle" ) )
    {
      if ( "atg.achsLastMessStelle".equals( atgInfo.getAtg() )
          || "atg.punktKoordinaten".equals( atgInfo.getAtg() ))
      {
        pruefeDatenDefinition( atgInfo, daten, result );
      }
    }

    return result;
  }
	  
   
  private boolean pruefeDatenDefinition(IAttributInfo attInfo, Data daten, MultiStatus status)
  {
		Data item = daten.getItem(attInfo.getName());	
		
		if (!item.isDefined())
		{
			status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Wert ist nicht korrekt definiert"));  
			return false;
		}
		else
		{
			status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, "Prfen Daten-Definition: OK"));
		}
		
		// Plichtfeld-Prfung:
		if (item.isPlain())
		{	
			if (attInfo.isPflichtFeld())
			{	
				String wert = item.asTextValue().getText();
				if ("".equals(wert) || (!(item.getAttributeType() instanceof StringAttributeType) && 
						"undefiniert".equals(wert) || "_Undefiniert_".equals(wert)))
				{
					status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Wert fr Pflichtfeld nicht angegeben"));  
					return false;
				}
				else
				{
					status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, "Prfen Pflichtfeld-Eigenschaft: OK"));
				}
			}	
		}		
  	return true;
  }
  
  private void pruefeDatenDefinition(AtgInfo atgInfo, Data daten, MultiStatus status)
  {
  	for (String attribut: atgInfo.getAttribute())
  	{
  		MultiStatus	meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert fr Attribut " +
    			attribut, null);
  		IAttributInfo attInfo = atgInfo.getAttributInfo(attribut);
  		pruefeDatenDefinition(attInfo, daten,  meldung);
  		status.add(meldung);
  	}
  }

}
  