/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.aktionen;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.action.IAction;

/**
 * Registrierung fr Aktionen ({@link IAction})
 * 
 * @author Krause
 * 
 */
public class ActionRegistry
{
  /*
   * Hashmap, die die Aktionen beinhaltet
   */
  private Map<String, IAction> _actionRegistry = new HashMap<String, IAction>();

  /**
   * Liefert <code>null</code> oder <code>IAction</code> zum gegebenen Schlssel.
   * 
   * @param key
   *          die ID der Aktion
   * @return <code>null</code> oder die Aktion mit der passenden ID
   */
  public IAction getAction(Object key)
  {
    return (IAction)_actionRegistry.get(key);
  }

  /**
   * Liefert einen {@link Iterator} ber alle Aktionen.
   * 
   * @return Iterator ber alle Aktionen
   */
  public Iterator<IAction> getActions()
  {
    return _actionRegistry.values().iterator();
  }

  /**
   * Registrieren einer Aktion in diesem Register. Die Aktion muss eine ID haben.
   * 
   * @param action
   *          zu registrierende Aktion
   */
  public void registerAction(IAction action)
  {
    Assert.isNotNull(action.getId(), "Aktion muss eine ID haben in " + //$NON-NLS-1$
        getClass().getName() + " :registerAction(IAction)");//$NON-NLS-1$
    registerAction(action.getId(), action);
  }

  /**
   * Registieren einer Aktion in diesem Register mit der gegeben ID.
   * 
   * @param id
   *          ID, zu der die Aktion registriert werden soll
   * @param action
   *          zu registrierende Aktion
   */
  public void registerAction(String id, IAction action)
  {
    _actionRegistry.put(id, action);
  }

  /**
   * Lschen einer Aktion aus diesem Register. Die Aktion muss eine ID haben und mit dieser registriert sein.
   * 
   * @param action
   *          zu lschende Aktion
   */
  public void removeAction(IAction action)
  {
    _actionRegistry.remove(action.getId());
  }

  /**
   * Lschen einer Aktion aus diesem Register mit der gegeben ID.
   * 
   * @param id
   *          ID, zu der die Aktion gelscht werden soll
   */
  public void removeAction(String id)
  {
    _actionRegistry.remove(id);
  }

}
