/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.datenbereich.editor;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.datenbereich.datenbasis.DatenidentifikationGtmOffline;
import de.bsvrz.buv.plugin.tkabasis.datenbereich.editor.widgetChangedListener.WidgetChangedListener;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IAttributInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.TypInfo;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Attribute;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeListDefinition;
import de.bsvrz.dav.daf.main.config.AttributeSet;
import de.bsvrz.dav.daf.main.config.AttributeType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DoubleAttributeType;
import de.bsvrz.dav.daf.main.config.IntegerAttributeType;
import de.bsvrz.dav.daf.main.config.IntegerValueRange;
import de.bsvrz.dav.daf.main.config.IntegerValueState;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.ReferenceAttributeType;
import de.bsvrz.dav.daf.main.config.ReferenceType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectInfo;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.pat.sysbed.dataEditor.DataEditorPanel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;

/**
 * {@link Data}-Editor in SWT. Entspricht von der Funktionalitt her dem {@link DataEditorPanel}
 * mit virtueller Darstellung fr groe Arrays
 * 
 * @author Krause
 * 
 */
public class DataEditorComposite extends Composite
{
  /**
   * Flag zum Debuggen
   */
  protected static final boolean _debug = false;
  
  /**
   * Darstellung von SystemObjekten und ObjektTypen
   */
  public static final int _darstlungSystemObjekteObjektTypen = 1;

  /** ein Zahlenformat */
  private static final NumberFormat _integerNumberFormat = NumberFormat.getNumberInstance();

  /** Genauigkeit des Zahlenformats */
  private static final NumberFormat _precisionTestNumberFormat;

  /**
   * Hintergrundfarbe eines Attributwertes, der nicht definiert ist und nicht ber den Datenverteiler verschickt werden
   * kann.
   */
  private static final Color _backgroundUndefinedValue = new Color(PlatformUI.getWorkbench().getDisplay(), 255, 0, 0);

  /**
   * Hintergrundfarbe eines Attributwertes, der ber den Datenverteiler verschickt werden kann.
   */
  private static final Color _backgroundColorDefinedValue = new Color(PlatformUI.getWorkbench().getDisplay(), 255, 255,
      255);
  
  /**
   * Hintergrundfarbe, wenn "keine Daten" angezeigt wird.
   */
  private static final Color _backgroundNoDataValue = new Color(PlatformUI.getWorkbench().getDisplay(), 255, 255, 0);

  
  /**
   * String der ausgegeben wird, wenn der Attributwert "undefiniert" ist. Der String entspricht dabei nicht dem wahren
   * undefiniert Wert.
   */
  private static final String _undefinedString = "_Undefiniert_"; // @jve:decl-index=0:

  /**
   * String, der in Comboboxen angezeigt wird und anzeigt, dass der Default-Wert benutzt werden soll.
   */
  private static final String _defaultValueString = "Default-Wert";

  /**
   * Das Zahlenformat erhlt eine Formatierung.
   */
  static
  {
    _integerNumberFormat.setMinimumIntegerDigits(1);
    _integerNumberFormat.setMaximumIntegerDigits(999);
    _integerNumberFormat.setGroupingUsed(false);
    DecimalFormatSymbols symbols = new DecimalFormatSymbols();
    symbols.setDecimalSeparator(',');
    _precisionTestNumberFormat = new DecimalFormat("0.#", symbols);
    _precisionTestNumberFormat.setMaximumFractionDigits(999);
  }

  /** speichert die aktuellen Daten */
  private Data _data;

  /** gibt an, ob die dargestellten Felder generell editierbar sein sollen, je nach Bearbeitungsmodus */
  private boolean _editable = false;

  /** gibt an, ob die dargestellten Felder verndert wurden */
  private boolean _dirty = false;

  /**
   * Lokaler Verweis
   */
  private DataEditorComposite _this = null;

  /**
   * Speichert zu einem {@link ComboViewer} einen zugehrigen {@link ISelectionChangedListener}. Dieser Listener muss
   * vom {@link ComboViewer} detached werden, wenn die Aktion hinter dem Listener ausgefhrt wird, da diese u.U. eigene
   * Selektionen auf dem {@link ComboViewer} durchfhrt. Ist diese Aktion am Ende, attached sie den Listener wieder.
   */
  private HashMap<ComboViewer, ISelectionChangedListener> _mapComboListener = new HashMap<ComboViewer, ISelectionChangedListener>();

  /**
   * In diesem {@link ScrolledComposite} findet die Darstellung des {@link DataEditorComposite} statt und dieses
   * {@link ScrolledComposite} muss ber nderungen der Gre des {@link DataEditorComposite} informiert werden.
   */
  private Canvas _scrolledArea = null;

  private TabbedDatenBereichEditorComposite _tabbedComposite;

  private DataModel _dataModel;

  private DatenidentifikationGtmOffline _dia;

  /**
   * AtgInfo fr die Bestimmung der Pflichtfelder aus der Kapsel ermitteln
   */
  private AtgInfo _atgInfo;
  
  // minimale Anzahl neuer Array-Elemente fr die virtuelle Ansicht
  final static int _minAnzahlNeu = 5;

  // Scrollhhe fr die virtuelle Ansicht
  private int _scrollHoehe = 0;
  
  // nicht enthaltenes virtuelles Array-Element vor dem Anzeigebereich
  private VirtuellesElement _virtuellElementVor;

  //nicht enthaltenes virtuelles Array-Element nach dem Anzeigebereich
  private VirtuellesElement _virtuellElementNach;

  // direkte Spinnereingabe per Tastatur
  private boolean _spinnerEingabe = false;

  // vorheriger Spinnerwert
  private int _spinnerWert = 0;
  
  private AttributeGroup _tooltipAtg = null;
    
  
  public DataEditorComposite(Composite parent, int style, DataModel dataModel)
  {
    super(parent, style);

    if (parent instanceof Canvas)
    {
      this._scrolledArea = (Canvas)parent;
    }

    Composite tabbedParent = parent;
    while (!(tabbedParent instanceof TabbedDatenBereichEditorComposite))
    {
      tabbedParent = tabbedParent.getParent();
    }
    _tabbedComposite = (TabbedDatenBereichEditorComposite)tabbedParent;
    
    this._dataModel = dataModel;
    
    //AtgInfo fr die Bestimmung der Pflichtfelder aus der Kapsel ermitteln
    setAtgInfo();
    
    this._this = this;

    initialize();
  }

  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    setLayout(new GridLayout());
  }
  
  /**
   * Setzen des {@link Data}
   * 
   * @param data
   *          Datensatz zur Attributgruppe.
   */
  public void setData(Data data)
  {
    this._data = data;
  }
  
  /**
   * Gibt die Daten zurck, die aktuell angezeigt werden.
   * 
   * @return die aktuellen Daten
   */
  public Data getData()
  {
    return _data;
  }
  
  /**
   * Setzen des {@link Data} und erzeugen einer neuen Gruppe dazu ohne virtuelle Anzeige
   * Achtung: setVirtuell() im Parent auf false setzen
   * 
   * @param data
   *          Datensatz zur Attributgruppe.
   */
  public void setDataGroup(Data data)
  {
    setData(data);
    createGroup(0, 0, null, false);
  }

  /**
   * Setzen des {@link Data} und erzeugen einer neuen Gruppe dazu mit virtueller Anzeige
   * 
   * @param data
   *          Datensatz zur Attributgruppe.
   * @param x 
   *          X-Wert der oberen linken Ecke des Parent relativ zum Anzeigebereich.
   * @param y 
   *          Y-Wert der oberen linken Ecke des Parent relativ zum Anzeigebereich.
   * @param rect
   *          Rechteck des Anzeigebereichs.
   * @param virtuell  virtuelle Darstellung (true/false)   
   */
  protected void setDataGroup(Data data, int x, int y, Rectangle rect, boolean virtuell)
  {
    setData(data);
    createGroup(x, y, rect, virtuell);
  }

  /**
   * Erzeugen einer neuen Gruppe zum Data
   * 
   * @param x 
   *          X-Wert der oberen linken Ecke des Parent relativ zum Anzeigebereich.
   * @param y 
   *          Y-Wert der oberen linken Ecke des Parent relativ zum Anzeigebereich.
   * @param rect
   *          Rechteck des Anzeigebereichs.
   * @param virtuell  virtuelle Darstellung (true/false)     
   */
  protected void createGroup(int x, int y, Rectangle rect, boolean virtuell)
  {
    if (_debug)
      System.out.println("createGroup: virtuell: " + virtuell);
    Data data = getData();
    
    //AtgInfo fr die Bestimmung der Pflichtfelder aus der Kapsel ermitteln
    if (_atgInfo == null)
      setAtgInfo();
    
    Control[] children = _this.getChildren();
    if (children != null)
    {
      for (Control control : children)
      {
        if (control != null)
          control.dispose();
      }
    }

    _scrollHoehe = 0;
    
    if (data == null)
    {
      Text text = new Text(_this, SWT.BORDER);
      text.setBackground(_backgroundNoDataValue);
      text.setText("keine Daten");
      _this.layout();
    }
    else
    {
      children = _this.getChildren();
      for (Control child : children)
      {
        child.dispose();
      }
      _scrollHoehe = 0;
      _virtuellElementVor = null;
      _virtuellElementNach = null;  
      // ist nicht notwendig, besser ohne
//      _this.setVisible(false);
      
      createBox(_this, _data, x, y, rect, null, virtuell, 0, null, null);
      
      _this.layout();
//      _this.setVisible(true);
    }

    if (_scrolledArea != null)
    {
      _scrollHoehe += _this.computeSize(SWT.DEFAULT, SWT.DEFAULT).y;
      ((ScrolledDatenBereichEditorComposite)_scrolledArea.getParent()).resizeScrollBars(0, _scrollHoehe);

      if (_debug)
        System.out.println("createGroup:: _scrollHoehe: " + _scrollHoehe);

      _scrolledArea.layout();
    }  
    
  }

  /**
   * Erzeugt eine gescrollte Gruppe zur Darstellung des {@link Data} innerhalb des Anzeigebereichs.
   * 
   * @param diffY
   *          Differenz und Richtung des Scrolls (negativ = runter / positiv = rauf).
   * @param rect
   *          Rechteck des Anzeigebereichs.
   * @return  verschiebung
   *            virtuell vernderte Verschiebung (negativ = runter / positiv = rauf)     
   */
  protected int scrollEditor(int diffY, Rectangle rect)
  {
    int verschiebung = 0;

    if ( _debug )
      System.out.println( "DiffY: " + diffY + " _virtuellElementVor: " + _virtuellElementVor );

    if ( diffY < 0 && _virtuellElementNach != null )
    {
      // Scroll nach unten
      VirtuellesElement veSubData = null;

      // nchstes Data aus _virtuellElementNach holen
      veSubData = _virtuellElementNach;

      if ( veSubData == null || veSubData.getData() == null )
        return verschiebung;

      // Elternteil und dessen Kinder ermitteln
      Composite parent = veSubData.getParent();
      Data dataParent = null;
      if ( parent != null )
      {
        if ( parent.getData() instanceof Data )
          dataParent = (Data) parent.getData();
      }

      // Anzahl anzufgender Array-Elemente ermitteln mit folgenden Kriterien:
      // 1. minimal eine feste Zahl
      // 2. minimal so viele Elemente, wie fr das Auffllen der Differenz notwendig sind
      // 3. maximal so viele Elemente, wie noch vorhanden sind

      int anzahlNeu = _minAnzahlNeu;

      int arrayL = 0;
      if ( dataParent != null && dataParent.isArray() )
        arrayL = dataParent.asArray().getLength();

      verschiebung = anzahlNeu * veSubData.getVirtuelleHoehe();

      if ( verschiebung < -diffY )
      {
        // Scroll-Differenz grer als <anzahl> Elemente
        anzahlNeu = (int) Math.floor( -diffY / veSubData.getVirtuelleHoehe() );
        if (_debug)
          System.out.println("AnzahlNeu: " + anzahlNeu + " = diffY: " + diffY + " / VirtuelleHhe: " + veSubData.getVirtuelleHoehe() );
      }

      if ( (veSubData.getIndex() + anzahlNeu) >= arrayL )
      {
        // Anzahl vorhandener Elemente
        anzahlNeu = arrayL - veSubData.getIndex();
      }

      if ( _debug )
        System.out.println( "Anzahl Array-Elemente anfgen: " + anzahlNeu );
      verschiebung = anzahlNeu * veSubData.getVirtuelleHoehe();
      if ( _debug )
        System.out.println( "Verschiebung: " + verschiebung );

      // um die Anzahl entsprechend verschobenes Parent im DataEditorComposite
      // neu aufbauen
      addNextCompositesNeu( parent, anzahlNeu );

    }
    else if ( diffY > 0 && _virtuellElementVor != null )
    {
      // Scroll nach oben
      VirtuellesElement veSubData = null;

      // vorheriges Data aus _virtuellElementVor holen
      veSubData = _virtuellElementVor;

      if ( veSubData == null || veSubData.getData() == null )
      {
        if ( _debug )
          System.out.println( "Data: " + veSubData.getData() );
        return verschiebung;
      }

      // Elternteil und dessen Kinder ermitteln
      Composite parent = veSubData.getParent();

      // Anzahl vorheriger Array-Elemente ermitteln mit folgenden Kriterien:
      // 1. minimal eine feste Zahl
      // 2. minimal so viele Elemente, wie fr das Auffllen der Differenz notwendig sind
      // 3. maximal so viele Elemente, wie noch vorhanden sind

      int anzahlNeu = _minAnzahlNeu;
      verschiebung = -anzahlNeu * veSubData.getVirtuelleHoehe();

      if ( -verschiebung < diffY )
      {
        // Scroll-Differenz grer als <anzahl> Elemente
        anzahlNeu = (int) Math.floor( diffY / veSubData.getVirtuelleHoehe() );
        if ( _debug )
          System.out.println( "mehr Elemente: " + anzahlNeu );
      }

      if ( (veSubData.getIndex() + 1 - anzahlNeu) <= 0 )
      {
        // Anzahl vorhandener Elemente
        anzahlNeu = veSubData.getIndex() + 1;
        if ( _debug )
          System.out.println( "vorhandene Elemente: " + anzahlNeu );
      }

      if ( _debug )
        System.out.println( "Anzahl Array-Elemente voranstellen: " + anzahlNeu );
      verschiebung = -anzahlNeu * veSubData.getVirtuelleHoehe();
      if ( _debug )
        System.out.println( "Verschiebung: " + verschiebung );

      // um die Anzahl entsprechend verschobenes Parent im DataEditorComposite
      // neu aufbauen
      addPrevCompositesNeu( parent, anzahlNeu );

    }

    this.layout();
    // das Layout der _scrolledArea darf nicht gesetzt werden, weil sonst die
    // (x,y)-Koordinaten von this auf den Anfangswert gesetzt werden
    // _scrolledArea.layout();

    return verschiebung;

  }
  
  /**
   * Neuaufbau des um die Anzahl entsprechend nach unten verschobenen Elternteils
   * 
   * @param parent    Eltern-{@link Composite}
   * @param anzahl    Anzahl Kinder, um die das Elternteil nach unten verschoben werden soll
   */
  private void addNextCompositesNeu(Composite parent, int anzahl)
  {
    if (_debug)
      System.out.println("addNextCompositesNeu:");
    
    // Data des Elternteils
    Data data = (Data)parent.getData();
    
    if (!data.isArray() || anzahl == 0)
      return;
    
    Array dataArray = data.asArray();

    VirtuellesElement veSubDataVor = null;
    VirtuellesElement veSubDataNach = null;
    
    if (_virtuellElementNach != null)
      veSubDataNach = _virtuellElementNach;
    
    if (_virtuellElementVor != null)
      veSubDataVor = _virtuellElementVor;
    else if ( veSubDataNach != null )
      veSubDataVor = new VirtuellesElement(parent, null, veSubDataNach.getX(), veSubDataNach.getY(), 0, true,  veSubDataNach.getVirtuelleHoehe());
    
    if ( veSubDataNach == null || veSubDataVor == null )
      return;
    
    int index = 0;

    // Virtuelles Nach-Element neu setzen
    if ( veSubDataNach.getData() == null )
    {
      // also kein virtuelles Element
      index = dataArray.getLength();
    }
    else
    {
      // virtuelles Element bereits vorhanden
      index = veSubDataNach.getIndex() + anzahl;
      index = index > dataArray.getLength() ? dataArray.getLength() : index;
    }

    //keine nderung
    if (veSubDataNach.getIndex() == index)
      return;
    
    if (_debug)
      System.out.println("veSubDataNach: " + veSubDataNach.getIndex() + " ==> " + index);
    veSubDataNach.setIndex(index);
    if ( dataArray.getLength() > index )
      veSubDataNach.setData(dataArray.getItem(index));
    else
      veSubDataNach.setData(null);    
    
    // Virtuelles Vor-Element neu setzen
    if ( veSubDataVor.getData() == null )
    {
      // also kein virtuelles Element
      index = anzahl - 1;
    }
    else
    {
      // virtuelles Element bereits vorhanden
      index = veSubDataVor.getIndex() + anzahl;
      index = index >= dataArray.getLength() ? dataArray.getLength()-1 : index;
    }
    
    veSubDataVor.setIndex(index);
    if ( dataArray.getLength() > index )
      veSubDataVor.setData(dataArray.getItem(index));
    else
      veSubDataVor.setData(null);
    
    // ist nicht notwendig, besser ohne
    parent.setVisible( false );
    
    // Alle Elemente des Parent bis auf das Erste lschen
    Control[] parentChildren = parent.getChildren();
    List<Composite> listExep = new ArrayList<Composite>();
    listExep.add( (Composite) parentChildren[0] );

    leereCompositeMitAusnahme( parent, listExep );
    parent.layout();

    // Anzuzeigende Array-Elemente ermitteln
    int start = veSubDataVor.getIndex() + 1;
    int ende = veSubDataNach.getIndex();
    
    start = start<0 ? 0 : start;
    ende = ende>dataArray.getLength() ? dataArray.getLength() : ende;

    if (_debug)
      System.out.println("Array-Elemente von " + start + " bis " + ende + "(ausschlielich) anzeigen!");
    
    for (int i = start; i < ende; i++)
    {
      Data subData = (Data)dataArray.getItem(i);
      createBox(parent, subData, 0, 0, null, data.getName(), false, 0, null, null);
    }
    
    parent.layout();
    parent.setVisible( true );
    
  }

  /**
   * Neuaufbau des um die Anzahl entsprechend nach oben verschobenen Elternteils
   * 
   * @param parent    Eltern-{@link Composite}
   * @param anzahl    Anzahl Kinder, um die das Elternteil nach unten verschoben werden soll
   */
  private void addPrevCompositesNeu(Composite parent, int anzahl)
  {
    if (_debug)
      System.out.println("addPrevCompositesNeu:");
    
    // Data des Elternteils
    Data data = (Data)parent.getData();
    
    if (!data.isArray() || anzahl == 0)
      return;
    
    Array dataArray = data.asArray();

    VirtuellesElement veSubDataVor = null;
    VirtuellesElement veSubDataNach = null;
    
    if (_virtuellElementNach != null)
      veSubDataNach = _virtuellElementNach;
    
    if (_virtuellElementVor != null)
      veSubDataVor = _virtuellElementVor;
    else if ( veSubDataNach != null )
      veSubDataVor = new VirtuellesElement(parent, null, veSubDataNach.getX(), veSubDataNach.getY(), 0, true,  veSubDataNach.getVirtuelleHoehe());
    
    if ( veSubDataNach == null || veSubDataVor == null )
      return;
    
    int index = 0;
    
    // Virtuelles Vor-Element neu setzen
    if ( veSubDataVor.getData() == null )
    {
      // also kein virtuelles Element
      index = -1;
    }
    else
    {
      // virtuelles Element bereits vorhanden
      index = veSubDataVor.getIndex() - anzahl;
      index = index < 0 ? -1 : index;
    }
    
    //keine nderung
    if (veSubDataVor.getIndex() == index)
      return;
    
    if (_debug)
      System.out.println("veSubDataVor: " + veSubDataVor.getIndex() + " ==> " + index);
    veSubDataVor.setIndex(index);
    if ( index >= 0 )
      veSubDataVor.setData(dataArray.getItem(index));
    else
      veSubDataVor.setData(null);

    // Virtuelles Nach-Element neu setzen
    if ( veSubDataNach.getData() == null )
    {
      // also kein virtuelles Element
      index = dataArray.getLength() - anzahl;
    }
    else
    {
      // virtuelles Element bereits vorhanden
      index = veSubDataNach.getIndex() - anzahl;
      index = index < 0 ? -1 : index;
    }

    if (_debug)
      System.out.println("veSubDataNach: " + veSubDataNach.getIndex() + " ==> " + index);
    veSubDataNach.setIndex(index);
    if ( index >= 0 )
      veSubDataNach.setData(dataArray.getItem(index));
    else
      veSubDataNach.setData(null);    
    
    // ist nicht notwendig, besser ohne
    parent.setVisible( false );
    
    // Alle Elemente des Parent bis auf das Erste lschen
    Control[] parentChildren = parent.getChildren();
    List<Composite> listExep = new ArrayList<Composite>();
    listExep.add( (Composite) parentChildren[0] );

    leereCompositeMitAusnahme( parent, listExep );
    parent.layout();

    // Anzuzeigende Array-Elemente ermitteln
    int start = veSubDataVor.getIndex() + 1;
    int ende = veSubDataNach.getIndex();
    
    start = start<0 ? 0 : start;
    ende = ende<0 ? 0 : ende;

    if (_debug)
      System.out.println("Array-Elemente von " + start + " bis " + ende + "(ausschlielich) anzeigen!");
    
    for (int i = start; i < ende; i++)
    {
      Data subData = (Data)dataArray.getItem(i);
      createBox(parent, subData, 0, 0, null, data.getName(), false, 0, null, null);
    }
    
    parent.layout();
    parent.setVisible( true );
    
  }
   
  /**
   * Komponenten sind editierbar
   * 
   * @return gibt an, ob die angezeigten Komponenten editierbar sein sollen
   */
  protected boolean isEditable()
  {
    return _editable;
  }

  /**
   * Hierber kann bestimmt werden, ob die angezeigten Textfelder, etc. editierbar sind, oder nicht.
   * 
   * @param editable
   *          gibt an, ob die angezeigten Komponenten editierbar sein sollen
   */
  protected void setEditable(boolean editable)
  {
    if (editable != _editable)
    {
      _editable = editable;
      refreshCompositeRecursive(_this);
    }

  }

  protected void refreshCompositeRecursive(Control theControl)
  {
   
    if (theControl instanceof Text || theControl instanceof Combo || theControl instanceof Spinner
        || theControl instanceof Button)
    {
      
      boolean aenderbar = _editable;
      Object dataEnable = ((Widget)theControl).getData("Aenderbar");
      if (dataEnable != null && dataEnable instanceof Boolean)
        aenderbar &= (Boolean)dataEnable;
            
      if (theControl instanceof Text)
      {
        Text text = (Text)theControl;
        text.setEditable(aenderbar);
        
        if (!_backgroundUndefinedValue.equals(theControl.getBackground()) && 
            !_backgroundNoDataValue.equals(theControl.getBackground()))
        {
          if (!aenderbar)                   
            text.setBackground(theControl.getParent().getBackground());
          else{
            text.setBackground(_backgroundColorDefinedValue);
          }          
        }        
        
//        System.out.println("dataEnable zu " + text.getText() + " : " + dataEnable);
      }
      else if (theControl instanceof Combo)
      {
        Combo c = (Combo)theControl;
        c.setEnabled(aenderbar);
      }
      else if (theControl instanceof CCombo)
      {
        CCombo c = (CCombo)theControl;
        c.setEnabled(aenderbar);
      }
      else if (theControl instanceof Spinner)
      {
        Spinner s = (Spinner)theControl;
        s.setEnabled(aenderbar);
      }
      else if (theControl instanceof Button)
      {
        Button b = (Button)theControl;
        b.setEnabled(aenderbar);
      }
    }

    // Alle mglichen Composites
    if (theControl instanceof Composite || theControl instanceof Group || theControl instanceof DataEditorComposite)
    {
      Composite composite = (Composite)theControl;

      if (!composite.isDisposed())
      {
        Control[] children = composite.getChildren();

        for (int i = 0; i < children.length; i++)
        {
          Control control = children[i];
          refreshCompositeRecursive(control);
        }
      }
    }
    else
    {
      if (_debug)
        System.out.println("else: " + theControl.getClass().getName());
    }
  }

  private String formatTooltip(String tooltip, int len)
  {
    StringBuffer buffer = new StringBuffer(); 
    
    int lenTotal = 0;
    
    // Ersetze berflssige Leerzeichen und HTML-Tags
    for (String text: tooltip.replaceAll("[ \\f\\t\\r\\n\\v]+", " ").replaceAll("(<[^>]*>)", "").trim().split(" "))
    {
      // Umbruch bei letztem Leerzeichen <= len
      if (lenTotal > 0)
      {
        if (lenTotal <= len)
        {  
          buffer.append(" ");
          lenTotal += 1;
        }
        else
        {
          buffer.append("\n");
          lenTotal = 0;
        }
      }  
          
      buffer.append(text);
      lenTotal += text.length();
    }
    
    return buffer.toString();
  }  
  
  /**
   * Erzeugt eine neue Gruppe zur Darstellung eines {@link Data}. Ruft sich selbstttig rekursiv auf.
   * 
   * @param parent
   *          Eltern-{@link Composite}, auf dessen Basis die Darstellung erfolgen soll.
   * @param data
   *          {@link Data} zur Darstellung in der neuen Gruppe.
   * @param x 
   *          X-Wert der oberen linken Ecke des Parent zum Elternteil relativ zum Anzeigebereich; nur bei virtuell relevant.
   * @param y 
   *          Y-Wert der oberen linken Ecke des Parent zum Elternteil relativ zum Anzeigebereich; nur bei virtuell relevant.
   * @param rect
   *          Rechteck des Anzeigebereichs; nur bei virtuell relevant.
   * @param isArrayElement  {@link Data} ist ein ArrayElement (true/false); nur bei virtuell relevant.
   * @param virtuell        virtuelle Darstellung (true/false).
   * @param hoehe           Hhe des Darstellungsobjektes zum {@link Data}, falls bekannt; nur bei virtuell relevant.
   * @return virtuelle Hhe des Darstellungsobjektes zum {@link Data}, falls keine Anzeige, ansonsten 0.         
   */
  private int createBox(Composite parent, final Data data, int x, int y, Rectangle rect, String parentArray, boolean virtuell, int hoehe, 
      Boolean attAenderbarKopie, AttributeSet parentAttributeSet)
  {
//    Boolean attAenderbarKopie = attAenderbar != null ? new Boolean(attAenderbar) : null;
    
    if (_debug)
      System.out.println("\ncreateBox: zu " + data.getName());
    
    boolean isVisible = true;
    boolean isSpinnerActive = true;
    int virtuelleHoehe = 0;
       
    if (data.isPlain())
    {
      Composite composite = new Composite(parent, SWT.NONE);
      GridData gdComposite = new GridData();
      gdComposite.grabExcessHorizontalSpace = true;
      gdComposite.horizontalAlignment = GridData.FILL;
      composite.setLayoutData(gdComposite);

      /*
       * Name der Box setzen
       */
      Label labelNameBox = new Label(composite, SWT.NONE);
      // labelNameBox.setText(data.getName() + ": "); // Doppelpunkt wurde wegen der einheitlichen Darstellung
      // eliminiert
      String attName = data.getName();
      labelNameBox.setText(attName);
      
      
      AttributeSet attributeSet = parentAttributeSet;
      if (attributeSet == null)
      {
        attributeSet = _tooltipAtg;
      }
      
      Attribute attribute = null;
      
      if (attributeSet != null)
      {
        if (parentArray != null) // einfaches Array, da isPlain() == true und Parent is ein Array
        {
          attribute = attributeSet.getAttribute(parentArray);
        }
        else
        {  
          attribute = attributeSet.getAttribute(attName);
        }  
      }
     
      SystemObjectInfo info = null;
      
      if (attribute != null)
      {
        info = attribute.getInfo();
      }
      
      if (_debug && attributeSet != null)
        System.out.println(attributeSet.getPid() + ": Feldname " + attName + " Attribut " +
            (attribute != null ? attribute.getId() : null) +
            "\n" + info);
      
      if (info == null || ((info.getShortInfo() == null || info.getShortInfo().isEmpty()) && 
          (info.getDescription() == null || info.getShortInfo().isEmpty())))
      {
        info = data.getAttributeType().getInfo();
      }
      
      String tooltip = "";
      
      if (info != null)
      {
        tooltip += formatTooltip(info.getShortInfo(), 120);
        String text = formatTooltip(info.getDescription(), 120);
        if (!tooltip.isEmpty() && !text.isEmpty())
        {
          tooltip += "\n";
        }
        tooltip += text;
      }
      
      // Flag, dass eine Aenderung des Attributs nach HOT erlaubt ist
      if (attAenderbarKopie == null)
        attAenderbarKopie = true;
      
      if (attName != null && _atgInfo != null)
      {
        IAttributInfo attInfo = _atgInfo.getAttributInfoTiefe(attName);
        
        if (attInfo != null)
        {
          if (attInfo.isPflichtFeld())
            labelNameBox.setText(attName + "*");
          
          if (attInfo != null){
            attAenderbarKopie = attInfo.isAenderbarNeu() && attInfo.isAenderbarBearb();
//            System.out.println("-----> Aenderbar: " + attAenderbarKopie);
          }           
          
          }else{
          if (parentArray != null)
          {
            attInfo = _atgInfo.getAttributInfoTiefe(parentArray);
            if (attInfo != null && !attInfo.hasInnereAttribute())
            {
              attAenderbarKopie = attInfo.isAenderbarNeu() && attInfo.isAenderbarBearb();
//              System.out.println("-----> Aenderbar: " + attAenderbarKopie);
            }
          }
          
//          System.out.println("attInfo ist null fr Name: " + attName);
//          
//          Map<String, IAttributInfo> attInfos = _atgInfo.getAttributInfos();
//          
//          for (Map.Entry<String, IAttributInfo> me :  attInfos.entrySet())
//          {                        
//            if (me.getValue().isArray())
//            {
//              ArrayInfo arrayInfo = (ArrayInfo)me.getValue();
//              
//              if (arrayInfo.isPflichtFeld())
//                labelNameBox.setText(attName + "*");
//              
//              if (arrayInfo != null){
//                attAenderbarKopie = arrayInfo.isAenderbarNeu() && arrayInfo.isAenderbarBearb();
//                System.out.println("-----> Aenderbar2: " + attAenderbarKopie);
//                isSpinnerActive = false;                
//              }
//              else{
//                System.out.println("arrayInfo ist null!!!");            
//              }              
//            }                         
//          }
        }
      }
      else if (attName != null && attName.equals( "verwaltung" ))
      {
        labelNameBox.setText(attName + "*");        
      }

      // Flag fr den momentanten Aenderungszustand des Attributs
      boolean attEnable = attAenderbarKopie && _editable;
      
//      System.out.println("---------> Attribut -> " + attName + " : " + attAenderbarKopie + " - " + attEnable + " # " + _editable);
      
      if (data.getAttributeType() instanceof ReferenceAttributeType)
      {
        // Referenzen (Textfeld Label Button)
        ReferenceAttributeType att = (ReferenceAttributeType)data.getAttributeType();
        SystemObjectType objectType = att.getReferencedObjectType();
        final List<SystemObjectType> types = new LinkedList<SystemObjectType>();
        final boolean flagUndefiniert = att.isUndefinedAllowed() ? true : false;            
//        attAenderbarKopie = flagUndefiniert ? attAenderbarKopie : true;
//        System.out.println("Nach flagUndefiniert: attEnable zu " + attName + " : " + attEnable);
         
        if (objectType == null)
        {
          // alle Typen anzeigen
          objectType = _dataModel.getType("typ.konfigurationsObjekt");
          // if (!objectType.isBaseType() || objectType.getElements().size() > 0)
          types.add(objectType);
          List<SystemObjectType> subTypen1 = ermittelnSubTypen(objectType);
          if (subTypen1 != null)
            types.addAll(subTypen1);

          objectType = _dataModel.getType("typ.dynamischesObjekt");
          // if (!objectType.isBaseType() || objectType.getElements().size() > 0)
          types.add(objectType);
          List<SystemObjectType> subTypen2 = ermittelnSubTypen(objectType);
          if (subTypen2 != null)
            types.addAll(subTypen2);
        }
        else
        {
          // if (!objectType.isBaseType() || objectType.getElements().size() > 0)
          types.add(objectType);
          List<SystemObjectType> subTypen = ermittelnSubTypen(objectType);
          if (subTypen != null)
            types.addAll(subTypen);
        }

        /*
         * Infotext zur Referenz
         */
        final Text textBox = new Text(composite, SWT.BORDER);
//        textBox.setEditable(attEnable);
//        textBox.setData("Aenderbar", attAenderbarKopie);
        
        boolean editableRefBox = att.isUndefinedAllowed() && 
          att.getReferenceType() == ReferenceType.ASSOCIATION;
        textBox.setEditable(attEnable && editableRefBox);
        textBox.setData("Aenderbar", attAenderbarKopie && editableRefBox);

        final Label suffixBox = new Label(composite, SWT.NONE);
        if (data.asTextValue() != null)
          suffixBox.setText(data.asTextValue().getSuffixText());
        
        if (editableRefBox)
        {  
          textBox.addFocusListener(new FocusListener() {
            @Override
            public void focusGained(FocusEvent e)
            {
            }

            @Override
            public void focusLost(FocusEvent e)
            {
              if (_editable)
              {  
                refTextBoxAttributeModifiedPruefung(textBox, suffixBox, data);
              }  
            }
          });

          textBox.addListener(SWT.DefaultSelection, new Listener() {
            public void handleEvent(Event e)
            {
              if (_editable)
              {
                refTextBoxAttributeModifiedPruefung(textBox, suffixBox, data);
//                WidgetChangedListener.getInstanz()
//                  .notifyWidgetChangedListener(e.widget, _tabbedComposite, null, null);
              }
            }
          });

          textBox.addListener(SWT.Modify, new Listener() {
            public void handleEvent(Event e)
            {
              if (_editable)
              {
//                textBox.setBackground(_backgroundColorDefinedValue);
                suffixBox.setText("");
                suffixBox.getParent().layout();
                textBox.setData("Geaendert", true);
                WidgetChangedListener.getInstanz()
                  .notifyWidgetChangedListener(e.widget, _tabbedComposite, null, null);
              }
            }
          });
        }
        
        textBox.setToolTipText(tooltip);

        if (data.isDefined())
        {
//          textBox.setText(data.asTextValue().getValueText());
          String wert = data.asReferenceValue().getSystemObjectPid();
          if (wert == null || wert.isEmpty())
          {
            wert =_undefinedString;
          }
          textBox.setText(wert);
          textBox.setData("Geaendert", false);
          
          if (_editable)
            textBox.setBackground(_backgroundColorDefinedValue);
          
//          if (textBox.getEditable())
//            textBox.setBackground(_backgroundColorDefinedValue);
        }
        else
        {
          textBox.setText(_undefinedString);
          textBox.setBackground(_backgroundUndefinedValue);
        }

        Button buttonRefAend = new Button(composite, SWT.PUSH);
        buttonRefAend.setText("Referenz ndern");
        buttonRefAend.setEnabled(attEnable);
        buttonRefAend.setData("Aenderbar", attAenderbarKopie);
        
        buttonRefAend.addSelectionListener(new SelectionAdapter() {

          public void widgetSelected(org.eclipse.swt.events.SelectionEvent e)
          {
            _dia = new DatenidentifikationGtmOffline(PlatformUI.getWorkbench().getDisplay().getActiveShell(), null,
                _dataModel);
            
            
            LinkedList<SystemObject> sos = _dia.oeffnen(types, _darstlungSystemObjekteObjektTypen, flagUndefiniert);

            if (sos.size() > 0)
            {
              data.asReferenceValue().setSystemObject(sos.getFirst());
              if (data.isDefined())
              {
                textBox.setText(data.asTextValue().getValueText());
                suffixBox.setText(data.asTextValue().getSuffixText());
                suffixBox.getParent().layout();
                
                boolean aenderbar = _editable;
                Object dataEnable = textBox.getData("Aenderbar");
                if (dataEnable != null && dataEnable instanceof Boolean)
                  aenderbar &= (Boolean)dataEnable;
                
                if (aenderbar)
                {  
                  textBox.setBackground(_backgroundColorDefinedValue);
                }  
                else
                {  
                  textBox.setBackground(textBox.getParent().getBackground());
                }  
              }
              else
              {
                textBox.setText(_undefinedString);
                textBox.setBackground(_backgroundUndefinedValue);
              }
            }

            WidgetChangedListener.getInstanz()
                .notifyWidgetChangedListener(e.widget, _tabbedComposite, null, null);

          }
        });

        /*
         * Layout
         */
        GridLayout gridLayout = new GridLayout(4, false);
        composite.setLayout(gridLayout);

        // TextBox
        GridData gdTextBox = new GridData();
        gdTextBox.grabExcessHorizontalSpace = true;
        gdTextBox.horizontalAlignment = GridData.FILL;
        textBox.setLayoutData(gdTextBox);
      }
      else if (data.getAttributeType() instanceof IntegerAttributeType)
      {
        // ComboBox
        IntegerAttributeType att = (IntegerAttributeType)data.getAttributeType();

        final Label suffixBox = new Label(composite, SWT.NONE);

        final IntegerValueRange range = att.getRange();
        if (range != null)
        {
          String suffixText = range.getUnit();
          if (suffixText != null)
          {
            suffixText = suffixText.trim();
            if (!suffixText.equals(""))
              suffixBox.setText("[" + suffixText + "] ");
          }
        }

        Combo comboBox = null;

        if (range != null)
          comboBox = new Combo(composite, SWT.BORDER | SWT.NONE);
        else
          comboBox = new Combo(composite, SWT.BORDER | SWT.READ_ONLY);

        comboBox.setEnabled(attEnable);
        comboBox.setData("Aenderbar", attAenderbarKopie);
        
        comboBox.setToolTipText(tooltip);

        final ComboViewer cvComboBox = new ComboViewer(comboBox);

        // final Label suffixBox = new Label(composite, SWT.NONE);
        // if (data.asTextValue() != null )
        // suffixBox.setText(data.asTextValue().getSuffixText());
        // else if (att.getRange() != null && att.getRange().getUnit() != null)
        // suffixBox.setText(att.getRange().getUnit());

        List<IntegerValueState> states = att.getStates();
        for (Iterator<IntegerValueState> statesIterator = states.iterator(); statesIterator.hasNext();)
        {
          IntegerValueState state = (IntegerValueState)statesIterator.next();
          cvComboBox.add(state.getName());
        }

        if (range != null)
        {
          final long unscaledMinimum = range.getMinimum();
          final long unscaledMaximum = range.getMaximum();
          final double conversionFactor = range.getConversionFactor();

          cvComboBox.add(getScaledValueText(unscaledMinimum, conversionFactor));
          cvComboBox.add(getScaledValueText(unscaledMaximum, conversionFactor));
        }

        // Wert, der gesetzt werden muss
        if (data.isDefined())
        {
          comboBox.setText(data.asTextValue().getValueText());
          if (_editable)
            comboBox.setBackground(_backgroundColorDefinedValue);
          
          // Der Wert des Attributes ist definiert
          StructuredSelection ss = new StructuredSelection(data.asTextValue().getValueText());
          cvComboBox.setSelection(ss, true);
        }
        else
        {
          comboBox.setBackground(_backgroundUndefinedValue);
          cvComboBox.setSelection(new StructuredSelection(_undefinedString));
        }

        // Soll in diesem Projekt nicht auswhlbar sein
        // cvComboBox.add(_defaultValueString);

        ISelectionChangedListener cvListener = new ISelectionChangedListener() {
          @Override
          public void selectionChanged(SelectionChangedEvent event)
          {
            comboBoxAttributeModifiziertWertPruefung(cvComboBox, suffixBox, data);
          }
        };

        cvComboBox.addSelectionChangedListener(cvListener);

        // Neu 20101112
        comboBox.addModifyListener(new ModifyListener() {
          public void modifyText(ModifyEvent e)
          {
            cvComboBox.removeSelectionChangedListener(_mapComboListener.get(cvComboBox));
            String text = null;

            ISelection selection = cvComboBox.getSelection();
            if (selection instanceof StructuredSelection)
            {
              StructuredSelection ss = (StructuredSelection)selection;

              if (ss.getFirstElement() != null)
              {
                text = ss.getFirstElement().toString();
              }
            }
            
            if (text == null)
            {
              text = cvComboBox.getCombo().getText();
            }

            try
            {
              if (text.equals(_defaultValueString) || text.equals(_undefinedString) || text.equals("undefiniert"))
              {
                data.setToDefault();
              }
              else
              {
                data.asTextValue().setText(text);
              }

              if (!data.isDefined())
              {
                if (_editable)
                  cvComboBox.getCombo().setBackground(_backgroundUndefinedValue);
//                StructuredSelection ss = new StructuredSelection(_undefinedString);
//                cvComboBox.setSelection(ss, true);
              }
              else
              {
//                StructuredSelection ss = new StructuredSelection(data.asTextValue().getValueText());
//                cvComboBox.setSelection(ss, true);
                if (_editable)
                  cvComboBox.getCombo().setBackground(_backgroundColorDefinedValue);
              }
            }
            catch (Exception ex)
            {
              data.setToDefault();
              cvComboBox.getCombo().setBackground(_backgroundUndefinedValue);
            }

            cvComboBox.addSelectionChangedListener(_mapComboListener.get(cvComboBox));
            
            WidgetChangedListener.getInstanz().notifyWidgetChangedListener(e.widget, _tabbedComposite, null, null);
          }
        });
        
//        comboBox.addModifyListener(new ModifyListener() {
//          @Override
//          public void modifyText(ModifyEvent e)
//          {
//            if (e.getSource() instanceof Combo)
//            {
//              Combo sourceCombo = (Combo)e.getSource();
//              String text = sourceCombo.getText().trim();
//              boolean wertGesetzt = false;
//              
//              if (text != null && !text.isEmpty())
//              {  
//                if (range != null)
//                {
//                  double dMin = range.getMinimum() * range.getConversionFactor();
//                  double dMax = range.getMaximum() * range.getConversionFactor();
//                
//                  NumberFormat nf = NumberFormat.getNumberInstance();
//                  ParsePosition pp = new ParsePosition(0);
//                  
//                  try
//                  {
//                    double wert = nf.parse(text, pp).doubleValue();
//                    
//                    System.out.println("ERRORINDEX: " + pp.getErrorIndex());
//                    System.out.println("INDEX: " + pp.getIndex());
//                    
//                    if (pp.getErrorIndex() < 0 && pp.getIndex() == text.length())
//                    {
//                      // Kein Fehler beim Parsen und ganzer Text geparst
//                      if (dMin <= wert && dMax >= wert)
//                      {
//                        data.asTextValue().setText(text);
//                      }
//                      else
//                      {
//                        data.setToDefault();
//                      }
//                      wertGesetzt = true;
//                    }
//                    System.out.println("WERT GESETZT: " + wertGesetzt);
//                  }
//                  catch (Exception ex)
//                  {
//                  }
//                }
//              }
//              
//              if (!wertGesetzt)
//              {  
//                try
//                {
//                  data.asTextValue().setText(text);
//                }
//                catch (Exception ex)
//                {
//                  data.setToDefault();
//                }  
//              }  
//              
//              if (data.isDefined())
//              {
//                cvComboBox.getCombo().setBackground(_backgroundColorDefinedValue);
//              }
//              else
//              {
//                cvComboBox.getCombo().setBackground(_backgroundUndefinedValue);
//              }
//
//              WidgetChangedListener.getInstanz().notifyWidgetChangedListener(e.widget, _tabbedComposite, null, null);
//
//            }
//          }
//        });
        
//        comboBox.addModifyListener(new ModifyListener() {
//          @Override
//          public void modifyText(ModifyEvent e)
//          {
//            if (e.getSource() instanceof Combo)
//            {
//              Combo sourceCombo = (Combo)e.getSource();
//              String text = sourceCombo.getText();
//
//              int itemCount = cvComboBox.getCombo().getItemCount();
//
//              if (text.length() > 0 && itemCount > 1)
//              {
//                if (_editable)
//                  cvComboBox.getCombo().setBackground(_backgroundColorDefinedValue);
//
//                String itemMax = (String)cvComboBox.getElementAt(itemCount - 1);
//                String itemMin = (String)cvComboBox.getElementAt(itemCount - 2);
//
//                try
//                {
//                  if (itemMax.contains(",") || itemMax.contains("."))
//                  {
//                    // nur Double-Werte sind gltig
//                    String dMax = itemMax.replace(',', '.');
//                    Double max = new Double(dMax);
//                    String dMin = itemMin.replace(',', '.');
//                    Double min = new Double(dMin);
//                    String dtext = text.replace(',', '.');
//                    Double wert = new Double(dtext);
//                    if (wert <= max && wert >= min)
//                      data.asTextValue().setText(wert.toString());
//                    else
//                    {
//                      data.setToDefault();
//                      cvComboBox.getCombo().setBackground(_backgroundUndefinedValue);
//                    }
//
//                    // berflssig, weil die Einheit beim Erstellen der SuffixBox aus dem DataModell geholt wird
//                    // if (data.isDefined())
//                    // {
//                    // String suffixText = null;
//                    // if (!suffixBox.getText().contains("["))
//                    // suffixText = data.asTextValue().getSuffixText();
//                    //                      
//                    // if ( suffixText != null )
//                    // {
//                    // suffixText = suffixText.trim();
//                    // if (!suffixText.equals(""))
//                    // {
//                    // suffixBox.setText(" [" + suffixText + "] " );
//                    // suffixBox.getParent().layout();
//                    // }
//                    // }
//                    // }
//
//                  }
//                  else
//                  {
//                    // nur Long-Werte sind gltig
//                    Long max = new Long(itemMax);
//                    Long min = new Long(itemMin);
//                    Long wert = new Long(text);
//                    if (wert <= max && wert >= min)
//                      data.asTextValue().setText(wert.toString());
//                    else
//                    {
//                      data.setToDefault();
//                      cvComboBox.getCombo().setBackground(_backgroundUndefinedValue);
//                    }
//
//                    // berflssig, weil die Einheit beim Erstellen der SuffixBox aus dem DataModell geholt wird
//                    // if (data.isDefined())
//                    // {
//                    // String suffixText = null;
//                    // if (!suffixBox.getText().contains("["))
//                    // suffixText = data.asTextValue().getSuffixText();
//                    //                      
//                    // if ( suffixText != null )
//                    // {
//                    // suffixText = suffixText.trim();
//                    // if (!suffixText.equals(""))
//                    // {
//                    // suffixBox.setText(" [" + suffixText + "] " );
//                    // suffixBox.getParent().layout();
//                    // }
//                    // }
//                    // }
//
//                  }
//
//                }
//                catch (Exception ex)
//                {
//                  data.setToDefault();
//                  cvComboBox.getCombo().setBackground(_backgroundUndefinedValue);
//                }
//              }
//              else
//              {
//                // keine Eingabe oder Auswahl leer
//                data.setToDefault();
//                cvComboBox.getCombo().setBackground(_backgroundUndefinedValue);
//              }
//
//              WidgetChangedListener.getInstanz().notifyWidgetChangedListener(e.widget, _tabbedComposite, null, null);
//
//            }
//          }
//        });


        _mapComboListener.put(cvComboBox, cvListener);

        /*
         * Layout
         */
        GridLayout gridLayout = new GridLayout(3, false);
        composite.setLayout(gridLayout);

        // ComboBox
        GridData gdComboBox = new GridData();
        gdComboBox.grabExcessHorizontalSpace = true;
        gdComboBox.horizontalAlignment = GridData.FILL;
        comboBox.setLayoutData(gdComboBox);
      }
      else
      {
        AttributeType att = data.getAttributeType();

        final Label suffixBox = new Label(composite, SWT.NONE);
        String suffixText = null;
        if (att instanceof IntegerAttributeType)
        {
          IntegerAttributeType iAtt = (IntegerAttributeType)att;
          if (iAtt.getRange() != null)
            suffixText = iAtt.getRange().getUnit();
        }
        else if (att instanceof DoubleAttributeType)
        {
          DoubleAttributeType dAtt = (DoubleAttributeType)att;
          suffixText = dAtt.getUnit();
        }

        if (suffixText != null)
        {
          suffixText = suffixText.trim();
          if (!suffixText.equals(""))
            suffixBox.setText("[" + suffixText + "] ");
        }

        final Text textBox = new Text(composite, SWT.BORDER);
        textBox.setEditable(attEnable);
        textBox.setData("Aenderbar", attAenderbarKopie);
    
        textBox.setToolTipText(tooltip);
    
        if (data.isDefined())
        {
          textBox.setText(data.asTextValue().getValueText());
          if (_editable)
            textBox.setBackground(_backgroundColorDefinedValue);
        }
        else
        {
          textBox.setText(_undefinedString);
          textBox.setBackground(_backgroundUndefinedValue);
        }
        textBox.setData("Geaendert", false);

        textBox.addFocusListener(new FocusListener() {
          @Override
          public void focusGained(FocusEvent e)
          {
          }

          @Override
          public void focusLost(FocusEvent e)
          {
            if (_editable)
              textBoxAttributeModifiedPruefung(textBox, suffixBox, data);
          }
        });

        textBox.addListener(SWT.DefaultSelection, new Listener() {
          public void handleEvent(Event e)
          {
            if (_editable)
            {
              textBoxAttributeModifiedPruefung(textBox, suffixBox, data);
//              WidgetChangedListener.getInstanz()
//                .notifyWidgetChangedListener(e.widget, _tabbedComposite, null, null);
            }
          }
        });

        textBox.addListener(SWT.Modify, new Listener() {
          public void handleEvent(Event e)
          {
            if (_editable)
            {
//              textBox.setBackground(_backgroundColorDefinedValue);
              textBox.setData("Geaendert", true);
              WidgetChangedListener.getInstanz()
                .notifyWidgetChangedListener(e.widget, _tabbedComposite, null, null);
            }
          }
        });

        /*
         * Layout
         */
        GridLayout gridLayout = new GridLayout(3, false);
        composite.setLayout(gridLayout);

        // TextField
        GridData gdTextFeld = new GridData();
        gdTextFeld.grabExcessHorizontalSpace = true;
        gdTextFeld.horizontalAlignment = GridData.FILL;
        textBox.setLayoutData(gdTextFeld);
      }
    }
    else
    {
      /*
       * Urlasser ausfiltern
       */
      if (!data.getName().equals("Urlasser"))
      {
        String attName = data.getName();
        
        if (attAenderbarKopie == null)
          attAenderbarKopie = true;
        if (attName != null && _atgInfo != null)
        {
          IAttributInfo attInfo = _atgInfo.getAttributInfoTiefe(attName);
          if (attInfo != null)
          {
            attAenderbarKopie = attInfo.isAenderbarNeu() && attInfo.isAenderbarBearb();
          }
        }
//        boolean attEnable = attAenderbar & _editable;
        
        // fr die virtuelle Ansicht
        Rectangle bounds = parent.getBounds();

        if (_debug)
          System.out.println("Parent: " + parent + " " + bounds + " Child-Name: " + data.getName() + " Hhe: " + hoehe + " x: " + x + " y: " + y);
        
        if (!bounds.isEmpty())
        {
          x += bounds.x;
          y += bounds.y;
        }
        
        if (virtuell && rect != null && parentArray != null)
        {
          if (!bounds.isEmpty())
          {           
            int w = bounds.width;
            int h = bounds.height + hoehe;
            Rectangle neu = new Rectangle(x, y, w, h + 10);
            if (rect.union(neu).equals(rect))
            {
              // Parent ganz enthalten
              isVisible = true;
            }
            else
            {
              //da die intersects-Methode mit negativen Zahlen kein korrektes Ergebnis liefert, ist folgende Abfrage notwendig
              if ( (y+h+10) > 0 )
                isVisible = rect.intersects(neu);
              else
                isVisible = false;
              
              if (isVisible && hoehe == 0)
              {
                // Parent teilweise enthalten und keine Hhe angegeben
                if (_debug)
                  System.out.println("Parent nur teilweise enthalten");
                // Sichtbarkeit ber den Vorgnger ermitteln
                virtuelleHoehe = isVisibleNachfolger(parent, x, y, rect, hoehe);
                isVisible = virtuelleHoehe == 0 ? true: false;
                if (!isVisible)
                  if (_debug)
                    System.out.println("Virtuelle Hhe aus Vorgnger ermitteln: " + virtuelleHoehe);
              }
            }
            if (_debug)
              System.out.println("Parent: " + parent + " Child: " + data.getName() + " " + rect + " enthlt --- " + neu
                + " ### " + isVisible);  
          }
          else
          {
            // Parent noch leer
            if (_debug)
              System.out.println("Parent leer");
            // Sichtbarkeit ber den Vorgnger des Parents ermitteln
            virtuelleHoehe = isVisibleNachfolger(parent.getParent(), x, y, rect, hoehe);
            isVisible = virtuelleHoehe == 0 ? true : false;

            if (_debug)
            {
              if (!isVisible)
                System.out.println("Virtuelle Hhe aus Vorgnger der Parent ermitteln: " + virtuelleHoehe);
              System.out.println("Parent: " + parent + " Child: " + data.getName() + " " + rect + " ### " + isVisible);
            }
          }

        }  

        if (isVisible || parentArray == null)
        {
          if (_debug)
            System.out.println("Control ist visible oder nicht ArrayElement");
          final Group arrayBox = new Group(parent, SWT.NONE);
          arrayBox.setText(data.getName());
          arrayBox.setData(data);
          
          final GridData gdBox = new GridData();
          gdBox.grabExcessHorizontalSpace = true;
          gdBox.horizontalAlignment = GridData.FILL;
          arrayBox.setLayoutData(gdBox);
          arrayBox.setLayout(new GridLayout());
          
          if (data.isArray())
          {
            if (_debug)
              System.out.println("Data ist Array");
            Composite arrayHeaderBox = new Composite(arrayBox, SWT.NONE);
            GridLayout glArrayHeaderBox = new GridLayout(2, false);
            arrayHeaderBox.setLayout(glArrayHeaderBox);

            GridData gdArrayHeaderBox = new GridData();
            gdArrayHeaderBox.grabExcessHorizontalSpace = true;
            gdArrayHeaderBox.horizontalAlignment = GridData.FILL;
            arrayHeaderBox.setLayoutData(gdArrayHeaderBox);
            
            Label labelSpinner = new Label(arrayHeaderBox, SWT.NONE);
            labelSpinner.setText("Arraygre ");

            final Spinner spinner;
            final Label labelArraySize;

            Data.Array array = data.asArray();
            if (array.isCountVariable())
            {
              spinner = new Spinner(arrayHeaderBox, SWT.BORDER);
              spinner.setMinimum(0);
              spinner.setMaximum(Integer.MAX_VALUE);
              spinner.setIncrement(1);
              spinner.setPageIncrement(100);
              spinner.setSelection(array.getLength());
              _spinnerWert = spinner.getSelection();
              
              if (array.isCountLimited())
              {
                spinner.setMaximum(new Integer(array.getMaxCount()));
                spinner.setToolTipText("Minimum: 0, Maximum: " + array.getMaxCount());
              }
              else
              {
                spinner.setToolTipText("Minimum: 0, Maximum: " + Integer.MAX_VALUE);
              }

              spinner.addKeyListener(new KeyListener() {

                @Override
                public void keyReleased(KeyEvent e)
                {
                  _spinnerEingabe = false;
                }
                
                @Override
                public void keyPressed(KeyEvent e)
                {
                  // dient zur Unterscheidung der direkten Eingabe ber die Tastatur
                  // und wird vor der Selektion gesetzt
                  _spinnerEingabe = true;
                }
              });

              spinner.addSelectionListener(new SelectionListener() {
                
                @Override
                public void widgetSelected(SelectionEvent e)
                {
                  int selection = spinner.getSelection(); 
                 
                  if (!_spinnerEingabe)
                  {
                    // keine direkte Tastatureingabe (also Up- oder Down-Button)
                    
                    // bei Dauerbettigung kann die Anzahl auch grer als 1 bzw. kleiner als -1 sein
                    int anzahl = selection - _spinnerWert;
                    
                    if (anzahl > 0)
                    {
                      if (_debug)
                        System.out.println("Spinner up " + selection + " > " + _spinnerWert);
                      
                      Control[] arrayChildren = arrayBox.getChildren();
                      if (arrayChildren.length >= _spinnerWert)
                      {
                        // Alle Array-Kinder werden angezeigt, dann neues Kind hinzufgen und ScrollHhe anpassen
                        data.asArray().setLength(selection);

                        WidgetChangedListener.getInstanz().notifyWidgetChangedListener(e.widget, _tabbedComposite, data,
                            arrayBox); 
                      }
                      else if (_virtuellElementNach != null)
                      {
                        // nicht alle Array-Kinder werden angezeigt, dann nur ScrollHhe anpassen 
                        _scrollHoehe += _virtuellElementNach.getVirtuelleHoehe() * anzahl;
                        if (_scrolledArea != null)
                        {
                          ((ScrolledDatenBereichEditorComposite)_scrolledArea.getParent()).resizeScrollBars(0,
                              _scrollHoehe);
                        }
                        data.asArray().setLength(selection);
                        // nur noch Dirty-Flag setzen
                        WidgetChangedListener.getInstanz().notifyWidgetChangedListener(e.widget, _tabbedComposite, null,
                            null);
                      }
                      
                      if (_scrolledArea != null)
                      {
                        _scrolledArea.layout();
                        _scrolledArea.getVerticalBar().setSelection( 0 );
                        ((ScrolledDatenBereichEditorComposite) _scrolledArea.getParent()).resetScrollPoint();
                      }
                      _spinnerEingabe = false;
                      _spinnerWert = selection;
                      
                    }
                    else
                    {
                      if (_debug)
                        System.out.println("Spinner down " + selection + " <= " + _spinnerWert);
                      
                      anzahl *= (-1);       // negative Anzahl zur besseren Verwendung positiv machen

                      Control[] arrayChildren = arrayBox.getChildren();
                      if (arrayChildren.length > 1 && arrayChildren.length > selection + 1)
                      {
                        // Alle Array-Kinder werden angezeigt, dann Letztes lschen und ScrollHhe anpassen
                        if (_debug)
                          System.out.println("Spinner down "  + selection + " +1 < " + arrayChildren.length
                            + " SpinnerWert: " + _spinnerWert + " alle Array-Kinder angezeigt: letztes lschen");
                        
                        int anzahlChild = arrayChildren.length - selection;
                        for (int i = 1; i < anzahlChild; i++)
                        {
                          arrayChildren[arrayChildren.length - i].dispose();                          
                        }
                        if (_virtuellElementNach != null && _virtuellElementNach.getData() != null)
                        {
                          VirtuellesElement ve = _virtuellElementNach;
                          ve.setData( null );
                          ve.setIndex( -1 );
                        }
                        _this.layout();
                        if (_scrolledArea != null)
                        {
                          if (_virtuellElementNach != null)
                          {
                            _scrollHoehe -= _virtuellElementNach.getVirtuelleHoehe() * anzahl;
                            ((ScrolledDatenBereichEditorComposite)_scrolledArea.getParent()).resizeScrollBars(0,
                                _scrollHoehe);
                          }
                          else
                          {
                            ((ScrolledDatenBereichEditorComposite)_scrolledArea.getParent()).resizeScrollBars(0, 0);
                          }
                          if (_debug)
                            System.out.println("widgetSelected Spinner:: Default _scrollHoehe: " + _scrollHoehe);
                        }
                      }
                      else if (_virtuellElementNach != null)
                      {
                        // nicht alle Array-Kinder werden angezeigt, dann nur ScrollHhe anpassen
                        _scrollHoehe -= _virtuellElementNach.getVirtuelleHoehe() * anzahl;
                        if (_scrolledArea != null)
                        {
                          ((ScrolledDatenBereichEditorComposite)_scrolledArea.getParent()).resizeScrollBars(0,
                              _scrollHoehe);

                          if (arrayChildren.length == selection + 1 && _virtuellElementNach.getData() != null)
                          {
                            VirtuellesElement ve = _virtuellElementNach;
                            ve.setData(null);
                            ve.setIndex(-1);
                          }
                        }
                      }
                      data.asArray().setLength(selection);
                      // nur noch Dirty-Flag setzen
                      WidgetChangedListener.getInstanz().notifyWidgetChangedListener(e.widget, _tabbedComposite, null,
                          null);
                    }
                    
                    if (_scrolledArea != null)
                    {
                      _scrolledArea.layout();
                      _scrolledArea.getVerticalBar().setSelection( 0 );
                      ((ScrolledDatenBereichEditorComposite) _scrolledArea.getParent()).resetScrollPoint();
                    }
                    _spinnerEingabe = false;
                    _spinnerWert = selection;
                    
                  }

                }
                
                @Override
                public void widgetDefaultSelected(SelectionEvent e)
                {
                  // direkte Spinnereingabe mit Datenfreigabe                 
                  int selection = spinner.getSelection(); 
                  _scrolledArea.getVerticalBar().setSelection(0);
                  
                  data.asArray().setLength(selection);
                  WidgetChangedListener.getInstanz().notifyWidgetChangedListener(e.widget, _tabbedComposite, data,
                      arrayBox);

                  if (_scrolledArea != null)
                  {
                    _scrolledArea.layout();
                    _scrolledArea.getVerticalBar().setSelection( 0 );
                    ((ScrolledDatenBereichEditorComposite) _scrolledArea.getParent()).resetScrollPoint();
                  }
                  
                  _spinnerEingabe = false;
                  _spinnerWert = selection;
                }
              });

              /*
               * Editable
               */
              if (spinner != null)
              {
                spinner.setEnabled(attAenderbarKopie && isSpinnerActive && _editable);
                spinner.setData("Aenderbar", attAenderbarKopie);
                if (_editable)
                  spinner.setBackground(_backgroundColorDefinedValue);
              }
            }
            else
            {
              // (!array.isCountVariable())
              array.setLength(array.getMaxCount());
              labelArraySize = new Label(arrayHeaderBox, SWT.NONE);
              labelArraySize.setText("" + array.getLength());
            }

          }

          // die Elemente des Arrays ermitteln und anzeigen, falls nicht auerhalb des Anzeigebereichs

          @SuppressWarnings("unchecked")
          Iterator iterator = data.iterator();
          int arrayL = 0;
          if (data.isArray())
            arrayL = data.asArray().getLength();
          
          int index = -1;
          int virtHoehe = 0;
          Data subData = null;
          
          Attribute parentAtt = null;
          
          AttributeListDefinition listDefinition = null;
          if (data.isList())
          {
            AttributeType attributeType = data.getAttributeType();
            if (attributeType instanceof AttributeListDefinition)
            {
              listDefinition = (AttributeListDefinition)attributeType;
            }
          }
          
          //solange Sub-Daten vorliegen und bei einem Array diese nicht auerhalb des Anzeigebereichs liegen
          while (iterator.hasNext() && (arrayL == 0 || virtHoehe == 0) )
          {
            index++;
            // geht nicht ohne neues Layout, weil sonst der virtuelle Bereich nicht richtig ermittelt werden kann 
            _this.layout();
            subData = (Data)iterator.next();
            
            virtHoehe = createBox(arrayBox, subData, x, y, rect, data.isArray() ? data.getName() : null, virtuell, 0, 
                attAenderbarKopie, listDefinition);
          }
          
          if (_debug)
          {
            if ( virtHoehe > 0 )
            {
              System.out.println( "Elemente bis ausschlielich " + index + " zu " + data.getName() + " angelegt." );
              System.out.println("Anzahl Array-Anzeige-Elemente: " + arrayBox.getChildren().length);
            }
          }
          
          //Scrollhhe entsprechend der noch fehlenden Array-Elemente anpassen und nchstes Element sichern
          if (virtuell && virtHoehe > 0 && index < arrayL)
          {
            // Ausgleich fr den Abstand zwischen den Array-Elementen
            int verticalSpacing = ((GridLayout)arrayBox.getLayout()).verticalSpacing;
            virtHoehe += verticalSpacing;
            _scrollHoehe += virtHoehe * (arrayL-index);
            
            //nchstes SubData sichern mit den x,y-Werten relativ zum DataEditorComposite
            int subX = x - _this.getBounds().x;
            int subY = y - _this.getBounds().y;
            VirtuellesElement veSubDataNach = new VirtuellesElement(arrayBox, subData, subX, subY, index, data.isArray(), virtHoehe);
            if (_debug)
              System.out.println("createBox: _virtuellElementNach: " + veSubDataNach.getData().getName() + " Hhe: " + veSubDataNach.getVirtuelleHoehe());
            if (_virtuellElementNach != null)
            {
              VirtuellesElement ve = _virtuellElementNach;
              ve.setData( veSubDataNach.getData() );
              ve.setIndex( veSubDataNach.getIndex() );
            }
            else
              _virtuellElementNach = veSubDataNach;
            
            // Initialisierung
            if (_virtuellElementVor == null)
            {
              VirtuellesElement veSubDataVor = new VirtuellesElement(arrayBox, null, subX, subY, -1, data.isArray(), virtHoehe); 
              _virtuellElementVor = veSubDataVor;
            }
          }
          else if (virtuell && data.isArray())
          {
            virtuell = false;
            ((ScrolledDatenBereichEditorComposite)_scrolledArea.getParent()).setVirtuell(virtuell);
          }
        }
      }
    }
    
    return virtuelleHoehe;
  }

  /**
   * Liegt der Nachfolger des letzten Kindes des Parent mit Bound im Anzeigebereich
   * 
   * @param parent  Parent
   * @param x 
   *          X-Wert der oberen linken Ecke des Parent relativ zum Anzeigebereich
   * @param y 
   *          Y-Wert der oberen linken Ecke des Parent relativ zum Anzeigebereich
   * @param rect    Rechteck des Anzeigebereichs
   * @param hoehe   Hhe des Darstellungsobjektes zum {@link Data}, falls bekannt
   * @return virtuelle Hhe des Darstellungsobjektes zum {@link Data}, falls keine Anzeige, ansonsten 0         
   */
  private int isVisibleNachfolger(Composite parent, int x, int y, Rectangle rect, int hoehe)
  {
    boolean isVisible = true;

    // letztes Kind mit Bounds ermitteln
    Control[] children = parent.getChildren();

    if (children.length == 0)
    {
      //entspricht isVisible = true;
      return 0;
    }

    int index = children.length - 1;
    Control child = null;
    Rectangle bounds = new Rectangle(0, 0, 0, 0);

    while (bounds.isEmpty() && bounds.y == 0 && index >= 0 )
    {
      child = children[index];
      bounds = child.getBounds();
      index--;
    }

    if (_debug)
      System.out.println("Child: " + child + " " + bounds + " X: " + x + " Y: " + y);

    // Position des Nachfolgers ermitteln
    x += bounds.x;
    y += bounds.y + bounds.height;
    int w = bounds.width;
    int h = bounds.height;
    if (hoehe>0)
      h = hoehe;

    if (w == 0 && h == 0)
    {
      w = 10;
      h = 10;
    }

    Rectangle neu = new Rectangle(x, y + 5, w, h + 10);
    if (rect.equals(rect.union(neu)))
    {
      // Nachfolger ganz enthalten
    }
    else
    {
      // Nachfolger teilweise oder nicht enthalten
      //da die intersects-Methode mit negativen Zahlen kein korrektes Ergebnis liefert, ist folgende Abfrage notwendig
      if ( (y+h+10) > 0 )
        isVisible = rect.intersects(neu);
      else
        isVisible = false;
    }
    if (_debug)
      System.out.println("Nachfolger von: " + child + " " + rect + " enthlt --- " + neu + " ### " + isVisible);

    if (!isVisible)
    {
      //virtuelle Hhe
      return h;
    }
    else
      return 0;
  }

  /**
   * AtgInfo fr die Bestimmung der Pflichtfelder aus der Kapsel ermitteln
   */
  private void setAtgInfo()
  {
    if (_tabbedComposite == null )
      return;
    DatenBereichEditorKapsel kapsel = _tabbedComposite.getAktuelleKapsel();
    if (kapsel == null)
      return;
    
    if (kapsel.isTypAtg())
    {  
      HierarchieObjekt ho = kapsel.getElternKapsel().getHierarchieObjekt();
      String typPid = kapsel.getElternKapsel().getSystemObjekt().getType().getPid();
      
      IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
      TypInfo typInfo = hot.getTypInfo(typPid);   
      _atgInfo = typInfo.getAtgInfo(kapsel.getName());
      _tooltipAtg = _dataModel.getAttributeGroup(kapsel.getName());
      
    }
    else if (kapsel.isTypMenge())
    {
      _tooltipAtg = _dataModel.getAttributeGroup("atg.dynamischeMenge");     
    }
    
  }

  /**
   * Rekursive Ermittlung der SubTypen zu einem ObjektTyp
   * 
   * @param objectType  ObjektTyp, zu dem die SubTypen ermittelt werden sollen
   * @return  Liste der SystemObjectTypen
   */
  private List<SystemObjectType> ermittelnSubTypen(SystemObjectType objectType)
  {
    List<SystemObjectType> types = new LinkedList<SystemObjectType>();

    List<SystemObjectType> subTypes = objectType.getSubTypes();
    for (SystemObjectType sot : subTypes)
    {
//      if (!objectType.isBaseType() || objectType.getElements().size() > 0)
        types.add(sot);
      List<SystemObjectType> subSubTypen = ermittelnSubTypen(sot);
      if (subSubTypen != null)
        types.addAll(subSubTypen);
    }

    return types;
  }

  /**
   * Fhrt den Update der Gruppe zum Array aus, falls Data ein Array ist.
   * Anzeigebereich ist immer am Anfang, da Spinner verndert wurde.
   * 
   * @param data
   *          {@link Data} zur Darstellung in der Gruppe des Arrays.
   * @param arrayBox
   *          Array-{@link Group}, auf dessen Basis die Darstellung erfolgen soll.
   * @param virtuell        virtuelle Darstellung (true/false).
   * @param rect
   *          Rechteck des Anzeigebereichs.
   *                    
   */
  public void updateArrayBox(Data data, Group arrayBox, boolean virtuell, Rectangle rect)
  {
    if (_debug)
      System.out.println("updateArrayBox: ");

    if (!data.isArray())
      return;

    _scrollHoehe = 0;

    // ab 20 Array-Elemente virtueller Aufbau
    if ( data.asArray().getLength() > 20 )
    {
      virtuell = true;
      ((ScrolledDatenBereichEditorComposite)_scrolledArea.getParent()).setVirtuell(virtuell);
    }

    // X- und Y-Wert der oberen linken Ecke der Parent zur ArrayBox relativ zum Anzeigebereich.
    int x = 0;
    int y = 0;

    Composite comp = arrayBox.getParent();
    while (comp != null && comp != _this)
    {
      Rectangle bounds = comp.getBounds();
      x += bounds.x;
      y += bounds.y;

      comp = comp.getParent();
    }

    Control[] arrayChildren = arrayBox.getChildren();
    Array dataArray = data.asArray();
    int arrayL = dataArray.getLength();

    if (_debug)
      System.out.println("Anzahl Kinder: " + arrayChildren.length + " Anzahl Data: " + arrayL);
    
    if (arrayChildren.length > arrayL + 1)
    {
      // Mehr Elemente sichtbar als ausgewhlt, dann nicht alles neu aufbauen, sondern nur entfernen
      // Alle Elemente des Parent bis auf das Erste und die gewnschten lschen
      List<Composite> listExep = new ArrayList<Composite>();
      for (int i = 0; i < arrayL + 1; i++)
      {
        listExep.add((Composite)arrayChildren[i]);
      }
      leereCompositeMitAusnahme(arrayBox, listExep);
    }
    else
    {     
      // Weniger Elemente sichtbar als ausgewhlt, dann die entsprechende Anzahl hinzufgen  
      int virtHoehe = 0;
      Data subData = null;
      int start = arrayChildren.length > 0 ? arrayChildren.length - 1 : 0;
      int index = start - 1;
     
      // ist nicht notwendig
//      arrayBox.setVisible(false);
      
      // nchste Elemente anfgen bis diese ausserhalb des Anzeigebereiches sind
      for (int i = start; i < arrayL; i++)
      {
        index++;
        subData = dataArray.getItem( i );

        if (_debug)
          System.out.println("subData( " + index + "): " + subData.valueToString());

        // geht nicht ohne neues Layout, weil sonst der virtuelle Bereich nicht richtig ermittelt werden kann 
        _this.layout();
        virtHoehe = createBox(arrayBox, subData, x, y, rect, data.getName(), virtuell, 0, null, null);
        if (virtHoehe > 0)
          break;
      }

      // Scrollhhe entsprechend der noch fehlenden Array-Elemente anpassen und nchstes Element sichern
      if (virtuell && virtHoehe > 0 && index < arrayL)
      {
        // Ausgleich fr den Abstand zwischen den Array-Elementen
        int verticalSpacing = ((GridLayout)arrayBox.getLayout()).verticalSpacing;
        virtHoehe += verticalSpacing;
        _scrollHoehe += virtHoehe * (arrayL - index);

        // nchstes SubData sichern mit den x,y-Werten relativ zum DataEditorComposite
        int subX = x - _this.getBounds().x;
        int subY = y - _this.getBounds().y;
        VirtuellesElement veSubDataNach = new VirtuellesElement(arrayBox, subData, subX, subY, index, data.isArray(),
            virtHoehe);
        if (_debug)
          System.out.println("updateArrayBox: _virtuellElementNach: " + veSubDataNach.getData().getName() + " Hhe: "
              + veSubDataNach.getVirtuelleHoehe() + " Index: " + veSubDataNach.getIndex());
        if (_virtuellElementNach != null)
        {
          VirtuellesElement ve = _virtuellElementNach;
          ve.setData(veSubDataNach.getData());
          ve.setIndex(veSubDataNach.getIndex());
        }
        else
          _virtuellElementNach = veSubDataNach;

        // Initialisierung
        if (_virtuellElementVor == null)
        {
          VirtuellesElement veSubDataVor = new VirtuellesElement(arrayBox, null, subX, subY, -1, data.isArray(),
              virtHoehe);
          _virtuellElementVor = veSubDataVor;
        }
      }
      else if (virtuell)
      {
        virtuell = false;
        ((ScrolledDatenBereichEditorComposite)_scrolledArea.getParent()).setVirtuell(virtuell);
      }
    }
    
    _this.layout();
//    arrayBox.setVisible(true);

    if (_scrolledArea != null)
    {
      _scrollHoehe += _this.computeSize(SWT.DEFAULT, SWT.DEFAULT).y;
      if (_debug)
        System.out.println("updateArrayBox:: _scrollHoehe: " + _scrollHoehe);
      ((ScrolledDatenBereichEditorComposite)_scrolledArea.getParent()).resizeScrollBars(0, _scrollHoehe);
    }

    //TODO besser ohne
//  if (_scrolledArea != null)
//  {
//    _scrolledArea.layout();
//    _scrolledArea.getVerticalBar().setSelection( 0 );
//    ((ScrolledDatenBereichEditorComposite) _scrolledArea.getParent()).resetScrollPoint();
//  }
    
  }

  /**
   * Entfernt von einem {@link Composite} alle Kinder. Es knnen Ausnahmen definiert werden.
   * 
   * @param toClearComposite
   *          Zu leerendes {@link Composite}.
   * @param exceptionComposites
   *          Liste von Ausnahme-{@link Composite} oder <code>null</code>, wenn alle entfernt werden sollen.
   */
  private void leereCompositeMitAusnahme(Composite toClearComposite, List<Composite> exceptionComposites)
  {
    if (_debug)
      System.out.println("leereCompositeMitAusnahme:: Anzahl Exceptions: " + exceptionComposites.size());
    
    Control[] children = toClearComposite.getChildren();
    for (Control control : children)
    {
      if (exceptionComposites != null)
      {
        if (!exceptionComposites.contains(control))
        {
          control.dispose();
        }
      }
      else
      {
        control.dispose();
      }
    }
  }

  private String getScaledValueText(final long unscaledValue, double conversionFactor)
  {

    if (conversionFactor == 1)
    {
      return String.valueOf(unscaledValue);
    }
    else
    {
      int precision = 0;
      synchronized (_integerNumberFormat)
      {
        String formatted = _precisionTestNumberFormat.format(conversionFactor);
        int kommaPosition = formatted.lastIndexOf(',');
        if (kommaPosition >= 0)
          precision = formatted.length() - kommaPosition - 1;
        _integerNumberFormat.setMinimumFractionDigits(precision);
        _integerNumberFormat.setMaximumFractionDigits(precision);
        return _integerNumberFormat.format(unscaledValue * conversionFactor);
      }
    }
  }

  /**
   * berprft, ob der eingegebene Wert auch ein zulssiger Wert ist. Bevor die Aktionen durchgefhrt werden, wird vom
   * jeweiligen {@link ComboViewer} der registrierte {@link ISelectionChangedListener} detached. Am Ende der Methode
   * wird der Listener wieder attached. Da dies Methode selbst Selektionen im ComboViewer ndern kann, wrde es sonst zu
   * Endlos-Loops im SWT und damit verbunden zu Stack-Fehlern und Speicherlecks fhren.
   * 
   * @param cvComboBox
   *          Komponente, wo der neue Wert eingegeben wurde
   * @param suffixBox
   *          das Feld mit dem Suffix
   * @param data
   *          die modifizierten Daten
   */
  private void comboBoxAttributeModifiziertWertPruefung(ComboViewer cvComboBox, Label suffixBox, Data data)
  {
    cvComboBox.removeSelectionChangedListener(_mapComboListener.get(cvComboBox));
    String text = null;

    ISelection selection = cvComboBox.getSelection();
    if (selection instanceof StructuredSelection)
    {
      StructuredSelection ss = (StructuredSelection)selection;

      if (ss.getFirstElement() != null)
      {
        text = ss.getFirstElement().toString();
      }
      else
      {
        text = cvComboBox.getCombo().getText();
      }
    }

    try
    {
      if (text.equals(_defaultValueString) || text.equals(_undefinedString) || text.equals("undefniert"))
      {
        data.setToDefault();
      }
      else
      {
        data.asTextValue().setText(text);
      }

      if (!data.isDefined())
      {
        if (_editable)
          cvComboBox.getCombo().setBackground(_backgroundUndefinedValue);
        StructuredSelection ss = new StructuredSelection(_undefinedString);
        cvComboBox.setSelection(ss, true);
      }
      else
      {
        StructuredSelection ss = new StructuredSelection(data.asTextValue().getValueText());
        cvComboBox.setSelection(ss, true);
        if (_editable)
          cvComboBox.getCombo().setBackground(_backgroundColorDefinedValue);

      }
    }
    catch (Exception ex)
    {
      // System.out.println("Viewer: " + cvComboBox);
      if (cvComboBox != null)
        // System.out.println("Combo: " + cvComboBox.getCombo());
        // cvComboBox.getCombo().setBackground(_backgroundUndefinedValue);
        // cvComboBox.setSelection(new StructuredSelection(_undefinedString), true);
        ex.printStackTrace();
    }

    cvComboBox.addSelectionChangedListener(_mapComboListener.get(cvComboBox));
  }
  
  /**
   * berprft, ob der eingegebene Wert auch ein zulssiger Wert ist.
   * 
   * @param textBox
   *          Komponente, wo der neue Wert eingegeben wurde
   * @param suffixBox
   *          das Feld mit dem Suffix
   * @param data
   *          die modifizierten Daten
   */
  private void textBoxAttributeModifiedPruefung(final Text textBox, final Label suffixBox, final Data data)
  {
    boolean geaendert = false;
    Object obj = textBox.getData("Geaendert");
    if (obj instanceof Boolean)
    {
      geaendert = (Boolean)obj;
    }
    
    if (!geaendert)
    {
      return;
    }
    
    textBox.setData("Geaendert", false);
    
    String text = textBox.getText();
    try
    {
      if (text.equals(_defaultValueString) || text.equals(_undefinedString))
      {
        data.setToDefault();
      }
      else
      {
        data.asTextValue().setText(text);
      }

      if (!data.isDefined())
      {
        textBox.setBackground(_backgroundUndefinedValue);
//        textBox.setText(_undefinedString);
      }
      else
      {
        textBox.setText(data.asTextValue().getValueText());
        if (_editable)
          textBox.setBackground(_backgroundColorDefinedValue);       
      }
    }
    catch (Exception ex)
    {
      // Wenn beim setzen des Textes ein Fehler Auftritt, dann wird der Wert auf
      // "undefiniert" gesetzt
      // und der Benutzer muss sich Gedanken um den Wert machen
      textBox.setBackground(_backgroundUndefinedValue);
//      textBox.setText(_undefinedString);
    }
  }

  /**
   * berprft, ob der eingegebene Wert auch ein zulssiger Wert ist.
   * 
   * @param textBox
   *          Komponente, wo der neue Wert eingegeben wurde
   * @param suffixBox
   *          das Feld mit dem Suffix
   * @param data
   *          die modifizierten Daten
   */
  private void refTextBoxAttributeModifiedPruefung(final Text textBox, final Label suffixBox, final Data data)
  {
    boolean geaendert = false;
    Object obj = textBox.getData("Geaendert");
    if (obj instanceof Boolean)
    {
      geaendert = (Boolean)obj;
    }
    
    if (!geaendert)
    {
      return;
    }
    
    textBox.setData("Geaendert", false);
    
    String text = textBox.getText();
    try
    {
      if (text.equals("") || text.equals(_undefinedString))
      {
        data.setToDefault();
      }
      else
      {
        ObjectLookup lookUp = LookupFactory.createLookupForModifiableVersion((ConfigDataModel)_dataModel);
        data.asReferenceValue().setSystemObjectPid(text, lookUp);
      }

      if (!data.isDefined())
      {
        textBox.setBackground(_backgroundUndefinedValue);
//        textBox.setText(_undefinedString);
      }
      else
      {
        String wert = data.asReferenceValue().getSystemObjectPid();
        if (wert == null || wert.isEmpty())
        {
          wert =_undefinedString;
        }
        textBox.setText(wert);
        suffixBox.setText(data.asTextValue().getSuffixText());
        suffixBox.getParent().layout();
        
        if (_editable)
          textBox.setBackground(_backgroundColorDefinedValue);       
      }
    }
    catch (Exception ex)
    {
      // Wenn beim setzen des Textes ein Fehler Auftritt, dann wird der Wert auf
      // "undefiniert" gesetzt
      // und der Benutzer muss sich Gedanken um den Wert machen
      textBox.setBackground(_backgroundUndefinedValue);
//      textBox.setText(_undefinedString);
    }
  }
  
  public boolean isDirty()
  {
    return _dirty;
  }

  public void setDirty(boolean dirty)
  {
    // System.out.println("DataEditorComposite: Set Dirty: " + dirty);
    _dirty = dirty;

  }

  public VirtuellesElement getVirtuellElementVor()
  {
    return _virtuellElementVor;
  }
  
}
