/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.datenbereich.editor;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Kapselt alle notwendigen Daten zur eindeutigen Identifikation eines
 * Datensatzes zu einem Systemobjekt.
 * 
 * @author Krause
 * 
 */
public class DatenBereichSOKapsel implements Comparable<String>
{
  
  /**
   * HierarchieObjekt
   */
  private HierarchieObjekt _hierarchieObjekt = null;
  
  /**
   * SystemObjekt
   */
  private SystemObject _systemObjekt = null;

  /**
   * Pid des SystemObjektes
   */
  private String _pidSystemObjekt = null;
  
  /**
   * Name des SystemObjektes
   */
  private String _nameSystemObjekt = null;
   
  /**
   * Liste der direkten Kinder {@link DatenBereichEditorKapsel} (untergeordnete Elemente)
   */
  private List<DatenBereichEditorKapsel> _kinderListe;
  
  /**
   * Konstruktor. bernimmt die notwendigen Parameter zur eindeutigen
   * Identifikation eines Datensatzes zu einem Systemobjekt.
   * 
   * @param systemObjekt
   *          SystemObjekt
   */
  public DatenBereichSOKapsel(SystemObject systemObjekt)
  {
    this(systemObjekt, null);  
  }

  /**
   * Konstruktor. bernimmt die notwendigen Parameter zur eindeutigen
   * Identifikation eines Datensatzes zu einem Systemobjekt.
   * 
   * @param pidSystemObjekt
   *          Pid des SystemObjektes
   * @param hierarchieObjekt
   *          HierarchieObjekt          
   */
  public DatenBereichSOKapsel(SystemObject systemObjekt, HierarchieObjekt hierarchieObjekt)
  {
    this._systemObjekt = systemObjekt;
    this._pidSystemObjekt = systemObjekt.getPid();
    this._hierarchieObjekt = hierarchieObjekt;
    this._kinderListe = new ArrayList<DatenBereichEditorKapsel>();

  }

  @Override
  public int compareTo(String o)
  {
    int re = 0;

    if (_pidSystemObjekt != null)
    {
      re = _pidSystemObjekt.compareTo(o);
    }

    return re;
  }
  
  /**
   * Liefert das HierarchieObjekt.
   * 
   * @return HierarchieObjekt
   */
  public HierarchieObjekt getHierarchieObjekt()
  {
    return _hierarchieObjekt;
  }
  
  /**
   * Liefert das SystemObjekt.
   * 
   * @return SystemObjekt
   */  
  public SystemObject getSystemObjekt()
  {
    return _systemObjekt;
  }
  
  /**
   * Liefert die Pid des SystemObjektes.
   * 
   * @return Pid des SystemObjektes
   */
  public String getPidSystemObjekt()
  {
    return _pidSystemObjekt;
  }

   /**
   * Setzt die Pid des SystemObjektes.
   * 
   * @param pid Pid des SystemObjektes
   */
  public void setPidSystemObjekt(String pid)
  {
    _pidSystemObjekt = pid;  
  }

  /**
   * Liefert den Namen des SystemObjektes.
   * 
   * @return Name des SystemObjektes
   */
  public String getNameSystemObjekt()
  {
    return _nameSystemObjekt;    
  }

  /**
   * Setzt den Namen des SystemObjektes.
   * 
   * @param name Name des SystemObjektes
   */
  public void setNameSystemObjekt(String name)
  {
    _nameSystemObjekt = name;    
  }

  /**
   * Liefert die Kinder {@link DatenBereichEditorKapsel} der Kapsel.
   * 
   * @return Kinder der Kapsel
   */
  public List<DatenBereichEditorKapsel> getKinderListe()
  {
    return _kinderListe;
  }

  /**
   * Setzt die Liste der Kinder {@link DatenBereichEditorKapsel}.
   * 
   * @param kinder Liste der Kinder
   */
  public void setKinder(ArrayList<DatenBereichEditorKapsel> kinder)
  {
    _kinderListe = kinder;
  }
  
  /**
   * Fgt ein Kind {@link DatenBereichEditorKapsel} der Liste der Kinder hinzu.
   * 
   * @param kind Kind der Kapsel
   */
  public void addKind(DatenBereichEditorKapsel kind)
  {
    _kinderListe.add(kind);
  }

}
