/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.datenbereich.editor;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Widget;

import de.bsvrz.buv.plugin.tkabasis.DataFactory;
import de.bsvrz.buv.plugin.tkabasis.TKABasisView;
import de.bsvrz.buv.plugin.tkabasis.datenbereich.editor.widgetChangedListener.IWidgetChangedListener;
import de.bsvrz.buv.plugin.tkabasis.datenbereich.editor.widgetChangedListener.WidgetChangedEvent;
import de.bsvrz.buv.plugin.tkabasis.datenbereich.editor.widgetChangedListener.WidgetChangedListener;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.MengeInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.TypInfo;
import de.bsvrz.buv.plugin.tkabasis.regeln.IBildungsRegeln;
import de.bsvrz.buv.plugin.tkabasis.zustaende.Zustand;
import de.bsvrz.buv.plugin.tkabasis.zustaende.ZustandsFactory.Bearbeitungsmodus;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Die Klasse stellt die beiden Tabbereiche fr die Systemobjekte und Attributgruppen dar.
 * 
 * @author Pittner
 *
 */
public class TabbedDatenBereichEditorComposite extends Composite implements Listener, IWidgetChangedListener
{
  /**
   * Flag zum Debuggen
   */
  protected static final boolean _debug = false;
  
  /**
   * Id zur Identifizierung der View
   */
  private String _secondaryId = null;
  
  /**
   * Gibt an, ob die Felder der erzeugten Tab-Seiten editierbar sind oder nicht.
   */
  private boolean _editable = false;
  
  /**
   * Datenmodell
   */
  private DataModel _dataModel = null;

  /**
   * Folder fr die SystemObjekte
   */
  private CTabFolder _folder = null;

  /**
   * Index des selektieren Tabs zum SystemObjekt
   */
  private int _selectionTabSoIndex = -1;
  
  /**
   * Selektiertes innerstes Tab mit den Atribut-Daten zum selektierten SystemObjekten
   */
  private CTabItem _selectionTabAtg = null;
  
  /** Flag das angibt, ob die dargestellten Felder verndert wurden */
  private boolean _dirty = false;

  /**
   * Aktuelle Kapsel fr das selektierte SystemObjekt
   */
  private DatenBereichSOKapsel _aktuelleSOKapsel;

  /**
   * Aktuelle Kapsel fr die selektierte Attributgruppe des SystemObjekts
   */
  private DatenBereichEditorKapsel _aktuelleKapsel;

  /**
   * Zuordnung zwischen Kapsel fr die Attributgruppe und dem ScrolledDatenBereichEditorComposite
   */
  private Map<String, ScrolledDatenBereichEditorComposite> _mapKapselEditorZuordnung = new HashMap<String, ScrolledDatenBereichEditorComposite>();

  /**
   * Folder fr die Attributgruppen zu den SystemObjekten
   */
  private LinkedList<CTabFolder> _folderSo = new LinkedList<CTabFolder>();
  
  /**
   * Checkboxen zu den SystemObjekten
   */
  private LinkedList<Button> _checkBox = new LinkedList<Button>();

  /**
   * NeueDaten-Buttons zu den SystemObjekten
   */
  private LinkedList<Button> _buttonNeueDaten = new LinkedList<Button>();

  /**
   * LoescheDaten-Buttons zu den SystemObjekten
   */
  private LinkedList<Button> _buttonLoescheDaten = new LinkedList<Button>();

  /**
   * Pids zu den SystemObjekten
   */
  private LinkedList<Text> _textPid = new LinkedList<Text>();

  /**
   * Namen zu den SystemObjekten
   */
  private LinkedList<Text> _textName = new LinkedList<Text>();
  
  /**
   * Flag das angibt, ob die Editor-Darstellung um die Optional-Checkbox und die Buttons erweitert sein soll
   */
  private boolean _erweitert = false;

  /**
   * bernehmen-Button des Elternteils
   */
  private Button _button;

  /**
   * Listener fr den Folder der SystemObjekte
   */
  private SelectionListener _selectionFolderListener;

  /**
   * Listener fr die Folder der Attributgruppen der SystemObjekte
   */
  private SelectionAdapter _selectionFolderSoListener;
 
  /**
   * Konstruktor
   * 
   * @param parent  Elternteil
   * @param style   Style
   * @param dataModel Datenmodell
   * @param secId   Id der View
   * @param erweitert erweiterte Darstellung des So-Tabs
   * @param button  bernehmen-Button des Elternteils
   */
  public TabbedDatenBereichEditorComposite(Composite parent, int style, DataModel dataModel, String secId,
      boolean erweitert, Button button)
  {
    super(parent, style);
    this._erweitert = erweitert;
    this._dataModel = dataModel;
    this._secondaryId = secId;
    this._button = button;

    initialize();   

    WidgetChangedListener.getInstanz().addWidgetChangedListener(this);
    
  }

  public TabbedDatenBereichEditorComposite(Composite parent, int style, DataModel dataModel, String secId)
  {
    this(parent, style, dataModel, secId, false, null);
  }

  private void initialize()
  {
    createFolder();
    
    setSize(new Point(300, 200));
    setLayout(new GridLayout(1, false));
  }

  private void addListeners(int folderIndex)
  {
    _textPid.get(folderIndex).addListener(SWT.Modify, this);
    _textName.get(folderIndex).addListener(SWT.Modify, this);
    if (_erweitert)
    {
      _buttonNeueDaten.get(folderIndex).addListener(SWT.Selection, this);
      _buttonLoescheDaten.get(folderIndex).addListener(SWT.Selection, this);
    }

  }

  private void removeAllListeners()
  {
    if (_debug)
      System.out.println("--->>> Remove alle Listener!!!");
    
    CTabItem[] items = _folder.getItems();
    for (int i = 0; i < items.length; i++)
    {
      _textPid.get(i).removeListener(SWT.Modify, this);
      _textName.get(i).removeListener(SWT.Modify, this);
      if (_erweitert)
      {
        _buttonNeueDaten.get(i).removeListener(SWT.Selection, this);
        _buttonLoescheDaten.get(i).removeListener(SWT.Selection, this);
      }
      
     if(_selectionFolderSoListener != null)
      _folderSo.get(i).removeSelectionListener(_selectionFolderSoListener);
           
    }
    
    if(_selectionFolderListener != null)
      _folder.removeSelectionListener(_selectionFolderListener);

    // _selectionFolderSoListener darf nicht gelscht werden, weil dieser sich nie verndert 
    //_selectionFolderListener darf nicht gelscht werden, weil dieser in createFolder erstellt wird
    
  }

  /**
   * Initialisierung des TabFolders
   * 
   */
  private void createFolder()
  {
    GridData gridDataTabFolder = new GridData();
    gridDataTabFolder.horizontalAlignment = GridData.FILL;
    gridDataTabFolder.grabExcessHorizontalSpace = true;
    gridDataTabFolder.grabExcessVerticalSpace = true;
    gridDataTabFolder.verticalAlignment = GridData.FILL;

    _folder = new CTabFolder(this, SWT.NONE);
    _folder.setLayout(new FillLayout());
    _folder.setLayoutData(gridDataTabFolder);
    
    _selectionFolderListener = new SelectionListener() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        if(_debug)
          System.out.println("\nTabSO selektiert!");

        // aktuelle Kapsel aus dem Datenbereich zum selektierten Tab fllen
        CTabFolder folder = (CTabFolder)e.getSource();
        _selectionTabSoIndex = folder.getSelectionIndex();

        DatenBereichSOKapsel kapselSo = (DatenBereichSOKapsel)folder.getSelection().getData("kapselSo");
        _aktuelleSOKapsel = kapselSo;

        CTabItem selection = _folderSo.get(_selectionTabSoIndex).getSelection();

        if (_debug)
        {
          System.out.println("   Index: " + _selectionTabSoIndex + " SoKapsel: "
              + _aktuelleSOKapsel.getPidSystemObjekt());
          System.out.println("   Selektierte Kapsel von FolderSo: " + selection);
          System.out.println("   0.Tab von FolderSo: " + _folderSo.get(_selectionTabSoIndex).getItem(0));
        }

        if (selection == null || selection.isDisposed())
          _selectionTabAtg = _folderSo.get(_selectionTabSoIndex).getItem(0);
        else
          _selectionTabAtg = selection;

        if(_debug)
          System.out.println("   _selectionTabAtg: " + _selectionTabAtg);
        
         DatenBereichEditorKapsel kapsel = (DatenBereichEditorKapsel)_selectionTabAtg.getData("kapsel");
               
         _aktuelleKapsel = kapsel;
         
         updateButtonEditor(_aktuelleKapsel, _selectionTabSoIndex);

        if (_debug)
        {
          System.out.println("_selectionTabAtg: " + _selectionTabAtg.getText());
          if (_selectionTabAtg != null)
            System.out.println("_selectionTabAtg: "
                + ((DatenBereichEditorKapsel)_selectionTabAtg.getData("kapsel")).getName());
          System.out.println("   Index: " + _selectionTabSoIndex + " SoKapsel: "
              + _aktuelleSOKapsel.getPidSystemObjekt());
          System.out.println("   Aktuelle Kapsel des SO: " + _aktuelleKapsel.getName());
        }

      }

      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
        if(_debug)
          System.out.println("TabSO default selektiert!");

      }
    };
    
    // wird in createTabPage removed und added
    _folder.addSelectionListener(_selectionFolderListener);
    
  }
  
  private Composite createTabSysObj(Composite parent)
  {
    Composite compTabSysObj = createCompTabSysObj(parent);

    GridData gridDataTabFolder = new GridData();
    gridDataTabFolder.horizontalAlignment = GridData.FILL;
    gridDataTabFolder.grabExcessHorizontalSpace = true;
    gridDataTabFolder.grabExcessVerticalSpace = true;
    gridDataTabFolder.verticalAlignment = GridData.FILL;
    
    CTabFolder folderSo = null;
    if (_erweitert)
    {
      Composite compositeScrolledEditor = createCompositeScrolledEditor(compTabSysObj);
      folderSo = new CTabFolder(compositeScrolledEditor, SWT.NONE);
      _folderSo.add(folderSo);

    }
    else
    {
      folderSo = new CTabFolder(compTabSysObj, SWT.NONE);
      _folderSo.add(folderSo);
    }
    
    folderSo.setLayout(new FillLayout());
    folderSo.setLayoutData(gridDataTabFolder);

    if (_selectionFolderSoListener == null)
    {
      _selectionFolderSoListener = new SelectionAdapter() {

        @Override
        public void widgetSelected(SelectionEvent e)
        {
          if(_debug)
            System.out.println("TabAtg selektiert!");
          
          CTabFolder folderSo = (CTabFolder)e.getSource();
          _selectionTabAtg = folderSo.getSelection();

          DatenBereichEditorKapsel kapsel = (DatenBereichEditorKapsel)_selectionTabAtg.getData("kapsel");

          _aktuelleKapsel = kapsel;
          
          //Inhalt des Tabs setzen, falls leer
          Control control = _selectionTabAtg.getControl();
          if ( control instanceof ScrolledDatenBereichEditorComposite )
          {
            ScrolledDatenBereichEditorComposite in = (ScrolledDatenBereichEditorComposite)control;
            if (!in._editorInhaltErzeugt )
              in.erzeugeEditorInhalt();
          }

          if (_erweitert)
          {
            updateButtonEditor(kapsel, _selectionTabSoIndex);
          }

          // System.out.println("Selektierte Kapsel: " + kapsel.getDataEmpfangen());
        }
      };
    }

    folderSo.addSelectionListener(_selectionFolderSoListener);
    
    return compTabSysObj;
  }

  private Composite createCompTabSysObj(Composite parent)
  {
    Composite compTabSysObj = new Composite(parent, SWT.NONE);
    compTabSysObj.setLayout(new GridLayout());

    Composite composite = new Composite(compTabSysObj, SWT.NONE);
    GridData gdComposite = new GridData();
    gdComposite.grabExcessHorizontalSpace = true;
    gdComposite.horizontalAlignment = GridData.FILL;
    GridData gd1 = new GridData();
    gd1.grabExcessHorizontalSpace = true;
    gd1.horizontalAlignment = GridData.FILL;
    GridData gd2 = new GridData();
    gd2.grabExcessHorizontalSpace = true;
    gd2.horizontalAlignment = GridData.FILL;
    composite.setLayout(new GridLayout(2, false));
    composite.setLayoutData(gdComposite);
    Label labelPid = new Label(composite, SWT.NONE);
    labelPid.setText("Pid*");
    Text textPid = new Text(composite, SWT.BORDER);
    textPid.setLayoutData(gd1);
    _textPid.add(textPid);

    Label labelName = new Label(composite, SWT.NONE);
//    labelName.setText("Name*");
    labelName.setText("Name");
    Text textName = new Text(composite, SWT.BORDER);
    textName.setLayoutData(gd2);
    _textName.add(textName);
    
    return compTabSysObj;

  }

  private Composite createCompositeScrolledEditor(Composite compTabSysObj)
  {
    GridData gd = new GridData();
    gd.horizontalAlignment = GridData.FILL;
    gd.grabExcessHorizontalSpace = true;
    gd.verticalAlignment = GridData.FILL;
    gd.grabExcessVerticalSpace = true;
    Composite compositeScrolledEditor = new Composite(compTabSysObj, SWT.NONE);
    compositeScrolledEditor.setLayout(new GridLayout());
    compositeScrolledEditor.setLayoutData(gd);

    GridData gd21 = new GridData();
    gd21.horizontalAlignment = GridData.FILL;
    gd21.grabExcessHorizontalSpace = true;
    gd21.verticalAlignment = GridData.FILL;
    gd21.grabExcessVerticalSpace = false;
    Composite composite2 = new Composite(compTabSysObj, SWT.NONE);
    composite2.setLayout(new GridLayout(2, false));
    composite2.setLayoutData(gd21);
    Button checkBox = new Button(composite2, SWT.CHECK);
    checkBox.setText("Optional");
    checkBox.setEnabled(false);
    _checkBox.add(checkBox);

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.END;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.CENTER;
    RowLayout rowLayout1 = new RowLayout();
    rowLayout1.pack = false;
    rowLayout1.wrap = false;
    Composite compButton = new Composite(composite2, SWT.NONE);
    compButton.setLayout(rowLayout1);
    compButton.setLayoutData(gridData);

    Button buttonNeueDaten = new Button(compButton, SWT.None);
    buttonNeueDaten.setText("Neue Daten");
    _buttonNeueDaten.add(buttonNeueDaten);
    
    buttonNeueDaten.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionNeueDaten();
        
        //_buttonNeueDaten.get(_selectionTabSoIndex).setEnabled(false);
        //_buttonLoescheDaten.get(_selectionTabSoIndex).setEnabled(true);
        updateButtonEditor(_aktuelleKapsel, _selectionTabSoIndex);
        
      }

    });

    Button buttonLoescheDaten = new Button(compButton, SWT.None);
    buttonLoescheDaten.setText("Lsche Daten");
    _buttonLoescheDaten.add(buttonLoescheDaten);
    
    buttonLoescheDaten.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionLoescheDaten();
        //_buttonNeueDaten.get(_selectionTabSoIndex).setEnabled(true);
        //_buttonLoescheDaten.get(_selectionTabSoIndex).setEnabled(false);
        
        updateButtonEditor(_aktuelleKapsel, _selectionTabSoIndex);

      }

    });
    
    return compositeScrolledEditor;
  }

  private void aktionNeueDaten()
  {

    CTabItem tab = _selectionTabAtg;

    if (tab != null)
    {
      DatenBereichEditorKapsel kapsel = (DatenBereichEditorKapsel) tab.getData( "kapsel" );

      ScrolledDatenBereichEditorComposite editor = _mapKapselEditorZuordnung.get( kapsel.getId() );

      DataEditorComposite dataEditorPanel = editor.getDataEditorPanel();
      dataEditorPanel.setVisible( false );

      
      AttributeGroup atg = null;
      DatenBereichSOKapsel elternKapsel = kapsel.getElternKapsel();
      
      if ( elternKapsel != null )
      {
        SystemObject so = elternKapsel.getSystemObjekt();
        
        if ( kapsel.isTypAtg() )
        {
          String pidAtg = kapsel.getName();
          atg = _dataModel.getAttributeGroup( pidAtg );
        }
        else if ( kapsel.isTypMenge() )
        {
          atg = _dataModel.getAttributeGroup( "atg.dynamischeMenge" );
          MutableSet menge = ((ConfigurationObject)so).getMutableSet(kapsel.getName());
          so = menge;
        }
  
        Data data = DataFactory.createData(atg);
  
        kapsel.setDataEmpfangen( data );
  
        // Data dataDefault = data.createModifiableCopy();

        TKABasisView view = TKABasisView.getView( _secondaryId );
        if ( view != null )
        {
          HierarchieObjekt ho = elternKapsel.getHierarchieObjekt();
          
          Data dataNeu = view.getModelInfo().getBildungsRegeln().getDatensatz( ho, so, atg, null, false );
          // dataDefault.setToDefault();

          editor.setVirtuell( false );
          dataEditorPanel.setDataGroup( dataNeu );
          dataEditorPanel.setVisible( true );
        }
      }
    }
  }

  private void aktionLoescheDaten()
  {
    CTabItem tab = _selectionTabAtg;

    if (tab != null)
    {
      DatenBereichEditorKapsel kapsel = (DatenBereichEditorKapsel)tab.getData("kapsel");
      if (kapsel != null)
      {
        ScrolledDatenBereichEditorComposite editor = _mapKapselEditorZuordnung.get(kapsel.getId());
        DataEditorComposite dataEditorPanel = editor.getDataEditorPanel();
        kapsel.setDataEmpfangen(null);
        dataEditorPanel.setDataGroup(null);
      }
    }
  }

  private void updateButtonEditor(DatenBereichEditorKapsel kapsel, int folderIndex)
  {
    if (kapsel != null)
    {
      DatenBereichSOKapsel elternKapsel = kapsel.getElternKapsel();
      if (elternKapsel != null)
      {
        SystemObject so = elternKapsel.getSystemObjekt();

        HierarchieObjekt ho = elternKapsel.getHierarchieObjekt();
        IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();

        TypInfo ti = hot.getTypInfo(so.getType().getPid());
        AtgInfo ai = null;
        MengeInfo mi = null;
        if (kapsel.isTypAtg())
          ai = ti.getAtgInfo(kapsel.getName());
        else
          mi = ti.getMengeInfo( kapsel.getName() );
        
        ScrolledDatenBereichEditorComposite editor = _mapKapselEditorZuordnung.get(kapsel.getId());
       
        if (ai != null)
        {
          boolean optional = ai.isOptional();

          if (_erweitert)
          {
            _checkBox.get(folderIndex).setSelection(optional);

            if (!_editable || (editor != null && !editor.isEditable()))
            {
              _buttonLoescheDaten.get(folderIndex).setEnabled(false);
              _buttonNeueDaten.get(folderIndex).setEnabled(false);
            }
            else
            {
              Data data = kapsel.getDataEmpfangen();
              if (data != null)
              {
                // Achtung: diesen Button entsprechend dem Wert optional setzen
                _buttonLoescheDaten.get(folderIndex).setEnabled(optional);
                _buttonNeueDaten.get(folderIndex).setEnabled(false);
              }
              else
              {
                _buttonLoescheDaten.get(folderIndex).setEnabled(false);
                _buttonNeueDaten.get(folderIndex).setEnabled(true);
              }
            }
          }
        }
        else if (mi != null)
        {
//          boolean optional = false;

          if (_erweitert)
          {
            _checkBox.get(folderIndex).setSelection(false);
            _buttonLoescheDaten.get(folderIndex).setEnabled(false);
            
            if (!_editable || (editor != null && !editor.isEditable()))
            {
//              _buttonLoescheDaten.get(folderIndex).setEnabled(false);
              _buttonNeueDaten.get(folderIndex).setEnabled(false);
            }
            else
            {
              Data data = kapsel.getDataEmpfangen();
              if (data != null)
              {
                // Achtung: diesen Button entsprechend dem Wert optional setzen
//                _buttonLoescheDaten.get(folderIndex).setEnabled(optional);
                _buttonNeueDaten.get(folderIndex).setEnabled(false);
              }
              else
              {
//                _buttonLoescheDaten.get(folderIndex).setEnabled(false);
                _buttonNeueDaten.get(folderIndex).setEnabled(true);
              }
            }
          }
                  
        }
        else
        {
          // keine Editor-Daten vorhanden
          if (_buttonLoescheDaten.size() > 0)
            _buttonLoescheDaten.get(folderIndex).setEnabled(false);
          if (_buttonNeueDaten.size() > 0)          
            _buttonNeueDaten.get(folderIndex).setEnabled(false);
        }
      }
    }
  }

  public ScrolledDatenBereichEditorComposite createTabPage(DatenBereichEditorKapsel kapsel, boolean setzteErstesSoItem, boolean setzteErstesAtgItem)
  {
    if ( _debug )
      System.out.println( "\ncreateTabPage zu: " + kapsel );

    if ( setzteErstesSoItem )
    {
      loescheDarstellungsBereich();
      _folder.addSelectionListener( _selectionFolderListener );
    }

    _aktuelleSOKapsel = kapsel.getElternKapsel();
    _aktuelleKapsel = kapsel;

    if ( _debug )
      System.out.println( "SOKapsel: " + _aktuelleSOKapsel.getPidSystemObjekt() + " Kapsel: "
          + kapsel.getPidSystemObjekt() );

    if ( setzteErstesAtgItem && _folder != null && _aktuelleSOKapsel != null )
    {
      HierarchieObjekt ho = _aktuelleSOKapsel.getHierarchieObjekt();
      IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
      SystemObject so = _aktuelleSOKapsel.getSystemObjekt();
      if ( so != null )
      {
        CTabItem item = new CTabItem( _folder, SWT.NONE );

        item.setData( "kapselSo", _aktuelleSOKapsel );

        GridData gridData1 = new GridData();
        gridData1.horizontalAlignment = GridData.FILL;
        gridData1.grabExcessHorizontalSpace = true;
        gridData1.grabExcessVerticalSpace = true;
        gridData1.verticalAlignment = GridData.FILL;

        ScrolledComposite scrollCompTab = new ScrolledComposite( _folder, SWT.H_SCROLL | SWT.V_SCROLL );
        scrollCompTab.setLayout( new GridLayout() );
        scrollCompTab.setLayoutData( gridData1 );

        Composite compTabSysObj = createTabSysObj( scrollCompTab );
        String itemText = hot.getName() + ": " + so.getType().getPid();
        item.setText( itemText );
        
        ConfigurationArea kb = so.getConfigurationArea();
        String itemTTText = itemText + "\nKV: " + kb.getConfigurationAuthority().getPid();
        itemTTText += "\nKB: " + kb.getPid();
        
        // vorerst nicht im Tooltip
//        ConfigurationObject co = (ConfigurationObject)so;
//        AttributeGroup atg = _dataModel.getAttributeGroup("atg.info");
//        Data data = co.getConfigurationData(atg);
//        
//        if (data != null)
//        {          
//          String info = data.getItem("kurzinfo").asTextValue().getValueText();
//          String beschreibung = data.getItem("beschreibung").asTextValue().getValueText();
//          if ( !info.equals("") && !beschreibung.equals("") )
//            itemTTText += "\nKurzinfo: " + info + "\nBeschreibung: " + beschreibung;
//        }

        item.setToolTipText( itemTTText );
        item.setControl( scrollCompTab );

        scrollCompTab.setContent( compTabSysObj );
        compTabSysObj.layout();
        scrollCompTab.setExpandHorizontal( true );
        scrollCompTab.setExpandVertical( true );
        scrollCompTab.setMinSize( compTabSysObj.computeSize( SWT.DEFAULT, SWT.DEFAULT ) );
        compTabSysObj.layout();

        if ( _folder.indexOf( item ) == 0 )
        {
          if ( _debug )
            System.out.println( "Folder auf 0.Item setzen" );
          _selectionTabSoIndex = 0;
          _folder.setSelection( item );
        }

        _textPid.get( _folder.indexOf( item ) ).setText( so.getPid() );

        TKABasisView view = TKABasisView.getView( _secondaryId );
        if ( view != null )
        {
          IBildungsRegeln bildungsRegeln = view.getModelInfo().getBildungsRegeln();
          if ( bildungsRegeln != null )
          {
            Zustand momentZustand = view.getTkaComposite().getZustandsmaschine().getMomentZustand();

            String s = momentZustand.toString();

            Text textPid = _textPid.get( _folder.indexOf( item ) );
            if ( Bearbeitungsmodus.NEUANLAGE.toString().equals( s ) )
            {
              textPid.setEditable(false);
              textPid.setData("Editable", false);
              String name = bildungsRegeln.getName( so );
              _textName.get( _folder.indexOf( item ) ).setText( name );
              String nameSO = _aktuelleSOKapsel.getSystemObjekt().getName();  // wichtig bei SystemObjektVorhanden und SystemObjektAenderbar
              _aktuelleSOKapsel.setNameSystemObjekt( name );
              if (!hot.isSystemObjektVorhanden() || !name.equals(nameSO))
                setDirty( true );

            }
            else
            {
              textPid.setEditable(true);
              textPid.setData("Editable", true);
              if ( _aktuelleSOKapsel.getNameSystemObjekt() != null )
                _textName.get( _folder.indexOf( item ) ).setText( _aktuelleSOKapsel.getNameSystemObjekt() );
              else
              {
                _textName.get( _folder.indexOf( item ) ).setText( so.getName() );
                _aktuelleSOKapsel.setNameSystemObjekt( so.getName() );
              }
            }
          }
        }

        addListeners( _folder.indexOf( item ) );

        updateButtonEditor( _aktuelleKapsel, _folder.indexOf( item ) );

      }

    }

    ScrolledDatenBereichEditorComposite in = null;
    CTabFolder folderSo = null;

    if ( _folder != null && _folder.getItemCount() > 0 )
      folderSo = _folderSo.get( _folder.getItemCount() - 1 );

    if ( folderSo != null )
    {
      CTabItem tab = new CTabItem( folderSo, SWT.NONE );

      tab.setData( "kapsel", kapsel );

      if ( _debug )
        System.out.println( "Atg/Menge zum SO: " + kapsel.getName() );

      // Atg oder Menge setzen
      tab.setText( kapsel.getName() );

      /*** ***/
      /*
      boolean editorAenderbar = true;
      SystemObject so = kapsel.getElternKapsel().getSystemObjekt();

    
      if ( kapsel.isTypAtg() )
      {
        AttributeGroup atg = _dataModel.getAttributeGroup( kapsel.getName() );
        if ( so != null && atg != null )
        {
          editorAenderbar = HierarchieObjekt.isAttributgruppeAenderbar( so, atg );
        }
      }
      else if ( kapsel.isTypMenge() )
      {
        if ( so != null )
        {
          NonMutableSet menge = ((ConfigurationObject)so).getNonMutableSet(kapsel.getName());
          if (menge != null)
          {
            editorAenderbar = HierarchieObjekt.isSystemObjektAenderbar(menge);
          }
        }
      }
      */
      
      /*** ***/
      tab.setToolTipText( kapsel.toString() /*+ "\nEditierbar: " + editorAenderbar*/ );

      in = new ScrolledDatenBereichEditorComposite( folderSo, SWT.NONE, _dataModel, _secondaryId );

      // ATG ist generell nicht nderbar -> Nicht-Editierbarkeit des Tabs setzen
      
      /*** ***/
      /*
      if ( !editorAenderbar )
        in.setEditable( false );
      */  

      if ( _mapKapselEditorZuordnung.containsKey( kapsel.getId() ) )
      {
        _mapKapselEditorZuordnung.remove( kapsel.getId() );
      }

      _mapKapselEditorZuordnung.put( kapsel.getId(), in );

      tab.setControl( in );
      in.setKapsel( kapsel );

      if ( folderSo.indexOf( tab ) == 0 )
      {
        // nur beim 1.Item den Editor aufbauen, ansonsten bei der Selektion der
        // Tabs
        in.erzeugeEditorInhalt();

        if ( _debug )
          System.out.println( "FolderSo auf 0.Item setzen" );
        _selectionTabAtg = tab;
        folderSo.setSelection( tab );
      }

    }

    return in;
  }

  public void loescheDarstellungsBereich()
    {
      if (_debug)
        System.out.println("Lsche Darstellungsbereich");
      
      if (_folder != null)
      {
        removeAllListeners();
        
        for (CTabItem itemSo : _folder.getItems())
        {
          //wird dies wirklich bentigt, oder erfolgt dies automatisch?
          int indexOf = _folder.indexOf(itemSo);
          if (indexOf > 0 && _folderSo.get(indexOf) != null)
          {
            CTabItem[] itemAtgs = _folderSo.get(indexOf).getItems();
            for (int i = 0; i < itemAtgs.length; i++)
            {
              itemAtgs[i].getControl().dispose();
              itemAtgs[i].dispose();
            }
          }
          itemSo.getControl().dispose();
          itemSo.dispose();
        }
        
        //Listen erst hier lschen, weil dies zum Dispose noch gebraucht werden
        _folderSo.clear();
  
        _checkBox.clear();
        _buttonNeueDaten.clear();
        _buttonLoescheDaten.clear();
        _textPid.clear();
        _textName.clear();
        _mapKapselEditorZuordnung.clear();
  
        _selectionTabSoIndex = -1;
        _selectionTabAtg = null;
        
        _aktuelleKapsel = null;
        _aktuelleSOKapsel = null;
        
      }
     
    }

  /**
   * Liefert den Folder fr die SystemObjekte
   * 
   * @return  Folder fr die SystemObjekte
   */
  public CTabFolder getTabFolder()
  {
    return _folder;
  }
  
  /**
   * Liefert den Folder fr die Attributgruppen zum SystemObjekt
   * 
   * @return  Folder fr die Attributgruppen
   */
  public CTabFolder getTabFolderSo()
  {
    return _selectionTabSoIndex > -1 ? _folderSo.get(_selectionTabSoIndex) : null;
  }

  /**
   * Liefert den Folder fr die Attributgruppen zum index-ten SystemObjekt
   * 
   * @return  Folder fr die Attributgruppen
   */
  public CTabFolder getTabFolderSo(int index)
  {
    return index > -1 ? _folderSo.get(index) : null;
  }
  
  /**
   * Liefert selektierten Tab mit der Attributgruppe zum SystemObjekt
   * 
   * @return  Tab mit der Attributgruppe
   */
  public CTabItem getSelektionTabAtg()
  {
    return _selectionTabAtg;
  }

  /**
   * Setzt die Selektion auf die angegebenen TabFolder
   * 
   * @param tabFolder     Index fr den TabFolder zu den SystemObjekten
   * @param tabFolderSo   Index fr den TabFolderSo zu den Attributruppen eines SystemObjekts
   */
  public void setSelektionTab(int tabFolder, int tabFolderSo)
  {
    if (_debug)
      System.out.println("setSelektionTab auf Folder " + tabFolder + " FolderSo " + tabFolderSo);
    if (tabFolder > -1 && tabFolder < _folder.getItems().length)
    {
      _selectionTabSoIndex = tabFolder;
      _folder.setSelection(_selectionTabSoIndex);
      _aktuelleSOKapsel = (DatenBereichSOKapsel)_folder.getSelection().getData("kapselSo");
      
      _folderSo.get(_selectionTabSoIndex).setSelection(tabFolderSo);
      _selectionTabAtg = _folderSo.get(_selectionTabSoIndex).getSelection();
      _aktuelleKapsel = (DatenBereichEditorKapsel)_selectionTabAtg.getData("kapsel");
      
      if (_erweitert)
      {
        updateButtonEditor(_aktuelleKapsel, _selectionTabSoIndex);
      }
    }
  }
  
  /**
   * Liefert die aktuelle Kapsel zum selektierten Tab der Attributgruppe
   * 
   * @return  aktuelle Kapsel
   */
  public DatenBereichEditorKapsel getAktuelleKapsel()
  {
    return _aktuelleKapsel;
  }

  /**
   * Liefert die Kapsel zum selektierten Tab der Attributgruppe
   * 
   * @return  aktuelle Kapsel
   */
  public DatenBereichEditorKapsel getSelektierteKapsel()
  {
    DatenBereichEditorKapsel kapsel = null;
    
    CTabItem sel = getTabFolderSo().getSelection();
    if (sel != null)
    {
      Object object = sel.getData("kapsel");      
      if (object != null && object instanceof DatenBereichEditorKapsel)
      {
        kapsel = (DatenBereichEditorKapsel)object;
      }
    }
    
    return kapsel;
  }

  /**
   * Liefert das aktuelle ScrollPanel zum selektierten Tab der Attributgruppe
   * 
   * @return  aktuelles ScrollPanel
   */
  public ScrolledDatenBereichEditorComposite getAktuellesScrollPanel()
  {
    ScrolledDatenBereichEditorComposite page = null;
    
    if (_mapKapselEditorZuordnung != null && _aktuelleKapsel != null)
    {
      page = _mapKapselEditorZuordnung.get(_aktuelleKapsel.getId());
    }
    
    return page;
  }
  
  /**
   * Liefert das aktuelle Panel zum selektierten Tab der Attributgruppe
   * 
   * @return  aktuelles Panel
   */
  public DataEditorComposite getAktuellesPanel()
  {
    DataEditorComposite panel = null;
    
    if (_mapKapselEditorZuordnung != null && _aktuelleKapsel != null)
    {
      ScrolledDatenBereichEditorComposite page = _mapKapselEditorZuordnung.get(_aktuelleKapsel.getId());
      panel = page.getDataEditorPanel();
    }
    
    return panel;
  }
  
  public Text getAktuelleTextPid()
  {
    return _selectionTabSoIndex > -1 ? _textPid.get(_selectionTabSoIndex) : null;
  }

  public Text getAktuellenTextName()
  {
    return _selectionTabSoIndex > -1 ? _textName.get(_selectionTabSoIndex) : null;
  }

  /**
   * Liefert eine Liste von {@link DatenBereichEditorKapsel}n zum selektierten SystemObjekt zurck, welche gesichert werden sollen.
   * 
   * @return Liste von {@link DatenBereichEditorKapsel}n.
   */
  public LinkedList<DatenBereichEditorKapsel> getKapselnZumSichern()
  {
    LinkedList<DatenBereichEditorKapsel> kapselListe = new LinkedList<DatenBereichEditorKapsel>();

    CTabItem[] items = _folderSo.get(_selectionTabSoIndex).getItems();
    for (CTabItem tabItem : items)
    {
      Control control = tabItem.getControl();
      if (control instanceof ScrolledDatenBereichEditorComposite)
      {
        ScrolledDatenBereichEditorComposite editor = (ScrolledDatenBereichEditorComposite)control;
        editor.synchronisiereBearbeitung();
        kapselListe.add(editor.getKapsel());
      }      
    }

    return kapselListe;
  }

  /**
   * Liefert eine Liste von {@link DatenBereichSOKapsel}n zum HierarchieObjekt zurck, welche gesichert werden sollen.
   * 
   * @return Liste von {@link DatenBereichSOKapsel}n.
   */
  public LinkedList<DatenBereichSOKapsel> getSOKapselnZumSichern()
  {
    if (_debug)
      System.out.println("getSOKapselnZumSichern: KapselListe liefern: ");
    
    LinkedList<DatenBereichSOKapsel> kapselListe = new LinkedList<DatenBereichSOKapsel>();

    CTabItem[] itemSos = _folder.getItems();
    for (CTabItem itemSo : itemSos)
    {
      CTabItem[] itemAtgs = _folderSo.get(_folder.indexOf(itemSo)).getItems();
      
      if (_debug)
        System.out.println(_folder.indexOf(itemSo) + ".ItemSo: " + itemSo.getText());
      
      for (CTabItem itemAtg : itemAtgs)
      {
        Control control = itemAtg.getControl();
        if (control instanceof ScrolledDatenBereichEditorComposite)
        {
          if (_debug)
            System.out.println("Synchronisiere Editor und Kapsel!!");
          ScrolledDatenBereichEditorComposite editor = (ScrolledDatenBereichEditorComposite)control;
          editor.synchronisiereBearbeitung();
        }
      }
      Object kapselSo = itemSo.getData("kapselSo");
      if (kapselSo instanceof DatenBereichSOKapsel)
      {
        kapselListe.add((DatenBereichSOKapsel)kapselSo);
        
        if(_debug)
          System.out.println("Kapsel nach Synchronisation: " 
              + ((DatenBereichSOKapsel)kapselSo).getPidSystemObjekt()
              + ((DatenBereichSOKapsel)kapselSo).getNameSystemObjekt());
      }
    }
    
    return kapselListe;
  }

  /**
   * Setzen des Flags, ob nderungen an den Feldern erlaubt sind 
   * 
   * @param editable  nderbar ja/nein
   */
  public void setEditable(boolean editable)
  {
    if(_debug)
      System.out.println("TabbedDatenBereichEditorComposite.setEditable(): " + _editable+ " -> " + editable);
    
    this._editable = editable;
    //Dirty-Flag setzen, falls editable false
    if (!_editable)
      this._dirty = false;

    //Tabs der Systemobjekte
    CTabItem[] items = _folder.getItems();
    for (int i = 0; i < items.length; i++)
    {
      
      if (_textName.size() > 0 && !_textName.get(i).isDisposed())
        _textName.get(i).setEditable(editable);
      if (_textPid.size() > 0 && !_textPid.get(i).isDisposed())
      {
        Object aenderbar = _textPid.get(i).getData("Editable");
        if ( aenderbar instanceof Boolean )
        {
          if ( (Boolean)aenderbar )
            _textPid.get(i).setEditable(_editable);
        }
        else
          _textPid.get(i).setEditable(_editable);
      }

      //Tabs der Attributgruppen zu einem Systemobjekt
      for (Map.Entry<String, ScrolledDatenBereichEditorComposite> me : _mapKapselEditorZuordnung.entrySet())
      {
        ScrolledDatenBereichEditorComposite editor = me.getValue();
        
        /*** ***/
        /*
        DatenBereichEditorKapsel kapsel = editor.getKapsel();
               
        if (kapsel != null)
        {
          SystemObject so = kapsel.getElternKapsel().getSystemObjekt();
          if (kapsel.isTypAtg())
          {
            AttributeGroup atg = _dataModel.getAttributeGroup( kapsel.getName() );
            boolean atgAenderbar = true;
            if ( so != null && atg != null )
            {
              atgAenderbar = HierarchieObjekt.isAttributgruppeAenderbar( so, atg );
              if ( !atgAenderbar )
                editor.setEditable( false );
              else
                editor.setEditable( editable );
            }
          }
          else if (kapsel.isTypMenge())
          {           
            boolean mengeAenderbar = true;
            if ( so != null )
            {
              NonMutableSet menge = ((ConfigurationObject)so).getNonMutableSet( kapsel.getName() );
              if (menge != null)
              {
                mengeAenderbar = HierarchieObjekt.isSystemObjektAenderbar( menge );
              }

              if ( !mengeAenderbar )
                editor.setEditable( false );
              else
                editor.setEditable( editable );
            }
          }
        }
        else
        */
        {
          editor.setEditable(editable);
        }
        
      }

      if (_erweitert)
      {
        if (!editable)
        {
          _buttonNeueDaten.get(i).setEnabled(false);
          _buttonLoescheDaten.get(i).setEnabled(false);
        }
        else
        {
          if (_selectionTabAtg != null && !_selectionTabAtg.isDisposed())
          {
            DatenBereichEditorKapsel kapsel = (DatenBereichEditorKapsel)_selectionTabAtg.getData("kapsel");

            if (kapsel != null)
            {
              updateButtonEditor(kapsel, i);
            }  
          }
        }
      }
    }

  }

  /**
   * Komponenten sind editierbar
   * 
   * @return  gibt an, ob die angezeigten Komponenten editierbar sein sollen
   */
  public boolean isEditable()
  {
    return _editable;
  }

  public void setDataModel(DataModel dataModel)
  {
    _dataModel = dataModel;
  }

  public boolean isSelectedAtgPanelDirty()
  {
    DataEditorComposite panel = getAktuellesPanel();
    if ( panel != null )
      return panel.isDirty();
    else
      return false;
  }
  
  public boolean isDirty()
  {

//    System.out.println("Is Dirty!!!: " + _dirty);
    return _dirty;
  }

  public boolean isDirty(String kapselId)
  {
    boolean result = false;
    ScrolledDatenBereichEditorComposite editor =  _mapKapselEditorZuordnung.get(kapselId);
    
    if (editor != null)
    {
      result = editor.isNeu() || editor.getDataEditorPanel().isDirty();
    }
    
    return result;
  }
  
  public void setDirty(boolean dirty)
  {
    _dirty = dirty;

    if (_button != null && _button.isEnabled() != dirty)
    {
      _button.setEnabled(dirty);
      TKABasisView view = TKABasisView.getView(_secondaryId);
      if (view != null)
      {
        view.firePropertyDirty();
      }
    }
     
    // Rcksetzen auch aller Dirty-Flags der DataEditorComposites
    if (!dirty)
    {
      Collection<ScrolledDatenBereichEditorComposite> scrollPanels = _mapKapselEditorZuordnung.values();
      for (ScrolledDatenBereichEditorComposite scrollPanel : scrollPanels)
      {
//        System.out.println("ScrollPanel: " + scrollPanel.toString());
        scrollPanel.getDataEditorPanel().setDirty(false);
      }
    }

  }

  public void setDirty(DataEditorComposite panel, boolean dirty)
  {  
      panel.setDirty(dirty);
      
      if ( _button != null && _button.isEnabled() != dirty )
      {
        _button.setEnabled(dirty);
        TKABasisView view = TKABasisView.getView(_secondaryId);
        if (view != null)
        {
          view.firePropertyDirty();
        }
      }

      //Rcksetzen des Dirty-Flags, falls alle DataEditorComposites dirty sind
      if (!dirty)
      {
        boolean allDirty = false;
        Collection<ScrolledDatenBereichEditorComposite> scrollPanels = _mapKapselEditorZuordnung.values();  
        for (ScrolledDatenBereichEditorComposite scrollPanel : scrollPanels)
        {
          if (scrollPanel.getDataEditorPanel().isDirty())
            allDirty = true;
        }
        _dirty = allDirty;
      }
      else
      {
        _dirty = dirty;
      }
    
  }

  @Override
  public void handleEvent(Event event)
  {
  
    if (_textName.contains(event.widget) )
    {
  
      if (_aktuelleSOKapsel != null && _selectionTabSoIndex > -1)
      {       
        String name = _textName.get(_selectionTabSoIndex).getText();
        String nameAktKapsel = _aktuelleSOKapsel.getNameSystemObjekt();
  
        if (name != null)
        {
          if (_debug)
          {
            System.out.println("\nhandleEvent: _textName: neuer Name: " + name);
  
            System.out.println("ndern des SO-Namens im Tab(" + _selectionTabSoIndex + ") " + _folder.getItem(_selectionTabSoIndex).getText());
            System.out.println("nderung vornehmen in der Kapsel mit SO-Pid " + _aktuelleSOKapsel.getPidSystemObjekt());
          }
          _aktuelleSOKapsel.setNameSystemObjekt(name);
                   
          if (_editable && !name.equals(nameAktKapsel))
          {
            setDirty(true);
          }
        }
  
      }
    }
    else if (_textPid.contains(event.widget))
    {      
  
      if (_aktuelleSOKapsel != null && _selectionTabSoIndex > -1)
      {
        String pid = _textPid.get(_selectionTabSoIndex).getText();
        String pidAktKapsel = _aktuelleSOKapsel.getPidSystemObjekt();
  
        if (pid != null)
        {
          if (_debug)
          {
            System.out.println("\nhandleEvent: _textPid: neue Pid: " + pid);
  
            System.out.println("ndern des SO-Pids im Tab(" + _selectionTabSoIndex + ") " + _folder.getItem(_selectionTabSoIndex).getText());
            System.out.println("nderung vornehmen in der Kapsel mit SO-Pid " + _aktuelleSOKapsel.getPidSystemObjekt());
          }          
          _aktuelleSOKapsel.setPidSystemObjekt(pid);
  
          if (_editable && !pid.equals(pidAktKapsel))
          {
            setDirty(true);
          }
        }
      }
    }
    else
    {
      // alle sonstigen Widgets: nur Dirty-Flag setzen
      CTabItem ti = _selectionTabAtg;
  
      DatenBereichEditorKapsel kapsel = (DatenBereichEditorKapsel)ti.getData("kapsel");
      String id = kapsel.getId();
  
      ScrolledDatenBereichEditorComposite scrollPanel = _mapKapselEditorZuordnung.get(id);
      if (scrollPanel == null)
        setDirty(true);
      else
      {
        // fr den DefaultParameterEditor
        DataEditorComposite panel = scrollPanel.getDataEditorPanel();
        setDirty(panel, true);
      }
  
    }
  }

  @Override
  /**
   * nderungen am Editor und Dirty-Flag durchfhren, wenn sich ein Widget insbesondere der Spinner gendert hat.
   * 
   */
  public void widgetChanged(WidgetChangedEvent e)
  {
    // diese Methode kann auf Grund der bentigten TKABasisView und damit dem Zugriff auf die Bildungsregeln
    // nicht in das DataEditorComposite verschoben werden.
    
    Widget w = e.getWidgetObjekt();
       
    if (w instanceof Spinner && !w.isDisposed() && e.getDataobjekt() != null)
    {
      // Widget ist Spinner (nicht disposed) und Daten zur Aktualisierung wurden bertragen  
      String arrayName = e.getDataobjekt().getName();

      CTabItem ti = _selectionTabAtg;
      DatenBereichEditorKapsel kapsel = null;
      if (ti != null)
        kapsel = (DatenBereichEditorKapsel)ti.getData("kapsel");
      else
        kapsel = _aktuelleKapsel;

      ScrolledDatenBereichEditorComposite scrollPanel = _mapKapselEditorZuordnung.get(kapsel.getId());
      TKABasisView view = TKABasisView.getView(_secondaryId);

      if (view != null && scrollPanel != null)
      {
        String pid = kapsel.getPidSystemObjekt();
        HierarchieObjekt ho = kapsel.getElternKapsel().getHierarchieObjekt();
        List<SystemObject> systemObjekte = ho.getSystemObjekte();
        SystemObject so = null;
        for (SystemObject soItem : systemObjekte)
        {
          if (soItem.getPid().equals(pid))
            so = soItem;
        }
        AttributeGroup atg = _dataModel.getAttributeGroup(kapsel.getName());

        DataEditorComposite dataEditorPanel = scrollPanel.getDataEditorPanel();
        Data dataArray = e.getDataobjekt();
        
        Data dataArrayNeu = view.getModelInfo().getBildungsRegeln().getDatenArrayElement(null, so, atg, dataArray, arrayName,
            -1, false);
        
        if (dataArrayNeu != null)
        {
          // neues Array-Data mit Vorbelegungen aus den Bildungsregen setzen         
          dataEditorPanel.updateArrayBox(dataArrayNeu, e.getGroupObjekt(), scrollPanel.isVirtuell(), scrollPanel.getScrolledAnzeigefeld());
        }
        else
        {
          // ohne bestimmte Vorbelegungen muss nur die Array-Gruppe neu erstellt werden
          dataEditorPanel.updateArrayBox(dataArray, e.getGroupObjekt(), scrollPanel.isVirtuell(), scrollPanel.getScrolledAnzeigefeld());
        }
      }

    }

    //Dirty-Flag setzen
    CTabItem ti = _selectionTabAtg;

    DatenBereichEditorKapsel kapsel = (DatenBereichEditorKapsel)ti.getData("kapsel");
    String id = kapsel.getId();

    ScrolledDatenBereichEditorComposite scrollPanel = _mapKapselEditorZuordnung.get(id);
    if (scrollPanel == null)
      setDirty(true);
    else
    {
      // fr den DefaultParameterEditor
      DataEditorComposite panel = scrollPanel.getDataEditorPanel();
      setDirty(panel, true);
    }

  }
  
}
