/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.datenbereich.editor.widgetChangedListener;

import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Widget;

import de.bsvrz.dav.daf.main.Data;

/**
 * Die Klasse kapselt die Verwaltungsmethoden fr die Listener-Funktionalitt 
 * des {@link WidgetChangedListener}. Diese Klasse implementiert das Singleton-Pattern
 * und stellt dadurch sicher dass der Listener nur einmal Instanziiert wird. 
 * 
 * 
 * @author Krause
 *
 */
public class WidgetChangedListener
{
  /**
   * Liste der {@link IWidgetChangedListener}, die ber eine nderung eines Widgets informiert werden mchten.
   */
  private LinkedList<IWidgetChangedListener> _widgetChangedListener = new LinkedList<IWidgetChangedListener>();
  
  /**
   * Singleton-Instanz
   */
  private static WidgetChangedListener _instanz = null;

  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  public static WidgetChangedListener getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new WidgetChangedListener();
    }
    return _instanz;
  }
    

  /**
   * Fgt einen Listener zur Liste der {@link WidgetChangedListener} hinzu.
   * 
   * @param listener
   */
  public void addWidgetChangedListener(IWidgetChangedListener listener)
  {
    if (!_widgetChangedListener.contains(listener))
    {
      _widgetChangedListener.add(listener);
    }
    else
    {
//      System.err
//          .println("Der Listener ist bereits in der Zuhrerliste vorhanden und wird nicht erneut hinzugefgt");
    }
  }
  
  /**
   * Benachrichtigt alle registrierten {@link IWidgetChangedListener} darber, dass nderungen vorhanden sind.
   *
   * @param widget auslsendes Widget.
   * @param parent Elternteil zum Widget, das informiert werden soll.
   * @param data   Dataobjekt zum Widget.
   * @param grp    Gruppenobjekt zum Widget.
   */
  public void notifyWidgetChangedListener(Widget widget, Composite parent, Data data, Group grp)
  {

    WidgetChangedEvent event = new WidgetChangedEvent(this, widget, data, grp);

    Iterator<IWidgetChangedListener> it = _widgetChangedListener.iterator();
    while (it.hasNext())
    {
      IWidgetChangedListener listener = it.next();
      if ( listener == parent )
        listener.widgetChanged(event);
    }
  }
  
  /**
   * Entfernt einen {@link IWidgetChangedListener} von der Liste der Zuhrer.
   * 
   * @param IKonsoleAusgabeUpdateListener
   *          Zu entfernender {@link IWidgetChangedListener}.
   */
  public void removeDatenModellUpdateListener(IWidgetChangedListener listener)
  {
    if (_widgetChangedListener.contains(listener))
    {
      _widgetChangedListener.remove(listener);
    }
    else
    {
//      System.err.println("Der Listener ist in der Zuhrerliste nicht vorhanden und wird somit nicht entfernt.");
    }
  }
  
}
