/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.hierarchie;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Vector;

/**
 * Es werden Informationen ber ein Array-Attribut modelliert, das von einem Hierarchieobjekttypen 
 * spezifiziert ist
 * 
 * @author Sans
 *
 */
public class ArrayInfo extends AttributInfo
{
  /**
   * Erzeugt eine neue Instanz vom Typ ArrayInfo<br>
   * Dieser Konstruktor wird fr Arrays von Attributlisten  benutzt
   * 
   * @param name Name des Array-Attributs
   * @param innereAttribute Liste der Beschreibungen der inneren Attribute
   */
  public ArrayInfo(String name, IAttributInfo[] innereAttribute)
  {
    super(name, false, false, false, true, true, true); // Array wird immer mit Lnge 0 vorbelegt
        
    if (innereAttribute != null)
    {
      _innereAttribute = new Vector<IAttributInfo>();
      _innereAttribute.addAll(Arrays.asList(innereAttribute));
    }
  }
  
  /**
   * Erzeugt eine neue Instanz vom Typ ArrayInfo<br>
   * Dieser Konstruktor wird fr Arrays von atomaren Werten benutzt
   * 
   * @param name Name des Array-Attributs
   * @param isPflichtFeld ist dieses Attribut Pflichtfeld?
   * @param isEindeutig sind die Werte dieses Attributs eindeutig?
   * @param isAutoGeneriert werden die Werte dieses Attributs automatisch generiert?
   * @param isAenderbarNeu sind die Werte dieses Attributs nderbar im Zustand 'Neu'?
   * @param isAenderbarBearb sind die Werte dieses Attributs nderbar im Zustand 'Bearbeiten'?
   * @param isVorbelegt sind die Werte dieses Attributs vorbelegt)
   */
  public ArrayInfo(String name, boolean isPflichtFeld, boolean isEindeutig, 
      boolean isAutoGeneriert, boolean isAenderbarNeu, boolean isAenderbarBearb)
  {
    super(name, isPflichtFeld, isEindeutig, isAutoGeneriert, isAenderbarNeu, 
        isAenderbarBearb, true); // Array wird immer mit Lnge 0 vorbelegt
  }
     
  /**
   * Liste der inneren Attribute 
   */
  private List<IAttributInfo> _innereAttribute = null;
  
  @Override
  public List<IAttributInfo> getInnereAttribute()
  {
    return _innereAttribute != null ? 
        Collections.unmodifiableList(_innereAttribute) : super.getInnereAttribute();
  }
  
  @Override
  public boolean hasInnereAttribute()
  {
    return _innereAttribute != null ? 
        _innereAttribute.size() > 0 : super.hasInnereAttribute();
  }
  
  @Override
  public boolean isArray()
  {
    return true;
  }
  
}
