/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.hierarchie;

import java.util.List;

/**
 * Schnittstelle, die Informationen ber ein Attribut modelliert, das von einem Hierarchieobjekttypen 
 * spezifiziert ist
 * 
 * @author Sans
 *
 */
public interface IAttributInfo
{
  /**
   * Bestimmt den Namen des Attributs
   * 
   * @return den Namen des Attributs
   */
  public String getName();
  
  /**
   * Beschreibt das Attribut ein Pflichtfeld?
   *   
   * @return true, wenn das Attribut ein Pflichtfeld beschreibt, false sonst
   */
  public boolean isPflichtFeld();
 
  /**
   * Mssen die Werte des Attributs eindeutig sein?
   * 
   * @return true, wenn eindeutige Werte verlangt werden, false sonst
   */
  public boolean isEindeutig();
  
  /**
   * Werden die Werte des Attributs automatisch generiert?
   * 
   * @return true, wenn die Werte automatisch generiert werden, false sonst
   */
  public boolean isAutoGeneriert();
  
  /**
   * Sind die Werte des Attributs nderbar im Zustand 'Neu'?
   * 
   * @return true, wenn die Werte im Zustand 'Neu' nderbar sind, false sonst
   */
  public boolean isAenderbarNeu();
  
  /**
   * Sind die Werte des Attributs nderbar im Zustand 'Bearbeiten'?
   * 
   * @return true, wenn die Werte im Zustand 'Bearbeiten' nderbar sind, false sonst
   */
  public boolean isAenderbarBearb();
   
  /**
   * Werden die Werte des Attributs vorbelegt?
   * 
   * @return true, wenn die Werte vorbelegt werden, false sonst
   */
  public boolean isVorbelegt();
   
  /**
   * Bestimmt die Liste innererer Attribute
   * 
   * @return ie Liste innererer Attribute
   */
  public List<IAttributInfo> getInnereAttribute();
 
  /**
   * Hat dieses Attribut innere Attribute?
   * 
   * @return true beim Vorhandensein innerer Attribute, false sonst
   */
  public boolean hasInnereAttribute();
  
  /**
   * Ist dieses Attribut ein Array?
   * 
   * @return true, wenn es sich bei diesem Attribut um ein Array handelt, false sonst
   */
  public boolean isArray();
}
