/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.hierarchie;

import java.util.List;

import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Die Schnittstelle IHierarchieObjektTyp modelliert den Hierarchieobjekttyp (HOT) von 
 * Hierarchieobjekten
 * 
 * @author Sans
 *
 */
public interface IHierarchieObjektTyp
{
  /**
   * Bestimmt den Namen dieses HOT
   * 
   * @return den Namen dieses HOT
   */
  public String getName();
  
  /**
   * Bestimmt die Systemobjekttypen, also die Typen der Systemobjekte, die Systemobjekten
   * eines Hierarchieobjekts von diesem HOT zugeordnet sind.
   * 
   * @return die Liste der Systemobjekttypen
   */
  public List<String> getTypen();
  
  /**
   * Bestimmt die TypInfo-Instanz fr einen Systemobjekttyp dieses HOT. Diese TypInfo-Instanz
   * ist die Kopfstruktur fr weitere detailierte Informationen, siehe {@link TypInfo}.<br>
   * Diese Methode liefert nur die Informataionen fr den direkten Typ, d. h. dass der bergebene
   * Typ evtl. ein Untertyp des gespeicheten Typs sein kann, wird NICHt bercksichtigt
   * 
   * @param typ die PID des Systemobjekttyps, fr den die TypInfo-Instanz bestimmt werden soll
   * @return die TypInfo-Instanz zum spezifizierten Systemobjekttyp
   */
  public TypInfo getTypInfo(String typ);
  
  /**
   * Bestimmt die TypInfo-Instanz fr einen Systemobjekttyp dieses HOT. Diese TypInfo-Instanz
   * ist die Kopfstruktur fr weitere detailierte Informationen, siehe {@link TypInfo}.<br>
   * Diese Methode liefert nur die Informataionen auch, wenn der bergebene Typ ein Untertyp 
   * des gespeicheten Typs ist.
   * 
   * @param sot Systemobjekttyp, fr den die TypInfo-Instanz bestimmt werden soll
   * @return die TypInfo-Instanz zum spezifizierten Systemobjekttyp
   */
  public TypInfo getTypInfo(SystemObjectType sot);
  
  /**
   * Bestimmt den (Grund-)Namen der Symboldatei dieses HOT
   * 
   * @return den (Grund-)Namen der Symboldatei dieses HOT
   */
  public String getSymbolName();
  
  /**
   * Flagge, die anzeigt ob das SystemObjekt bereits vorhanden ist.
   * Dies impliziert dass das/die entsprechende(n) SystemObjekte zum HO nicht angelegt werden.
   * 
   * @return true, wenn das SystemObjekt bereits vorhanden ist
   */
  public boolean isSystemObjektVorhanden();
  
  /**
   * Flagge, die anzeigt dass das SystemObjekt nicht nderbar ist.
   * 
   * @return true, wenn das SystemObjekt nderbar ist, false sonst
   */
  public boolean isSystemObjektNichtAenderbar();
  
}