/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.hierarchie;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.SystemObject;

public class SystemObjektDaten
{
  private String _pid;
  private String _name;
  
  private Map<String, Data> _atgDaten = new HashMap<String, Data>();
  private List<String> _atg = new Vector<String>();
  private Map<String, Data> _dynMengenDaten = new HashMap<String, Data>();
  private List<String> _dynMengen = new Vector<String>();
  private Set<String> _datenGeaendert = new HashSet<String>();
  private Map<String, List<SystemObject>> _mengenElemente = new HashMap<String, List<SystemObject>>();
  private List<String> _mengen = new Vector<String>();
    
  public String getPid()
  {
    return _pid;
  }

  public void setPid(String pid)
  {
    _pid = pid;
  }

  public String getName()
  {
    return _name;
  }

  public void setName(String name)
  {
    _name = name;
  }

  public void setAtgDaten(String atg, Data data)
  {
    _atg.remove(atg);
    if (data != null)
    {  
      _atg.add(atg);
      _atgDaten.put(atg, data);
    }
    else
    {
      _atgDaten.remove(atg);
    }
  }
  
  public List<String> getAtg()
  {
    return Collections.unmodifiableList(_atg);
  }
  
  public Data getAtgDaten(String atg)
  {
    return _atgDaten.get(atg);
  }

  
  public void setDynMengenDaten(String menge, Data data)
  {
    _dynMengen.remove(menge);
    if (data != null)
    {  
      _dynMengen.add(menge);
      _dynMengenDaten.put(menge, data);
    }
    else
    {
      _dynMengenDaten.remove(menge);
    }
  }
  
  public List<String> getDynMengen()
  {
    return Collections.unmodifiableList(_dynMengen);
  }
  
  public Data getDynMengenDaten(String menge)
  {
    return _dynMengenDaten.get(menge);
  }
  
  public void setDatenGeaendert(String daten, boolean isGeaendert)
  {
    if (isGeaendert)
    {
      _datenGeaendert.add(daten);
    }
    else
    {
      _datenGeaendert.remove(daten);
    }
  }
  
  public boolean isDatenGeaendert(String daten)
  {
    return _datenGeaendert.contains(daten);
  }
  
  public void setMengenElemente(String menge, List<SystemObject> elemente)
  {
    _mengen.remove(menge);
    if (elemente != null)
    {  
      _mengen.add(menge);
      _mengenElemente.put(menge, new Vector<SystemObject>(elemente));
    }
    else
    {
      _mengenElemente.remove(menge);
    }
  }
  
  public List<String> getMengen()
  {
    return Collections.unmodifiableList(_mengen);
  }
  
  public List<SystemObject> getMengenElemente(String menge)
  {
    return _mengenElemente.get(menge);
  }
}
