/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.hierarchie;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

public class TypInfo
{
  private String _typ;
  private Map<String, AtgInfo> _atgInfos = new HashMap<String, AtgInfo>();
  private List<String> _atgs = new Vector<String>();
  private Map<String, MengeInfo> _mengeInfos = new HashMap<String, MengeInfo>();
  private List<String> _mengen = new Vector<String>();
  
  public TypInfo(String typ, AtgInfo[] atgInfos, MengeInfo[] mengeInfos)
  {
    _typ = typ;
    
    if (atgInfos != null)
    {
      for (AtgInfo info: atgInfos)
      {
        _atgs.add(info.getAtg());
        _atgInfos.put(info.getAtg(), info);
      }
    }
    
    if (mengeInfos != null)
    {
      for (MengeInfo info: mengeInfos)
      {
        _mengen.add(info.getName());
        _mengeInfos.put(info.getName(), info);
      }
    }
    
  }
  
  public String getTyp()
  {
    return _typ;
  }
  
  public List<String> getAtgs()
  {
    return Collections.unmodifiableList(_atgs);
  }
  
  public AtgInfo getAtgInfo(String atg)
  {
    return _atgInfos.get(atg);
  }

  public List<String> getMengen()
  {
    return _mengen;
  }

  public List<String> getDynMengen()
  {
    List<String> dynMengen = new Vector<String>();

    for (String menge: _mengen)
    {
      MengeInfo info = getMengeInfo(menge);
      if ( info.isDynamisch() )
      {  
        dynMengen.add( info.getName() );
      }  
    }
//    for (Map.Entry<String, MengeInfo> mengeInfo : _mengeInfos.entrySet())
//    {
//      MengeInfo info = mengeInfo.getValue();
//      if ( info.isDynamisch() )
//        dynMengen.add( info.getName() );
//    }
    return dynMengen;
  }
  
  public MengeInfo getMengeInfo(String menge)
  {
    return _mengeInfos.get(menge);
  }

}
