/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.navigator;


import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import de.bsvrz.buv.plugin.tkabasis.navigator.navigatorModell.INavigatorElement;

/**
 * Bereitet die Inhalte des TreeViewers auf. {@link ITreeContentProvider}
 * 
 * @author Krause
 * 
 */
public class NavigatorContentProvider implements ITreeContentProvider
{
  /**
   * Flag zum Debuggen
   */
  protected static final boolean _debug = false;

  /**
   * Baumstruktur selbst 
   */
//  private TreeViewer m_viewer;

  /**
   * Liefert die Kinder eines Elements
   */
  public Object[] getChildren( Object parentElement )
  {
    if ( parentElement instanceof INavigatorElement )
      return ((INavigatorElement) parentElement).getChildren();
    return new Object[0];
  }

  /**
   * Liefert den Elternteil eines Elements
   */
  public Object getParent( Object element )
  {
    if ( element instanceof INavigatorElement )
      return ((INavigatorElement) element).getParent();
    return null;
  }

  /**
   * Liefert, ob ein Element Kinder hat
   */
  public boolean hasChildren( Object element )
  {
    // Kinder holen
    Object[] obj = getChildren( element );

    // Rckgabe, ob das Element Kinder hat
    return obj == null ? false : obj.length > 0;
  }

  /**
   * Liefert die Root-Elemente, kann daher meist getChildren() aufrufen
   */
  public Object[] getElements( Object inputElement )
  {
    return getChildren( inputElement );
  }

  /**
   * Verwerfen aller erzeugter Ressourcen
   */
  public void dispose()
  {
  }

  /**
   * Called when the input changes
   * 
   * @param arg0
   *            the viewer
   * @param arg1
   *            the old input
   * @param arg2
   *            the new input
   */
  public void inputChanged( Viewer viewer, Object oldInput, Object newInput )
  {
    if ( _debug )
      System.out.println( "InputChanged: " + newInput );

//    this.m_viewer = (TreeViewer) viewer;
  }

}