/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.regeln;

/**
 * Die Klasse EingabeVorbelegung modelliert Vorgabe-Werte verschiedener Typen:<br>
 * Texte und Texlisten<br>
 * Instanzen vom Typ Eingabevorbelegung werden direkt vom Bildungsregel-Assistenten
 * dazu verwendet:<br>
 * - entsprechende Controls, Editfelder oder Comboboxen zu erzeugen<br>
 * - die Controls mit den entsprechenden Daten zu versorgen<br>
 *  
 * @author Sans
 *
 */
public class EingabeVorbelegung
{
  public EingabeVorbelegung(String wert, String[] liste, boolean leerErlaubt, boolean editierbar)
  {
    _wert = wert;
    _liste = liste;
    _leerErlaubt = leerErlaubt;
    _editierbar = editierbar;
  }
  
  /**
   * Erzeugt eine Eingabevorbeleung, die ein Textfeld darstellt. Wird im Bildungsregel-Assistenten
   * als Textfeld mit text wert angezeigt
   * 
   * @param wert der Text, den das Textfeld im Bildungsregel-Assistenten erhlt
   * @param leerErlaubt Flag, das angibt, ob leere Eingaben erlaubt sind
   */
  public EingabeVorbelegung(String wert, boolean leerErlaubt)
  {
    this(wert, null, leerErlaubt, true);
  }
  
  /**
   * Erzeugt eine Eingabevorbeleung, die ein Textliste darstellt. Wird im Bildungsregel-Assistenten
   * als Combobox mit der bergebenen Textliste angezeigt, der bergebene Wert wird im Texfeld der
   * Combobox angezeigt
   * 
   * @param wert der Text, den das Textfeld der Combobox im Bildungsregel-Assistenten erhlt
   * @param liste die Texte, die das Listenfeld der Combobox im Bildungsregel-Assistenten erhlt
   */
  public EingabeVorbelegung(String wert, String[] liste)
  {
    this(wert, liste, false, true);
  }
  
  /**
   * Erzeugt eine Eingabevorbeleung, die ein Textliste darstellt. Wird im Bildungsregel-Assistenten
   * als Combobox mit der bergebenen Textliste angezeigt, der bergebene Wert wird im Texfeld der
   * Combobox angezeigt
   * 
   * @param wert der Text, den das Textfeld der Combobox im Bildungsregel-Assistenten erhlt
   * @param liste die Texte, die das Listenfeld der Combobox im Bildungsregel-Assistenten erhlt
   * @param leerErlaubt Flag, das angibt ob fr das Texfeld der Combobox eine leere Eingabe erlaubt ist
   */
  public EingabeVorbelegung(String wert, String[] liste, boolean leerErlaubt)
  {
    this(wert, liste, leerErlaubt, true);
  }
  
  public EingabeVorbelegung(String[] liste)
  {
    this(null, liste, false, true);
  }
  
  /**
   * Erzeugt eine Eingabevorbeleung, die ein Textliste darstellt. Wird im Bildungsregel-Assistenten
   * als Combobox mit der bergebenen Textliste angezeigt, der bergebene Wert wird im Texfeld der
   * Combobox angezeigt
   * 
   * @param wert der Text, den das Textfeld der Combobox im Bildungsregel-Assistenten erhlt
   * @param liste die Texte, die das Listenfeld der Combobox im Bildungsregel-Assistenten erhlt
   * @param leerErlaubt Flag, das angibt ob fr das Texfeld der Combobox eine leere Eingabe erlaubt ist
   * @param editierbar Flag, das angibt ob das Texfeld der Combobox editierbar ist
   */
  public EingabeVorbelegung(String[] liste, boolean leerErlaubt, boolean editierbar)
  {
    this(null, liste, leerErlaubt, editierbar);
  }
  
  /**
   * Aktueller Datenwert dieser Instanz
   */
  private String _wert;
  
  /**
   * Liste der Texte, wenn diese Instanz eine Textliste darstellt
   */
  private String[] _liste;
  
  /**
   * Ist den entsprechenden Wert im Bildungsregel-Assistenten eine leere Eingabe erlaubt?
   */
  private boolean _leerErlaubt;
  
  /**
   * Ist der entsprechende Wert im Bildungsregel-Assistenten editierbar?
   */
  private boolean _editierbar;
  

  private boolean _isNummer = false;
  private long _minNummer = Long.MIN_VALUE;
  private long _maxNummer = Long.MAX_VALUE;

  public boolean isNummer()
  {
    return _isNummer;
  }
  
  public long getMinNummer()
  {
    return _minNummer;
  }

  public long getMaxNummer()
  {
    return _maxNummer;
  }
  
  public EingabeVorbelegung(long minNummer, long maxNummer)
  {
    this(null, null, false, true);
    _isNummer = true;
    _minNummer = minNummer;
    _maxNummer = maxNummer;
  }


  public void setWert(String wert)
  {
    _wert = wert;
  }

  
  
  
  /**
   * 
   * @return Den aktuellen Text-Wert dieser EingabeVorbelegung
   */
  public String getWert()
  {
    return _wert != null ? _wert : BildungsRegelnTools.LEERER_WERT;
  }
  
  /**
   * 
   * @return Wenn diese EingabeVorbelegung eine die Textliste ist, die Liste der Test-Werte,
   * ansonsten eine leere Liste
   */
  public String[] getListe()
  {
    return _liste != null ? _liste : BildungsRegelnTools.LEERE_TEXT_LISTE;
  }
  
  /**
   * 
   * @return ist den entsprechenden Wert im Bildungsregel-Assistenten eine leere Eingabe erlaubt?
   */
  public boolean isLeerErlaubt()
  {
    return _leerErlaubt;
  }
  
  /**
   * 
   * @return ist der entsprechende Wert im Bildungsregel-Assistenten editierbar?
   */
  public boolean isEditierbar()
  {
    return _editierbar;
  }
}