/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.tkaUpdateManagement;

import java.util.List;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Suchen mindestens eines Systemobjekts aus einer Liste von Systemobjekten in den Hierarchieobjekten 
 * einer Hierarchie
 * 
 * @author Sans
 *
 */
public class FindeObjektInHierarchie extends HierarchieIterierer
{
  /**
   * Liste der Systemobjekte, nach denen gesucht wird
   */
  private List<SystemObject> _liste;
  
  private boolean _einSystemobjektGefunden;
  
  /**
   * Wurde mindestens eines der bergenenen Systemobjekte in den Hierarchieobjekten der Hierarchie
   * gefunden?
   * 
   * @return true, wenn mindesten eines der Systemobjekte in der Hierarchie vorkommt, 
   * false sonst
   */
  public boolean isEinSystemobjektGefunden()
  {
    return _einSystemobjektGefunden;
  }

  /**
   * Erzeugt eine neue Instanz und durchsucht die bergebene Hierarchie. Nach Durchlauf des
   * Konstruktors kann das Ergebnis mit isEinSystemobjektGefunden() abgefragt werden
   * 
   * @param liste die Liste der Systemobjekte, nach denen gesucht wird
   * @param hierarchie die zu durchsuchende Hierarchie
   */
  public FindeObjektInHierarchie(List<SystemObject> liste, IHierarchie hierarchie)
  {
    super(hierarchie);
    _liste = liste;
    if (liste != null && !liste.isEmpty())
    {  
      alleObjekteIterieren();
    }  
  }
  
  @Override
  protected boolean iteration(Object objekt)
  {
    if (objekt instanceof HierarchieObjekt)
    {
      HierarchieObjekt ho = (HierarchieObjekt)objekt;
      
      for (SystemObject so: ho.getSystemObjekte())
      {
        if (_liste.contains(so))
        {
//          System.out.println("Gefunden: " + so.getPid());
          _einSystemobjektGefunden = true;
          return false;
        }
      }
    }  
    return true;
  }
  
}
