/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 *//*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkamqgrp;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.bsvrz.buv.plugin.tkabasis.DataFactory;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.regeln.BildungsRegelnBeobachter;
import de.bsvrz.buv.plugin.tkabasis.regeln.BildungsRegelnTools;
import de.bsvrz.buv.plugin.tkabasis.regeln.EingabeVorbelegung;
import de.bsvrz.buv.plugin.tkabasis.regeln.IBildungsRegeln;
import de.bsvrz.dav.daf.communication.dataRepresentation.AttributeBaseValueDataFactory;
import de.bsvrz.dav.daf.communication.dataRepresentation.AttributeHelper;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.SystemObject;

public class BildungsRegelnAllgemeinMQGrp implements IBildungsRegeln
{
  private IHierarchie _hierarchie;

  protected Map<String, String> _praefixe = new HashMap<String, String>();

  protected Map<IHierarchieObjektTyp, String[]> _felder = new HashMap<IHierarchieObjektTyp, String[]>();

  protected Map<IHierarchieObjektTyp, Map<String, String>> _feldWerte = new HashMap<IHierarchieObjektTyp, Map<String, String>>();

  protected Map<String, Map<String, String>> _historie = new HashMap<String, Map<String, String>>();

  public IHierarchie getHierarchie()
  {
    return _hierarchie;
  }

  public void setHierarchie(IHierarchie hierarchie)
  {
    _hierarchie = hierarchie;
  }

  protected void initPraefixe()
  {
    _praefixe.put("typ.messStellenGruppe", "messStellenGruppe");
    _praefixe.put("typ.messStelle", "messStelle");
  }

  protected void initFelder()
  {
    _felder.put(HOT_MQGrp.MessStellenGruppe, new String[] { "Bezeichnung" });
    _felder.put(HOT_MQGrp.MessStelle, new String[] { "Bezeichnung" });
    _felder.put(HOT_MQGrp.MQPruefling, new String[] { "Bezeichnung" });
    _felder.put(HOT_MQGrp.MQZufahrt, new String[] { "Bezeichnung" });
    _felder.put(HOT_MQGrp.MQAbfahrt, new String[] { "Bezeichnung" });
  }

  public BildungsRegelnAllgemeinMQGrp()
  {
    initPraefixe();
    initFelder();
  }

  public String getPraefix(String pidTypNeuesObjekt)
  {
    String result = _praefixe.get(pidTypNeuesObjekt);

    return result != null ? result : BildungsRegelnTools.LEERER_WERT;
  }

  public String[] getEingabeFelder(IHierarchieObjektTyp hotNeuesObjekt)
  {
    String[] result = _felder.get(hotNeuesObjekt);

    return result != null ? result : BildungsRegelnTools.LEERE_TEXT_LISTE;
  }

  private static EingabeVorbelegung STANDARD_VORBELEGUNG = new EingabeVorbelegung(null, false);

  public EingabeVorbelegung getEingabeVorbelegung(IHierarchieObjektTyp hotNeuesObjekt, String eingabeFeld,
      HierarchieObjekt verbundenesHO)
  {
    if (Arrays.asList(_felder.get(hotNeuesObjekt)).contains(eingabeFeld))
    {
// Keine Vererbung der Feldinhalte  		
//      if (verbundenesHO != null)
//      {
//        Map<String, String> werte = _historie.get(verbundenesHO.getSystemObjekte().get(0).getPid());
//
//        if (werte != null)
//        {
//          String text = werte.get(eingabeFeld);
//          if (text != null)
//          {
//            return new EingabeVorbelegung(text, false);
//          }
//        }
//      }
      return STANDARD_VORBELEGUNG;
    }

    return BildungsRegelnTools.LEERE_VORBELEGUNG;
  }

  public void setWert(IHierarchieObjektTyp hotNeuesObjekt, String eingabeFeld, String wert)
  {
    Map<String, String> werte = _feldWerte.get(hotNeuesObjekt);

    if (werte == null)
    {
      werte = new HashMap<String, String>();
      _feldWerte.put(hotNeuesObjekt, werte);
    }

    werte.put(eingabeFeld, wert);
  }

  public String getWert(IHierarchieObjektTyp hotNeuesObjekt, String eingabeFeld)
  {
    Map<String, String> werte = _feldWerte.get(hotNeuesObjekt);

    if (werte == null)
    {
      return BildungsRegelnTools.LEERER_WERT;
    }

    String result = werte.get(eingabeFeld);

    return result != null ? result : BildungsRegelnTools.LEERER_WERT;
  }

  public boolean setWerte(IHierarchieObjektTyp hotNeuesObjekt, HierarchieObjekt uebergeordnetesHO,
      HierarchieObjekt vorlageHO)
  {
    boolean result = false;

    if (HOT_MQGrp.MQPruefling.equals(hotNeuesObjekt) || HOT_MQGrp.MQZufahrt.equals(hotNeuesObjekt)
        || HOT_MQGrp.MQAbfahrt.equals(hotNeuesObjekt))
    {
      if (uebergeordnetesHO != null)
      {
        EingabeVorbelegung belegung = getEingabeVorbelegung(hotNeuesObjekt, "Bezeichnung", uebergeordnetesHO);
        String wert = belegung.getWert();

        Map<IHierarchieObjektTyp, Integer> typMap = new HashMap<IHierarchieObjektTyp, Integer>();

        for (HierarchieObjekt ho : getHierarchie().getUntergeordneteObjekte(uebergeordnetesHO))
        {
          IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
          Integer anzahl = typMap.get(hot);
          typMap.put(hot, anzahl == null ? 1 : anzahl.intValue() + 1);
        }
        Integer suffix = typMap.get(hotNeuesObjekt);
        // if (suffix != null)
        // {
        // wert += (suffix.intValue() + 1);
        // }
        //        
        // setWert(hotNeuesObjekt, "Bezeichnung", wert);
        //
        // if (suffix == null)
        // {
        // result = true;
        // }

        if (suffix == null)
        {
          setWert(hotNeuesObjekt, "Bezeichnung", wert);
          result = true;
        }
      }
    }

    return result;
  }

  public String getPidRegelText(IHierarchieObjektTyp hot)
  {
    String felder[] = _felder.get(hot);
    if (felder != null)
    {
      String typPid = hot.getTypen().get(0);
      String result = getPraefix(typPid) + ".Bezeichnung";

      if (typPid.equals("typ.messStelle"))
      {
        result += ".<PID bergeordnetes Systemobjekt>";
      }

      return result;
    }
    return BildungsRegelnTools.LEERER_WERT;
  }

  public String getNameRegelText(IHierarchieObjektTyp hot)
  {
    String felder[] = _felder.get(hot);
    if (felder != null)
    {
      return "<PID Systemobjekt>";
    }
    return BildungsRegelnTools.LEERER_WERT;
  }

  public String[] getPids(IHierarchieObjektTyp hotNeuesObjekt, HierarchieObjekt uebergeordnetesHO)
  {
    String[] result = new String[1];
    String pidTypNeuesObjekt = hotNeuesObjekt.getTypen().get(0);

    StringBuffer buffer = new StringBuffer();

    buffer.append(getPraefix(pidTypNeuesObjekt));
    buffer.append('.');

    String bezeichnung = getWert(hotNeuesObjekt, "Bezeichnung");
    buffer.append(BildungsRegelnTools.getPidText(bezeichnung, false, false));

    if (uebergeordnetesHO != null)
    {
      buffer.append('.');
      buffer.append(uebergeordnetesHO.getSystemObjekte().get(0).getPid());
    }

    String pid = buffer.toString();
    Map<String, String> werte = _feldWerte.get(hotNeuesObjekt);
    if (werte != null && !werte.isEmpty())
    {
      _historie.put(pid, new HashMap<String, String>(werte));
    }

    result[0] = pid;

    return result;
  }

  public String getName(SystemObject objekt)
  {
    String pid = objekt.getPid();

    if (true /* _historie.get(pid) != null */)
    {
      return pid;
    }

    return BildungsRegelnTools.LEERER_WERT;
  }

  public Data getDatensatz(HierarchieObjekt ho, SystemObject objekt, AttributeGroup atg, Data daten,
      boolean nurPflichtFelder)
  {
    if (!ho.getSystemObjekte().contains(objekt))
    {
      throw new IllegalArgumentException("Systemobjekt " + objekt.getPid() + " ist nicht im Hierarchieobjet enthalten");
    }

    if (!objekt.getType().getAttributeGroups().contains(atg))
    {
      throw new IllegalArgumentException("Attributgruppe " + atg.getName() + " kann nicht mit Systemobjekt "
          + objekt.getPid() + "verwendet werden ");
    }

    IHierarchie hierarchie = getHierarchie();
    // ConfigDataModel dataModel = hierarchie.getDataModel();

    Data data;

    if (daten == null)
    {
      data = DataFactory.createData(atg);
      // Default und undefiniert Werte setzen
      data.setToDefault();
    }
    else
    {
    	ObjectLookup lookUp = LookupFactory.createLookupForModifiableVersion(_hierarchie.getDataModel());
  		data = DataFactory.createModifiableCopy(daten, lookUp);
    }

    if (objekt.getType().getPid().equals("typ.messStellenGruppe"))
    {
      if (atg.getPid().equals("atg.messStellenGruppe"))
      {
        data.getItem("MessStellen").asArray().setLength(0);
        data.getItem("SystematischeDetektorfehler").asTextValue().setText("Ja");
      }
    }

    return data;
  }

  public void beobachterAnmelden(BildungsRegelnBeobachter beobachter)
  {
  }

  public void alleBeobachterInformieren(String feldName, String wert)
  {
  }

  public void beobachterAbmelden(BildungsRegelnBeobachter beobachter)
  {
  }

  public void wertGeaendert(String feldName, String wert)
  {
  }

  @Override
  public Data getDatenArrayElement(HierarchieObjekt ho, SystemObject objekt, AttributeGroup atg, Data daten,
      String arrayName, int arrayIndex, boolean nurPflichtFelder)
  {
    // TODO Auto-generated method stub
    return null;
  }

}
