/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkamqgrp;

import java.util.List;

import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.konfigass.IKonfigAssDatenModell;
import de.bsvrz.buv.plugin.konfigass.MeldungenSpeicher;
import de.bsvrz.buv.plugin.konfigass.datenModellUpdateListener.DatenModellUpdateEvent;
import de.bsvrz.buv.plugin.konfigass.datenModellUpdateListener.DatenModellUpdateListener;
import de.bsvrz.buv.plugin.konfigass.datenModellUpdateListener.IDatenModellUpdateListener;
import de.bsvrz.buv.plugin.tkabasis.ITKABasisComposite;
import de.bsvrz.buv.plugin.tkabasis.TKABasisComposite;
import de.bsvrz.buv.plugin.tkabasis.TKABasisView;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.TypInfo;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.IPruefungen;
import de.bsvrz.buv.plugin.tkabasis.regeln.IBildungsRegeln;
import de.bsvrz.buv.plugin.tkamqgrp.preferences.DefsPara;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;

public class TKAMQGrpPart implements ITKABasisComposite, IKonfigAssDatenModell, IDatenModellUpdateListener {

	private IAction _action;

	public TKAMQGrpPart() {
		_action = erzeugeAktion();
		DatenModellUpdateListener.getInstanz().addDatenModellUpdateListener(this);		
	}

	private IAction erzeugeAktion() {
		IAction aktion = new Action() {
			public void run() {
				IWorkbenchWindow dw = PlatformUI.getWorkbench()
						.getActiveWorkbenchWindow();
				IWorkbenchPage page = dw.getActivePage();

				try {
					starteView(page);
				} catch (PartInitException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		};
		aktion.setText("TKA-MQGrp");
		aktion.setId("de.bsvrz.buv.plugin.tkamqgrp");

		return aktion;
	}

	@Override
	public String holeName() {
		// TODO Auto-generated method stub
		return "TKA-MQGrp";
	}

	@Override
	public String holePluginId() {
		// TODO Auto-generated method stub
		return Activator.PLUGIN_ID;
	}

	@Override
	public String holeBeschreibung() {
		// TODO Auto-generated method stub
		return "Teilkonfigurationsassistent fr das Teilmodell Messstellengruppe/Messstelle";
	}

	@Override
	public void setzeKonfigDatenModell(ConfigDataModel dataModel, String pfad) {
		// TODO Auto-generated method stub
		IPreferenceStore preferences = Activator.getDefault()
				.getPreferenceStore();
		HOT_MQGrp[] hotMqs = HOT_MQGrp.values();

		for (int i = 0; i < hotMqs.length; i++) {
			HOT_MQGrp hotMQ = hotMqs[i];

			boolean istAtgMandatory = true;

			List<String> typen = hotMQ.getTypen();
			for (String typ : typen) {

				TypInfo ti = hotMQ.getTypInfo(typ);
				List<String> atgs = ti.getAtgs();

				for (String atg : atgs) {
					AtgInfo ai = ti.getAtgInfo(atg);
					istAtgMandatory = ai.isMandatory();
					if (!istAtgMandatory) {
						boolean optional = preferences.getBoolean(typ
								+ "," + atg);
						ai.setOptional(optional);
					}
				}
			}
		}

		if (dataModel != null) {
			DatenSpeicher.getInstanz().setzeConfigDataModel(dataModel);
			DatenSpeicher.getInstanz().setzeDateiPfad(pfad);
//			DatenSpeicher.getInstanz().setzeSecId(secId);

			if (DatenSpeicher.getInstanz().getHierarchieManager() == null) {
				DatenSpeicher.getInstanz().setHierarchieManager(
						new HierachieManagerMQGrp());
			}

			if (DatenSpeicher.getInstanz().getBildungsRegeln() == null) {
				String klassenName = Activator.getDefault()
						.getPreferenceStore().getString(
								DefsPara.KlasseBildungsregeln);
				String defKlassenName = Activator.getDefault()
						.getPreferenceStore().getDefaultString(
								DefsPara.KlasseBildungsregeln);
				IBildungsRegeln bildungsRegeln = (IBildungsRegeln) ladeKlassenInstanz(
						klassenName, IBildungsRegeln.class,
						DefsPara.NameBildungsregeln, null, defKlassenName);

				if (bildungsRegeln != null) {
					DatenSpeicher.getInstanz()
							.setBildungsRegeln(bildungsRegeln);
				}
			}

			if (DatenSpeicher.getInstanz().getPruefungen() == null) {
				String klassenName = Activator.getDefault()
						.getPreferenceStore().getString(
								DefsPara.KlassePruefungen);
				String defKlassenName = Activator.getDefault()
						.getPreferenceStore().getDefaultString(
								DefsPara.KlassePruefungen);
				IPruefungen pruefungen = (IPruefungen) ladeKlassenInstanz(
						klassenName, IPruefungen.class,
						DefsPara.NamePruefungen, null, defKlassenName);

				if (pruefungen != null) {
					DatenSpeicher.getInstanz().setPruefungen(pruefungen);
				}
			}

			TKABasisView view = DatenSpeicher.getInstanz().holeTKABasisView();

			if (view != null) {

				view.setModelInfo(DatenSpeicher.getInstanz());

				TKABasisComposite tkaComposite = DatenSpeicher.getInstanz()
						.holeTKABasisView().getTkaComposite();

				if (tkaComposite != null && !tkaComposite.isDisposed()) {
					tkaComposite.setConfigDataModel(dataModel, pfad);
				}

			} else {
				IWorkbenchWindow dw = PlatformUI.getWorkbench()
						.getActiveWorkbenchWindow();
				IWorkbenchPage page = dw.getActivePage();
				IViewReference ref = page.findViewReference(TKABasisView.ID,
						"MQGrp");
				if (ref != null) {
					if (_action != null) {
						_action.run();
					}
				}
			}
		} else {
		  
		  DatenSpeicher.getInstanz().setzeConfigDataModel(dataModel);
      DatenSpeicher.getInstanz().setzeDateiPfad(pfad);
//      DatenSpeicher.getInstanz().setzeSecId(secId);
		  
			TKABasisView view = DatenSpeicher.getInstanz().holeTKABasisView();

			if (view != null) {

				view.setModelInfo(DatenSpeicher.getInstanz());

				TKABasisComposite tkaComposite = DatenSpeicher.getInstanz()
						.holeTKABasisView().getTkaComposite();

				if (tkaComposite != null && !tkaComposite.isDisposed()) {
					tkaComposite.setConfigDataModel(null, "");

				}

			}
		}		
	}

	@Override
	public void erzeugeTKABasisView(TKABasisView view) {
		// TODO Auto-generated method stub
		if (view != null) {
			DatenSpeicher.getInstanz().setzeTKABasisView(view);
		}
	}

	@Override
	public void setReiterName(String name) {
		// TODO Auto-generated method stub

	}

	@Override
	public void ladeKonfigDatenModell(String pfad) {
		// TODO Auto-generated method stub

	}

	protected static Object ladeKlassenInstanz(String klassenName, Class typ,
			String text, Object instanz, String defKlassenName) {
		Object result = null;

		Class klasse = null;

		MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK,
				"Laden der Klassen-Instanz fr " + text + " vom Typ \""
						+ klassenName + "\"", null);
		MeldungenSpeicher.getInstanz().setStatus(status);

		try {
			klasse = Class.forName(klassenName);
		} catch (Throwable e) {
			status
					.merge(new Status(Status.WARNING, Activator.PLUGIN_ID,
							"Fr \"" + klassenName
									+ "\" konnte keine Klasse geladen werden",
							e));

			if (instanz == null && !klassenName.equals(defKlassenName)) {
				try {
					klasse = Class.forName(defKlassenName);

					status.merge(new Status(Status.INFO, Activator.PLUGIN_ID,
							"Als Klassen-Instanz fr " + text + " wird \""
									+ defKlassenName + "\" benutzt", null));
				} catch (Throwable ex) {
					status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
							"Es ist keine Klassen-Instanz fr " + text
									+ " geladen", ex));
				}
			} else {
				if (instanz != null) {
					status.merge(new Status(Status.INFO, Activator.PLUGIN_ID,
							"Als Klassen-Instanz fr " + text
									+ " wird weiterhin \"" + instanz.getClass()
									+ "\" benutzt", null));
				} else {
					status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
							"Es ist keine Klassen-Instanz fr " + text
									+ " geladen", e));
				}

			}
		}

		if (klasse != null) {
			try {
				result = klasse.newInstance();
			} catch (Exception e) {
				status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID,
						"Die Klasse \"" + klassenName
								+ "\" kann nicht instanziiert werden", e));
			}

			if (result != null) {
				if (!typ.isAssignableFrom(klasse)) {
					status
							.merge(new Status(
									Status.ERROR,
									Activator.PLUGIN_ID,
									"Die Klasse \""
											+ klassenName
											+ "\" ist keine Unterklasse von/erfllt nicht die Schnittstelle \""
											+ typ.getName() + "\""));
				}
			}
		}

//		IWorkbenchWindow dw = PlatformUI.getWorkbench()
//				.getActiveWorkbenchWindow();
//		IWorkbenchPage page = dw.getActivePage();
//		IViewReference ref = page.findViewReference(ErrorView.ID);
//
//		if (ref != null) {
//			ErrorView view = (ErrorView) ref.getPart(false);
//
//			if (view != null) {
//				view.getProtokoll().hinzufuegenMeldung(status,
//						System.currentTimeMillis());
//			}
//			// Activator.getDefault().getLog().log(status);
//		}
    
    
 
		MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(status, System.currentTimeMillis(), false);
				
		return result;
	}

	@Deprecated
	static IBildungsRegeln ladeBildungsRegeln(String klassenName,
			IBildungsRegeln bildungsRegeln) {
		IBildungsRegeln result = null;

		Class klasse = null;

		MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK,
				"Laden der Klassen-Instanz fr Bildungsregeln vom Typ \""
						+ klassenName + "\"", null);

		try {
			klasse = Class.forName(klassenName);
		} catch (Throwable e) {
			status
					.add(new Status(Status.WARNING, Activator.PLUGIN_ID,
							"Fr \"" + klassenName
									+ "\" konnte keine Klasse geladen werden",
							e));

			if (bildungsRegeln == null) {
				try {
					klasse = Class.forName(DefsPara.KlasseBildungsregeln);

					status.add(new Status(Status.INFO, Activator.PLUGIN_ID,
							"Als Klassen-Instanz fr Bildungsregeln wird \""
									+ DefsPara.KlasseBildungsregeln
									+ "\" benutzt", null));
				} catch (Throwable ex) {
					status
							.add(new Status(
									Status.ERROR,
									Activator.PLUGIN_ID,
									"Es ist keine Klassen-Instanz fr Bildungsregeln geladen",
									ex));
				}
			} else {
				status.add(new Status(Status.INFO, Activator.PLUGIN_ID,
						"Als Klassen-Instanz fr Bildungsregeln wird weiterhin \""
								+ bildungsRegeln.getClass() + "\" benutzt",
						null));
			}
		}

		if (klasse != null) {
			try {
				result = (IBildungsRegeln) klasse.newInstance();
			} catch (Exception e) {
				if (e instanceof ClassCastException) {
					status
							.add(new Status(
									Status.ERROR,
									Activator.PLUGIN_ID,
									"Die Klasse \""
											+ klassenName
											+ "\" erfllt nicht die Schnittstelle \""
											+ IBildungsRegeln.class.getName()
											+ "\"", e));
				} else {
					status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,
							"Die Klasse \"" + klassenName
									+ "\" kann nicht instanziiert werden", e));
				}
			}
		}

//		IWorkbenchWindow dw = PlatformUI.getWorkbench()
//				.getActiveWorkbenchWindow();
//		IWorkbenchPage page = dw.getActivePage();
//		IViewReference ref = page.findViewReference(ErrorView.ID);
//
//		if (ref != null) {
//			ErrorView view = (ErrorView) ref.getPart(false);
//
//			if (view != null) {			  
//				view.getProtokoll().hinzufuegenMeldung(status,
//						System.currentTimeMillis());
//			}
//			// Activator.getDefault().getLog().log(status);
//		}
		
		
		
		return result;

	}

	static {
		Activator.getDefault().getPreferenceStore().addPropertyChangeListener(
				new IPropertyChangeListener() {
					public void propertyChange(PropertyChangeEvent event) {
						// Wenn (noch) kein HierarchieManager zugewiesen ist, erfolgt 
						// hier keine weitere Aktion - das Klassenladen erfolgt dann, 
						// sobald ein neues Datenmodell und damit der HierarchieManager
						// zugewiesen wird
						if (DatenSpeicher.getInstanz().getHierarchieManager() == null)
						{
							return;
						}
						if (event.getProperty().equals(
								DefsPara.KlasseBildungsregeln)) {
							String defKlassenName = Activator.getDefault()
									.getPreferenceStore().getDefaultString(
											DefsPara.KlasseBildungsregeln);
							IBildungsRegeln bildungsRegelnAlt = DatenSpeicher
									.getInstanz().getBildungsRegeln();
							IBildungsRegeln bildungsRegelnNeu = (IBildungsRegeln) ladeKlassenInstanz(
									event.getNewValue().toString(),
									IBildungsRegeln.class,
									DefsPara.NameBildungsregeln,
									bildungsRegelnAlt, defKlassenName);
							if (bildungsRegelnNeu != null) {
								IHierarchie hierarchie = DatenSpeicher
										.getInstanz().getHierarchieManager()
										.getHierarchie();
								bildungsRegelnNeu.setHierarchie(hierarchie);
								DatenSpeicher.getInstanz().setBildungsRegeln(
										bildungsRegelnNeu);
							}
						}
						if (event.getProperty().equals(
								DefsPara.KlassePruefungen)) {
							String defKlassenName = Activator.getDefault()
									.getPreferenceStore().getDefaultString(
											DefsPara.KlassePruefungen);
							IPruefungen pruefungenAlt = DatenSpeicher
									.getInstanz().getPruefungen();
							IPruefungen pruefungenNeu = (IPruefungen) ladeKlassenInstanz(
									event.getNewValue().toString(),
									IPruefungen.class, DefsPara.NamePruefungen,
									pruefungenAlt, defKlassenName);
							if (pruefungenNeu != null) {
								IHierarchie hierarchie = DatenSpeicher
										.getInstanz().getHierarchieManager()
										.getHierarchie();
								pruefungenNeu.setHierarchie(hierarchie);
								DatenSpeicher.getInstanz().setPruefungen(
										pruefungenNeu);
							}
						}
					}
				});
	}

	@Override
	public IAction holeAktion() {

		return _action;
	}

	private void starteView(IWorkbenchPage page) throws PartInitException {
		// String id = DatenSpeicher.getInstanz().holeTKABasisView().ID;

		IPreferenceStore preferences = Activator.getDefault()
				.getPreferenceStore();
		HOT_MQGrp[] hotMqs = HOT_MQGrp.values();

		for (int i = 0; i < hotMqs.length; i++) {
			HOT_MQGrp hotMQ = hotMqs[i];

			boolean istAtgMandatory = true;

			List<String> typen = hotMQ.getTypen();
			for (String typ : typen) {

				TypInfo ti = hotMQ.getTypInfo(typ);
				List<String> atgs = ti.getAtgs();

				for (String atg : atgs) {
					AtgInfo ai = ti.getAtgInfo(atg);
					istAtgMandatory = ai.isMandatory();
					if (!istAtgMandatory) {
						boolean optional = preferences.getBoolean(typ
								+ "," + atg);
						ai.setOptional(optional);
					}
				}
			}
		}

		String id = TKABasisView.ID;
		TKABasisView view = (TKABasisView) page.showView(id, "MQGrp",
				IWorkbenchPage.VIEW_CREATE);
		
		//Hilfefunktion (F1-Taste)		
		view.setHelp("MQGrp", Activator.PLUGIN_ID + ".tkamqgrp");

		DatenSpeicher.getInstanz().setzeTKABasisView(view);

		// TKABasisView view = DatenSpeicher.getInstanz().holeTKABasisView();

		ConfigDataModel dataModel = DatenSpeicher.getInstanz()
				.holeConfigDataModel();
		
		String pfad = DatenSpeicher.getInstanz().holeDateiPfad();

		if (DatenSpeicher.getInstanz().getHierarchieManager() == null) {
			DatenSpeicher.getInstanz().setHierarchieManager(
					new HierachieManagerMQGrp());
		}

		if (DatenSpeicher.getInstanz().getBildungsRegeln() == null) {
			String klassenName = Activator.getDefault().getPreferenceStore()
					.getString(DefsPara.KlasseBildungsregeln);
			String defKlassenName = Activator.getDefault().getPreferenceStore()
					.getDefaultString(DefsPara.KlasseBildungsregeln);

			IBildungsRegeln bildungsRegeln = (IBildungsRegeln) TKAMQGrpPart
					.ladeKlassenInstanz(klassenName, IBildungsRegeln.class,
							DefsPara.NameBildungsregeln, null, defKlassenName);

			if (bildungsRegeln != null) {
				DatenSpeicher.getInstanz().setBildungsRegeln(bildungsRegeln);
			}
		}

		if (DatenSpeicher.getInstanz().getPruefungen() == null) {
			String klassenName = Activator.getDefault().getPreferenceStore()
					.getString(DefsPara.KlassePruefungen);
			String defKlassenName = Activator.getDefault().getPreferenceStore()
					.getDefaultString(DefsPara.KlassePruefungen);

			IPruefungen pruefungen = (IPruefungen) TKAMQGrpPart
					.ladeKlassenInstanz(klassenName, IPruefungen.class,
							DefsPara.NamePruefungen, null, defKlassenName);

			if (pruefungen != null) {
				DatenSpeicher.getInstanz().setPruefungen(pruefungen);
			}
		}

		view.setModelInfo(DatenSpeicher.getInstanz());

		TKABasisComposite tkaComposite = view.getTkaComposite();

		tkaComposite.setConfigDataModel(dataModel, pfad);
	}

	@Override
	public void datenModellUpdate(DatenModellUpdateEvent e) {
		// TODO Auto-generated method stub
		List<SystemObject> liste = e.getListeSystemObjekte();
		
		for (SystemObject so : liste) {
//			System.out.println("DatenModell update fr : " + so);			
		}
		
		
	}

//  @Override
//  public String holePfad()
//  {
//    // TODO Auto-generated method stub
//    return DatenSpeicher.getInstanz().holeDateiPfad();
//  }
//
//  @Override
//  public String holeSecId()
//  {
//    // TODO Auto-generated method stub
//    return DatenSpeicher.getInstanz().holeSecId();
//  }

}
