/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkanba;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Vector;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.tkabasis.Activator;
import de.bsvrz.buv.plugin.tkabasis.DataFactory;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HODaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HOHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HOMitStatus;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieManager;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.MengeInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.SystemObjektDaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.TypInfo;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.IPruefungen;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.LoeschenChecker;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.PruefungenTools;
import de.bsvrz.buv.plugin.tkabasis.regeln.IBildungsRegeln;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeGroupUsage;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.ObjectSetType;
import de.bsvrz.dav.daf.main.config.ObjectTimeSpecification;
import de.bsvrz.dav.daf.main.config.ReferenceType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.puk.config.configFile.datamodel.ConfigConfigurationObject;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigSystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

public class HierarchieManagerNBA implements IHierarchieManager
{
	/**
   * Debug-Logger fr Logging-Ausgaben
   */
	private final static Debug _debug = Debug.getLogger();
  
  private IHierarchie _hierarchie;
  
  private static IHierarchieObjektTyp[] EBENE1 = new IHierarchieObjektTyp[] {HOT_NBA.Masche};                                                                      
  private static IHierarchieObjektTyp[] EBENE2 = new IHierarchieObjektTyp[] {HOT_NBA.HauptRoute, HOT_NBA.NebenRoute};
  private static IHierarchieObjektTyp[] EBENE20 = new IHierarchieObjektTyp[] {HOT_NBA.HauptRoute};
  private static IHierarchieObjektTyp[] EBENE21 = new IHierarchieObjektTyp[] {HOT_NBA.NebenRoute};
  private static IHierarchieObjektTyp[] EBENE3 = new IHierarchieObjektTyp[] {HOT_NBA.RoutenStueck};
  private static IHierarchieObjektTyp[] EBENE4 = new IHierarchieObjektTyp[] {HOT_NBA.ASS};
	
  public IHierarchieObjektTyp[] getUntergeordneteHOTs(Object objekt)
  {
    if (objekt instanceof ConfigurationArea)
    {
      return EBENE1;
    }
    else if (objekt instanceof HierarchieObjekt)
    {
      HierarchieObjekt ho = (HierarchieObjekt)objekt;
      IHierarchieObjektTyp hot = null;
      if ( ho != null )
        hot = ho.getHierarchieObjektTyp();
      
      if (hot.equals(HOT_NBA.Masche))
      {
        return EBENE2;
      }
      else if (hot.equals(HOT_NBA.HauptRoute) || hot.equals(HOT_NBA.NebenRoute))
      {
        return EBENE3;
      }
      else if (hot.equals(HOT_NBA.RoutenStueck))
      {
        return EBENE4;
      }      
    }

    return null;
  }
  
  public IHierarchie getHierarchie()
	{
		return _hierarchie;
	}
  
	public void setHierarchie(IHierarchie hierarchie)
	{
		_hierarchie = hierarchie;
	}
  
  public void bestimmeUntergeordneteHOs( Object objekt )
  {
    ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion( _hierarchie.getDataModel() );

    bestimmeUntergeordneteHOs( objekt, lookup );
  }
	
  public void bestimmeUntergeordneteHOs(Object objekt, ObjectLookup lookup)
  {
    ConfigDataModel dataModel = _hierarchie.getDataModel();
    
    if (objekt instanceof ConfigurationArea)  
    {
      ConfigurationArea kb = (ConfigurationArea)objekt;
      Collection<SystemObjectType> typeSpez = new Vector<SystemObjectType>();
      
      for (IHierarchieObjektTyp hot: EBENE1)
      {
        String typPid = hot.getTypen().get(0);
        SystemObjectType  typ = _hierarchie.getDataModel().getType(typPid);
        
        if (typ == null)
        {
          throw new RuntimeException("Typ " + typPid + " wurde im aktuellen Datemmodell nicht gefunden");
        }
        
        typeSpez.add( typ );
               
        for (SystemObject so : kb.getObjects( typeSpez, ObjectTimeSpecification.valid() ))
        {
          ConfigurationObject ko = (ConfigurationObject) so;
          if ( !(ko.getNotValidSince() >= ko.getConfigurationArea().getModifiableVersion()) )
          // Objekt wird in zuknftiger Version nicht ungltig
          {
            HierarchieObjekt ho = new HierarchieObjekt( hot, new SystemObject[] { so }, EBENE1 );
            //Hinzufgen HierarchieObjekt-Verbindung
            _hierarchie.objektHinzufuegen( kb, ho );
            bestimmeUntergeordneteHOs( ho );
          }
        }

        for (SystemObject so : kb.getNewObjects())
        {
          if ( so.isOfType( typ ) )
          {
            HierarchieObjekt ho = new HierarchieObjekt( hot, new SystemObject[] { so }, EBENE1 );
            //Hinzufgen HierarchieObjekt-Verbindung
            _hierarchie.objektHinzufuegen( kb, ho );
            bestimmeUntergeordneteHOs( ho );
          }
        }
      }
    }
    else if (objekt instanceof HierarchieObjekt)
    {
      HierarchieObjekt ho = (HierarchieObjekt)objekt;     
      IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp(); 
      
      if (hot.equals(HOT_NBA.Masche))
      {
        ConfigSystemObject kso = (ConfigSystemObject)ho.getSystemObjekt(0);
        Data data = kso.getConfigurationData( dataModel.getAttributeGroup( "atg.routen"), lookup );
        
        if (data != null)
        {
          SystemObject[] soArray = data.getItem( "Route" ).asReferenceArray().getSystemObjectArray();
          List<SystemObject> liste = Arrays.asList( soArray );

          int index = 0;
          for (SystemObject so : liste)
          {
            if ( so != null )
            {
              if ( so.isOfType( "typ.route" ) )
              {
                IHierarchieObjektTyp hotKind = null;
                if ( index == 0 )
                  hotKind = HOT_NBA.HauptRoute;
                else
                  hotKind = HOT_NBA.NebenRoute;

                HierarchieObjekt kind = new HierarchieObjekt( hotKind, new SystemObject[] { so },
                    getUntergeordneteHOTs( objekt ) );
                //Hinzufgen HierarchieObjekt-Verbindung
                _hierarchie.objektHinzufuegen( ho, kind );
                bestimmeUntergeordneteHOs( kind );
              }
              else
              {
                // Log Fehler: Mengenelement falscher Typ
                _debug.warning( "Unzulssiger Objekttyp " + so.getPid() + " in Attributgruppe Routen vom Objekt " + kso.getPid() );
              }
            }
            index++;
          }
        }
        else
        {
          // Log Fehler: Menge fehlt
          _debug.warning("Attributgruppe Routen nicht gefunden fr Objekt " + kso.getPid());
        }
      }
      else if (hot.equals(HOT_NBA.HauptRoute) || hot.equals(HOT_NBA.NebenRoute))
      {
        ConfigSystemObject kso = (ConfigSystemObject)ho.getSystemObjekt(0);
        Data data = kso.getConfigurationData( dataModel.getAttributeGroup( "atg.bestehtAusLinienObjekten"), lookup );
        
        if (data != null)
        {
          SystemObject[] soArray = data.getItem( "LinienReferenz" ).asReferenceArray().getSystemObjectArray();
          List<SystemObject> liste = Arrays.asList( soArray );

          for (SystemObject so : liste)
          {
            if ( so != null )
            {
              if ( so.isOfType( "typ.routenStck" ) )
              {
                IHierarchieObjektTyp hotKind = HOT_NBA.RoutenStueck;

                HierarchieObjekt kind = new HierarchieObjekt( hotKind, new SystemObject[] { so },
                    getUntergeordneteHOTs( objekt ) );
                //Hinzufgen HierarchieObjekt-Verbindung
                _hierarchie.objektHinzufuegen( ho, kind );
                bestimmeUntergeordneteHOs( kind );
              }
              else
              {
                // Log Fehler: Mengenelement falscher Typ
                _debug.warning( "Unzulssiger Objekttyp " + so.getPid() + " in Attributgruppe BestehtAusLinienObjekten vom Objekt " + kso.getPid() );
              }
            }
          }
        }
        else
        {
          // Log Fehler: Menge fehlt
          _debug.warning("Attributgruppe BestehtAusLinienObjekten nicht gefunden fr Objekt " + kso.getPid());
        }
      }
      else if (hot.equals(HOT_NBA.RoutenStueck))
      {
        ConfigSystemObject kso = (ConfigSystemObject)ho.getSystemObjekt(0);
        Data data = kso.getConfigurationData( dataModel.getAttributeGroup( "atg.bestehtAusLinienObjekten"), lookup );
        
        if (data != null)
        {
          SystemObject[] soArray = data.getItem( "LinienReferenz" ).asReferenceArray().getSystemObjectArray();
          List<SystemObject> liste = Arrays.asList( soArray );

          for (SystemObject so : liste)
          {
            if ( so != null )
            {
              if ( so.isOfType( "typ.ueresStraenSegment" ) )
              {
//                System.out.println("KB von typ.ueresStraenSegment: " + so.getConfigurationArea().getPidOrId());
                IHierarchieObjektTyp hotKind = HOT_NBA.ASS;

                HierarchieObjekt kind = new HierarchieObjekt( hotKind, new SystemObject[] { so },
                    getUntergeordneteHOTs( objekt ) );
                //Hinzufgen HierarchieObjekt-Verbindung
                _hierarchie.objektHinzufuegen( ho, kind );
                // keine weitere Hierarchie-Ebene
//                bestimmeUntergeordneteHOs( kind );
              }
              else
              {
                // Log Fehler: Mengenelement falscher Typ
                _debug.warning( "Unzulssiger Objekttyp " + so.getPid() + " in Attributgruppe ueresStraenSegmente vom Objekt " + kso.getPid() );
              }
            }
          }
        }
        else
        {
          // Log Fehler: Menge fehlt
          _debug.warning("Attributgruppe ueresStraenSegmente nicht gefunden fr Objekt " + kso.getPid());
        }
      
      }
      else
      {
        _debug.warning("bergebenes HO " + ho + " ist vom unzulssigen (HO-)Typ + " +
            ho.getHierarchieObjektTyp());
      }
    }
    else
    {
      _debug.warning("bergebenes Objekt " + objekt + " ist von unzulssigem Typ");
    }
  }
    
  private List<ObjectSet> erzeugeMengen(IHierarchieObjektTyp hot, SystemObjectType typ, 
  		ConfigurationArea kb, ConfigDataModel dataModel, MultiStatus status)
  {
  	List<ObjectSet> result = new Vector<ObjectSet>();
		
		for (String mengenName: hot.getTypInfo(typ.getPid()).getMengen())
		{
			MengeInfo info = hot.getTypInfo(typ.getPid()).getMengeInfo(mengenName);
			
			SystemObjectType mengenTyp = dataModel.getType(info.getTyp());
			ObjectSet menge;
			try
			{
				menge = (ObjectSet)kb.createConfigurationObject((ConfigurationObjectType)mengenTyp, "", mengenName, null);
				
				status.add(new Status(Status.OK, Activator.PLUGIN_ID, 
						"Menge " + mengenName + " vom Typ " + mengenTyp.getPid() + " erzeugt"));
				
			}
			catch (ConfigurationChangeException e)
			{
				status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Fehler beim Erzeugen der Menge " 
						+ mengenName + " vom Typ " + mengenTyp.getName() + " erzeugt", e));
				return result;
			}
			
			result.add(menge);
		}
		
		return result;
  }
  
  private SystemObject erzeugeSystemObjekt(String pid, ConfigurationObjectType typ, ConfigurationArea kb,
  		List<ObjectSet> mengen, MultiStatus status)
  {
	  SystemObject result = null;
		
		try
		{
			result = kb.createConfigurationObject(typ, pid, "", mengen);
						
			if (mengen != null && !mengen.isEmpty())
			{
				MultiStatus unterStatus = new MultiStatus(Activator.PLUGIN_ID, IStatus.OK,
						"Systemobjekt " +	pid + " vom Typ " + typ.getPid() + " erzeugt", null);
				status.add(unterStatus);
						
				for (ObjectSet set: mengen)
				{
					unterStatus.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Menge " + set.getName() + 
							" dem Systemobjekt " + pid + " vom Typ " + typ.getPid() + " zugewiesen"));
				}
			}
			else
			{
				status.add(new Status(Status.OK, Activator.PLUGIN_ID, 
						"Systemobjekt " + pid + " vom Typ " + typ.getPid() + " erzeugt"));
			}
		}
		catch (ConfigurationChangeException e)
		{
			status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, 
					"Systemobjekt " + pid + " vom Typ " + typ.getPid() + " konnte nicht erzeugt werden", e));
		}
		
		return result;
  }

    
  public HOMitStatus erzeugeNeuesHO(ConfigurationArea kb, Object vater, String pid[], IHierarchieObjektTyp hot, boolean pruefeEindeutigkeit)
  {
    // System.out.println( "erzeugeNeuesHO zu hot " + hot + " in " + vater );

    ConfigDataModel dataModel = _hierarchie.getDataModel();

    ConfigurationObjectType typ = (ConfigurationObjectType) dataModel.getType( hot.getTypen().get( 0 ) );

    if ( typ == null )
    {
      throw new IllegalArgumentException( "Fr " + hot.getTypen().get( 0 )
          + " kann kein Typ im Datenmodell bestimmt werden" );
    }

    MultiStatus status = new MultiStatus( Activator.PLUGIN_ID, Status.OK, "Erzeuge neues Hierarchieobjekt vom Typ "
        + hot, null );

    HierarchieObjekt ho = null;

    // Hierarchieobjekt (sowie gegebenenfalls zugehriges Systemobjekt)
    // erzeugen, sowie SO in die Menge bzw. ins Array eintragen

    // MengenListe im HOT ermitteln
    List<String> mengenListe = hot.getTypInfo( typ.getPid() ).getMengen();
    List<ObjectSet> mengen = null;
    if ( mengenListe != null && mengenListe.size() == 0 )
      mengenListe = null;

    if ( !status.matches( IStatus.ERROR | IStatus.CANCEL ) )
    {
      SystemObject so = null;

      // System.out.println( "isSystemObjektVorhanden zu hot " + hot.getName() +
      // " : " + hot.isSystemObjektVorhanden() );
      if ( !hot.isSystemObjektVorhanden() )
      {
        // zugehriges Systemobjekt erzeugen
        if (pruefeEindeutigkeit)
          status.add( new PruefungenTools( dataModel ).pruefePid( pid[0] ) );
        
        if ( !status.matches( IStatus.ERROR | IStatus.CANCEL ) )
        {
          // Menge erzeugen, falls im HOT vorhanden
          if ( mengenListe != null )
            mengen = erzeugeMengen( hot, typ, kb, dataModel, status );
          
          so = erzeugeSystemObjekt( pid[0], typ, kb, mengen, status );
        }
      }
      else
      {
        // Menge erzeugen, falls im HOT vorhanden
//        if ( mengenListe != null )
//          mengen = erzeugeMengen( hot, typ, kb, dataModel, status );
        
        // zugehriges Systemobjekt ermitteln
        so = dataModel.getObject( pid[0] );
        if ( so == null )
        {
          // System.out.println( "-->>So ist Null!!" );
          Collection<ConfigurationArea> areas = dataModel.getAllConfigurationAreas().values();
          for (ConfigurationArea ca : areas)
          {
            Collection<SystemObject> elements = ca.getNewObjects();
            for (SystemObject soNeu : elements)
            {
              if ( pid[0].equals( soNeu.getPid() ) )
                so = soNeu;
            }
          }
        }
      }

      if ( so != null && !status.matches( IStatus.ERROR | IStatus.CANCEL ) )
      {
        // Systemobjekt zum HO beim Array des Vaters eintragen und speichern
        if ( mengenListe == null )
          setzeHOArrayZuordnung( so, vater, status );

        if ( !status.matches( IStatus.ERROR | IStatus.CANCEL ) )
        {
          ho = new HierarchieObjekt( hot, new SystemObject[] { so }, getUntergeordneteHOTs(vater) );
          // Hinzufgen HierarchieObjekt-Verbindung
          getHierarchie().objektHinzufuegen( vater, ho );
        }
        else
        {
          if (!hot.isSystemObjektVorhanden())
          {
            try
            {
              so.invalidate();
            }
            catch (ConfigurationChangeException e)
            {
              // Keine weitere Fehlerbehebung mglich
            }
          }
        }

      }

    }

    return new HOMitStatus( ho, status );
  }
  
  /**
   * Setze Systemobjekt des HierarchieObjekts in der Menge oder im Array des bergeordneten Objekts
   * und HierarchieObjekt-Verbindung
   * 
   * @param ho
   * @param vater
   * @param status
   */
  private void setzeHOZuordnung( HierarchieObjekt ho, Object vater, MultiStatus status )
  {
    IHierarchie hierarchie = getHierarchie();
    SystemObject so = ho.getSystemObjekt( 0 );

    setzeHOArrayZuordnung( so, vater, status );
    
    // Hinzufgen HierarchieObjekt-Verbindung
    if ( !status.matches( IStatus.CANCEL | IStatus.ERROR ) )
      hierarchie.objektHinzufuegen( vater, ho );

  }
  
  /**
  * Systemobjekt des Hierarchieobjekts in das 1.Array der 1.Attributgruppe des 1.Systemobjekts beim Vater eintragen,
  * falls dieses nicht bereits vorhanden ist
  * 
  * @param so       Systemobjekt, das eingefgt werden soll
  * @param vater    Vater-Objekt
  * @param status   Status
  */
  private void setzeHOArrayZuordnung( SystemObject so, Object vater, MultiStatus status )
  {
    if (vater instanceof HierarchieObjekt)
    {
      HierarchieObjekt ho = (HierarchieObjekt)vater;
      IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();

      // 1.Systemobjekt des Hierarchieobjektes ermitteln
      SystemObject vaterSo = ho.getSystemObjekt(0);
      String pidSo = vaterSo.getType().getPid();
      
      // Pid der 1.Attributgruppe und Name der Attribute aus dem Hierarchieobjekttyp ermitteln
      String atgPid = hot.getTypInfo( pidSo ).getAtgs().get( 0 );
      AtgInfo atgInfo = hot.getTypInfo( pidSo ).getAtgInfo( atgPid );
      List<String> attNamenListe = atgInfo.getAttribute();

      AttributeGroup atg = _hierarchie.getDataModel().getAttributeGroup(atgPid);
      
      ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(getHierarchie().getDataModel());
      Data data = ((ConfigConfigurationObject)vaterSo).getConfigurationData(atg, lookup);
      
      if (data == null)
      {
        data = DataFactory.createData(atg);
        // Default und undefiniert Werte setzen
        data.setToDefault();
      }

      // 1.Array-Element aus den Attributen ermitteln
      String arrayName = "";     
      for (String attName : attNamenListe)
      {
        if (data.getItem( attName ).isArray())
        {
          arrayName = attName;
          break;
        }
      }
      
      if (arrayName.isEmpty())
      {
        status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
            + " konnte nicht als Referenz in " + vaterSo.getPid() + " gesetzt werden, weil kein Array gefunden wurde."));
        return;
      }
      
      ReferenceArray ra = data.getReferenceArray(arrayName);
      int len = ra.getLength();
      
      Array array = data.getItem( arrayName ).asArray();
      if (array.getMaxCount() != 0 && array.getMaxCount() < (len+1))
      {
        status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
            + " konnte nicht als Referenz in " + vaterSo.getPid() + " gesetzt werden, weil die maximale Anzahl von Array-Elementen erreicht ist."));
        return;
      }
      
      if ( Arrays.asList( ra.getSystemObjectArray() ).contains( so ) )
      {
        status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
            + " konnte nicht als Referenz in " + vaterSo.getPid() + " gesetzt werden, weil dieses bereits vorhanden ist."));
//        System.out.println("setzeHOZuordnung: Status: " + status);
        return;        
      }
           
      ra.setLength(len + 1);
      ra.getReferenceValue(len).setSystemObject(so);
      
//      IPruefungen pruefInstanz = DatenSpeicher.getInstanz().getPruefungen();
//      if ( !so.isOfType( "typ.masche" ) || so.isOfType( "typ.masche" ) && len > 1 )
//        status.merge(pruefInstanz.pruefeDaten( ho, vaterSo, atgInfo, data ));
//      
//      if (status.matches( IStatus.ERROR | IStatus.CANCEL ))
//        return;
      
      try
      {
        if (!HierarchieObjekt.isAttributgruppeAenderbar(vaterSo, atg))
        {  
          vaterSo = behandleSystemObjektAenderung(ho, (ConfigurationObject)vaterSo, vaterSo.getPid());
          ho.setSystemObjekt(0, vaterSo);
        }  
        
        vaterSo.setConfigurationData(atg, data);
        status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
            + " als Referenz in " + vaterSo.getPid() + " gesetzt"));
      }
      catch (ConfigurationChangeException e)
      {
        status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
            + " konnte nicht als Referenz in " + vaterSo.getPid() + " gesetzt werden ", e));
      }
    }
    
  }

/**
 * Kopieren eines HierarchieObjekts in ein neues Vater-Objekt mit den angegeben Pids
 * 
 * @param ho          HierarchieObjekt, das kopiert werden soll
 * @param neuerVater  Vater-Objekt, in das das HierarchieObjekt kopiert werden soll
 * @param neuePid     Pids fr die SystemObjekte des HierarchieObjekts
 * @param rekursiv    Flag, ob eine rekursive Kopie gewnscht wird
 * 
 */
  public HOMitStatus kopiereHO(HierarchieObjekt ho, ConfigurationArea kb, Object neuerVater, 
      String[] neuePid, boolean rekursiv, boolean isWurzel)
  {
    return kopiereHO(ho, kb, neuerVater, neuePid, rekursiv, true, true);
  }
  
  private HOMitStatus kopiereHO(HierarchieObjekt ho, ConfigurationArea kb, Object neuerVater, 
      String neuePid[], boolean rekursiv, boolean anwendenBR, boolean pruefePid)
  
  {
  	MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Hierarchieobjekt " + ho, null);
  
  	IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
  	String[] pid = neuePid;
  	HOMitStatus result = null;
  	HierarchieObjekt neu = null;
  	
  	if (pid == null || pid.length == 0)
    {
      pid = new String[1];
      pid[0] = ho.getSystemObjekt( 0 ).getPid();
//      System.out.println("Pid setzen auf " + pid[0]);
    }
  	
  	if (!hot.isSystemObjektVorhanden())
  	{ 	  
  	  //Ho und zugehriges SystemObjekt, sowie HO-Daten, erzeugen
//      System.out.println("kopiereHO " + ho.getSystemObjekt( 0 ).getPid() + " neue Pid: " + neuePid[0]);      
      
      result = erzeugeNeuesHO(kb, neuerVater, pid, ho.getHierarchieObjektTyp(),pruefePid);
      neu = result.getHO();
      status.add(result.getStatus());
      
//      System.out.println("Neues HO: " + neu + " Status: " + result.getStatus());
      
      if (neu != null)
      {
        MultiStatus statusDaten = kopiereDaten(ho, neu, anwendenBR, anwendenBR);
        status.merge(statusDaten);
        
        //Prfung, ob HOT gendert werden muss; es darf nur eine Hauptroute geben
        if (hot.equals( HOT_NBA.HauptRoute ) &&  _hierarchie.getUntergeordneteObjekte( neuerVater ).size() > 1)
        {
          neu.setHierarchieObjektTyp( HOT_NBA.NebenRoute );
        }
        
        //Leere Array des neuen Objekts, falls keine Menge vorhanden ist
        String pidTyp = hot.getTypen().get( 0 );
        List<String> mengenListe = hot.getTypInfo( pidTyp ).getMengen();
        if ( mengenListe != null && mengenListe.size() == 0 )
          mengenListe = null;
        
        if ( mengenListe == null )
        {
          //Leere Array des neuen Objekts
          loescheHOArrayZuordnung( neu, status );
        }
      }      
  	}
  	else
  	{
  	  //SystemObjekt zum Hot immer vorhanden, nur HO neu erzeugen
      
      result = erzeugeNeuesHO(kb, neuerVater, pid, ho.getHierarchieObjektTyp(), pruefePid);
      neu = result.getHO();
      status.add(result.getStatus());
  	}
  	
  	//Rekursive Kopie fr die Kinder
  	if (!status.matches(IStatus.ERROR | IStatus.CANCEL) && rekursiv)
		{
  		for (HierarchieObjekt kind: _hierarchie.getUntergeordneteObjekte(ho))
  		{
  			MultiStatus meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
  					"Kopiere untergeordnetes Hierarchieobjekt " + kind + " zu " + ho, null);
  			result = kopiereHO(kind, kb, neu, null, true, anwendenBR, pruefePid);
  			meldung.merge(result.getStatus());
  			status.add(meldung);
  			//es sollte Alles kopiert werden was geht, ohne Abbruch
//  			if (status.matches(IStatus.ERROR | IStatus.CANCEL))
//				{
//  				break;
//				}
  		}
		}
  	
  	return new HOMitStatus(neu, status);
  }
  
  
  private MultiStatus kopiereDaten(HierarchieObjekt srcHO, HierarchieObjekt destHO, 
  		boolean bildungsRegelnFuerName, boolean bildungsRegelnFuerAtg)
  {
  	if (!srcHO.getHierarchieObjektTyp().equals(destHO.getHierarchieObjektTyp()))
  	{
  		throw new IllegalArgumentException("Beide bergebenen Hierarchieobjekte mssen vom gleichen Hierarchieobjekttyp sein");
  	}
  	
		int index = 0;
		MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Daten von " + srcHO +	" nach " + destHO, null);
		IBildungsRegeln br = DatenSpeicher.getInstanz().getBildungsRegeln();
		ConfigDataModel dataModel = _hierarchie.getDataModel();
		
		ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
		
		for (SystemObject src: srcHO.getSystemObjekte())
		{
			SystemObject dest = destHO.getSystemObjekt(index);
			String name;
			if (bildungsRegelnFuerName)
			{
				name = br.getName(dest);
			}
			else
			{
				name = src.getName();
			}
			
			try
			{
				dest.setName(name);
				status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Setze Name \"" + name + 
						"\" fr Systemobjekt " +	dest.getPid()));
			}
			catch (ConfigurationChangeException e)
			{
				status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Name \"" + name + 
						"\" fr Systemobjekt " +	dest.getPid() + " konnte nicht gesetzt werden", e));
				return status;
			}
			
			for (AttributeGroupUsage atgu: src.getUsedAttributeGroupUsages())
			{
				Data dataSrc = ((ConfigSystemObject)src).getConfigurationData(atgu, lookup);
				if (dataSrc != null)
				{
					try
					{
						Data dataDest = DataFactory.createModifiableCopy(dataSrc, lookup);;
						if (bildungsRegelnFuerAtg)
						{
							dataDest = br.getDatensatz(destHO, dest, atgu.getAttributeGroup(), dataDest, true);
						}
						dest.setConfigurationData(atgu, dataDest);
	  				status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Kopiere Daten der ATG-Verwendung " +
	  						atgu.getAttributeGroup().getPid() + "/" + atgu.getAspect().getPid() + 
	  						" fr Systemobjekt " +	dest.getPid())); 
					}
					catch (ConfigurationChangeException e)
					{
						status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Daten der ATG-Verwendung " +
								atgu.getAttributeGroup().getPid() + "/" + atgu.getAspect().getPid() + 
								" fr Systemobjekt " +	dest.getPid() + " konnten nicht kopiert werden", e));
						return status;
					}
				}
			}
			
			index++;
		}
		
		return status;
  }
 
  
  public HOMitStatus verschiebeHO(HierarchieObjekt ho, Object neuerVater)
  {
//    System.out.println("verschiebeHo in HierarchieManagerNBA.");
    
  	IHierarchie hierarchie = getHierarchie();
  	ConfigDataModel dataModel = hierarchie.getDataModel();
  	IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
  	Object alterVater = hierarchie.getUebergeordnetesObjekt(ho);
  	
  	ConfigurationArea srcKB = hierarchie.getKonfigurationsBereich( alterVater );
  	ConfigurationArea destKB = hierarchie.getKonfigurationsBereich( neuerVater );
  	
  	MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Verschiebe Hierarchieobjekt " + ho, null);
  	
  	HierarchieObjekt neu = null;
  	
  	if (srcKB.equals(destKB)) // HO wird im gleichen kb verschoben
  	{
  		if (neuerVater.equals(alterVater))
  		{
  			status.add(new Status(Status.INFO, Activator.PLUGIN_ID,	"Objekt " + ho +
				" kann nicht in sich selbst verschoben werden"));
  		}
  		else
      {
        // Menge ermitteln, falls im HOT vorhanden
        ConfigurationObjectType typ = (ConfigurationObjectType) dataModel.getType( hot.getTypen().get( 0 ) );
        List<String> mengenListe = null;
        if ( hot != null && typ != null )
        {
          mengenListe = hot.getTypInfo( typ.getPid() ).getMengen();
          if ( mengenListe != null && mengenListe.size() == 0)
            mengenListe = null;
        }
        
        if ( mengenListe != null )
        {
          //Menge erzeugen
          //TODO noch zu testen, hier nicht bentigt
//          MengeInfo mengeInfo = hot.getTypInfo( typ.getPid() ).getMengeInfo( mengenListe.get(0) );
//          behandleMengenVerschieben(ho, (HierarchieObjekt)alterVater, (HierarchieObjekt)neuerVater, mengeInfo.getTyp(),
//              mengeInfo.getName(), 0, status);         
        }
        else
        {
          // Systemobjekt zum HO beim Array des neuen Vaters eintragen und speichern
          SystemObject so = ho.getSystemObjekt( 0 );

          setzeHOArrayZuordnung( so, neuerVater, status );

          // Lsche Systemobjekt zum HO im Array des alten Vater-Objekts
          if (!status.matches( IStatus.CANCEL | IStatus.ERROR ))
            loescheHOArrayZuordnung( ho, alterVater, status );
        }
        
        if (!status.matches( IStatus.CANCEL | IStatus.ERROR ))
        {
          hierarchie.objektVerschieben( alterVater, neuerVater, ho );

          // Falls das verschobene HO vom Typ HauptRoute ist, dann beim ersten
          // HO den Typ auf HauptRoute ndern
          // und falls der neue Vater bereits eine Hauptroute hat diese in eine
          // NebenRoute ndern
          if ( hot.equals( HOT_NBA.HauptRoute ) )
          {
            aendernHOT( alterVater, HOT_NBA.HauptRoute, status );
            if ( _hierarchie.getUntergeordneteObjekte( neuerVater ).size() > 1 )
              ho.setHierarchieObjektTyp( HOT_NBA.NebenRoute );
          }
        }

        if ( !status.matches( IStatus.CANCEL | IStatus.ERROR ) )
          neu = ho;
      }
  	}
  	else // HO wird in einen anderen kb verschoben
    {
  	  HOHierarchie hoHierarchie = new HOHierarchie(ho, hierarchie);
  	  
  	  status.add(loescheHO(ho));
  	  
  	  if (!status.matches( IStatus.ERROR | IStatus.CANCEL ))
  	  {
  	    HOMitStatus result = kopiereHORekursiv(hoHierarchie.getWurzel(), destKB, neuerVater,
  	        hoHierarchie);
  	    status.add( result.getStatus() );
  	    neu = result.getHO();
  	  }
    }
  	
  	return new HOMitStatus( neu, status );
  }
  
  private HOMitStatus kopiereHORekursiv(HierarchieObjekt ho, ConfigurationArea kb, Object vater,
      HOHierarchie hierarchie)
  {
    MultiStatus meldung;
    
    if (ho == hierarchie.getWurzel())
    {  
      meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Hierarchieobjekt " + ho, null);
    }
    else
    {
      meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere untergeordnetes Hierarchieobjekt " + 
          ho + " zu " + vater, null);
    }
    
    HOMitStatus result = kopiereHO(ho, kb, vater, null, false, false, false);
    meldung.merge( result.getStatus() );
    HierarchieObjekt neu = result.getHO();

    if (neu != null)
    {  
      for (HierarchieObjekt kind: hierarchie.getUntergeordneteObjekte( ho ))
      {
        result = kopiereHORekursiv(kind, kb, neu, hierarchie);
        meldung.add( result.getStatus() );
      }
    }
    
    return new HOMitStatus(neu, meldung);
  }
  
  public IStatus pruefeLoeschen(HierarchieObjekt ho)
  {
		return new Status(Status.ERROR, Activator.PLUGIN_ID, Status.OK, "NICHT IMPLEMENTIERT", null);
	} 
  
	public IStatus loescheHO(HierarchieObjekt ho)
	{
		MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Lsche Hierarchieobjekt " + ho, null);
				
		loescheHO(ho, status);
		
		return status;
	}
  
  
  protected void loescheHO(HierarchieObjekt ho, MultiStatus status)
  {
//    System.out.println("loescheHo in HierarchieManagerNBA zu HO: " +ho.getSystemObjekt( 0 ));
    
    // berprfung Plausibilitt der Parameter
  	IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
  	Object vater = getHierarchie().getUebergeordnetesObjekt(ho);
  	if (vater == null)
  	{
  		throw new IllegalArgumentException("Kein bergeordnetes Objekt fr " + ho +
  				" gefunden  in der Hierarchie Netzbeeinflussungsanlagen");
  	}  	
  	if (Arrays.asList(EBENE1).contains(hot) && !(vater instanceof ConfigurationArea))
  	{
  		throw new IllegalArgumentException("Als bergeordnetes Objekt ist nur ein Konfigurationsbereich zulssig");
  	}
  	
    if (!Arrays.asList(getUntergeordneteHOTs(vater)).contains(hot))
    {
      throw new IllegalArgumentException("Unzulssiges bergeordnetes Objekt " + vater +
          " fr Hierarchieobjekt " + ho);
    }
  	
    MultiStatus unterStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Lsche Hierarchieobjekt " + ho +
        " vom Typ " + hot, null);
    
  	// alle untergeordneten Objekte lschen, falls SO nicht erhalten bleiben soll
    if ( !hot.isSystemObjektVorhanden() )
    {     
      // Rekursiver Aufruf: Lschen der untergeordneten Objekte
      while (!getHierarchie().getUntergeordneteObjekte(ho).isEmpty())
      {
        HierarchieObjekt objekt = getHierarchie().getUntergeordneteObjekte(ho).get(0);
        
        loescheHO(objekt, unterStatus);
        
        status.add(unterStatus);
        
        //Lschen darf deswegen nicht mitten drin beendet werden
        if (status.matches(IStatus.CANCEL | IStatus.ERROR))
        {
          break;
        }
      }
      
      if ( status.matches( IStatus.CANCEL | IStatus.ERROR ) )
      {
        return;
      }

//      System.out.println("Lsche HoZuordnung!");
      
      //Lsche Systemobjekt zum HO in der Menge oder im Array des bergeordneten Vater-Objekts und HierarchieObjekt-Verbindung     
      loescheHOZuordnung( ho, vater, status );
      
      if ( status.matches( IStatus.CANCEL | IStatus.ERROR ) )
      {
        return;
      }
      
//      System.out.println("Lsche ConfigurationObject!");
      
      // Lsche ConfigurationObject
      ConfigurationObject ko = (ConfigurationObject) ho.getSystemObjekt( 0 );
      
      try
      {
        ko.invalidate();

        status.add( new Status( Status.OK, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
            + ko.getType().getName() + " gelscht." ) );
      }
      catch (ConfigurationChangeException e)
      {
        status.add( new Status( IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
            + ko.getType().getName() + " konnte nicht gelscht werden.", e ) );
        setzeHOZuordnung( ho, vater, status );
        return;
      }
      catch (RuntimeException e)
      {
        status.add( new Status( IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
            + ko.getType().getName() + " konnte nicht gelscht werden.", e ) );
        // wegen NullPointerException bei AttributeHelper auskommentiert
        //        setzeHOZuordnung( ho, vater, status );
        return;
      }

      //Falls das zu lschende HO vom Typ HauptRoute ist, dann beim ersten HO den Typ auf HauptRoute ndern
      if ( hot.equals( HOT_NBA.HauptRoute ) )
      {
        aendernHOT( vater, HOT_NBA.HauptRoute, status );      
      }
      
    }
    else
    {
      //SystemObjekt bleibt erhalten
      //Lsche Systemobjekt zum HO in der Menge oder im Array des bergeordneten Vater-Objekts und HierarchieObjekt-Verbindung
      loescheHOZuordnung( ho, vater, status );
    
      if ( status.matches( IStatus.CANCEL | IStatus.ERROR ) )
      {
        return;
      } 
      
    }

  }

  /**
   * Lsche Systemobjekt des HierarchieObjekts in der Menge oder im Array des bergeordneten Objekts
   * und HierarchieObjekt-Verbindung
   * 
   * @param ho      HierarchieObjekt
   * @param vater   Vater-Objekt
   * @param status  Status
   */
  private void loescheHOZuordnung( HierarchieObjekt ho, Object vater, MultiStatus status )
  {
    IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
    String pid = hot.getTypen().get( 0 );
    List<String> mengenListe = hot.getTypInfo( pid ).getMengen();
    
    if ( mengenListe != null && mengenListe.size() == 0)
      mengenListe = null;
    
    if ( mengenListe != null )
    {
      // Lsche Objekt in der Menge
      //TODO Objekt in der Menge lschen
      
      // Entferne Referenz aus Menge des bergeordneten Objekts
    }
    else
    {
      //Lsche Systemobjekt zum HO im Array des bergeordneten Vater-Objekts
      loescheHOArrayZuordnung( ho, vater, status );
    }
    
    // Lsche HierarchieObjekt-Verbindung
    if ( !status.matches( IStatus.CANCEL | IStatus.ERROR ) )
      getHierarchie().objektEntfernen( vater, ho );

  }

  /**
   * Leere 1.Array der 1.Attributgruppe des 1.Systemobjekts beim Vater
   * 
   * @param vater   bergeordnetes Hierarchieobjekt
   * @param status  Status
   */
  private void loescheHOArrayZuordnung( Object vater, MultiStatus status )
  {
    HierarchieObjekt vaterHo = null;
    if ( vater == null || !(vater instanceof HierarchieObjekt) )
      return;
    else
      vaterHo = (HierarchieObjekt)vater;
    
    IHierarchieObjektTyp vaterHot = vaterHo.getHierarchieObjektTyp();
    
    // 1.Systemobjekt des bergeordneten Hierarchieobjektes ermitteln
    SystemObject vaterSo = vaterHo.getSystemObjekt(0);
    String pidSo = vaterSo.getType().getPid();
    
    // Pid der 1.Attributgruppe und Name der Attribute aus dem Hierarchieobjekttyp des Vaters ermitteln
    String atgPid = vaterHot.getTypInfo( pidSo ).getAtgs().get( 0 );
    List<String> attNamenListe = vaterHot.getTypInfo( pidSo ).getAtgInfo( atgPid ).getAttribute();
    
    AttributeGroup atg = getHierarchie().getDataModel().getAttributeGroup(atgPid);
    
    ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(getHierarchie().getDataModel());
    Data data = ((ConfigConfigurationObject)vaterSo).getConfigurationData(atg, lookup);
    
    if (data != null)
    {
      // 1.Array-Element aus den Attributen ermitteln
      String arrayName = "";     
      for (String attName : attNamenListe)
      {
        if (data.getItem( attName ).isArray())
        {
          arrayName = attName;
          break;
        }
      }
      
      if (arrayName.isEmpty())
      {
        status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Das Array in "
            + vaterSo.getPid() + " konnte nicht geleert werden, weil der Name des Arrays nicht gefunden wurde."));
        return;
      }
      
      ReferenceArray ra = data.getReferenceArray(arrayName);
      
      ra.setLength( 0 );
      
      try
      {
        if (!HierarchieObjekt.isAttributgruppeAenderbar(vaterSo, atg))
        {  
          vaterSo = behandleSystemObjektAenderung(vaterHo, (ConfigurationObject)vaterSo, vaterSo.getPid());
          vaterHo.setSystemObjekt(0, vaterSo);
        }  
        vaterSo.setConfigurationData(atg, data);
        status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Das Array in "
            + vaterSo.getPid() + " wurde geleert"));
      }
      catch (ConfigurationChangeException e)
      {
        status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Das Array in "
        		+ vaterSo.getPid() + " konnte nicht geleert werden ", e));
      }                  
    }
  }

  
  /**
   * Lsche 1.Systemobjekt des Hierarchieobjekts aus dem 1.Array der 1.Attributgruppe des 1.Systemobjekts beim Vater
   * 
   * @param ho      Hierarchieobjekt, dessen Systemobjekt entfernt werden soll
   * @param vater   bergeordnetes Hierarchieobjekt
   * @param status  Status
   */
  private void loescheHOArrayZuordnung( HierarchieObjekt ho, Object vater, MultiStatus status )
  {
//    System.out.println("loescheHOZuordnung: " + status.getCode());
    
    SystemObject so = ho.getSystemObjekt(0);
    
    HierarchieObjekt vaterHo = null;
    if ( vater == null || !(vater instanceof HierarchieObjekt) )
      return;
    else
      vaterHo = (HierarchieObjekt)vater;
    
    IHierarchieObjektTyp vaterHot = vaterHo.getHierarchieObjektTyp();
    
    // 1.Systemobjekt des bergeordneten Hierarchieobjektes ermitteln
    SystemObject vaterSo = vaterHo.getSystemObjekt(0);
    String pidSo = vaterSo.getType().getPid();
    
    // Pid der 1.Attributgruppe und Name der Attribute aus dem Hierarchieobjekttyp des Vaters ermitteln
    String atgPid = vaterHot.getTypInfo( pidSo ).getAtgs().get( 0 );
    AtgInfo atgInfo = vaterHot.getTypInfo( pidSo ).getAtgInfo( atgPid );
    List<String> attNamenListe = atgInfo.getAttribute();
    
    AttributeGroup atg = getHierarchie().getDataModel().getAttributeGroup(atgPid);
    
    ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(getHierarchie().getDataModel());
    Data data = ((ConfigConfigurationObject)vaterSo).getConfigurationData(atg, lookup);
    
    if (data != null)
    {
      // 1.Array-Element aus den Attributen ermitteln
      String arrayName = "";     
      for (String attName : attNamenListe)
      {
        if (data.getItem( attName ).isArray())
        {
          arrayName = attName;
          break;
        }
      }
      
      if (arrayName.isEmpty())
      {
        status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
            + " konnte nicht als Referenz in " + vaterSo.getPid() + " gelscht werden, weil der Name des Arrays nicht gefunden wurde."));
        return;
      }
      
      ReferenceArray ra = data.getReferenceArray(arrayName);
      
      if ( !Arrays.asList( ra.getSystemObjectArray() ).contains( so ) )
      {
        status.add(new Status(Status.INFO, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
            + " konnte nicht als Referenz in " + vaterSo.getPid() + " gelscht werden, weil dieses nicht vorhanden ist."));
        return;        
      }        
      
      List<SystemObject> delList = new ArrayList<SystemObject>();
      List<SystemObject> list = new ArrayList<SystemObject>();
      
      for (int i = 0; i < ra.getLength(); i++)
      {
         SystemObject ref = ra.getReferenceValue(i).getSystemObject();             
         list.add(ref);             
         if (ref.equals(so))
           delList.add(ref);
      }
      
      list.removeAll(delList);
       
      ra.setLength(list.size());
      
      for (int i = 0; i < ra.getLength(); i++)
      {
         ra.getReferenceValue(i).setSystemObject(list.get(i));                        
      }
      
//      System.out.println("data: " + data);
//      IPruefungen pruefInstanz = DatenSpeicher.getInstanz().getPruefungen();
//      status.merge(pruefInstanz.pruefeDaten( ho, vaterSo, atgInfo, data ));
//      
//      if (status.matches( IStatus.ERROR | IStatus.CANCEL ))
//        return;
      
      try
      {
        if (!HierarchieObjekt.isAttributgruppeAenderbar(vaterSo, atg))
        {  
          vaterSo = behandleSystemObjektAenderung(vaterHo, (ConfigurationObject)vaterSo, vaterSo.getPid());
          vaterHo.setSystemObjekt(0, vaterSo);
        }  
        vaterSo.setConfigurationData(atg, data);
        status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
            + " als Referenz in " + vaterSo.getPid() + " gelscht"));
      }
      catch (ConfigurationChangeException e)
      {
        status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
            + " konnte nicht als Referenz in " + vaterSo.getPid() + " gelscht werden ", e));
      }                  
    }
  }

  /**
   * HOT des 1.Hierarchieobjekts zum bergebenen Vater-Objekt ndern 
   * 
   * @param vater     bergeordnetes Hierarchieobjekt
   * @param neuerHot  neuer HierarchieObjektTyp    
   */
  private void aendernHOT( Object vater, IHierarchieObjektTyp neuerHot, MultiStatus status )
  {
//    System.out.println("HOT der nchsten Route ndern auf: " + neuerHot);
    
    HierarchieObjekt vaterHo = null;
    if ( vater instanceof HierarchieObjekt )
      vaterHo = (HierarchieObjekt)vater;
    else
      return;
    
    IHierarchieObjektTyp vaterHot = vaterHo.getHierarchieObjektTyp();
    
    // 1.Systemobjekt des bergeordneten Hierarchieobjektes ermitteln
    SystemObject vaterSo = vaterHo.getSystemObjekt(0);
    String pidSo = vaterSo.getType().getPid();
    
    // Pid der 1.Attributgruppe und Name der Attribute aus dem Hierarchieobjekttyp des Vaters ermitteln
    String atgPid = vaterHot.getTypInfo( pidSo ).getAtgs().get( 0 );
    List<String> attNamenListe = vaterHot.getTypInfo( pidSo ).getAtgInfo( atgPid ).getAttribute();
    
    AttributeGroup atg = getHierarchie().getDataModel().getAttributeGroup(atgPid);
    
    ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(getHierarchie().getDataModel());
    Data data = ((ConfigConfigurationObject)vaterSo).getConfigurationData(atg, lookup);
    
    if (data != null)
    {
      // 1.Array-Element aus den Attributen ermitteln
      String arrayName = "";     
      for (String attName : attNamenListe)
      {
        if (data.getItem( attName ).isArray())
        {
          arrayName = attName;
          break;
        }
      }
      
      if (arrayName.isEmpty())
      {
        status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "HierarchieObjektTyp der 1.Route in " + vaterSo.getPid() + " konnte nicht gendert werden, weil der Name des Arrays nicht gefunden wurde."));
        return;
      }
      
      ReferenceArray ra = data.getReferenceArray(arrayName);
      
      if (ra.getLength() > 0)
      {
        SystemObject so = ra.getSystemObject( 0 );

        List<HierarchieObjekt> uos = getHierarchie().getUntergeordneteObjekte( vaterHo );

        for (HierarchieObjekt ho : uos)
        {
          if ( ho.getSystemObjekt( 0 ).equals( so ) )
          {
            ho.setHierarchieObjektTyp( neuerHot );
            _hierarchie.notifyHOGeaendert( ho );
            break;
          }
        }
      }
    }
    
  }
  
  public IStatus pruefeDaten(HierarchieObjekt ho, HODaten daten)
	{
		IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
		IPruefungen pruefInstanz = DatenSpeicher.getInstanz().getPruefungen();
		MultiStatus result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Hierarchieobjekt " + ho, null);
		
		// Prfe Hierarchieobjekt
		result.merge(pruefInstanz.pruefeHierarchieObjekt(ho, daten));
		
		// Prfe auf offene Referenzen bei PID-nderungen
    List<SystemObject> listeEltern = new Vector<SystemObject>();
    Object ueberObj = getHierarchie().getUebergeordnetesObjekt( ho );
    if ( ueberObj instanceof HierarchieObjekt )
    {
      HierarchieObjekt ueberHO = (HierarchieObjekt) ueberObj;
      listeEltern.add( ueberHO.getSystemObjekt( 0 ) );      
    }

    LoeschenChecker loeschenChecker = new LoeschenChecker(this);
    List<SystemObject> liste = new Vector<SystemObject>();
    StringBuffer aend = new StringBuffer();
		StringBuffer loesch = new StringBuffer();
		
		for (SystemObject so: ho.getSystemObjekte())
		{
		 SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
		 
		 if (!so.getPid().equals(soDaten.getPid()))
		 {
			 liste.add(so);
			 if (aend.length() > 0)
			 {
				 aend.append(", ");
			 }
			 aend.append(soDaten.getPid());
			 if (loesch.length() > 0)
			 {
				 loesch.append(", ");
			 }
			 loesch.append(so.getPid());
		 }
		}
		
		if (liste.size() > 0)
		{
			 MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK,  "Durch PID-nderung(en) auf " + 
					 aend + " werden folgende Objekte gelscht: " + loesch, null);
			 
//			 status.add(loeschenChecker.bestimmeOffeneReferenzen(liste));
       
       Collection<SystemObjectType> typen = loeschenOffeneReferenzenVerboten(ho.getHierarchieObjektTyp());
       boolean flag = verwendeZuLoeschendeObjekteZurPruefungOffeneReferenzen();
       status.add(loeschenChecker.bestimmeOffeneReferenzen(liste, typen, listeEltern, flag));
       
			 result.add(status);
		 }
		
		// Prfe ATG-Daten
		for (SystemObject so: ho.getSystemObjekte())
  	{
			 SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
			 if (soDaten != null)
			 {
				 MultiStatus objektStatus;
				 if (ho.getSystemObjekte().size() > 1)
				 {
					 objektStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe ATG-Daten fr Systemobjekt " + 
							 so.getPid() + " vom Typ " + so.getType().getPid(), null);
				 }
				 else
				 {
					 objektStatus = result;
				 }
				 
				 // Prfe ATG-Daten  fr das Systemobjekt
				 TypInfo typInfo = hot.getTypInfo(so.getType().getPid());
				 if (typInfo != null)
				 {
					 for (String atgPid: typInfo.getAtgs())
					 {
						 AtgInfo atgInfo = typInfo.getAtgInfo(atgPid);
						 if (atgInfo != null)
						 {
							 Data data = soDaten.getAtgDaten(atgPid);
							 if (data == null && (atgInfo.isMandatory() || !atgInfo.isOptional()))
							 {
								 objektStatus.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Fr die nicht-optionale ATG " +
										 atgPid + " sind keine Daten vorhanden"));					 
							 }
							 if (data != null)
							 {
								 MultiStatus atgStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
										 "Prfe Daten der ATG " + atgPid, null);
								 atgStatus.merge(pruefInstanz.pruefeDaten(ho, so, atgInfo, data));
								 								 								 
								 objektStatus.add(atgStatus);
							 }
						 }
						 else
						 {
								// Kein AtgInfo fr den aktuellen Typ-Info-Typ im HOT fr aktuellen Systemobjekttyp
							 throw new IllegalStateException("AtgInfo fr ATG " + atgPid + " fehlt in TypInfo fr " + 
									 so.getType().getPid() + " im HierarchieObjektTyp " + hot);
						 }
					 }
				 }
				 else
				 {
				// Keine Typ-Info im HOT fr aktuellen Systemobjekttyp
					 throw new IllegalStateException("TypInfo fehlt fr  " + so.getType().getPid() + 
							 " im HierarchieObjektTyp " + hot);
				 }
				 if (result != objektStatus)
				 {
					 result.add(objektStatus);
				 }
				 
			 }
			 else
			 {
				 // Keine Daten fr Systemobjekt so bergeben
				 throw new IllegalStateException("Daten fr Systemobjekt " + so.getPid() +
						 " erforderlich, aber nicht bergeben");
			 }
  	}
  	
  	return result;
	}    
  
  private ConfigurationObject dupliziereObjekt( ConfigurationObject ko, String neuePid )
      throws ConfigurationChangeException
  {
    List<ObjectSet> sets = new Vector<ObjectSet>();
    ConfigurationObject neu = null;
    ConfigurationObject result = null;

    try
    {
      for (ObjectSet set : ko.getObjectSets())
      {
        ObjectSetType setTyp = set.getObjectSetType();
        assert (setTyp.getReferenceType() != ReferenceType.COMPOSITION);
        ObjectSet menge = (ObjectSet) ko.getConfigurationArea().createConfigurationObject( set.getObjectSetType(), "",
            set.getName(), null );
        List<SystemObject> elemente = set instanceof NonMutableSet ? ((NonMutableSet) set)
            .getElementsInModifiableVersion() : set.getElements();
        menge.add( elemente.toArray( new SystemObject[elemente.size()] ) );
        sets.add( menge );
      }

      neu = ko.getConfigurationArea().createConfigurationObject( (ConfigurationObjectType) ko.getType(),
          neuePid == null ? ko.getPid() : neuePid, ko.getName(), sets );

      ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion( (ConfigDataModel) ko.getDataModel() );

      for (AttributeGroupUsage atgu : ko.getUsedAttributeGroupUsages())
      {
        Data dataSrc = ((ConfigSystemObject) ko).getConfigurationData( atgu, lookup );
        if ( dataSrc != null )
        {
          Data dataDest = DataFactory.createModifiableCopy(dataSrc, lookup);;
          neu.setConfigurationData( atgu, dataDest );
        }
      }
      result = neu;
    }
    finally
    {
      // Rollback
      if ( result == null )
      {
        for (ObjectSet set : sets)
        {
          set.invalidate();
        }
        if ( neu != null )
        {
          neu.invalidate();
        }
      }
    }

    return result;
  }

  private ConfigurationObject behandleSystemObjektAenderung( HierarchieObjekt ho, ConfigurationObject ko, String neuePid )
      throws ConfigurationChangeException
  {
    ConfigurationObject result = dupliziereObjekt( ko, neuePid );

    Object vater = _hierarchie.getUebergeordnetesObjekt( ho );
    if ( vater instanceof HierarchieObjekt && !ko.getPid().equals( result.getPid() ))
    {
      HierarchieObjekt vaterHo = (HierarchieObjekt) vater;
      IHierarchieObjektTyp hot = vaterHo.getHierarchieObjektTyp();

      SystemObject vaterSo = vaterHo.getSystemObjekt( 0 );
      ConfigDataModel dataModel = _hierarchie.getDataModel();
      ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion( dataModel );
      AttributeGroup atg = null;
      Data neu = null;
      ReferenceArray refArray = null;
      if ( hot.equals( HOT_NBA.Masche ) )
      {
        atg = dataModel.getAttributeGroup( "atg.routen" );
        Data data = ((ConfigSystemObject) vaterSo).getConfigurationData( atg, lookup );
        neu = DataFactory.createModifiableCopy(data, lookup);;
        refArray = neu.getReferenceArray( "Route" );
      }
      else
      {
        atg = dataModel.getAttributeGroup( "atg.bestehtAusLinienObjekten" );
        Data data = ((ConfigSystemObject) vaterSo).getConfigurationData( atg, lookup );
        neu = DataFactory.createModifiableCopy(data, lookup);
        refArray = neu.getReferenceArray( "LinienReferenz" );
      }

      for (int i = 0; i < refArray.getLength(); i++)
      {
        SystemObject ref = refArray.getReferenceValue( i ).getSystemObject();
        if ( ref != null && ref.equals( (SystemObject) ko ) )
          refArray.getReferenceValue( i ).setSystemObject( result );
      }
      
      if (!HierarchieObjekt.isAttributgruppeAenderbar(vaterSo, atg))
      {  
        vaterSo = behandleSystemObjektAenderung(vaterHo, (ConfigurationObject)vaterSo, vaterSo.getPid());
        vaterHo.setSystemObjekt(0, vaterSo);
      }  
      vaterSo.setConfigurationData( atg, neu );
    }

    ko.invalidate();

    return result;
  }
  
  public IStatus uebernehmeDaten(HierarchieObjekt ho, HODaten daten)
  {
  	MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
  			"bernehme bearbeitete Daten fr Hierarchieobjekt " + ho, null);
  	
  	List<SystemObject> objektListe = daten.getSystemObjekte();
  	
//  	DatenModellUpdateListener.getInstanz().notifyDatenModellUpdateListener(objektListe);
  	
  	for (int i = 0; i < objektListe.size(); i += 1)
  	{
  		SystemObject so = objektListe.get(i);
  		
  		if (ho.getSystemObjekte().contains(so))
			{
  			SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
  			
  			// PID gendert
  			if (!so.getPid().equals(soDaten.getPid()))
  			{
          try
          {
	  				so = behandleSystemObjektAenderung(ho, (ConfigurationObject)so, soDaten.getPid());
	  				ho.setSystemObjekt(i, so);
	  				
	  				status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
	  						"Neues Objekt mit PID " + so.getPid() + 
	  						" vom Typ " + so.getType().getPid() + " erzeugt"));
          }
          catch (ConfigurationChangeException e)
        	{
          	status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
      					"Fehler beim behandeln der PID-nderung auf " + soDaten.getPid() +
      					" fr Objekt " + so.getPid(), e));
        	}
  			}
  			
  			// Name
        if (so.getName() == null || !so.getName().equals(soDaten.getName()))
        {
          try
          {
            so.setName(soDaten.getName());
            status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
                "Name \"" + soDaten.getName() + "\" bernommen"));
          }
          catch (ConfigurationChangeException e)
          {
            status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
                "Name \"" + soDaten.getName() + "\" konnte nicht bernommen werden" , e));
          }
        }   
  		
  	   	// ATGs gendert
  			List<String> atgList = ho.getHierarchieObjektTyp().getTypInfo(so.getType().getPid()).getAtgs();
  			
		    for (String atgPid: atgList)
    		{
		    	AttributeGroup atg = _hierarchie.getDataModel().getAttributeGroup(atgPid);
		    	if (atg != null)
		    	{
	    		
		    		ConfigDataModel dataModel = _hierarchie.getDataModel();
		    		ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
		    		
		        Data ist = ((ConfigSystemObject)so).getConfigurationData(atg, lookup);
		        		        
		    		Data soll = soDaten.getAtgDaten(atgPid);
		  
	          if (soll != null)
	          {
	            if (ist == null || soDaten.isDatenGeaendert(atgPid))
              { 
  	          	if (!HierarchieObjekt.isAttributgruppeAenderbar(so, atg))
  			    		{
  	              try
                  {
                   so = behandleSystemObjektAenderung(ho, (ConfigurationObject)so, so.getPid());
                   ho.setSystemObjekt(i, so);
                   status.add(new Status(IStatus.OK, Activator.PLUGIN_ID,
                       "ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
                       " bedingt Erzeugung neues Objekt mit PID " + so.getPid() + " vom Typ " + 
                       so.getType().getPid()));
                  }
                  catch (ConfigurationChangeException e)
                 {
                   status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
                       "Bei der durch ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
                       " bedingten Erzeugung neues Objekts mit PID " + so.getPid() + " vom Typ " + 
                       so.getType().getPid() + " ist ein Fehler aufgetreten", e));
                   continue;
                 }
//  	          		status.add(new Status(IStatus.INFO, Activator.PLUGIN_ID, 
//  			   						"Eventuelle nderungen an konfigurierenden Daten der ATG " + atgPid +
//  			   						" werden IGNORIERT, da wegen ATG-Verwendung (" + 
//  			   						AttributeGroupUsage.Usage.OptionalConfigurationData + ") nicht mglich"));
//  			    			continue;
  			    		}
  	          	try
  	          	{
  	          		so.setConfigurationData(atg, soll);
  	          		status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
  	       						"Konfigurierende Daten der ATG " + atgPid + " bernommen"));
  	          	}
  	          	catch (ConfigurationChangeException e)
  	          	{
  	          		status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
  	       						"Fehler beim bernehmen konfigurierender Daten der ATG " + atgPid, e));
  	          	}
  	            
  	            //TODO Ho zum Vater erzeugen bzw. lschen, wenn direkte nderungen im Array erfolgen
  //              if ( soll.getItem( atgPid ) != null )
  //              {
  //                if ( soll.getItem( atgPid ).isArray() )
  //                {
  //                  Array sollArray = soll.getItem( atgPid ).asArray();
  //
  //                }
  //                if ( ist.getItem( atgPid ) != null && ist.getItem( atgPid ).isArray() )
  //                {
  //                  Array istArray = ist.getItem( atgPid ).asArray();
  //
  //                }
  //              }
              }
	          }
	          else // Keine Daten fr diese ATG bergeben => vorhanden Daten lschen
	          {
	          	if (ist != null)
	          	{
	          		if (!HierarchieObjekt.isAttributgruppeAenderbar(so, atg))
				    		{
	          		  try
                  {
                   so = behandleSystemObjektAenderung(ho, (ConfigurationObject)so, so.getPid());
                   ho.setSystemObjekt(i, so);
                   status.add(new Status(IStatus.OK, Activator.PLUGIN_ID,
                       "ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
                       " bedingt Erzeugung neues Objekt mit PID " + so.getPid() + " vom Typ " + 
                       so.getType().getPid()));
                  }
                  catch (ConfigurationChangeException e)
                 {
                   status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
                       "Bei der durch ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
                       " bedingten Erzeugung neues Objekts mit PID " + so.getPid() + " vom Typ " + 
                       so.getType().getPid() + " ist ein Fehler aufgetreten", e));
                   continue;
                 }
//				    			status.add(new Status(IStatus.INFO, Activator.PLUGIN_ID, 
//				   						"Eventuelle nderungen an konfigurierenden Daten der ATG " + atgPid +
//				   						" werden IGNORIERT, da wegen ATG-Verwendung (" + 
//				   						AttributeGroupUsage.Usage.OptionalConfigurationData + ") nicht mglich"));
//				    			continue;
				    		}
	          		try
		          	{
		          		so.setConfigurationData(atg, null);
		          		status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
		       						"Konfigurierende Daten der ATG " + atgPid + " gelscht"));
		          	}
		          	catch (ConfigurationChangeException e)
		          	{
		          		status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
		       						"Fehler beim Lschen konfigurierender Daten der ATG " + atgPid, e));
		          		continue;
		          	}
		            //TODO Hos zum Vater lschen, wenn direkte nderungen im Array erfolgen
	          	}
	          }
		    	}
		    	else
		    	{
		    		_debug.warning("Bearbeitung Objekt " + so.getPid() + 
		    				": Keine Attributgruppe im Datenmodell gefunden fr " + atgPid);
		    	}
    		}	
	  	}   	
  	}    
  	
  	return status;
  }

  @Override
  public boolean pruefeNeuanlageVerboten( Object objekt, IHierarchieObjektTyp hot )
  {
    if (Arrays.asList( EBENE2 ).contains( hot ))
    {
      // fr das 1.HO und alle weiteren sind unterschiedliche HOTs zugelassen
      if ( getHierarchie().getUntergeordneteObjekte( objekt ).size() == 0 )
        return !Arrays.asList( EBENE20 ).contains( hot );
      else
        return !Arrays.asList( EBENE21 ).contains( hot );
    }
    
    return false;
  }

  @Override
  public Collection<SystemObjectType> loeschenOffeneReferenzenVerboten( IHierarchieObjektTyp hot )
  {
    return null;
  }

  @Override
  public boolean verwendeZuLoeschendeObjekteZurPruefungOffeneReferenzen()
  {
    // TODO Auto-generated method stub
    return false;
  }
  
  /**
   * Zeigt an, ob das bergebene Hierarchieobjekt beim rekursiven Kopieren als untergeordnetes 
   * Objekt mitkopiert werden soll oder nicht
   *  
   * @param ho das aktuell zu kopierende HO
   * @return <code>true</code>, wenn das aktuell zu kopierende Objekt nicht kopiert werden soll, 
   * <code>false</code> (default) sonst
   */
  public boolean kopierenNichtErlaubt(HierarchieObjekt ho)
  {
    return false;
  }
 
}
