/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkanetz;

import java.util.Arrays;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.konfigass.MeldungenSpeicher;
import de.bsvrz.buv.plugin.tkabasis.Activator;
import de.bsvrz.buv.plugin.tkabasis.DataFactory;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AbstractHierarchieManager;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HODaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HOMitStatus;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieManager;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.MengeInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.SystemObjektDaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.TypInfo;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.IPruefungen;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.LoeschenChecker;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.PruefungenTools;
import de.bsvrz.buv.plugin.tkabasis.regeln.IBildungsRegeln;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeGroupUsage;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.ObjectSetType;
import de.bsvrz.dav.daf.main.config.ObjectTimeSpecification;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigSystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

public class HierachieManagerNetz extends AbstractHierarchieManager implements IHierarchieManager
{
 /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
  
  private static IHierarchieObjektTyp[] EBENE1 = new IHierarchieObjektTyp[] {HOT_Netz.VerkehrsModellNetz, 
  	HOT_Netz.SchwachStelle};     
  
  private static IHierarchieObjektTyp[] EBENE2 = new IHierarchieObjektTyp[] {HOT_Netz.VerkehrsModellNetzReferenz, 
  	HOT_Netz.SchwachStelleReferenz, HOT_Netz.ASS};
  
  
    
//	private Collection<NonMutableSet> getAlleNetzBestandTeileMengen()
//	{
//		Collection<NonMutableSet> result = new LinkedList<NonMutableSet>();
//		ConfigDataModel dataModel = getHierarchie().getDataModel();
//		Collection<SystemObjectType> typeSpez = new Vector<SystemObjectType>();
//		ObjectSetType typ = dataModel.getObjectSetType("menge.netzBestandTeile");
//		
//		typeSpez.add(typ);
//		
//		for (SystemObject objekt: dataModel.getObjects(null, typeSpez, ObjectTimeSpecification.valid()))
//		{
//			if (objekt instanceof NonMutableSet)
//			{
//				if (!(((NonMutableSet)objekt).getNotValidSince() >= objekt.getConfigurationArea().getModifiableVersion()))
//	    	// Objekt wird in zuknftiger Version nicht ungltig	
//	    	{		
//					if ("NetzBestandTeile".equals(objekt.getName()))
//					{
//						result.add((NonMutableSet)objekt);
//					}
//	    	}  
//			}
//		}
//
//		// Neue Objekte
//		for (ConfigurationArea kb: dataModel.getAllConfigurationAreas().values())
//		{
//			for (SystemObject objekt: kb.getNewObjects())
//			{
//				if (objekt instanceof NonMutableSet && objekt.isOfType(typ) && "NetzBestandTeile".equals(objekt.getName()))
//				{
//					result.add((NonMutableSet)objekt);
//				}
//			}
//		}
//		
//		return result;
//	}

	private boolean pruefeObjektAufUnendlicheRekursion(SystemObject so, HierarchieObjekt vater, MultiStatus status, 
			boolean rekursiv, boolean warnung)
	{
		boolean result = false;
		
		if (so != null && (so.isOfType("typ.verkehrsModellNetz") || so.isOfType("typ.schwachStelle")))
		{	
			Object objekt = vater;
			while (objekt instanceof HierarchieObjekt)
			{
				HierarchieObjekt ho = (HierarchieObjekt)objekt;
				
				if (ho.getSystemObjekt(0) == so)
				{
					status.add(new Status(warnung ? Status.WARNING : Status.ERROR, Activator.PLUGIN_ID,	
							"Systemobjekt " + so.getPid() +	" ist bereits in der bergeordneten Hierarchie enthalten"));
					result = true;
				}
				
				objekt = getHierarchie().getUebergeordnetesObjekt(ho);
			}
			
			if (rekursiv )
			{
				NonMutableSet menge = ((ConfigurationObject)so).getNonMutableSet("NetzBestandTeile");
				if (menge != null)
				{
					for (SystemObject kind: menge.getElementsInModifiableVersion())
					{
						result |= pruefeObjektAufUnendlicheRekursion(kind, vater, status, true, warnung);
					}
				}
			}
		}
		
		return result;
	}
	
	@Override
  public void  bestimmeUntergeordneteHOs(Object objekt)
  {
		IHierarchie hierarchie = getHierarchie();
		
  	if (objekt instanceof ConfigurationArea)	
    {
			ConfigurationArea kb = (ConfigurationArea)objekt;
			//Collection<NonMutableSet> netzBestandTeileMengen = getAlleNetzBestandTeileMengen();
						
			for (IHierarchieObjektTyp hot: EBENE1)
			{
				String typPid = hot.getTypen().get(0);
			  SystemObjectType  typ = hierarchie.getDataModel().getType(typPid);
			  
			  if (typ == null)
			  {
			  	throw new RuntimeException("Typ " + typPid + " wurde im aktuellen Datemmodell nicht gefunden");
			  }
		  
		  	Collection<SystemObjectType> typeSpez = new Vector<SystemObjectType>();
		    typeSpez.add(typ);
		    
		    for (SystemObject so: kb.getObjects(typeSpez, ObjectTimeSpecification.valid()))
		    {
		    	ConfigurationObject ko = (ConfigurationObject)so;
		    	if (!(ko.getNotValidSince() >= ko.getConfigurationArea().getModifiableVersion()))
		    	// Objekt wird in zuknftiger Version nicht ungltig	
		    	{		
						 HierarchieObjekt ho = new HierarchieObjekt(hot, new SystemObject[]{ so }, EBENE1);
						 hierarchie.objektHinzufuegen(kb, ho);
			       bestimmeUntergeordneteHOs(ho);
		    	}  
		    }
		    
		    for (SystemObject so: kb.getNewObjects())
		    {
		    	if (so.isOfType(typ))
    			{
						 HierarchieObjekt ho = new HierarchieObjekt(hot, new SystemObject[]{ so }, EBENE1);
						 hierarchie.objektHinzufuegen(kb, ho);
						 bestimmeUntergeordneteHOs(ho);
    			}
		    }
			}
    }
    else if (objekt instanceof HierarchieObjekt)
    {
      HierarchieObjekt ho = (HierarchieObjekt)objekt;
      IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp(); 
      
      if ( hot.equals( HOT_Netz.VerkehrsModellNetz ) || 
      		hot.equals( HOT_Netz.SchwachStelle ) ||
      		hot.equals( HOT_Netz.VerkehrsModellNetzReferenz ) || 
      		hot.equals( HOT_Netz.SchwachStelleReferenz ))
      {
        ConfigurationObject co = (ConfigurationObject)ho.getSystemObjekte().get(0);
        NonMutableSet nmSet = co.getNonMutableSet("NetzBestandTeile");
        MultiStatus meldung = new MultiStatus(Activator.PLUGIN_ID, IStatus.ERROR,
						"Es gibt als Hierarchieobjekte unterhalb von " + ho + " ungltige Objekte", null);

        
        if (nmSet != null)
        {
          for (SystemObject so : nmSet.getElementsInModifiableVersion())
          {
            if ( so == null )
            {
            	_debug.warning("Null-Referenz in  der Menge NetzBestandTeile des Objekts " + co.getPid());
            	continue;
            }
            
            if (pruefeObjektAufUnendlicheRekursion(so, ho, meldung, false, false))
            {
            	continue;
            }
            
            if ( so.isOfType( "typ.verkehrsModellNetz" ) )
            {
              IHierarchieObjektTyp hotKind = HOT_Netz.VerkehrsModellNetzReferenz;

              HierarchieObjekt kind = new HierarchieObjekt( hotKind, new SystemObject[] { so }, EBENE2);
              // Hinzufgen HierarchieObjekt-Verbindung
              hierarchie.objektHinzufuegen(ho, kind);
							// weitere Hierarchie-Ebene
              bestimmeUntergeordneteHOs( kind );
            }              
            else if ( so.isOfType( "typ.schwachStelle" ) )
            {
              IHierarchieObjektTyp hotKind = HOT_Netz.SchwachStelleReferenz;

              HierarchieObjekt kind = new HierarchieObjekt( hotKind, new SystemObject[] { so }, EBENE2);
              // Hinzufgen HierarchieObjekt-Verbindung
              hierarchie.objektHinzufuegen(ho, kind);
              bestimmeUntergeordneteHOs( kind );
            }
            else if ( so.isOfType( "typ.ueresStraenSegment" ) )
            {
              IHierarchieObjektTyp hotKind = HOT_Netz.ASS;

              HierarchieObjekt kind = new HierarchieObjekt( hotKind, new SystemObject[] { so }, EBENE2);
              // Hinzufgen HierarchieObjekt-Verbindung
              hierarchie.objektHinzufuegen(ho, kind);
							// keine weitere Hierarchie-Ebene
            }
            else
            {
              // Log Fehler: Mengenelement falscher Typ
              _debug.warning( "Unzulssiger Objekttyp " + so.getPid()+
                   " in Menge NetzBestandTeile von Objekt " + co.getPid());
            }
          }
        }
        else
        {
          // Log Fehler: Menge fehlt
          _debug.warning("Menge NetzBestandTeile nicht gefunden fr Objekt " + co.getPid());
        }
        
        if (meldung.getChildren().length > 0)
        {
        	MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(meldung, System.currentTimeMillis(), false);
        }
      }
      
    }
    else
    {
      _debug.warning("bergebenes Objekt " + objekt + " ist von unzulssigem Typ");
    
    }
  }
  
	@Override
  public IHierarchieObjektTyp[] getUntergeordneteHOTs(Object objekt)
  {
    if ( objekt instanceof ConfigurationArea )
    {
      return EBENE1;
    }
    else if ( objekt instanceof HierarchieObjekt )
    {
      HierarchieObjekt ho = (HierarchieObjekt) objekt;

      if ( ho.getHierarchieObjektTyp().equals( HOT_Netz.VerkehrsModellNetz ) || 
          ho.getHierarchieObjektTyp().equals( HOT_Netz.SchwachStelle ) ||
          ho.getHierarchieObjektTyp().equals( HOT_Netz.VerkehrsModellNetzReferenz ) || 
          ho.getHierarchieObjektTyp().equals( HOT_Netz.SchwachStelleReferenz ))
      {
        return EBENE2;
      }
    }

    return null;
  }
	
	 private ObjectSet erzeugeMengeAmObjekt(ConfigurationObject ko, String mengenTyp, 
			 String mengenName, MultiStatus status)
	  {
	    ObjectSet result = null;
	    
	    ObjectSetType setTyp = getHierarchie().getDataModel().getObjectSetType(mengenTyp);
	    try
	    {
	      result = (ObjectSet)ko.getConfigurationArea().createConfigurationObject(setTyp, null, mengenName, null);
	      ko.addSet(result);
	      
	      status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Menge " + mengenName+ 
	          " vom Typ " +  setTyp.getPid() +  " fr Systemobjekt " + ko.getPid() + " erzeugt"));
	      
	    }
	    catch (ConfigurationChangeException e)
	    {
	      status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,  "Menge " + mengenName + " vom Typ " + 
	          setTyp.getPid() + " konnte fr das Systemobjekt " + ko.getPid() + " nicht erstellt werden", e));
	    }
	    
	    return result;
	  }
  
	private void behandleMengenZuordung(HierarchieObjekt vater, IHierarchieObjektTyp hotNeuesHO, 
			SystemObject so, MultiStatus status, Map<ObjectSet, SystemObject> undoMengenListe)
	{
	  Assert.isLegal((Arrays.asList(HOT_Netz.values()).contains(hotNeuesHO)), 
	  		"HierarchieObjekttyp fr neues Objekt ist nicht zulssig");
	  Assert.isLegal(so.isOfType("typ.verkehrsModellNetz") || so.isOfType("typ.schwachStelle") || 
	  		so.isOfType("typ.ueresStraenSegment"), "Systemobjekt " + so.getPid() + " ist von unzulssigem Typ");
	  
		ConfigurationObject ko = (ConfigurationObject)vater.getSystemObjekt(0);
		Assert.isTrue(ko.isOfType("typ.verkehrsModellNetz") || ko.isOfType("typ.schwachStelle"));
		
		ObjectSet menge = ko.getNonMutableSet("NetzBestandTeile");
		
		if (!pruefeObjektAufUnendlicheRekursion(so, vater, status, true, true))
		{	
			if (menge instanceof NonMutableSet)
			{
				Collection<SystemObject> elemente = ((NonMutableSet)menge).getElementsInModifiableVersion();
				if (elemente.contains(so))
				{
					status.add(new Status(Status.WARNING, Activator.PLUGIN_ID,	"Systemobjekt " + so.getPid() + 
							" ist bereits in der Menge NetzBestandTeile des Systemobjekts " +	ko.getPid() + " enthalten"));		
				}
			}
		}
		
		
		if (!status.matches(IStatus.WARNING | IStatus.ERROR | IStatus.CANCEL))
		{
			if (menge == null)
			{
				menge = erzeugeMengeAmObjekt(ko, "menge.netzBestandTeile", "NetzBestandTeile", status);
			}
			
			try
			{
				
				menge.add(so);
				
				if (undoMengenListe != null)
				{	
					undoMengenListe.put(menge, so);
				}	
				status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Systemobjekt " + so.getPid() + 
						" der Menge NetzBestandTeile des Systemobjekts " +	ko.getPid() + " hinzugefgt"));		
			}
			catch (ConfigurationChangeException e)
			{
				status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Systemobjekt " + so.getPid() + 
						" konnte nicht der Menge NetzBestandTeile des Systemobjekts " +	ko.getPid() + " hinzugefgt werden", e));
			}
		}
	}
	
	private List<ObjectSet> erzeugeMengenFuerTyp(IHierarchieObjektTyp hot, String typ, 
      ConfigurationArea kb, MultiStatus status, List<SystemObject> undoListe)
  {
    List<ObjectSet> result = new Vector<ObjectSet>();
    
    for (String mengenName: hot.getTypInfo(typ).getMengen())
    {
      MengeInfo info = hot.getTypInfo(typ).getMengeInfo(mengenName);
      String mengenTyp = info.getTyp();
      ObjectSetType setTyp = kb.getDataModel().getObjectSetType(mengenTyp);
      ObjectSet menge;
      try
      {
        menge = (ObjectSet)kb.createConfigurationObject(setTyp, "", mengenName, null);
        
        undoListe.add(0, menge);
        status.add(new Status(Status.OK, Activator.PLUGIN_ID, 
            "Menge " + mengenName + " vom Typ " + setTyp.getPid() + " erzeugt"));
      }
      catch (ConfigurationChangeException e)
      {
        status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Fehler beim Erzeugen der Menge " 
            + mengenName + " vom Typ " + setTyp.getName() + " erzeugt", e));
        return result;
      }
      
      result.add(menge);
    }
    
    return result;
  }
	
	 private SystemObject erzeugeSystemObjekt(String pid, String typ, ConfigurationArea kb,
	      List<ObjectSet> mengen, MultiStatus status, List<SystemObject> undoListe)
	  {
	    SystemObject result = null;
	    ConfigurationObjectType koTyp = (ConfigurationObjectType)kb.getDataModel().getType(typ);
	    if (koTyp != null)
	    { 
	      try
	      {
	        result = kb.createConfigurationObject(koTyp, pid, "", mengen);
	          
	        undoListe.add(0, result);
	        MultiStatus meldung = new MultiStatus(Activator.PLUGIN_ID, IStatus.OK,
	            "Systemobjekt " + pid + " vom Typ " + typ + " erzeugt", null);
	        
	        if (mengen != null && !mengen.isEmpty())
	        {
	          for (ObjectSet set: mengen)
	          {
	            meldung.add(new Status(Status.OK, Activator.PLUGIN_ID,  "Menge " + set.getName() + 
	                " dem Systemobjekt " + pid + " vom Typ " + typ + " zugewiesen"));
	          }
	        }
	        
	        status.add(meldung);
	      }
	      catch (ConfigurationChangeException e)
	      {
	        status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, 
	            "Systemobjekt " + pid + " vom Typ " + typ + " konnte nicht erzeugt werden", e));
	      }
	    }
	    else
	    {
	      status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,  "Fr " + typ + 
	          " kann kein Typ im Datenmodell bestimmt werden", null));
	    }
	    return result;
	  }
	
	public HOMitStatus erzeugeNeuesHO(ConfigurationArea kb, Object vater, String pid[], IHierarchieObjektTyp hot,
			boolean pruefePid)
  {
      return  erzeugeNeuesHO(kb, vater, pid, hot, pruefePid, false);
  }
	
	
  private HOMitStatus erzeugeNeuesHO(ConfigurationArea kb, Object vater, String[] pid, IHierarchieObjektTyp hot, 
  		boolean pruefePid, boolean vonKopie)
  {
  	if (hot.isSystemObjektVorhanden())
  	{
  		Assert.isLegal(vater instanceof HierarchieObjekt, "Keine Referenzierung auf erster Hierarchieebene");
  		
  		MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK,
  				"Erzeuge neues Hierarchieobjekt vom Typ " + hot, null);
  		HierarchieObjekt ho = null;
  		ConfigDataModel dataModel = getHierarchie().getDataModel();
    	SystemObject so = dataModel.getObject(pid[0]);

      if (so == null)
      {
      	Collection<ConfigurationArea> areas = dataModel.getAllConfigurationAreas().values();
        for (ConfigurationArea ca : areas)
        {
        	Collection<SystemObject> elements = ca.getNewObjects();
          for (SystemObject neu : elements)
          {
          	if (pid[0].equals(neu.getPid()))
            {
          		so = neu;
          		break;
            }
          }
        }
      }

      if (so != null)
      {
     		behandleMengenZuordung((HierarchieObjekt)vater, hot, so, status, null);
     		
     		if (!status.matches(IStatus.WARNING | IStatus.ERROR | IStatus.CANCEL))
     		{	
     			ho = new HierarchieObjekt(hot, new SystemObject[] { so }, EBENE2);
     			objektHinzufuegen(vater, ho, !vonKopie);
     		}	
      }
      
      return new HOMitStatus(ho, status);
  	}
  	else
  	{
  		return erzeugeNeuesHOIntern(kb, vater, pid, hot, pruefePid);
  	}
  }
  
	private HOMitStatus erzeugeNeuesHOIntern(ConfigurationArea kb, Object vater, String[] pid, IHierarchieObjektTyp hot, 
			boolean pruefePid)
  {
    Assert.isLegal(Arrays.asList(getUntergeordneteHOTs(vater)).contains(hot),"Unzulssiger HOT " + hot + 
          " fr bergeordnetes Objekt " + vater);
    Assert.isLegal(pid.length == hot.getTypen().size(), "Die Lnge des Feldes pid muss der Anzahl der Systemoobjekttypen von " + 
        hot + " entsprechen");
    
    HierarchieObjekt ho = null;
      
    SystemObject[] neueObjekte = new SystemObject[hot.getTypen().size()];
    
    List<SystemObject> undoObjektListe = new LinkedList<SystemObject>();
    Map<ObjectSet, SystemObject> undoMengenListe = new LinkedHashMap<ObjectSet, SystemObject>();
    
    MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK,
        "Erzeuge neues Hierarchieobjekt vom Typ " + hot, null);
    
    if (pruefePid)
    { 
      PruefungenTools pruefungenTools = new PruefungenTools(getHierarchie().getDataModel());
      for (int i = 0; i < pid.length; i += 1)
      { 
        status.add(pruefungenTools.pruefePid(pid[i]));
      }
    } 
    
    if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
    { 
      try
      { 
        int index = 0;
        
        for (String typ: hot.getTypen())
        {
          List<ObjectSet> mengen = erzeugeMengenFuerTyp(hot, typ, kb, status, undoObjektListe);
          if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
          {
            neueObjekte[index] = erzeugeSystemObjekt(pid[index], typ, kb, mengen, status, undoObjektListe);
          }
          if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
          {
            if (vater instanceof HierarchieObjekt) // Keine Mengenzuordnung auf der 1. Ebene
            {
              behandleMengenZuordung((HierarchieObjekt)vater, hot, neueObjekte[index], status, 
                  undoMengenListe);
            }
          }
                  
          index += 1;
        }
      } 
      finally
      {
        if (status.matches(IStatus.ERROR | IStatus.CANCEL))
        {
          // Alle Aktionen rckgngig
          MultiStatus meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK,
              "Rollback Erzeugen Hierarchieobjekt", null);
          meldung.add(new Status(Status.INFO, Activator.PLUGIN_ID, "Rollback Erzeugen Hierarchieobjekt", null));
          
          for (ObjectSet menge: undoMengenListe.keySet())
          {
            SystemObject so = undoMengenListe.get(menge);
            try
            {
              menge.remove(so);
              meldung.add(new Status(Status.INFO, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid() + 
                  " aus der Menge " + menge.getName() + " vom Typ " + menge.getType().getPid() + 
                  " entfernt", null));
            }
            catch (ConfigurationChangeException e)
            {
              meldung.add(new Status(Status.WARNING, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid() + " " +
                  "konnte nicht aus der Menge " + menge.getName() + " vom Typ " + menge.getType().getPid() +
                  " entfernt werden", e));
            }
            catch (RuntimeException e)
            {
              meldung.add(new Status(Status.WARNING, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid() + " " +
                  "konnte nicht aus der Menge " + menge.getName() + " vom Typ " + menge.getType().getPid() +
                  " entfernt werden", e));
            }
          }
          for (SystemObject so: undoObjektListe)
          {
            try
            {
              so.invalidate();
              meldung.add(new Status(Status.INFO, Activator.PLUGIN_ID, "Systemobjekt " + so.getPidOrNameOrId() + 
                  " vom Typ " + so.getType().getPid() + " gelscht", null));
            }
            catch (ConfigurationChangeException e)
            {
              meldung.add(new Status(Status.WARNING, Activator.PLUGIN_ID, "Systemobjekt " + so.getPidOrNameOrId() + 
                  " vom Typ " + so.getType().getPid() + " konnte nicht gelscht werden", e));
            }
            catch (RuntimeException e)
            {
              meldung.add(new Status(Status.WARNING, Activator.PLUGIN_ID, "Systemobjekt " + so.getPidOrNameOrId() + 
                  " vom Typ " + so.getType().getPid() + " konnte nicht gelscht werden", e));
            }
          }
          status.add(meldung);
        }
      } 
      
      if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
      {
        ho = new HierarchieObjekt(hot, neueObjekte, getUntergeordneteHOTs(vater));
        getHierarchie().objektHinzufuegen(vater, ho);
      }
    } 
    
    return new HOMitStatus(ho, status);
  }
  
	@Override
	protected void bereinigeSystemObjektAenderung(HierarchieObjekt ho, ConfigurationObject koAlt,
      ConfigurationObject koNeu) throws ConfigurationChangeException
  {
		Assert.isLegal(koAlt != null);
		Assert.isLegal(koNeu != null);
		
		IHierarchie hierarchie = getHierarchie();
		
	  // Ersetze Systemobjekt in allen Mengen
    for (ConfigurationArea kb: hierarchie.getDataModel().getAllConfigurationAreas().values())
  	{
  		for (HierarchieObjekt kind: hierarchie.getUntergeordneteObjekte(kb))
  		{
  			NonMutableSet menge = ((ConfigurationObject)kind.getSystemObjekt(0)).getNonMutableSet("NetzBestandTeile");
  			if (menge != null && menge.getElementsInModifiableVersion().contains(koAlt))
				{
					menge.add(koNeu);
					menge.remove(koAlt);
				}
  		}
  	}
  }
	
	private Collection<NonMutableSet> entferneAusAllenMengen(ConfigurationObject ko)
		throws ConfigurationChangeException
	{
		Assert.isLegal(ko != null);
		
		Collection<NonMutableSet> result = new LinkedList<NonMutableSet>();
		IHierarchie hierarchie = getHierarchie();
		
		for (ConfigurationArea kb: hierarchie.getDataModel().getAllConfigurationAreas().values())
  	{
  		for (HierarchieObjekt kind: hierarchie.getUntergeordneteObjekte(kb))
  		{
  			NonMutableSet menge = ((ConfigurationObject)kind.getSystemObjekt(0)).getNonMutableSet("NetzBestandTeile");
  			if (menge != null && menge.getElementsInModifiableVersion().contains(ko))
				{
  				menge.remove(ko);
  				result.add(menge);
  			}	
			}
  	}
		
		return result;
	}
	
	private void einfuegenInAlleMengen(ConfigurationObject ko, Collection<NonMutableSet> mengen)
		throws ConfigurationChangeException
	{
		Assert.isLegal(ko != null);
		Assert.isLegal(mengen != null);
		
		for (NonMutableSet menge: mengen)
		{
			menge.add(ko);
		}
	}
 
	@Override
	protected ConfigurationObject behandleSystemObjektAenderung(HierarchieObjekt ho, ConfigurationObject ko, 
	     ConfigurationArea kb, String neuePid, boolean loescheVorab) throws ConfigurationChangeException
	{
		Collection<NonMutableSet> mengen = null;
		
    if (loescheVorab)
    {
    	mengen = entferneAusAllenMengen(ko);
      ko.invalidate();
    }
    
    ConfigurationObject result = dupliziereObjekt(ko, kb, neuePid);
    
    if (loescheVorab)
    {
    	einfuegenInAlleMengen(result, mengen);
    }
    else
    {
      bereinigeSystemObjektAenderung(ho, ko, result);
      ko.invalidate();
    }
    
    return result;
	}
	
	
	private MultiStatus kopiereDaten(HierarchieObjekt srcHO, HierarchieObjekt destHO, 
  		boolean bildungsRegelnFuerName, boolean bildungsRegelnFuerAtg)
  {
		Assert.isTrue(srcHO.getHierarchieObjektTyp().equals(destHO.getHierarchieObjektTyp()), 
				"Beide bergebenen Hierarchieobjekte mssen vom gleichen Hierarchieobjekttyp sein");
  	
		int index = 0;
		MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Daten von " + srcHO +	" nach " + destHO, null);
		IBildungsRegeln br = DatenSpeicher.getInstanz().getBildungsRegeln();
		ConfigDataModel dataModel = getHierarchie().getDataModel();
		
		ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
		
		for (SystemObject src: srcHO.getSystemObjekte())
		{
			SystemObject dest = destHO.getSystemObjekte().get(index);
			String name;
			if (bildungsRegelnFuerName)
			{
				name = br.getName(dest);
			}
			else
			{
				name = src.getName();
			}
			
			try
			{
				dest.setName(name);
				status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Setze Name \"" + name + 
						"\" fr Systemobjekt " +	dest.getPid()));
			}
			catch (ConfigurationChangeException e)
			{
				status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Name \"" + name + 
						"\" fr Systemobjekt " +	dest.getPid() + " konnte nicht gesetzt werden", e));
				return status;
			}
			
			for (AttributeGroupUsage atgu: src.getUsedAttributeGroupUsages())
			{
				Data dataSrc = ((ConfigSystemObject)src).getConfigurationData(atgu, lookup);
				if (dataSrc != null)
				{
					try
					{
						Data dataDest = DataFactory.createModifiableCopy(dataSrc, lookup);;
						if (bildungsRegelnFuerAtg)
						{
							dataDest = br.getDatensatz(destHO, dest, atgu.getAttributeGroup(), dataDest, true);
						}
						dest.setConfigurationData(atgu, dataDest);
	  				status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Kopiere Daten der ATG-Verwendung " +
	  						atgu.getAttributeGroup().getPid() + "/" + atgu.getAspect().getPid() + 
	  						" fr Systemobjekt " +	dest.getPid())); 
					}
					catch (ConfigurationChangeException e)
					{
						status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Daten der ATG-Verwendung " +
								atgu.getAttributeGroup().getPid() + "/" + atgu.getAspect().getPid() + 
								" fr Systemobjekt " +	dest.getPid() + " konnten nicht kopiert werden", e));
						return status;
					}
				}
			}
			
			
			AttributeGroup atg = dataModel.getAttributeGroup("atg.dynamischeMenge");
			
			for (ObjectSet mengeSrc: ((ConfigurationObject)src).getObjectSets())
			{
				if (mengeSrc instanceof MutableSet)
				{
					ObjectSet mengeDest = ((ConfigurationObject)dest).getMutableSet(mengeSrc.getName());
					if (mengeDest != null)
					{
						int anzahl = 0;
						try
						{
							for (SystemObject element: mengeSrc.getElements())
							{
								mengeDest.add(element);
								anzahl += 1;
							}
							if (anzahl > 0)
							{	
								status.add(new Status(Status.OK, Activator.PLUGIN_ID,	anzahl + " Elemente der Menge " + mengeSrc.getName() + 
										" fr Systemobjekt " + dest.getPid() + " kopiert"));
							}	
						}
						catch (ConfigurationChangeException e)
						{
							status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	anzahl + "Fehler beim Kopieren der Elemente der Menge " + 
									mengeSrc.getName() + " fr Systemobjekt " + dest.getPid(), e));
						}
						
						Data dataSrc = ((ConfigSystemObject)mengeSrc).getConfigurationData(atg, lookup);
						if (dataSrc != null)
						{
							try
							{
								Data dataDest = DataFactory.createModifiableCopy(dataSrc, lookup);;
								if (bildungsRegelnFuerAtg)
								{
									dataDest = br.getDatensatz(destHO, mengeDest, atg, dataDest, true);
								}
								mengeDest.setConfigurationData(atg, dataDest);
			  				status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Kopiere Daten der ATG " +
			  						atg.getPid() + " der Menge " + mengeSrc.getName() + " fr Systemobjekt " + dest.getPid()));
							}
							catch (ConfigurationChangeException e)
							{
								status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Daten der ATG " +
										atg.getPid() + " der Menge " + mengeSrc.getName() +	" fr Systemobjekt " +	
										dest.getPid() + " konnten nicht kopiert werden", e));
								return status;
							}
						}
					}
				}
			}
			index++;
		}
		
		return status;
  }
	
	public HOMitStatus kopiereHO(HierarchieObjekt ho, ConfigurationArea kb, Object neuerVater, 
			String neuePid[], boolean rekursiv, boolean isWurzel)
  {
    return kopiereHO(ho, kb, neuerVater, neuePid, rekursiv, true, true, isWurzel);
  }
	
	private HOMitStatus kopiereHO(HierarchieObjekt ho, ConfigurationArea kb, Object neuerVater, 
			String neuePid[], boolean rekursiv, boolean anwendenBR, boolean pruefePid, boolean isWurzel)
	  {
	    MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Hierarchieobjekt " + ho, null);
	    String[] pid = neuePid;
	    if (pid == null)
	    { 
	      pid = new String[1];
	      pid[0] = ho.getSystemObjekt(0).getPid();
	    } 
	    
	    IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
	    HierarchieObjekt neu = null;
	    IHierarchieObjektTyp hotVater =  neuerVater instanceof HierarchieObjekt ?
	    		((HierarchieObjekt)neuerVater).getHierarchieObjektTyp() : null;
	   	    		
	    if (hotVater == null || HOT_Netz.VerkehrsModellNetz.equals(hotVater) ||	HOT_Netz.SchwachStelle.equals(hotVater))
	    {
		    HOMitStatus result = erzeugeNeuesHO(kb, neuerVater, pid, hot, pruefePid, true);
		    neu = result.getHO();
		    status.add(result.getStatus());
	     
		    if (!status.matches(IStatus.WARNING | IStatus.ERROR | IStatus.CANCEL) && !hot.isSystemObjektVorhanden())
		    {
		      status.merge(kopiereDaten(ho, neu, anwendenBR, anwendenBR));
		    }
		    
		    if (!status.matches(IStatus.WARNING | IStatus.ERROR | IStatus.CANCEL) && !hot.isSystemObjektVorhanden() && rekursiv)
		    {
		      for (HierarchieObjekt lauf: getHierarchie().getUntergeordneteObjekte(ho))
		      {
		        MultiStatus meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
		            "Kopiere untergeordnetes Hierarchieobjekt " + lauf + " zu " + ho, null);
		        HOMitStatus ret = kopiereHO(lauf, kb, neu, null, true, anwendenBR, pruefePid, false);
		        meldung.merge(ret.getStatus());
		        status.add(meldung);
		        if (status.matches(IStatus.ERROR | IStatus.CANCEL))
		        {
		          break;
		        }
		      }
		    }
	    }
	    else
	    {
	    	Assert.isTrue(HOT_Netz.VerkehrsModellNetzReferenz.equals(hotVater) ||	
	    			HOT_Netz.SchwachStelleReferenz.equals(hotVater) || HOT_Netz.ASS.equals(hotVater));
	    	
	    	if (isWurzel)
	    	{
	    		HOMitStatus result = erzeugeNeuesHO(kb, neuerVater, pid, hot, pruefePid, true);
	 		    neu = result.getHO();
	 		    status.add(result.getStatus());
	    	}
	    	else
	    	{	
		      neu = new HierarchieObjekt( hot, new SystemObject[] { ho.getSystemObjekt(0) }, EBENE2);
		      objektHinzufuegen( neuerVater, neu, false );
		    	status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Erzeuge neues HierarchieObjekt vom Typ " + ho.getHierarchieObjektTyp()));
	    	}	
	    }
	   	    
	    return new HOMitStatus(neu, status);
	  }
 
  @Override
 public IStatus uebernehmeDaten(HierarchieObjekt ho, HODaten daten)
 {
   MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
       "bernehme bearbeitete Daten fr Hierarchieobjekt " + ho, null);
   
   List<SystemObject> objektListe = daten.getSystemObjekte();
//   DatenModellUpdateListener.getInstanz().notifyDatenModellUpdateListener(objektListe);
   
   for (int i = 0; i < objektListe.size(); i += 1)
   {
     SystemObject so = objektListe.get(i);
     
     if (ho.getSystemObjekte().contains(so))
     {
       SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
         
    // PID
 			if (!so.getPid().equals(soDaten.getPid()))
 			{
         try
         {
	  				so = behandleSystemObjektAenderung(ho, (ConfigurationObject)so, soDaten.getPid());
//	  				ho.setSystemObjekt(i, so);
	  				new ObjektErsetzen(ho.getSystemObjekt(0), so).alleObjekteAktualisieren();
	  				status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
	  						"Neues Objekt mit PID " + so.getPid() + 
	  						" vom Typ " + so.getType().getPid() + " erzeugt"));
         }
         catch (ConfigurationChangeException e)
       	{
         	status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
     					"Fehler beim Behandeln der PID-nderung auf " + soDaten.getPid() +
     					" fr Objekt " + so.getPid(), e));
       	}
 			}

 			// Name
 			if (so.getName() == null || !so.getName().equals(soDaten.getName()))
 			{
	  			try
	      	{
	  				so.setName(soDaten.getName());
	  				status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
	   						"Name \"" + soDaten.getName() + "\" bernommen"));
	      	}
	  	   	catch (ConfigurationChangeException e)
	      	{
	  	   		status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
	   						"Name \"" + soDaten.getName() + "\" konnte nicht bernommen werden" , e));
	      	}
 			  } 	
       
       TypInfo typInfo = ho.getHierarchieObjektTyp().getTypInfo(so.getType().getPid());
       IHierarchie hierarchie = getHierarchie();
            
       // ATGs
     	List<String> atgList = ho.getHierarchieObjektTyp().getTypInfo(so.getType().getPid()).getAtgs();
			
	    for (String atgPid: atgList)
  		{
	    	AttributeGroup atg = hierarchie.getDataModel().getAttributeGroup(atgPid);
	    	if (atg != null)
	    	{
	    		ConfigDataModel dataModel = hierarchie.getDataModel();
	    		ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
	    		
	    		//Data ist = so.getConfigurationData(atg);
	        Data ist = ((ConfigSystemObject)so).getConfigurationData(atg, lookup);
	        		        
	    		Data soll = soDaten.getAtgDaten(atgPid);
	  
          if (soll != null)
          {
          	if (ist == null || soDaten.isDatenGeaendert(atgPid))
          	{	
	          	if (!HierarchieObjekt.isAttributgruppeAenderbar(so, atg))
			    		{
	          		 try
	               {
	     	  				so = behandleSystemObjektAenderung(ho, (ConfigurationObject)so, so.getPid());
	     	  				ho.setSystemObjekt(i, so);
	     	  				status.add(new Status(IStatus.OK, Activator.PLUGIN_ID,
	     	  						"ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
	     	  						" bedingt Erzeugung neues Objekt mit PID " + so.getPid() + " vom Typ " + 
	     	  						so.getType().getPid()));
	               }
	               catch (ConfigurationChangeException e)
	             	{
	               	status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
	               			"Bei der durch ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
	     	  						" bedingten Erzeugung neues Objekts mit PID " + so.getPid() + " vom Typ " +	
	     	  						so.getType().getPid() + " ist ein Fehler aufgetreten", e));
	               	continue;
	             	}
	          		
//	          		status.add(new Status(IStatus.INFO, Activator.PLUGIN_ID, 
//			   						"Eventuelle nderungen an konfigurierenden Daten der ATG " + atgPid +
//			   						" werden IGNORIERT, da wegen ATG-Verwendung (" + 
//			   						AttributeGroupUsage.Usage.OptionalConfigurationData + ") nicht mglich"));
//			    			continue;
	               
			    		}
	          	try
	          	{
	          		so.setConfigurationData(atg, soll);
	          		status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
	       						"Konfigurierende Daten der ATG " + atgPid + " bernommen"));
	          	}
	          	catch (ConfigurationChangeException e)
	          	{
	          		status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
	       						"Fehler beim bernehmen konfigurierender Daten der ATG " + atgPid, e));
	          	}
          	}	
          }
          else // Keine Daten fr diese ATG bergeben => vorhanden Daten lschen
          {
          	if (ist != null)
          	{
          		if (!HierarchieObjekt.isAttributgruppeAenderbar(so, atg))
			    		{
          			 try
	               {
	     	  				so = behandleSystemObjektAenderung(ho, (ConfigurationObject)so, so.getPid());
	     	  				ho.setSystemObjekt(i, so);
	     	  				status.add(new Status(IStatus.OK, Activator.PLUGIN_ID,
	     	  						"ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
	     	  						" bedingt Erzeugung neues Objekt mit PID " + so.getPid() + " vom Typ " + 
	     	  						so.getType().getPid()));
	               }
	               catch (ConfigurationChangeException e)
	             	{
	               	status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
	               			"Bei der durch ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
	     	  						" bedingten Erzeugung neues Objekts mit PID " + so.getPid() + " vom Typ " +	
	     	  						so.getType().getPid() + " ist ein Fehler aufgetreten", e));
	               	continue;
	             	}
          			
//			    			status.add(new Status(IStatus.INFO, Activator.PLUGIN_ID, 
//			   						"Eventuelle nderungen an konfigurierenden Daten der ATG " + atgPid +
//			   						" werden IGNORIERT, da wegen ATG-Verwendung (" + 
//			   						AttributeGroupUsage.Usage.OptionalConfigurationData + ") nicht mglich"));
//			    			continue;
	               
			    		}
          		try
	          	{
	          		so.setConfigurationData(atg, null);
	          		status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
	       						"Konfigurierende Daten der ATG " + atgPid + " gelscht"));
	          	}
	          	catch (ConfigurationChangeException e)
	          	{
	          		status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
	       						"Fehler beim Lschen konfigurierender Daten der ATG " + atgPid, e));
	          	}
          	}
          }
	    	}
	    	else
	    	{
	    		_debug.warning("Bearbeitung Objekt " + so.getPid() + 
	    				": Keine Attributgruppe im Datenmodell gefunden fr " + atgPid);
	    	}
  		}	
       
       // Mengen
       for (String mengenName: typInfo.getDynMengen())
       {
         MutableSet menge = ((ConfigurationObject)so).getMutableSet(mengenName);
         
         if (menge != null)
         {
           ConfigDataModel dataModel = hierarchie.getDataModel();
           ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
           AttributeGroup atg = dataModel.getAttributeGroup("atg.dynamischeMenge");
           
           Data ist = ((ConfigSystemObject)menge).getConfigurationData(atg, lookup);
           
           Data soll = soDaten.getDynMengenDaten(mengenName);
           
           if (soll != null)
           {
             if (ist == null || soDaten.isDatenGeaendert(mengenName))
             {	
	             if (!HierarchieObjekt.isAttributgruppeAenderbar(menge, atg))
	             {
	            	 try
	               {
	     	  				so = behandleSystemObjektAenderung(ho, (ConfigurationObject)so, so.getPid());
	     	  				menge = ((ConfigurationObject)so).getMutableSet(mengenName);
	     	  				ho.setSystemObjekt(i, so);
	     	  				status.add(new Status(IStatus.OK, Activator.PLUGIN_ID,
	     	  						"ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
	     	  						" bedingt Erzeugung neues Objekt mit PID " + so.getPid() + " vom Typ " + 
	     	  						so.getType().getPid()));
	               }
	               catch (ConfigurationChangeException e)
	             	 {
	               	 status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
	               			"Bei der durch ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
	     	  						" bedingten Erzeugung neues Objekts mit PID " + so.getPid() + " vom Typ " +	
	     	  						so.getType().getPid() + " ist ein Fehler aufgetreten", e));
	               	 continue;
	             	 }
//	               status.add(new Status(IStatus.INFO, Activator.PLUGIN_ID, 
//	                   "Eventuelle nderungen an konfigurierenden Daten der ATG atg.dynamischeMenge der Menge " + 
//	                   mengenName + " werden IGNORIERT, da wegen ATG-Verwendung (" + 
//	                   AttributeGroupUsage.Usage.OptionalConfigurationData + ") nicht mglich"));
//	               continue;
	             }
	             try
	             {
	               menge.setConfigurationData(atg, soll);
	               status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
	                   "Konfigurierende Daten der ATG atg.dynamischeMenge der Menge " + mengenName + " bernommen"));
	             }
	             catch (ConfigurationChangeException e)
	             {
	               status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
	                   "Fehler beim bernehmen konfigurierender Daten der ATG atg.dynamischeMenge der Menge " + 
	                   mengenName, e));
	               return status;
	             }
             }  
           }
           else // Keine Daten fr diese ATG bergeben => vorhanden Daten lschen
           {
             if (ist != null)
             {
               if (!HierarchieObjekt.isAttributgruppeAenderbar(menge, atg))
               {
              	 try
	               {
	     	  				so = behandleSystemObjektAenderung(ho, (ConfigurationObject)so, so.getPid());
	     	  				menge = ((ConfigurationObject)so).getMutableSet(mengenName);
	     	  				ho.setSystemObjekt(i, so);
	     	  				status.add(new Status(IStatus.OK, Activator.PLUGIN_ID,
	     	  						"ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
	     	  						" bedingt Erzeugung neues Objekt mit PID " + so.getPid() + " vom Typ " + 
	     	  						so.getType().getPid()));
	               }
	               catch (ConfigurationChangeException e)
	             	 {
	               	 status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
	               			"Bei der durch ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
	     	  						" bedingten Erzeugung neues Objekts mit PID " + so.getPid() + " vom Typ " +	
	     	  						so.getType().getPid() + " ist ein Fehler aufgetreten", e));
	               	 continue;
	             	 }
//                 status.add(new Status(IStatus.INFO, Activator.PLUGIN_ID, 
//                     "Eventuelle nderungen an konfigurierenden Daten der ATG atg.dynamischeMenge der Menge " + 
//                     mengenName + " werden IGNORIERT, da wegen ATG-Verwendung (" + 
//                     AttributeGroupUsage.Usage.OptionalConfigurationData + ") nicht mglich"));
//                 continue;
               }
               try
               {
                 menge.setConfigurationData(atg, null);
                 status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
                     "Konfigurierende Daten der ATG atg.dynamischeMenge der Menge " + mengenName + " gelscht"));
               }
               catch (ConfigurationChangeException e)
               {
                 status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
                     "Fehler beim Lschen konfigurierender Daten der ATG atg.dynamischeMenge der Menge " + 
                     mengenName, e));
                 return status;
               }
             }
           }
         }
         else
         {
           _debug.warning("Bearbeitung Objekt " + so.getPid() + 
               ": Menge " + mengenName + " nicht am Objekt vorhanden");
         }
       }
     }     
   }    
   
   return status;
 }
  
  protected void behandleVerschieben(HierarchieObjekt ho, Object neuerVater, MultiStatus status)
  {
  	Assert.isLegal(neuerVater instanceof HierarchieObjekt, "Kein Verschiebung auf erster Hierarchieebene");
		
  	IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
		MultiStatus meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK,
				"Verschiebe Hierarchieobjekt " + ho + " vom Typ " + hot, null);
		SystemObject so = ho.getSystemObjekt(0);
    
 		behandleMengenZuordung((HierarchieObjekt)neuerVater, hot, so, meldung, null);
   		
 		if (!meldung.matches(IStatus.WARNING | IStatus.ERROR | IStatus.CANCEL))
 		{
 			meldung.add(loescheHO(ho));
 		}
 		if (!meldung.matches(IStatus.WARNING | IStatus.ERROR | IStatus.CANCEL))
 		{
 			objektHinzufuegen(neuerVater, ho, true);
 		}
 		status.add(meldung);
  }
	  
  @Override
  protected void behandleVerschiebenInAnderenKB(HierarchieObjekt ho, Object neuerVater, ConfigurationArea srcKB,
  		ConfigurationArea destKB, MultiStatus status)
	{
  	IHierarchie hierarchie = getHierarchie();
  	Object alterVater = hierarchie.getUebergeordnetesObjekt(ho);
  	hierarchie.objektVerschieben(alterVater, neuerVater, ho);
	}
  
  protected boolean zuordnenKB(HierarchieObjekt ho, ConfigurationArea kb, MultiStatus status)
  {
    ConfigurationObject ko = (ConfigurationObject)ho.getSystemObjekt(0);
    
    if (ko.getConfigurationArea().equals(kb))
    {
      return true;
    }
    
    int index = 0;
    boolean result = false;
    boolean aktiviert = ko.getValidSince() < ko.getConfigurationArea().getModifiableVersion();
    
    try
    {
      ConfigurationObject neu = behandleSystemObjektAenderung(ho, ko, kb, null, !aktiviert);
//      ho.setSystemObjekt(index, neu)
      new ObjektErsetzen(ko, neu).alleObjekteAktualisieren();
      result = true;
      
      status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, "Fr Hierarchieobjekt " + ho + 
          " Objekt mit PID " + ko.getPid() + " gelscht und im Konfigurationsbereich " + 
          kb.getPid() + " identisch neu erzeugt " + ko.getType().getPid() + " erzeugt"));
      
      index += 1;
    }
    catch (Exception e)
    {
      status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Beim Lschen oder Neuerzeugen von Objekt mit Pid " + 
          ko.getPid() + " bei Hierarchieobjekt" + ho + " ist eine Ausnahme aufgetreten", e));
      if (!ko.isValid())
      {
        try
        {
        	ConfigurationObject neu = behandleSystemObjektAenderung(ho, ko, null);
//          ho.setSystemObjekt(index, ko);
          new ObjektErsetzen(ko, neu).alleObjekteAktualisieren();
        } 
        catch(Exception ex)
        {
          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Weitere Fehlerbehandlung nicht mglich", e));
        }
      }
    }
 
    
    return result;
  }
  
  
//  public HOMitStatus verschiebeHO(HierarchieObjekt ho, Object neuerVater)
//  {
//  	Assert.isNotNull(ho, "ho darf nicht null sein");
//    Object alterVater = _hierarchie.getUebergeordnetesObjekt(ho);
//    Assert.isNotNull(alterVater, "bergeordnetes Objekt von ho darf nicht null sein");
//    ConfigurationArea srcKB = _hierarchie.getKonfigurationsBereich(alterVater);
//    Assert.isNotNull(srcKB, "Der Konfigurationsbereich zu " + alterVater + " darf nicht null sein");
//    ConfigurationArea destKB = _hierarchie.getKonfigurationsBereich(neuerVater);
//    Assert.isNotNull(destKB, "Der Konfigurationsbereich zu " + neuerVater + " darf nicht null sein");
//    
//    MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Verschiebe Hierarchieobjekt " + ho, null);
//    
//    HierarchieObjekt neu = null;
//        
//    if (ho.getHierarchieObjektTyp().isSystemObjektVorhanden() || srcKB.equals(destKB)) 
//    {
//     	// HO ist Referenz oder wird wird im gleichen kb verschoben
//    	
//      if (neuerVater.equals(alterVater))
//      {
//        status.add(new Status(Status.INFO, Activator.PLUGIN_ID, "Objekt " + ho +
//        " kann nicht in sich selbst verschoben werden"));
//      }
//      else
//      {
//        behandleVerschieben(ho, neuerVater, status);
//        
//        if (!status.matches(IStatus.WARNING | IStatus.ERROR | IStatus.CANCEL))
//        {
//        	neu = ho;
//        }
//      }
//    }
//    else // HO wird in einen anderen kb verschoben
//    {
//    	HOHierarchie hoHierarchie = new HOHierarchie(ho, _hierarchie);
//   	  
//   	  status.add(loescheHO(ho));
//   	  
//   	  if (!status.matches(IStatus.WARNING | IStatus.ERROR | IStatus.CANCEL))
//   	  {
//   	    HOMitStatus result = kopiereHORekursiv(hoHierarchie.getWurzel(), destKB, neuerVater,
//   	        hoHierarchie, true);
//   	    status.add( result.getStatus() );
//   	    neu = result.getHO();
//   	  }
//    }
//    return new HOMitStatus( neu, status );
//  }
   
//  private HOMitStatus kopiereHORekursiv(HierarchieObjekt ho, ConfigurationArea kb, Object vater,
//      HOHierarchie hierarchie, boolean isWurzel)
//  {
//    MultiStatus meldung;
//    
//    if (ho == hierarchie.getWurzel())
//    {  
//      meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Hierarchieobjekt " + ho, null);
//    }
//    else
//    {
//      meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere untergeordnetes Hierarchieobjekt " + 
//          ho + " zu " + vater, null);
//    }
//    
//    HOMitStatus result = kopiereHO(ho, kb, vater, null, false, false, false, isWurzel);
//    meldung.merge( result.getStatus() );
//    HierarchieObjekt neu = result.getHO();
//
//    if (neu != null)
//    {  
//      for (HierarchieObjekt kind: hierarchie.getUntergeordneteObjekte( ho ))
//      {
//        result = kopiereHORekursiv(kind, kb, neu, hierarchie, false);
//        meldung.add( result.getStatus() );
//      }
//    }
//    
//    return new HOMitStatus(neu, meldung);
//  }
//  
  private void behandleZuordnungLoeschen(HierarchieObjekt ho, SystemObject so, MultiStatus status)
  {
  	Object vater = getHierarchie().getUebergeordnetesObjekt(ho);
  	if (vater instanceof HierarchieObjekt) // Nur Zuordnungen tiefer als Ebene 1
  	{
  		ConfigurationObject ko = null;
  		ObjectSet menge = null;
  		
	  	IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
	  	
		  if (HOT_Netz.VerkehrsModellNetzReferenz.equals(hot) || HOT_Netz.SchwachStelleReferenz.equals(hot) || 
		  		HOT_Netz.ASS.equals(hot))
			{
		  	HierarchieObjekt vaterHO = (HierarchieObjekt)vater;
		  	IHierarchieObjektTyp vaterHOT = vaterHO.getHierarchieObjektTyp();
		  	Assert.isTrue(HOT_Netz.VerkehrsModellNetz.equals(vaterHOT) || HOT_Netz.SchwachStelle.equals(vaterHOT) || 
		  			HOT_Netz.VerkehrsModellNetzReferenz.equals(vaterHOT) || HOT_Netz.SchwachStelleReferenz.equals(vaterHOT));
		  	ko = (ConfigurationObject)vaterHO.getSystemObjekt(0);
		  	menge = ko.getNonMutableSet("NetzBestandTeile");
			}
		  
			if (ko != null && menge != null)
	  	{
				try
	    	{
	  			menge.remove(so);
	  			
	  			status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " +	so.getPid() + 
	  					" vom Typ " + so.getType().getPid() + " aus der Menge " + menge.getName() + 
	  					" des Objekts " + ko.getPid() + " vom Typ " + ko.getType().getPid() + " entfernt"));
	  			
	    	}
	    	catch (ConfigurationChangeException e)
	    	{
	    		status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid() + 
	    				" vom Typ " + so.getType().getPid() + " konnte nicht aus der Menge " + menge.getName() +
	    				" des Objekts " +	ko.getPid() + " vom Typ " + ko.getType().getPid() + " entfernt werden", e));
	    	}
	  	}
		}
  }
  
  private void loescheObjektAusAllenMengen(SystemObject so, MultiStatus status)
  {
  	IHierarchie hierarchie = getHierarchie();
  	ConfigDataModel dataModel = hierarchie.getDataModel();
  	
  	for (ConfigurationArea kb: dataModel.getAllConfigurationAreas().values())
  	{
  		for (HierarchieObjekt ho: hierarchie.getUntergeordneteObjekte(kb))
  		{
  			ConfigurationObject ko = ((ConfigurationObject)ho.getSystemObjekt(0)); 
  			NonMutableSet menge = ko.getNonMutableSet("NetzBestandTeile");
  			if (menge != null && menge.getElementsInModifiableVersion().contains(so))
				{
  				try
  	    	{
  	  			menge.remove(so);
  	  			
  	  			status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " +	so.getPid() + 
  	  					" vom Typ " + so.getType().getPid() + " aus der Menge " + menge.getName() + 
  	  					" des Objekts " + ko.getPid() + " vom Typ " + ko.getType().getPid() + " entfernt"));
  	    	}
  	    	catch (ConfigurationChangeException e)
  	    	{
  	    		status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid() + 
  	    				" vom Typ " + so.getType().getPid() + " konnte nicht aus der Menge " + menge.getName() +
  	    				" des Objekts " +	ko.getPid() + " vom Typ " + ko.getType().getPid() + " entfernt werden", e));
  	    	}
				}
  		}
  	}
  }
  
  @Override
  public IStatus loescheHO(HierarchieObjekt ho)
  {
    // berprfung Plausibilitt der Parameter
    IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
    Object vater = getHierarchie().getUebergeordnetesObjekt(ho); 
    
    Assert.isTrue(Arrays.asList(getUntergeordneteHOTs(vater)).contains(hot), "Unzulssiges bergeordnetes Objekt " + 
        vater + " fr Hierarchieobjekt " + ho);
    
    MultiStatus status = null;
    
    if (vater instanceof HierarchieObjekt)
    {
    	// Referenzen
    	status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Entferne Hierarchieobjekt " + ho +
          " vom Typ " + hot, null);	
    	behandleZuordnungLoeschen(ho, ho.getSystemObjekt(0), status);
    	
    	if (!status.matches(IStatus.CANCEL | IStatus.ERROR))
    	{
    		objektEntfernen(vater, ho);
    	}
    }
    else
    {	
    	// Objekte
    	status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Lsche Hierarchieobjekt " + ho +
          " vom Typ " + hot, null);	
    	
    	SystemObject so = ho.getSystemObjekt(0);
    	loescheObjektAusAllenMengen(so, status);
    	
      if (!status.matches(IStatus.CANCEL | IStatus.ERROR))
      // Lsche Systemobjekt  
      {
        try
        {
          so.invalidate();
          
          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + 
              so.getPid() + " vom Typ " + so.getType().getName() + " gelscht"));
        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + 
              so.getPid() + " vom Typ " + so.getType().getName() + " konnte nicht gelscht werden", e));
        }
      }
      
      if (!status.matches(IStatus.CANCEL | IStatus.ERROR))
      {
      	objektEntfernen(vater, ho);
      }
      
    }
    	
    return status;
  }
  
  public IStatus pruefeDaten(HierarchieObjekt ho, HODaten daten)
  {
    IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
    IPruefungen pruefInstanz = DatenSpeicher.getInstanz().getPruefungen();
    MultiStatus result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Hierarchieobjekt " + ho, null);
    
    // Prfe Hierarchieobjekt
    result.merge(pruefInstanz.pruefeHierarchieObjekt(ho, daten));
    
    // Prfe auf offene Referenzen bei PID-nderungen
    LoeschenChecker loeschenChecker = new LoeschenChecker(this);
    List<SystemObject> liste = new Vector<SystemObject>();
    StringBuffer aend = new StringBuffer();
    StringBuffer loesch = new StringBuffer();
    
    for (SystemObject so: ho.getSystemObjekte())
    {
     SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
     
     Assert.isNotNull(soDaten, "Zu dem Systemobjekt " + so.getPid() + 
           " wurden keine Daten vom Typ SODaten bergeben");
     
     if (!so.getPid().equals(soDaten.getPid()))
     {
       liste.add(so);
       if (aend.length() > 0)
       {
         aend.append(", ");
       }
       aend.append(soDaten.getPid());
       if (loesch.length() > 0)
       {
         loesch.append(", ");
       }
       loesch.append(so.getPid());
     }
    }
    
    if (liste.size() > 0)
    {
       MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK,  "Durch PID-nderung(en) auf " + 
           aend + " werden folgende Objekte gelscht: " + loesch, null);
       
//       status.add(loeschenChecker.bestimmeOffeneReferenzen(liste));
       
       //Collection<SystemObjectType> typen = loeschenOffeneReferenzenVerboten(ho.getHierarchieObjektTyp());
       //boolean flag = verwendeZuLoeschendeObjekteZurPruefungOffeneReferenzen();
       
       status.add(loeschenChecker.bestimmeOffeneReferenzen(liste, null, false));
       
       result.add(status);
     }
    
    // Prfe Systemobjekt-Daten
    for (SystemObject so: ho.getSystemObjekte())
    {
       SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
       Assert.isNotNull(soDaten, "Daten fr Systemobjekt " + so.getPid() +
             " erforderlich, aber nicht bergeben");
       
       MultiStatus objektStatus;
       if (ho.getSystemObjekte().size() > 1)
       {
         objektStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe ATG-Daten fr Systemobjekt " + 
             so.getPid() + " vom Typ " + so.getType().getPid(), null);
       }
       else
       {
         objektStatus = result;
       }
       
       // Prfe ATG-Daten fr das Systemobjekt
       TypInfo typInfo = hot.getTypInfo(so.getType().getPid());
       Assert.isNotNull(typInfo, "TypInfo fehlt fr  " + so.getType().getPid() + 
             " im HierarchieObjektTyp " + hot);
       
       for (String atgPid: typInfo.getAtgs())
       {
         AtgInfo atgInfo = typInfo.getAtgInfo(atgPid);
         Assert.isNotNull(atgInfo, "AtgInfo fr ATG " + atgPid + " fehlt in TypInfo fr " + 
               so.getType().getPid() + " im HierarchieObjektTyp " + hot);
         
         Data data = soDaten.getAtgDaten(atgPid);
         if (data == null && (atgInfo.isMandatory() || !atgInfo.isOptional()))
         {
           objektStatus.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Fr die nicht-optionale ATG " +
               atgPid + " sind keine Daten vorhanden"));           
         }
         if (data != null)
         {
           MultiStatus atgStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
               "Prfe Daten der ATG " + atgPid, null);
           atgStatus.merge(pruefInstanz.pruefeDaten(ho, so, atgInfo, data));
         
           objektStatus.add(atgStatus);
         }
       }
       
//       // Prfe Mengen-Daten fr das Systemobjekt
//       TypInfo typInfo = hot.getTypInfo(so.getType().getPid());
//       ConfigDataModel dataModel = getHierarchie().getDataModel();
//       AttributeGroup atg = dataModel.getAttributeGroup("atg.dynamischeMenge");
//       
//       for (String mengenName: typInfo.getDynMengen())
//       {
//      	 ObjectSet menge = ((ConfigurationObject)so).getObjectSet(mengenName);
//      	 if (menge != null)
//      	 {
//      		 Data data = soDaten.getDynMengenDaten(mengenName);
//      		 if (data != null)
//	         {
//	           MultiStatus atgStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
//	               "Prfe Daten der ATG " + atg.getPid(), null);
//	           AtgInfo info = new AtgInfo(atg.getPid(), false, new AttributInfo[]
//	             { new AttributInfo("verwaltung", true, true, true, true, true, true) });
//	           atgStatus.merge(pruefInstanz.pruefeDaten(ho, menge, info, data));
//	         
//	           objektStatus.add(atgStatus);
//	         }
//      	 }
//       }
       
       
       if (result != objektStatus)
       {
         result.add(objektStatus);
       }
     }
    
    return result;
  }
  

  @Override
  public boolean pruefeNeuanlageVerboten(Object objekt, IHierarchieObjektTyp hot)
  {
    // TODO Auto-generated method stub
    return false;
  }

  @Override
  public Collection<SystemObjectType> loeschenOffeneReferenzenVerboten(IHierarchieObjektTyp hot)
  {
    // TODO Auto-generated method stub
    return null;
  }

  @Override
  public boolean verwendeZuLoeschendeObjekteZurPruefungOffeneReferenzen()
  {
    // TODO Auto-generated method stub
    return false;
  }
  
  private void behandleLoeschenReferenz(HierarchieObjekt ho, MultiStatus status)
  {
  	IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
  	Assert.isTrue(HOT_Netz.VerkehrsModellNetzReferenz.equals(hot) || HOT_Netz.VerkehrsModellNetzReferenz.equals(hot) ||
  			HOT_Netz.ASS.equals(hot));
  	Object vater = getHierarchie().getUebergeordnetesObjekt(ho);
  	Assert.isTrue(vater instanceof HierarchieObjekt, "Lschen Referenz nur unterhalb der zweiten Ebene mglich");
  	HierarchieObjekt hoVater = (HierarchieObjekt)vater;
  	ConfigurationObject ko = (ConfigurationObject)hoVater.getSystemObjekt(0);
  	ObjectSet menge = ko.getObjectSet("NetzBestandTeile");
  	SystemObject so = ho.getSystemObjekt(0);
  	if (menge != null)
  	{
  		try
    	{
  			menge.remove(so);
  			
  			status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " +	so.getPid() + 
  					" vom Typ " + so.getType().getPid() + " aus der Menge " + menge.getName() + 
  					" des Objekts " + ko.getPid() + " vom Typ " + ko.getType().getPid() + " entfernt"));
  			
    	}
    	catch (ConfigurationChangeException e)
    	{
    		status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid() + 
    				" vom Typ " + so.getType().getPid() + " konnte nicht aus der Menge " + menge.getName() +
    				" des Objekts " +	ko.getPid() + " vom Typ " + ko.getType().getPid() + " entfernt werden", e));
    	}
  	}
  	if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
  	{
  		// entferneHierarchieObjekt(hoVater, ho);
  		objektEntfernen(hoVater, ho);
  		  		
  		status.add(new Status(Status.OK, Activator.PLUGIN_ID, "HierarchieObjekt " +	ho + 
					" aus der Hierarchie entfertn"));
  	}
  }

	private void objektEntfernen(Object vater, HierarchieObjekt ho)
	{
		new ObjektEntfernen(vater, ho).alleObjekteAktualisieren();
	}
			
	private void objektHinzufuegen(Object vater, HierarchieObjekt ho, boolean rekursiv)
	{
		new ObjektHinzufuegen(vater, ho, rekursiv).alleObjekteAktualisieren();
	}
	
	/**
	 * Rekursives Aktualisieren aller Objekte der Hierarchie
	 * 
	 * @author Sans
	 *
	 */
	public abstract class HierarchieAktualisierer
	{
		/**
		 * Die Methode aktualisiereHO() wird fr alle HierarchieObjekte der Hierarchie rekursiv 
		 * aufgerufen
		 */
		public final void alleObjekteAktualisieren()
		{
			List<ConfigurationArea> liste = new Vector<ConfigurationArea>(getHierarchie().getDataModel().getAllConfigurationAreas().values());
			for (ConfigurationArea kb: liste)
			{
				kinderAktualisieren(kb);
			}
		}
		
		/**
		 * Die Methode aktualisiere() wird das bergebene Objekt aufgerufen,
		 * danach erfolgen rekursive Aufrufe mit den ggf. vorhandenen Kindern
		 * des bergebenen Objekts
		 *  
		 * @param objekt das zu aktualisierende Objekt
		 */
		private void kinderAktualisieren(Object objekt)
		{
//		System.out.println("AKTUALISIEREN: " + objekt);
			aktualisiere(objekt);
			
			List<HierarchieObjekt> liste = new Vector<HierarchieObjekt>(getHierarchie().getUntergeordneteObjekte(objekt));
			for (HierarchieObjekt kind: liste)
			{
				kinderAktualisieren(kind);
			}
				
		}
		
		/**
		 * Diese Methode wird fr jedes Objekt in der der zugrundeliegenden Hierarchie aufgerufen
		 * 
		 * @param ho das zu aktualisierende Hierarchieobjekt
		 * @return true, wenn die Rekusion fr diese Unterhierarchie weitergefhrt werden soll, bei
		 * false wird die Rekursion fr die Unterhierarchie abgebrochen
		 */
		protected abstract void aktualisiere(Object objekt);
	}
	
	public class ObjektHinzufuegen extends HierarchieAktualisierer 
	{
		private Object _vater;
		private HierarchieObjekt _ho;
		private boolean _rekursiv;
		
		public ObjektHinzufuegen(Object vater, HierarchieObjekt ho,	boolean rekursiv)
		{
			super();
			_vater = vater;
			_ho = ho;
			_rekursiv = rekursiv;
		}
		
		private HierarchieObjekt findeHOMitSO(Object vater, SystemObject so)
		{
			HierarchieObjekt result = null;
			
			for (HierarchieObjekt kind: getHierarchie().getUntergeordneteObjekte(vater))
			{
				if (kind.getSystemObjekt(0) == so)
				{
					result = kind;
					break;
				}
			}
			
			return result;
		}
		
		protected void aktualisiere(Object objekt)
		{
			if (objekt == _vater)
			{
				getHierarchie().objektHinzufuegen(_vater, _ho);
				if (_rekursiv)
				{
					bestimmeUntergeordneteHOs(_ho);
				}
			}
			else if (_vater instanceof HierarchieObjekt)
			{
				if (objekt instanceof HierarchieObjekt)
				{
					HierarchieObjekt vater = (HierarchieObjekt)_vater;
					HierarchieObjekt ho = (HierarchieObjekt)objekt;
					
					if (vater.getSystemObjekt(0) == ho.getSystemObjekt(0))
					{
						SystemObject so = _ho.getSystemObjekt(0);
						
						if (findeHOMitSO(objekt, so ) == null)
						{	
							HierarchieObjekt neu = new HierarchieObjekt(_ho.getHierarchieObjektTyp(), 
									new SystemObject[]{ so }, EBENE2);
							
							getHierarchie().objektHinzufuegen(objekt, neu);
							if (_rekursiv)
							{
								bestimmeUntergeordneteHOs(neu);
							}
						}	
					}
				}
			}
		}
	}

	public class ObjektEntfernen extends HierarchieAktualisierer 
	{
		private Object _vater;
		private HierarchieObjekt _ho;
				
		public ObjektEntfernen(Object vater, HierarchieObjekt ho)
		{
			super();
			_vater = vater;
			_ho = ho;
		}
		
		protected void aktualisiere(Object objekt)
  	{
			IHierarchie hierarchie = getHierarchie();
			if (_vater instanceof HierarchieObjekt)
			{
				// Entfernen Referenz
				if (objekt instanceof HierarchieObjekt)
				{
					HierarchieObjekt ho = (HierarchieObjekt)objekt;
					Object vater = hierarchie.getUebergeordnetesObjekt(ho);
					if (vater instanceof HierarchieObjekt)
					{
						if (((HierarchieObjekt)_vater).getSystemObjekt(0) == ((HierarchieObjekt)vater).getSystemObjekt(0))
						{
							if (ho == _ho)
							{
								objektEntfernenRekursiv(_vater, _ho);
							}
							else
							{
								if (ho.getSystemObjekt(0) == _ho.getSystemObjekt(0))
								{
									objektEntfernenRekursiv(vater, ho);
								}
							}
						}
					}
				}	
			}
			else
			{
				// Entfernen Objekt
				if (objekt == _ho)
				{
					objektEntfernenRekursiv(_vater, _ho);
				}
				else
				{
					if (objekt instanceof HierarchieObjekt)
					{
						HierarchieObjekt ho = (HierarchieObjekt)objekt;
						if (ho.getSystemObjekt(0) == _ho.getSystemObjekt(0))
						{
							Object vater = hierarchie.getUebergeordnetesObjekt(ho);
							objektEntfernenRekursiv(vater, ho);
						}
					}
				}
			}
  	}
		
//		protected void aktualisiere(Object objekt)
//		{
//			if (_ho.getHierarchieObjektTyp().isSystemObjektVorhanden())
//			{
//				
//				if (_vater instanceof HierarchieObjekt)
//				{
//  				// Entfernen Referenz
//					if (objekt instanceof HierarchieObjekt)
//					{
//						HierarchieObjekt vater = (HierarchieObjekt)_vater;
//						HierarchieObjekt ho = (HierarchieObjekt)objekt;
//						
//						if (vater.getSystemObjekt(0) == ho.getSystemObjekt(0))
//						{
//							if (objekt == _vater)
//							{
//								objektEntfernenRekursiv(_vater, _ho, false);
//							}
//							else
//							{	
//								HierarchieObjekt kind = findeHOMitSO(objekt, _ho.getSystemObjekt(0));
//								if (kind != null)
//								{	
//									objektEntfernenRekursiv(objekt, kind, true);
//								}
//							}	
//						}
//					}
//				}
//				else
//				{
//					// Entfernen Objekt
//					if (objekt == _vater)
//					{
//						objektEntfernenRekursiv(_vater, _ho, false);
//					}
//					else
//					{
//						HierarchieObjekt kind = findeHOMitSO(objekt, _ho.getSystemObjekt(0));
//						if (kind != null)
//						{	
//							objektEntfernenRekursiv(objekt, kind, true);
//						}
//					}
//				}
//			}
//		}
//		
//		private HierarchieObjekt findeHOMitSO(Object vater, SystemObject so)
//		{
//			HierarchieObjekt result = null;
//			
//			for (HierarchieObjekt kind: _hierarchie.getUntergeordneteObjekte(vater))
//			{
//				if (kind.getSystemObjekt(0) == so)
//				{
//					result = kind;
//					break;
//				}
//			}
//			
//			return result;
//		}
		
		private void objektEntfernenRekursiv(Object vater, HierarchieObjekt ho)
		{
			IHierarchie hierarchie = getHierarchie();
			List<HierarchieObjekt> kinder = hierarchie.getUntergeordneteObjekte(ho);
			
			while(!kinder.isEmpty())
			{
				objektEntfernenRekursiv(ho, kinder.get(0));
			}
			
			hierarchie.objektEntfernen(vater, ho);
		}
	}
	
	public class ObjektErsetzen extends HierarchieAktualisierer 
	{
		private SystemObject _altesObjekt;
		
		private SystemObject _neuesObjekt;
				
		public ObjektErsetzen(SystemObject altesObjekt, SystemObject neuesObjekt)
		{
			super();
			_altesObjekt = altesObjekt;
			_neuesObjekt = neuesObjekt;
		}
		
		protected void aktualisiere(Object objekt)
		{
			if (objekt instanceof HierarchieObjekt)
			{
				HierarchieObjekt ho = (HierarchieObjekt)objekt;
				
				if (ho.getSystemObjekt(0) == _altesObjekt)
				{
					ho.setSystemObjekt(0, _neuesObjekt);
					getHierarchie().notifyHOGeaendert(ho);
				}
			}
		}
		
	}	
	
	/**
   * Zeigt an, ob das bergebene Hierarchieobjekt beim rekursiven Kopieren als untergeordnetes 
   * Objekt mitkopiert werden soll oder nicht
   *  
   * @param ho das aktuell zu kopierende HO
   * @return <code>true</code>, wenn das aktuell zu kopierende Objekt nicht kopiert werden soll, 
   * <code>false</code> (default) sonst
   */
  public boolean kopierenNichtErlaubt(HierarchieObjekt ho)
  {
  	return false;
  }
		
}
