/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkatls;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.tkabasis.Activator;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HODaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IAttributInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.SystemObjektDaten;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.IPruefungen;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.PruefungenTools;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.ObjectTimeSpecification;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigSystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Prfungen von Attributwerten fr den TKA TLS
 * 
 * @author Sans  
 *
 */
public class PruefungenTLS implements IPruefungen
{
	/**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();

  /**
   * Die Hierarchie, die fr die Prfungen benutzt werden kann
   */
	private IHierarchie _hierarchie;
	
	 /**
   * Bestimmt die Hierarchie, die fr die Prfungen benutzt werden kann
   * 
   * @return die Hierarchie, die fr die Prfungen benutzt werden kann
   */
	public IHierarchie getHierarchie()
	{
		return _hierarchie;
	}
  
	/**
   * Setzt die Hierarchie, die fr die Prfungen benutzt werden kann
   * @param hierarchie die Hierarchie, die fr die Prfungen benutzt werden kann
   */
	public void setHierarchie(IHierarchie hierarchie)
	{
		_hierarchie = hierarchie;
	}
	
	public IStatus pruefeHierarchieObjekt(HierarchieObjekt ho, HODaten daten)
	{
		MultiStatus result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
				"Prfe Hierarchieobjekt " + ho, null);
	  		
		for (SystemObject so: ho.getSystemObjekte())
  	{
			 SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
			 if (soDaten != null)
			 {
				 MultiStatus objektStatus;
				 if (ho.getSystemObjekte().size() > 1)
				 {
					 objektStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Systemobjekt " + 
							 so.getPid() + " vom Typ " + so.getType().getPid(), null);
				 }
				 else
				 {
					 objektStatus = result;
				 }
				 
				 //objektStatus.add(pruefePid(so, soDaten.getPid()));
				 objektStatus.add(new PruefungenTools(getHierarchie().getDataModel()).pruefePid(soDaten.getPid(), so));
					
				 if (objektStatus != result)
				 {
					 result.add(objektStatus);
				 }
			 }	
  	}
		
		return result;
	}
	
	public IStatus pruefeDaten(HierarchieObjekt ho, SystemObject so, AtgInfo atgInfo, Data daten)
	{
		MultiStatus result = new MultiStatus(Activator.PLUGIN_ID,	Status.OK, 
  			"Prfe Daten der ATG " + atgInfo.getAtg(), null);
		 
		 IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
		 
		 if (HOT_TLS.VIZ.equals(hot) || HOT_TLS.VRZ.equals(hot) ||
				 HOT_TLS.UZ.equals(hot) || HOT_TLS.KRI.equals(hot))
		 {	 
			 // VIZ, VRZ, UZ, KRI
			 if (so.isOfType("typ.viz") || so.isOfType("typ.vrz") ||
					 so.isOfType("typ.uz") || so.isOfType("typ.kri"))
			 {
				 if ("atg.gert".equals(atgInfo.getAtg()))
				 {
					 pruefeDatenAtgGeraet(so, atgInfo, daten, result);
				 }
				 else if ("atg.gertZustndigkeit".equals(atgInfo.getAtg()) ||
					 "atg.punktKoordinaten".equals(atgInfo.getAtg()))
				 {
				   result.merge(PruefungenTools.pruefeDatenDefinition(atgInfo, daten));
				 }
			 }
			 else if (so.isOfType("typ.anschlussPunkt"))
			 {
				 if ("atg.anschlussPunkt".equals(atgInfo.getAtg()))
				 {
					 pruefeDatenAtgAP(ho, so, atgInfo, daten, result);
				 }
			 }
			 else if (so.isOfType("typ.anschlussPunktKommunikationsPartner"))
			 {
				 if ("atg.anschlussPunktKommunikationsPartner".equals(atgInfo.getAtg()))
				 {
					 pruefeDatenAtgAPKP(ho, so, atgInfo, daten, result);
				 }
			 }
		 }
		 else if (HOT_TLS.Inselbus.equals(hot))
		 {
			 // Inselbus
		 	 if (so.isOfType("typ.anschlussPunkt"))
			 {
				 if ("atg.anschlussPunkt".equals(atgInfo.getAtg()))
				 {
					 pruefeDatenAtgAP(ho, so, atgInfo, daten, result);
				 }
			 }
		 }
		 else if (HOT_TLS.SM.equals(hot))
		 {	 
			 // SM
			 if (so.isOfType("typ.steuerModul"))
			 {
				 if ("atg.gert".equals(atgInfo.getAtg()))
				 {
					 pruefeDatenAtgGeraet(so, atgInfo, daten, result);
				 }
				 else if ("atg.gertZustndigkeit".equals(atgInfo.getAtg()) ||
					 "atg.punktKoordinaten".equals(atgInfo.getAtg()))
				 {
					 result.merge(PruefungenTools.pruefeDatenDefinition(atgInfo, daten));
				 }
			 }
			 else if (so.isOfType("typ.anschlussPunktKommunikationsPartner"))
			 {
				 if ("atg.anschlussPunktKommunikationsPartner".equals(atgInfo.getAtg()))
				 {
					 pruefeDatenAtgAPKP(ho, so, atgInfo, daten, result);
				 }
			 }
		 }
		 else if (HOT_TLS.EAK.equals(hot))
		 {
			 // EAK
			 if (so.isOfType("typ.eak"))
			 {
				 if ("atg.eak".equals(atgInfo.getAtg()))
				 {
					 pruefeDatenAtgEak(ho, so, atgInfo, daten, result);
				 }
				 else if ("atg.punktKoordinaten".equals(atgInfo.getAtg()))
				 {
					 result.merge(PruefungenTools.pruefeDatenDefinition(atgInfo, daten));
				 }
			 }
		 }
		 else if (hot instanceof DeUnterTyp)
		 {
			 // DE
			 if (so.isOfType("typ.de"))
			 {
				 if ("atg.de".equals(atgInfo.getAtg()))
				 {
					 pruefeDatenAtgDe(ho, so, atgInfo, daten, result);
				 }
				 else if ("atg.punktKoordinaten".equals(atgInfo.getAtg()))
				 {
					 result.merge(PruefungenTools.pruefeDatenDefinition(atgInfo, daten));
				 }
			 }
		 }
		
		return result;
	}

	/**
	 * Prfen der Daten fr die ATG atg.gert
	 * 
	 * @param so	so das Systemobjekt, zu dem ATG-Daten bernommen werden sollen
   * @param atgInfo Info ber die ATG der zu bernehmenden Daten
   * @param daten Datensatz der Daten, die geprft werden sollen
	 * @param status Multimeldung, die mit Meldungen ergnzt wird
	 */
  private void pruefeDatenAtgGeraet(SystemObject so, AtgInfo atgInfo, Data daten, MultiStatus status)
  {
  	for (String attribut: atgInfo.getAttribute())
  	{
  		MultiStatus	meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert fr Attribut " +
    			attribut, null);
  		IAttributInfo attInfo = atgInfo.getAttributInfo(attribut);
  		meldung.merge(PruefungenTools.pruefeDatenDefinition(attInfo, daten));
  		
  		if (!meldung.matches(IStatus.ERROR | IStatus.CANCEL))
			{
  			if ("KnotenNummer".equals(attribut))
    		{
    			int knotenNummer = daten.getItem(attribut).asUnscaledValue().intValue(); 
    			
    			ConfigDataModel dataModel = getHierarchie().getDataModel();
    			SystemObjectType typ = dataModel.getType("typ.gert");
    			Collection<SystemObjectType> typSpez = Arrays.asList(new SystemObjectType[] { typ });
    			Collection<Integer> vorhandeneNummern = new HashSet<Integer>();
    			ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
    			
    			for (ConfigurationArea kb: dataModel.getAllConfigurationAreas().values())
    			{
    				for (SystemObject lauf: kb.getObjects(typSpez, ObjectTimeSpecification.valid()))
    				{
    					if (!(((ConfigurationObject)lauf).getNotValidSince() >= kb.getModifiableVersion()))
   			    	// Objekt wird in zuknftiger Version nicht ungltig	
   			    	{		
	    					if (lauf != so)
	    					{	
	    						fuegeKnotenNummerHinzu(lauf, lookup, vorhandeneNummern);
	    					}
   			    	}	
    				}
    				for (SystemObject lauf: kb.getNewObjects())
    				{
    					if (lauf.isOfType(typ))
    					{	
	    					if (lauf != so)
	    					{
	    						fuegeKnotenNummerHinzu(lauf, lookup, vorhandeneNummern);
	    					}	
    					}	
    				}
    			}
    			
    			if (vorhandeneNummern.contains(knotenNummer))
    			{
    				meldung.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
    				"Die Knotennummer " + knotenNummer + " wird bereits verwendet"));
    			}
    			else
    			{
    				meldung.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
    				"Prfen Eindeutigkeit Knotennummer: OK"));
    			}
    		}
			}
  		
  		status.add(meldung);
  	}
  }
  
  private void  fuegeKnotenNummerHinzu(SystemObject geraet, ObjectLookup lookup, 
  		Collection<Integer> knotenNummern)
  {
  	AttributeGroup atg = getHierarchie().getDataModel().getAttributeGroup("atg.gert");
  	Data data = ((ConfigSystemObject)geraet).getConfigurationData(atg, lookup);
  	if (data != null)
  	{
  		int knotenNummer = data.getItem("KnotenNummer").asUnscaledValue().intValue();
  		knotenNummern.add(knotenNummer);
  	}
  }
  
  /**
	 * Prfen der Daten fr die ATG atg.anschlussPunkt
	 * 
	 * @param ho das Hierarchieobhekt, zu dem das bergebene Systemobjekt gehrt
	 * @param so	so das Systemobjekt, zu dem ATG-Daten bernommen werden sollen
   * @param atgInfo Info ber die ATG der zu bernehmenden Daten
   * @param daten Datensatz der Daten, die geprft werden sollen
	 * @param status Multimeldung, die mit Meldungen ergnzt wird
	 */
  private void pruefeDatenAtgAP(HierarchieObjekt ho, SystemObject so, AtgInfo atgInfo, Data daten, MultiStatus status)
  {
  	for (String attribut: atgInfo.getAttribute())
  	{
  		MultiStatus	meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert fr Attribut " +
    			attribut, null);
  		IAttributInfo attInfo = atgInfo.getAttributInfo(attribut);
  		meldung.merge(PruefungenTools.pruefeDatenDefinition(attInfo, daten));
  		
  		if (!meldung.matches(IStatus.ERROR | IStatus.CANCEL))
			{
  			if ("PortNummer".equals(attribut))
    		{
    			long portNummer = daten.getItem(attribut).asUnscaledValue().longValue(); 
    			
    			Collection<Long> vorhandeneNummern = new HashSet<Long>();
    			ConfigDataModel dataModel = getHierarchie().getDataModel();
    			AttributeGroup atg = dataModel.getAttributeGroup("atg.anschlussPunkt");
    			ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
    			HierarchieObjekt hoVater = (HierarchieObjekt)getHierarchie().getUebergeordnetesObjekt(ho);
    			
    			for (HierarchieObjekt hoKind: _hierarchie.getUntergeordneteObjekte(hoVater))
    	  	{
    	  		if (hoKind != ho)
    	  		{
    	  			SystemObject ap = null;
    	  			IHierarchieObjektTyp hot = hoKind.getHierarchieObjektTyp();
    	  			if (hot.equals(HOT_TLS.VRZ) || hot.equals(HOT_TLS.UZ) || hot.equals(HOT_TLS.KRI))
    	  			{
    	  				ap = hoKind.getSystemObjekt(1);
    	  			}
    	  			else if (hot.equals(HOT_TLS.Inselbus))
    	  			{
    	  				ap = hoKind.getSystemObjekt(0);
    	  			}
    	  				
    	  			if (ap != null)
    	  			{	
    		  			Data data = ((ConfigSystemObject)ap).getConfigurationData(atg, lookup);
    		  			if (data != null)
    		  			{
    							long nummer = data.getItem("PortNummer").asUnscaledValue().longValue();
    							vorhandeneNummern.add(nummer);
    						}
    	  			}	
    	  		}	
    	  	}
    			
    			if (vorhandeneNummern.contains(portNummer))
    			{
    				meldung.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
    				"Die Portnummer " + portNummer + " wird bereits verwendet"));
    			}
    			else
    			{
    				meldung.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
    				"Prfen Eindeutigkeit Portnummer fr AnschlussPunkt: OK"));
    			}
    		}
			}
  		
  		status.add(meldung);
  	}
  }
  
  /**
	 * Prfen der Daten fr die ATG atg.anschlussPunktKommunikationsPartner
	 * 
	 * @param ho das Hierarchieobhekt, zu dem das bergebene Systemobjekt gehrt
	 * @param so	so das Systemobjekt, zu dem ATG-Daten bernommen werden sollen
   * @param atgInfo Info ber die ATG der zu bernehmenden Daten
   * @param daten Datensatz der Daten, die geprft werden sollen
	 * @param status Multimeldung, die mit Meldungen ergnzt wird
	 */
  private void pruefeDatenAtgAPKP(HierarchieObjekt ho, SystemObject so, AtgInfo atgInfo, Data daten, MultiStatus status)
  {
  	for (String attribut: atgInfo.getAttribute())
  	{
  		MultiStatus	meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert fr Attribut " +
    			attribut, null);
  		IAttributInfo attInfo = atgInfo.getAttributInfo(attribut);
  		meldung.merge(PruefungenTools.pruefeDatenDefinition(attInfo, daten));
  		
  		if (!meldung.matches(IStatus.ERROR | IStatus.CANCEL))
			{
  			if ("PortNummer".equals(attribut))
    		{
  				long portNummer = daten.getItem(attribut).asUnscaledValue().longValue();
  				Collection<Long> vorhandeneNummern = new HashSet<Long>();
    			IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
    			SystemObject ap = null;

    			if (_hierarchie.getUebergeordnetesObjekt(ho) instanceof HierarchieObjekt) // Nicht fr HO unmittelbar unter KB
    			{	
		  			if (hot.equals(HOT_TLS.VRZ) || hot.equals(HOT_TLS.UZ) || hot.equals(HOT_TLS.KRI))
		  			{
	  					ap = ho.getSystemObjekt(1);
		  			}	
	  				else if (HOT_TLS.SM.equals(hot))
	  				{
	  					ap = ((HierarchieObjekt)_hierarchie.getUebergeordnetesObjekt(ho)).getSystemObjekt(0);
	  				}
    			}
    			
    			if (ap != null)
    			{
  					NonMutableSet apkpMenge = ((ConfigurationObject)ap).getNonMutableSet("AnschlussPunkteKommunikationsPartner");
  					if (apkpMenge != null)
  					{
  						for (SystemObject apkp: apkpMenge.getElementsInModifiableVersion())
  						{
  							if (apkp == null)
	  	          {
	  	          	_debug.warning("null-Referenz in Menge " + apkpMenge.getName() + " von " + ap.getPid());
	  	          	continue;
	  	          }
  							if (apkp != so)
  							{
  								ConfigDataModel dataModel = getHierarchie().getDataModel();
  		      			AttributeGroup atg = dataModel.getAttributeGroup("atg.anschlussPunktKommunikationsPartner");
  		      			ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);

		              Data data = ((ConfigSystemObject)apkp).getConfigurationData( atg, lookup );
		              if (data != null)
		              {	
		              	long nummer = data.getItem("PortNummer").asUnscaledValue().longValue();
	    							vorhandeneNummern.add(nummer);
		              }  
  							}  
  						}
  					}
  				}
    			
    			if (vorhandeneNummern.contains(portNummer))
    			{
    				meldung.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
    				"Die Portnummer " + portNummer + " wird bereits verwendet"));
    			}
    			else
    			{
    				meldung.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
    				"Prfen Eindeutigkeit Portnummer fr AnschlussPunktKomnunikationsPartner: OK"));
    			}
    		}
			}
  		
  		status.add(meldung);
  	}
  }
   
  /**
	 * Prfen der Daten fr die ATG atg.eak
	 * 
	 * @param ho das Hierarchieobhekt, zu dem das bergebene Systemobjekt gehrt
	 * @param so	so das Systemobjekt, zu dem ATG-Daten bernommen werden sollen
   * @param atgInfo Info ber die ATG der zu bernehmenden Daten
   * @param daten Datensatz der Daten, die geprft werden sollen
	 * @param status Multimeldung, die mit Meldungen ergnzt wird
	 */
  private void pruefeDatenAtgEak(HierarchieObjekt ho, SystemObject so, AtgInfo atgInfo, Data daten, MultiStatus status)
  {
  	for (String attribut: atgInfo.getAttribute())
  	{
  		MultiStatus	meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert fr Attribut " +
    			attribut, null);
  		IAttributInfo attInfo = atgInfo.getAttributInfo(attribut);
  		meldung.merge(PruefungenTools.pruefeDatenDefinition(attInfo, daten));
  		
  		if (!meldung.matches(IStatus.ERROR | IStatus.CANCEL))
			{
  			if ("OSI2Adresse".equals(attribut))
    		{
    			int osi2Adresse = daten.getItem(attribut).asUnscaledValue().intValue(); 
    			
    			Collection<Integer> vorhandeneAdressen = new HashSet<Integer>();
    			ConfigDataModel dataModel = getHierarchie().getDataModel();
    			AttributeGroup atg = dataModel.getAttributeGroup("atg.eak");
    			ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
    			HierarchieObjekt hoVater = (HierarchieObjekt)getHierarchie().getUebergeordnetesObjekt(ho);
    			
    			for (HierarchieObjekt hoEAK: _hierarchie.getUntergeordneteObjekte(hoVater))
    	  	{
    	  		if (hoEAK != ho)
    	  		{
    	  			SystemObject eak = hoEAK.getSystemObjekt(0);
    	  			Data data = ((ConfigSystemObject)eak).getConfigurationData(atg, lookup);
    	  			if (data != null)
    	  			{
    						int adresse = data.getItem("OSI2Adresse").asUnscaledValue().intValue();
    						vorhandeneAdressen.add(adresse);
    					}	
    	  		}	
    	  	}
    			
    			if (vorhandeneAdressen.contains(osi2Adresse))
    			{
    				meldung.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
    				"Die OSI2Adresse " + osi2Adresse + " wird fr das SM " + hoVater + " bereits verwendet"));
    			}
    			else
    			{
    				meldung.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
    				"Prfen Eindeutigkeit OSI2Adresse fr EAK: OK"));
    			}
    		}
			}
  		
  		status.add(meldung);
  	}
  }
  
  /**
	 * Prfen der Daten fr die ATG atg.de
	 * 
	 * @param ho das Hierarchieobhekt, zu dem das bergebene Systemobjekt gehrt
	 * @param so	so das Systemobjekt, zu dem ATG-Daten bernommen werden sollen
   * @param atgInfo Info ber die ATG der zu bernehmenden Daten
   * @param daten Datensatz der Daten, die geprft werden sollen
	 * @param status Multimeldung, die mit Meldungen ergnzt wird
	 */
  private void pruefeDatenAtgDe(HierarchieObjekt ho, SystemObject so, AtgInfo atgInfo, Data daten, MultiStatus status)
  {
  	for (String attribut: atgInfo.getAttribute())
  	{
  		MultiStatus	meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert fr Attribut " +
    			attribut, null);
  		IAttributInfo attInfo = atgInfo.getAttributInfo(attribut);
  		meldung.merge(PruefungenTools.pruefeDatenDefinition(attInfo, daten));
  		
  		if (!meldung.matches(IStatus.ERROR | IStatus.CANCEL))
			{
  			if ("DEKanal".equals(attribut))
    		{
  				Assert.isTrue(ho.getHierarchieObjektTyp() instanceof DeUnterTyp);
  		  	int fg = ((DeUnterTyp)ho.getHierarchieObjektTyp()).getFg();
  				
  		  	int pruefKanal = daten.getItem(attribut).asUnscaledValue().intValue();
  		  	
    			Collection<Integer> vorhandeneKanaele = new HashSet<Integer>();
    			ConfigDataModel dataModel = getHierarchie().getDataModel();
    			AttributeGroup atg = dataModel.getAttributeGroup("atg.de");
    			ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
    			HierarchieObjekt hoVater = (HierarchieObjekt)getHierarchie().getUebergeordnetesObjekt(ho);
    			HierarchieObjekt hoSM = (HierarchieObjekt)getHierarchie().getUebergeordnetesObjekt(hoVater);
    			
    			for (HierarchieObjekt hoEAK: _hierarchie.getUntergeordneteObjekte(hoSM))
    			{	
	    			for (HierarchieObjekt hoDE: _hierarchie.getUntergeordneteObjekte(hoEAK))
	    	  	{
	    	  		if (hoDE != ho)
	    	  		{
	      				if (hoDE.getHierarchieObjektTyp() instanceof DeUnterTyp)
	      				{	
		      		  	if (fg == ((DeUnterTyp)hoDE.getHierarchieObjektTyp()).getFg())
		      		  	{	
			    	  			SystemObject de = hoDE.getSystemObjekt(0);
			    	  			Data data = ((ConfigSystemObject)de).getConfigurationData(atg, lookup);
			    	  			if (data != null)
			    	  			{
			    						int kanal = data.getItem(attribut).asUnscaledValue().intValue();
			    						vorhandeneKanaele.add(kanal);
			    					}
		      		  	}	
	      				}	
	    	  		}	
	    	  	}
    			}
    			
    			if (vorhandeneKanaele.contains(pruefKanal))
    			{
    				meldung.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
    						"Der " + attribut + " " + pruefKanal + " wird fr das SM " + hoSM + " und die FG" + 
    						fg + " bereits verwendet"));
    			}
    			else
    			{
    				meldung.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
    				"Prfen Eindeutigkeit " + attribut + " fr SM und FG: OK"));
    			}
    		}
  			if ("EAKanal".equals(attribut))
    		{
  				Assert.isTrue(ho.getHierarchieObjektTyp() instanceof DeUnterTyp);
  		  	int fg = ((DeUnterTyp)ho.getHierarchieObjektTyp()).getFg();
  				
  		  	int pruefKanal = daten.getItem(attribut).asUnscaledValue().intValue();
  		  	
    			Collection<Integer> vorhandeneKanaele = new HashSet<Integer>();
    			ConfigDataModel dataModel = getHierarchie().getDataModel();
    			AttributeGroup atg = dataModel.getAttributeGroup("atg.de");
    			ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
    			HierarchieObjekt hoEAK = (HierarchieObjekt)getHierarchie().getUebergeordnetesObjekt(ho);
    		
    			for (HierarchieObjekt hoDE: _hierarchie.getUntergeordneteObjekte(hoEAK))
    	  	{
    	  		if (hoDE != ho)
    	  		{
      				if (hoDE.getHierarchieObjektTyp() instanceof DeUnterTyp)
      				{	
	      		  	if (fg == ((DeUnterTyp)hoDE.getHierarchieObjektTyp()).getFg())
	      		  	{	
		    	  			SystemObject de = hoDE.getSystemObjekt(0);
		    	  			Data data = ((ConfigSystemObject)de).getConfigurationData(atg, lookup);
		    	  			if (data != null)
		    	  			{
		    						int kanal = data.getItem(attribut).asUnscaledValue().intValue();
		    						vorhandeneKanaele.add(kanal);
		    					}
	      		  	}	
      				}	
    	  		}	
    	  	}
    			    			
    			if (vorhandeneKanaele.contains(pruefKanal))
    			{
    				meldung.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
    						"Der " + attribut + " " + pruefKanal + " wird fr das EAK " + hoEAK + " und die FG" + 
    						fg + " bereits verwendet"));
    			}
    			else
    			{
    				meldung.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
    				"Prfen Eindeutigkeit " + attribut + " fr EAK und FG: OK"));
    			}
    		}
			}
  		
  		status.add(meldung);
  	}
  }
  
}
  