/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkatlsae;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashMap;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.tkabasis.Activator;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HODaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IAttributInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.SystemObjektDaten;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.IPruefungen;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.PruefungenTools;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigSystemObject;

/**
 * Prfungen von Attributwerten fr den TKA TLS hnliche Kommunikationspartner
 * 
 * @author Sans
 *
 */
public class PruefungenTLSAE implements IPruefungen
{

	private IHierarchie _hierarchie;
	
	 /**
   * Bestimmt die Hierarchie, die fr die Prfungen benutzt werden kann
   * 
   * @return die Hierarchie, die fr die Prfungen benutzt werden kann
   */
	public IHierarchie getHierarchie()
	{
		return _hierarchie;
	}
  
	/**
   * Setzt die Hierarchie, die fr die Prfungen benutzt werden kann
   * @param hierachie die Hierarchie, die fr die Prfungen benutzt werden kann
   */
	public void setHierarchie(IHierarchie hierarchie)
	{
		_hierarchie = hierarchie;
	}
	
	public IStatus pruefeHierarchieObjekt(HierarchieObjekt ho, HODaten daten)
	{
		MultiStatus result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
				"Prfe Hierarchieobjekt " + ho, null);
//	  IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
		
		for (SystemObject so: ho.getSystemObjekte())
  	{
			 SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
			 if (soDaten != null)
			 {
				 MultiStatus objektStatus;
				 if (ho.getSystemObjekte().size() > 1)
				 {
					 objektStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Systemobjekt " + 
							 so.getPid() + " vom Typ " + so.getType().getPid(), null);
				 }
				 else
				 {
					 objektStatus = result;
				 }
				 
				 //objektStatus.add(pruefePid(so, soDaten.getPid()));
				 objektStatus.add(new PruefungenTools(getHierarchie().getDataModel()).pruefePid(soDaten.getPid(), so));
			 }	
  	}
		
		return result;
	}

  /**
   * Prft einen Attributwert auf Zulssigkeit
   * 
   * @param ho Hierarchieobjekt, dessen Daten geprft werden sollen
   * @param so Systemobjekt, gehrt zu ho
   * @param atgInfo Info ber das zu prfenden Attribut
   * @param daten Datensatz der Daten, die geprft werden sollen
   * @return eine Instanz vom Typ IStatus, die aussagt, ob es zulssig ist, die 
   * bergebenen Daten als Attributwert zu bernehmen, oder <code>null</code>, wenn
   * keine Prfung fr die bergebenen Parameter implementiert ist
   */
	public IStatus pruefeDaten(HierarchieObjekt ho, SystemObject so, AtgInfo atgInfo, Data daten)
  { 
    MultiStatus result = new MultiStatus( Activator.PLUGIN_ID, Status.OK, "Prfe Daten der ATG " + atgInfo.getAtg(),
        null );

    if ( so.isOfType( "typ.viz" ) || so.isOfType( "typ.vrz" ) || so.isOfType( "typ.uz" ) )
    {
      if ( "atg.gertTlshnlicheKommunikationsPartner".equals( atgInfo.getAtg() ) )
  		{
      	pruefeDatenAtgTlsAehnlicheKommunikationsPartner(atgInfo, daten, result);
  		}
      else if ( "atg.punktKoordinaten".equals( atgInfo.getAtg() ) )
   		{
      	result.merge(PruefungenTools.pruefeDatenDefinition(atgInfo, daten));
   		}
      else if ( "atg.gertTlshnlicheKommunikationsPartnerUmsetzungsModule".equals( atgInfo.getAtg() ) )
      {
      	pruefeDatenAtgTlsAehnlicheKommunikationsPartnerUmsetzungsModule(atgInfo, daten, result);
      }
    }

    return result;
  }
	  
	private void pruefeDatenAtgTlsAehnlicheKommunikationsPartner(AtgInfo atgInfo, Data daten, MultiStatus result)
	{
		for (String attribut: atgInfo.getAttribute())
  	{
  		MultiStatus	meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert fr Attribut " +
    			attribut, null);
  		IAttributInfo attInfo = atgInfo.getAttributInfo(attribut);
  		meldung.merge(PruefungenTools.pruefeDatenDefinition(attInfo, daten));
  		
  		if (!meldung.matches(IStatus.ERROR | IStatus.CANCEL))
			{
  			if ("KommunikationsPartner".equals(attribut))
    		{
  				MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
    					"Prfe Eindeutigkeit Attribut KommunikationsPartner.Pid", null);
  				
  				HashMap<String, Integer> vorhandeneKP = new LinkedHashMap<String, Integer>();
  				
  				Array array = daten.getItem("KommunikationsPartner").asArray();
  				
  				for (int i = 0; i < array.getLength(); i += 1)
  				{
  					Data data = array.getItem(i);
  					String pid = data.getItem("Pid").asReferenceValue().getSystemObjectPid();
  					
  					Integer wert = vorhandeneKP.get(pid);
  					if (wert == null)
  					{
  						vorhandeneKP.put(pid, 1);
  					}
  					else
  					{
  						vorhandeneKP.put(pid, wert + 1);
  					}
  				}	
    			
  				for (String pid: vorhandeneKP.keySet())
  				{
  					Integer wert = vorhandeneKP.get(pid);
  					if (wert > 1)
  					{
	    				status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
	    						"Der Wert " + pid + " kommt " + wert + "x vor"));
  					}	
  				}
  				meldung.add(status);
    		}
			}	
  		result.add(meldung);
		}
	}
	
	private void pruefeDatenAtgTlsAehnlicheKommunikationsPartnerUmsetzungsModule(AtgInfo atgInfo, Data daten, 
			MultiStatus result)
	{
		for (String attribut: atgInfo.getAttribute())
  	{
  		MultiStatus	meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert fr Attribut " +
    			attribut, null);
  		IAttributInfo attInfo = atgInfo.getAttributInfo(attribut);
  		meldung.merge(PruefungenTools.pruefeDatenDefinition(attInfo, daten));
  		
  		if (!meldung.matches(IStatus.ERROR | IStatus.CANCEL))
			{
  			if ("UmsetzungsModule".equals(attribut))
    		{
  				Array array = daten.getItem("UmsetzungsModule").asArray();
  				
  				MultiStatus typStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
    					"Prfe Eindeutigkeit Attribut UmsetzungsModule.Typ", null);
  				
  				HashMap<String, Integer> vorhandeneTypen = new LinkedHashMap<String, Integer>();
  				
  				for (int i = 0; i < array.getLength(); i += 1)
  				{
  					Data data = array.getItem(i);
  					String typ = data.getItem("Typ").asReferenceValue().getSystemObjectPid();
  					
  					Integer wert = vorhandeneTypen.get(typ);
  					if (wert == null)
  					{
  						vorhandeneTypen.put(typ, 1);
  					}
  					else
  					{
  						vorhandeneTypen.put(typ, wert + 1);
  					}
  				}	
    			
  				for (String typ: vorhandeneTypen.keySet())
  				{
  					Integer wert = vorhandeneTypen.get(typ);
  					if (wert > 1)
  					{
	    				typStatus.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
	    						"Der Wert " + typ + " kommt " + wert + "x vor"));
  					}	
  				}
  				meldung.add(typStatus);
  				
    			MultiStatus modulStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
    					"Prfe Eindeutigkeit Attribut UmsetzungsModule.UmsetzungsModul", null);
  				
  				HashMap<String, Integer> vorhandeneModule = new LinkedHashMap<String, Integer>();
  				
  				for (int i = 0; i < array.getLength(); i += 1)
  				{
  					Data data = array.getItem(i);
  					String modul = data.getItem("UmsetzungsModul").asTextValue().getText();
  					
  					Integer wert = vorhandeneModule.get(modul);
  					if (wert == null)
  					{
  						vorhandeneModule.put(modul, 1);
  					}
  					else
  					{
  						vorhandeneModule.put(modul, wert + 1);
  					}
  				}	
    			
  				for (String modul: vorhandeneModule.keySet())
  				{
  					Integer wert = vorhandeneModule.get(modul);
  					if (wert > 1)
  					{
      				modulStatus.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
      						"Der Wert " + modul + " kommt " + wert + "x vor"));
  					}	
  				}
  				meldung.add(modulStatus);

    		}
			}	
  		result.add(meldung);
		}
	}
	
	
	
//  private boolean pruefeDatenDefinition(IAttributInfo attInfo, Data daten, MultiStatus status)
//  {
//		Data item = daten.getItem(attInfo.getName());
//
//		//dies sollte in den Bildungsregeln erfolgen
////		if (item.isArray() && item.asArray().getLength() > 0)
////		{
////		  Array itemArray = item.asArray();
////		  for (int i = 0; i < itemArray.getLength(); i++)
////      {
////        Data item2 = itemArray.getItem( i );       
////        String valueText = item2.getItem( "PidSender" ).asTextValue().getValueText();
////        
////        System.out.println("PidSender(" + i+ ") setzen! Wert: " + valueText);
////        
////        if (valueText == null || valueText.equals( "undefiniert" ))
////        {
////          System.out.println("Pid: " + _so.getPidOrNameOrId() );
////          item2.getItem( "PidSender" ).asReferenceValue().setSystemObject(_so);
////  		    
////  		    System.out.println(item2.getItem( "PidSender" ).asTextValue().getValueText());
////        }
////      }
////		  
////		}
//		
//		
//		if (!item.isDefined())
//		{
//			status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Wert ist nicht korrekt definiert"));  
//			return false;
//		}
//		else
//		{
//			status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, "Prfen Daten-Definition: OK"));
//		}
//		
//		// Plichtfeld-Prfung:
//		if (item.isPlain())
//		{	
//			if (attInfo.isPflichtFeld())
//			{	
//				String wert = item.asTextValue().getText();
//				if ("".equals(wert) || (!(item.getAttributeType() instanceof StringAttributeType) && 
//						"undefiniert".equals(wert) || "_Undefiniert_".equals(wert)))
//				{
//					status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Wert fr Pflichtfeld nicht angegeben"));  
//					return false;
//				}
//				else
//				{
//					status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, "Prfen Pflichtfeld-Eigenschaft: OK"));
//				}
//			}	
//		}		
//  	return true;
//  }
//  
//  private void pruefeDatenDefinition(AtgInfo atgInfo, Data daten, MultiStatus status)
//  {
//  	for (String attribut: atgInfo.getAttribute())
//  	{
//  		MultiStatus	meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert fr Attribut " +
//    			attribut, null);
//  		IAttributInfo attInfo = atgInfo.getAttributInfo(attribut);
//  		pruefeDatenDefinition(attInfo, daten,  meldung);
//  		status.add(meldung);
//  	}
//  }
   
}
  