/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkaufd;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.regeln.BildungsRegelnBeobachter;
import de.bsvrz.buv.plugin.tkabasis.regeln.BildungsRegelnTools;
import de.bsvrz.buv.plugin.tkabasis.regeln.EingabeVorbelegung;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;

public class BildungsRegelnBWUFD extends BildungsRegelnAllgemeinUFD
{
  protected Map<String, EingabeVorbelegung> _vorbelegungen = new HashMap<String, EingabeVorbelegung>()
  ;
  protected Set<BildungsRegelnBeobachter> _beobachter = new LinkedHashSet<BildungsRegelnBeobachter>();
  
  protected Map<IHierarchieObjektTyp, EingabeVorbelegung> _belegungSensorTyp = new HashMap<IHierarchieObjektTyp, EingabeVorbelegung>();
   
  protected void initVorbelegungen()
  {
    _vorbelegungen.put("Bezeichnung", new EingabeVorbelegung(null, false));
    _vorbelegungen.put("Strae", new EingabeVorbelegung(null, false));
    _vorbelegungen.put("Knotennummer", new EingabeVorbelegung(null, false));
    _vorbelegungen.put("Fahrtrichtung",  new EingabeVorbelegung(new String[] {"N", "O", "S", "W"}));
    _vorbelegungen.put("Fernziel", new EingabeVorbelegung(null, false));
    _vorbelegungen.put("Sensor", new EingabeVorbelegung("HS", new String[] {"HS", "NS1", "NS2", "NS3", "NS4"}));
    _vorbelegungen.put("SensorTyp", new EingabeVorbelegung(null, false));
    _vorbelegungen.put("SensorName", new EingabeVorbelegung("Hauptsensor", false));
  }
  
  @Override
  protected void initPraefixe()
  {
		 _praefixe.put("typ.umfeldDatenMessStelle", "ufdms");
		 
		 UFDSTypenManager ufdsTypenManager = DatenSpeicher.getInstanz().getUFDSTypenManager();
		 for (String typ: ufdsTypenManager.getSystemObjektTypen())
		 {	
		   _praefixe.put(typ, "ufds");
		 }
  }
  
  @Override
  protected void initFelder()
  {
    _felder.put(HOT_UFD.UmfeldDatenMessStelle, new String[] { "Bezeichnung", "Strae", "Knotennummer", "Fahrtrichtung", "Fernziel"});
    _felder.put(HOT_UFD.UFDS, new String[] { "Bezeichnung", "Strae", "Knotennummer" });
    
    UFDSTypenManager ufdsTypenManager = DatenSpeicher.getInstanz().getUFDSTypenManager();
    for (IHierarchieObjektTyp hot: ufdsTypenManager.getUFDSEbene())
    {	
    	_felder.put(hot, new String[] { "Bezeichnung", "Strae", "Knotennummer", "Fahrtrichtung", "Fernziel", 
    			"Sensor", "SensorTyp", "SensorName" });
    }	
  }
  
  @Override
  public void setHierarchie(IHierarchie hierarchie)
	{
  	IHierarchie alteHierarchie = getHierarchie();
  	
  	super.setHierarchie(hierarchie);
  	
  	if (hierarchie != alteHierarchie)
  	{	
  		_vorbelegungen.clear();
	  	initVorbelegungen();
	  	
	  	_beobachter.clear();
	  	
	  	_belegungSensorTyp.clear();
	  	UFDSTypenManager ufdsTypenManager = DatenSpeicher.getInstanz().getUFDSTypenManager();
			for (UFDSUnterTyp unterTyp: ufdsTypenManager.getUFDSEbene())
			{
				_belegungSensorTyp.put(unterTyp, new EingabeVorbelegung(bestimmeSensorTyp(unterTyp), null, false, true));
			}
  	}	
	}
  
  public BildungsRegelnBWUFD()
  {
    super();

//  	initVorbelegungen();
//  	
//  	UFDSTypenManager ufdsTypenManager = DatenSpeicher.getInstanz().getUFDSTypenManager();
//	  
//		for (UFDSUnterTyp unterTyp: ufdsTypenManager.getUFDSEbene())
//		{
//			_belegungSensorTyp.put(unterTyp, new EingabeVorbelegung(bestimmeSensorTyp(unterTyp), null, false, true));
//		}
  }
  
  @Override
  public EingabeVorbelegung getEingabeVorbelegung(IHierarchieObjektTyp hotNeuesObjekt, String eingabeFeld, 
      HierarchieObjekt verbundenesHO)
  {
    if (hotNeuesObjekt.equals(HOT_UFD.UmfeldDatenMessStelle))
    {
      if (Arrays.asList(_felder.get(hotNeuesObjekt)).contains(eingabeFeld))
      {
        return _vorbelegungen.get(eingabeFeld);
      }
    }
    else if (hotNeuesObjekt instanceof UFDSUnterTyp)  
    {
      if (Arrays.asList(_felder.get(hotNeuesObjekt)).contains(eingabeFeld))
      {
         EingabeVorbelegung result = _vorbelegungen.get(eingabeFeld);
        
        if (Arrays.asList(_felder.get(hotNeuesObjekt)).contains(eingabeFeld))
        {
        	if ("SensorTyp".equals(eingabeFeld))
  	  		{	
  	  			return _belegungSensorTyp.get(hotNeuesObjekt);
  	  		}
        	else
        	{
        		String text = null;
        		
	          Map<String, String> werte = Collections.emptyMap();
	          
	          if (verbundenesHO != null)
	          {
	            werte = _historie.get(verbundenesHO.getSystemObjekte().get(0).getPid());
	          }
	          
	          if (werte != null)
	          {
	            text = werte.get(eingabeFeld);
	          }
	          
	          if (text != null && text.length() > 0)
	          {  
	            String[] liste = result.getListe();
	            boolean leerErlaubt = result.isLeerErlaubt();
	            return liste.length > 0 ? new EingabeVorbelegung(text, liste, leerErlaubt) : 
	            	new EingabeVorbelegung(text, leerErlaubt);
	          }
	          else
	          {
	            return result;
	          }
        	} 
        }
      }  
    }
    
    //return super.getEingabeVorbelegung(hotNeuesObjekt, eingabeFeld, verbundenesHO);
    return BildungsRegelnTools.LEERE_VORBELEGUNG;
  }
  
  @Override
  public String getPidRegelText(IHierarchieObjektTyp hot)
  {
  	String result = BildungsRegelnTools.LEERER_WERT;
  	
  	if (hot.equals(HOT_UFD.UmfeldDatenMessStelle))
  	{
  		result = getPraefix(hot.getTypen().get(0))
  				+ ".Strae"
  				+ ".Knotennummer"
  				+ ".Fahrtrichtung";
  	}
  	else if (hot instanceof UFDSUnterTyp)
  	{
  		result = getPraefix(hot.getTypen().get(0))
  		+ ".Strae"
			+ ".Knotennummer"
			+ ".Fahrtrichtung"
			+ ".Sensor"
			+ ".SensorTyp";
		}
  	
  	return result;
  }
  
  @Override
  public String getNameRegelText(IHierarchieObjektTyp hot)
  {
  	String result = BildungsRegelnTools.LEERER_WERT;
  	
  	if (hot.equals(HOT_UFD.UmfeldDatenMessStelle))
  	{
  		result = "UFD Messstelle"
  				+ " Bezeichnung"
  				+ " FR"
  				+ " Fernziel";
  	}
  	else if (hot instanceof UFDSUnterTyp)
  	{
  		result = "UFD"
  			+ " SensorName"
  			+ " Bezeichnung"
  			+ " FR"
				+ " Fernziel"
				+ " SensorTyp";
		}
  	
  	return result;
  }
  
  @Override
  public String[] getPids(IHierarchieObjektTyp hotNeuesObjekt, HierarchieObjekt uebergeordnetesHO)
  {
  	String text = BildungsRegelnTools.LEERER_WERT;
  	
    if (hotNeuesObjekt.equals(HOT_UFD.UmfeldDatenMessStelle))
    {  
      text += getWert(hotNeuesObjekt, "Strae");
      text += ".";
      text += getWert(hotNeuesObjekt, "Knotennummer");
      text += ".";
      text += getWert(hotNeuesObjekt, "Fahrtrichtung");
    }
    else if (hotNeuesObjekt instanceof UFDSUnterTyp)
    {
      text += getWert(hotNeuesObjekt, "Strae");
      text += ".";
      text += getWert(hotNeuesObjekt, "Knotennummer");
      text += ".";
      text += getWert(hotNeuesObjekt, "Fahrtrichtung");
      text += ".";
      text += getWert(hotNeuesObjekt, "Sensor");
      text += ".";
      text += getWert(hotNeuesObjekt, "SensorTyp");
    }
    
    String[] result = new String[1]; 
    
    String pidTypNeuesObjekt = hotNeuesObjekt.getTypen().get(0);
   
    StringBuffer buffer = new StringBuffer();
    
    buffer.append(getPraefix(pidTypNeuesObjekt));
    buffer.append('.');
    
    buffer.append(BildungsRegelnTools.getPidText(text, true, true));
    
    String pid = buffer.toString(); 
    Map<String, String> werte = _feldWerte.get(hotNeuesObjekt);
    if (werte != null && !werte.isEmpty())
    {
      _historie.put(pid, new HashMap<String, String>(werte));
    }
    
    result[0] = pid;
    
    return result; 
  }
  
  public String getName(SystemObject objekt)
  {
  	String result = BildungsRegelnTools.LEERER_WERT;
  	
  	String pid = objekt.getPid();
  	
  	Map<String, String> werte = _historie.get(pid);
  	
  	if (werte != null)
  	{
  		SystemObjectType typ = objekt.getType();
  		StringBuffer buffer = new StringBuffer();
      
      if (typ.getPid().equals("typ.umfeldDatenMessStelle"))
      {
        buffer.append("UFD Messstelle");
        buffer.append(' ');
        buffer.append(werte.get("Bezeichnung"));
        buffer.append(' ');
        buffer.append("FR");
        buffer.append(' ');
        buffer.append(werte.get("Fernziel"));
        
        result = buffer.toString();
      }
      else
      {	
      	ConfigDataModel dataModel = getHierarchie().getDataModel();
      	SystemObjectType deTyp = dataModel.getType("typ.umfeldDatenSensor");
    	
      	if (typ.inheritsFrom(deTyp))
      	{
      		buffer.append("UFD");
      		buffer.append(' ');
          buffer.append(werte.get("SensorName"));
          buffer.append(' ');
          buffer.append(werte.get("Bezeichnung"));
          buffer.append(' ');
          buffer.append("FR");
          buffer.append(' ');
          buffer.append(werte.get("Fernziel"));
          buffer.append(' ');
          buffer.append(werte.get("SensorTyp"));
          
          result = buffer.toString();
      	}
      }
      
  	}
  	
  	return result;
  }
  		
  
  @Override
  public void beobachterAnmelden(BildungsRegelnBeobachter beobachter)
  {
  	_beobachter.add(beobachter);
  }
      
  @Override
  public void beobachterAbmelden(BildungsRegelnBeobachter beobachter)
  {
  	_beobachter.remove(beobachter);
  }
  
  @Override
  public void alleBeobachterInformieren(String feldName, String wert)
  {
  	for (BildungsRegelnBeobachter beobachter: _beobachter)
  	{
  		beobachter.wertGeaendert(feldName, wert);
  	}
  }
  
  @Override
  public void wertGeaendert(String feldName, String wert)
  {
  	if (feldName.equals("Sensor"))
  	{
  		if (wert.equals("HS"))
  		{
  			alleBeobachterInformieren("SensorName", "Hauptsensor");
  		}
  		else if (wert.equals("NS1"))
  		{
  			alleBeobachterInformieren("SensorName", "Nebensensor");
  		}
  		else if (wert.equals("NS2"))
  		{
  			alleBeobachterInformieren("SensorName", "Nebensensor");
  		}
  		else if (wert.equals("NS3"))
  		{
  			alleBeobachterInformieren("SensorName", "Nebensensor");
  		}
  		else if (wert.equals("NS4"))
  		{
  			alleBeobachterInformieren("SensorName", "Nebensensor");
  		}
  		else
  		{
  			alleBeobachterInformieren("SensorName", "");
  		}
  	}
  }
  
  protected String bestimmeSensorTyp(IHierarchieObjektTyp hot)
  {
  	String result = BildungsRegelnTools.LEERER_WERT;
  	String name = hot.toString();  	
  	
  	if (name.equals("UFDS_FahrBahnFeuchte"))
  	{
  		result = "FBF";
  	}
  	else if (name.equals("UFDS_FahrBahnGltte"))
  	{
  		result = "FBG";
  	}
  	else if (name.equals("UFDS_FahrBahnOberFlchenTemperatur"))
  	{
  		result = "FBT";
  	}
  	else if (name.equals("UFDS_FahrBahnOberFlchenZustand"))
  	{
  		result = "FBZ";
  	}
  	else if (name.equals("UFDS_GefrierTemperatur"))
  	{
  		result = "GT";
  	}
  	else if (name.equals("UFDS_Helligkeit"))
  	{
  		result = "HK";
  	}
  	else if (name.equals("UFDS_LuftDruck"))
  	{
  		result = "LD";
  	}
  	else if (name.equals("UFDS_LuftTemperatur"))
  	{
  		result = "LT";
  	}
  	else if (name.equals("UFDS_NiederschlagsArt"))
  	{
  		result = "NS";
  	}
  	else if (name.equals("UFDS_NiederschlagsIntensitt"))
  	{
  		result = "NI";
  	}
  	else if (name.equals("UFDS_NiederschlagsMenge"))
  	{
  		result = "NM";
  	}
  	else if (name.equals("UFDS_RelativeLuftFeuchte"))
  	{
  		result = "RLF";
  	}
  	else if (name.equals("UFDS_RestSalz"))
  	{
  		result = "RS";
  	}
  	else if (name.equals("UFDS_SchneeHhe"))
  	{
  		result = "SH";
  	}
  	else if (name.equals("UFDS_SichtWeite"))
  	{
  		result = "SW";
  	}
  	else if (name.equals("UFDS_TaupunktTemperatur"))
  	{
  		result = "TPT";
  	}
  	else if (name.equals("UFDS_TemperaturInTiefe1"))
  	{
  		result = "TT1";
  	}
  	else if (name.equals("UFDS_TemperaturInTiefe2"))
  	{
  		result = "TT2";
  	}
  	else if (name.equals("UFDS_TemperaturInTiefe3"))
  	{
  		result = "TT3";
  	}
  	else if (name.equals("UFDS_WasserFilmDicke"))
  	{
  		result = "WFD";
  	}
  	else if (name.equals("UFDS_WindGeschwindigkeitMittelWert"))
  	{
  		result = "WGM";
  	}
  	else if (name.equals("UFDS_WindGeschwindigkeitSpitzenWert"))
  	{
  		result = "WGS";
  	}
  	else if (name.equals("UFDS_WindRichtung"))
  	{
  		result = "WR";
  	}
  	
  	return result;
  }
      
}
