/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.lib.einstellungen;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.plugin.lib.einstellungen.SpeicherFunktionen.ArtOrt;

//~ KLASSEN ===================================================================

/**
 * Klasse beschreibt die Art, Ort von Speicher und ID des zu speicherten Parameters.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 1.1 $ / $Date: 2010/05/07 16:03:36 $ / ($Author: Pittner $)
 */
public class SpeicherArtUndOrt {

    /**
     * Array von Art und Ort-Aufzhlungen.
     * Definiert wie und wo die Parameter gespeichert werden.
     * Falls der Speicher nicht synchronisiert wird,
     * definiert auch die Reihenfolge beim Lesen.
     *
     * WICHTIG_: array darf nicht leer sein.
     */
    private ArtOrt[] _arArtOrt;

    /**
     * ID des zu speicherten Parameters.
     */
    private String _parameterID;

    /**
     * Liegt fest, ob der Speicher synchronisiert werden soll.
     * BeimSynchronisieren wird auch das Datum mitgespeichert.
     */
    private boolean _synchronisieren;

    /**
     * Stellt fest, ob Warnungen beim Speichern in 'Error Log' ausgegeben werden sollen.
     */
    private boolean _warnungenAusgeben;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse SpeicherArtUndOrt.
     *
     * @param arArtOrt Array von Art und Ort-Aufzhlungen.
     * WICHTIG_: array darf nicht leer sein.
     *
     * @param synchronisieren Liegt fest, ob der Speicher synchronisiert werden soll.
     *
     * @param parameterID ID des zu speicherten Parameters.
     */
    public SpeicherArtUndOrt(ArtOrt[] arArtOrt, boolean synchronisieren, String parameterID) {
        this(arArtOrt, synchronisieren, parameterID, false);
    }

    /**
     * Erzeugt eine Instanz der Klasse SpeicherArtUndOrt.
     *
     * @param arArtOrt Array von Art und Ort-Aufzhlungen.
     * WICHTIG_: array darf nicht leer sein.
     *
     * @param synchronisieren Liegt fest, ob der Speicher synchronisiert werden soll.
     *
     * @param parameterID D des zu speicherten Parameters.
     *
     * @param warnungenAusgeben Stellt fest, ob Warnungen beim Speichern in 'Error Log' ausgegeben werden sollen.
     */
    public SpeicherArtUndOrt(ArtOrt[] arArtOrt, boolean synchronisieren, String parameterID, boolean warnungenAusgeben) {
        _arArtOrt          = arArtOrt;
        _synchronisieren   = synchronisieren;
        _parameterID       = parameterID;
        _warnungenAusgeben = warnungenAusgeben;
    }

    //~ GET METHODEN ==========================================================

    /**
     * Getter fr Eigenschaft 'arArtOrt'.
     *
     * @return Wert der Eigenschaft 'arArtOrt'.
     *
     * @see #_arArtOrt
     */
    public ArtOrt[] getArArtOrt() {
        return _arArtOrt;
    }

    /**
     * Getter fr Eigenschaft 'parameterID'.
     *
     * @return Wert der Eigenschaft 'parameterID'.
     *
     * @see #_parameterID
     */
    public String getParameterID() {
        return _parameterID;
    }

    /**
     * Getter fr Eigenschaft 'synchronisieren'.
     *
     * @return Wert der Eigenschaft 'synchronisieren'.
     *
     * @see #_synchronisieren
     */
    public boolean isSynchronisieren() {
        return _synchronisieren;
    }

    /**
     * Getter fr Eigenschaft 'warnungenAusgeben'.
     *
     * @return Wert der Eigenschaft 'warnungenAusgeben'.
     *
     * @see #_warnungenAusgeben
     */
    public boolean isWarnungenAusgeben() {
        return _warnungenAusgeben;
    }

    //~ SET METHODEN ==========================================================

    /**
     * Setter fr Eigenschaft 'arArtOrt'.
     *
     * @param arArtOrt Wert zum Setzen der Eigenschaft 'arArtOrt'.
     *
     * @see #_arArtOrt
     */
    public void setArArtOrt(ArtOrt[] arArtOrt) {
        _arArtOrt = arArtOrt;
    }

    /**
     * Setter fr Eigenschaft 'parameterID'.
     *
     * @param parameterID Wert zum Setzen der Eigenschaft 'parameterID'.
     *
     * @see #_parameterID
     */
    public void setParameterID(String parameterID) {
        _parameterID = parameterID;
    }

    /**
     * Setter fr Eigenschaft 'synchronisieren'.
     *
     * @param synchronisieren Wert zum Setzen der Eigenschaft 'synchronisieren'.
     *
     * @see #_synchronisieren
     */
    public void setSynchronisieren(boolean synchronisieren) {
        _synchronisieren = synchronisieren;
    }

    /**
     * Setter fr Eigenschaft 'warnungenAusgeben'.
     *
     * @param warnungenAusgeben Wert zum Setzen der Eigenschaft 'warnungenAusgeben'.
     *
     * @see #_warnungenAusgeben
     */
    public void setWarnungenAusgeben(boolean warnungenAusgeben) {
        _warnungenAusgeben = warnungenAusgeben;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 04.02.10
